<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Calendar
 */

/**
 * A user can select which users to view on the Calendar
 *
 * @package Calendar
 */
class CalendarAccessWebMATEdit extends WebMATEdit {
  
	function beforeDBWrite(&$item, &$fields, &$error_msg) {
		global $USER;
		
		//set previous selected items back to 0  
		$items = $this->db_functions->getItems('', '', "active=1 AND access>0 AND selected>0 AND user_id=" . $USER['id']);
		foreach ($items as $key => $access_item) {
			$access_item['selected'] = 0;
			$access_item['user_id'] = $USER['id'];
			$this->db_functions->setItem($access_item);
		}
		
		$new_item = array();
		//set new selected items back to 1 
		foreach ($item['selected'] as $key => $creator_id) {
			$result = $this->db_functions->getItems('', '', "creator_id=$creator_id AND user_id=" . $USER['id']);
			if ($result) {
				$result = current($result);
				$new_item[$key]['id'] = $result['id'];
				$new_item[$key]['selected'] = 1;
			} 
		}
		$item = $new_item;
	}
  
	/* need to override doSubmit Function */
	function setItem($item = false) {
		if (!$item) {
			$item = $this->item;
		}
		$this->beforeDBWrite($item, $fields, $error_msg);  //passed by reference
		
		$updated = '';
		foreach ($item as $key => $access_item) {
			$updated = $this->db_functions->setItem($access_item); //To add more cases you need to update setItem
			switch ($updated) { // Error catching for updates
				case 'Deleted': //If item has been deleted while editing
				case 'Updated': //If item has been changed while editing
					global $L;
					$this->setErrorMessage('errItem' . $updated);
					return $item['id'];
				break;
				default:
				// Do nothing
				break;
			}
		}
		
		$this->afterDBWrite($item, $fields, $error_msg, $updated);
		return $updated;
	} 
  
	function beforeForm(&$id, &$item, &$error_msg) {
		global $USER;
		//loads data from access table 
		$items = $this->db_functions->getItems('', '', 'creator_id=' . $USER['id']  . ' AND user_id=' . $USER['id']);
		
		if (!$items) {
		   $access_item = array(
				'creator_id' => $USER['id'],
				'user_id' => $USER['id'],
				'access' => 3,
				'selected' => 1
			);
			$this->db_functions->setItem($access_item);
			$item['selected'][] = $access_item['creator_id'];
		}
		
		$items = $this->db_functions->getViewUsers(true); // Phil 20061025 Selected users only
		
		if ($items) {
			foreach ($items as $key => $access_item) {
				$item['selected'][] = $access_item['creator_id'];
			}
		}
	}
	
	/**
	 * Disabled in this class
	 */
	function getDescriptionRow() {
		return false;
	}
	
	/**
	 * Disabled in this class
	 *
	 * @return boolean false
	 * @author Phil Calder
	 */
	function getNextPageRow() {
		return false;
	}

	/**
	 * set the log actions
	 * 
	 * @author Syed, Bosco 
	 */
	function setLogAction() {
		// Bosco 20051128 | Overridden to remove logging 'create'
		if ($this->form_values['submit']) {
			if ($this->form_values['item']['id']) { //Syed 20050831
				$this->db_functions->setLogAction('update');
			}
		} else {
			$this->db_functions->setLogAction('delete');
		}
	}

}
?>