<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 */
 
/**
 * @package Company
 * @author Phil Calder
 */
class CompanyItemWebMATEdit extends WebMATEdit {
	
	
	/**
	 * Disable the contact selector for contact_id
	 */
	function beforeForm(&$id, &$item, &$error_msg) {
		//any other processing before the Form is created (ie. adding extra items)
		// Phil 20051220 Removed - field is now read_only
		// if ($this->form_values['contact_id']) {
			//$this->output->setShowContactSelector(false);
		// }
		if ($this->dd->fieldExists('contact_id')) {
			$this->dd->changeAttributes(array('field_key' => 'contact_id', 'read_only' => 1));
		}
		if ($this->dd->fieldExists('contact_person')) {
			$this->dd->changeAttributes(array('field_key' => 'contact_person', 'contact_company' => $this->form_values['contact_id']));
			$this->dd->changeAttributes(array('field_key' => 'contact_person', 'search_filter' => 'off'));
		}
		
	}
	
	/**
	 * any processing required after adding to the db
	 *
	 * Update display name
	 *
	 * @param array $item
	 * @param array $fields
	 * @param array $error_msg 
	 */
	function afterDBWrite(&$item, &$fields, &$error_msg, $id = false) {  //passed by reference
		$this->updateDisplayName($item);
		// Phil 20051220 Removed - field is now read_only (see beforeForm), no need to reset contact selector setting
		// if ($this->form_values['contact_id']) {
			// $this->output->setShowContactSelector(true);
		// }
	}
	
	/**
	 * any processing required after adding to the db
	 *
	 * Update display name
	 *
	 * @param array $item
	 */
	function afterDelete($item) {
		// print '<p>afterDelete</p>';
		$deleted_item = array('contact_id'=>$item['contact_id']);
		$this->updateDisplayName($deleted_item);
		
	}
	
	/**
	 * any processing required after adding to the db
	 *
	 * Updates the contact display name for this company.
	 *
	 * @param array $item
	 */
	function updateDisplayName($item) {
		global $SERVICES;
		$SERVICES->getService('Modules');
		
		$contact_mod =& $SERVICES->modules->getModule('Contact');
			
		$contact_id = ( (isset($item['contact_id']) && $item['contact_id']) ? $item['contact_id'] : false);
		$contact_mod->updateContactDisplayName($contact_id, $item['display_name']);
		//$contact_mod->updateContactCompany($contact_id);
	}
	
	/**
	 * deletes the contact for this item.
	 */
	function doSubmitDelete() {
		global $SERVICES;
		$SERVICES->getService('Modules');
		if ( 
			($contact =& $SERVICES->modules->getModule('Contact')) && 
			($contact_id = $this->getContactForCompany($this->form_values['delete'])) 
		) {
			
			// $contact->deleteItem($contact_id);
			// print '<p>Redirect to Contact delete page id=' . $contact_id . ' for Company ' . $this->form_values['delete'] . '</p>';
			// $contact_delete_url = '../../modules/Contact/edit.php?delete=' . $contact_id;
			$contact_delete_url = $contact->getContactDeleteLink(true);
			$contact_delete_url = str_replace('%id%', $contact_id, $contact_delete_url);
			header('Location: ' . $contact_delete_url);
			exit();
			
		} else {
			// print '<p>NO LINK id=' . $contact_id . ' for Company ' . $this->form_values['delete'] . '</p>';
			return parent::dosubmitDelete();
		}
	}
	
	/**
	 * any processing required after adding to the db
	 *
	 * Update display name, show contact selector if there is a contact for this company.
	 *
	 * @param integer $id
	 * @return mixed integer contact id for this company, or false
	 */
	function getContactForCompany($id) {
		$item = $this->getItem($id);
		return $item['contact_id'];
	}
	
	/**
	 * Checks for a delete or update, runs the submit function
	 */
	function run() {
		if (!$this->form_values['contact_view'] && !isset($this->form_values['delete'])) {
			if (!isset($this->form_values['id']) || !$this->form_values['id']) {
				if ($new_contact_link = $this->mod->getNewCompanyLink()) {
					header('Location: ' . $new_contact_link);
					exit();
				}
			}
		}
		parent::run();
	}
	
}
?>