<?php
/**
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Contact
 */

/**
 * Displays a contact view list of the current contacts sub-contacts
 *
 * @package Contact
 * @author Phil Calder
 */
class ContactHierarchyWebMATContactView extends WebMATContactView {
	
	/**
	 * @private
	 */
	var $show_sub_contact_view = true;
	
	/**
	 * Determines whether to show sub-contacts
	 *
	 * Used by {@link main}. 
	 * If we arent showing sub-contacts (ie there arent any), main will return an empty string
	 * Calls parent constructor
	 */
	function ContactHierarchyWebMATContactView($module, $data_option, $post, $get) {
		$this->show_sub_contact_view = $this->showSubContactView($get['id']);
		parent::WebMATContactView($module, $data_option, $post, $get);
		// $dd =& $this->getDataDefinition();
		global $L;
		$this->setTitleText($L['contactContactViewChildMany']);
	}
	
	/*
	 * Force refresh
	 *
	 * @param boolean $refresh Ignored - will be set to true
	 * @author Phil Calder
	 */
	// function setDbFunctions($refresh = true) {
		// print '<div>########## ' . get_class($this) . '::setDbFunctions(' . ($refresh ? 'true' : 'false') . ')</div>';
		// // Phil 20060717 We need to force a refresh here to ensure 
		// // the db_function object uses the correct data definition
		// $refresh = true;
		// parent::setDbFunctions($refresh);
	// }

	/**
	 * Either an empty string or the content for this contacts sub-contact table
	 *
	 * Sets title text
	 *
	 * @return string Always returns a string (can be empty)
	 */
	function main($return_string = true) {
		if (!$this->show_sub_contact_view) {
			return '';
		}
		// Phil 20051101 - display_name will become a contact link so we dont want to trim it
		$this->dd->setNoTrimOption(); 
		return parent::main($return_string);
	}
	
	/**
	 * Custom - reorders the contact piece display so that the persons display name is first
	 *
	 * @author Phil Calder
	 */
	function addContactField() {
		global $SERVICES;
		$dd =& $this->getDataDefinition();
		$fields = $this->db_functions->getDataDefinitionFields();
		if (isset($fields['display_name']) && $fields['display_name']) { 
			// $dd->setDictionaryIncludeField($fields[$contact_id_field], true);
			$new_order = array('display_name'=>array());
			$dd->data_dictionary = $this->arrayMergeReplace($new_order, $dd->data_dictionary);
		}
	}
	
	/**
	 * @param integer Contact id
	 * @return boolean true if this is a company
	 */
	function showSubContactView($contact_id) {
		if ($contact_id) {
			$contact =& $this->getContactModule();
			$is_company = $contact->isCompany($contact_id);
			// print '<div>Is this a company [' . $contact_id . '] ' . ($is_company ? 'yes' : 'no') . '</div>';
			return $is_company;
		}
		return false;
	}
	
	/**
	 * Returns the contact module object
	 */
	function &getContactModule() {
		global $SERVICES;
		$SERVICES->getService('Modules');
		$contact =& $SERVICES->modules->getModule('Contact');
		return $contact;
	}
	
	/**
	 * @return object The contact module item database table object
	 */
	function &getContactModuleTable() {
		global $SERVICES;
		$SERVICES->getService('Modules');
		$contact =& $SERVICES->modules->getModule('Contact');
		$db_functions =& $contact->getDatabaseTable('item');
		return $db_functions;
	}
	
	function countItems() {
		if (!$this->show_sub_contact_view) {
			return 0;
		}
	}
	
	/** 
	 * @return integer Always return 0 (no alerts for this piece)
	 */
	function countAlerts() {
		return 0;
	}
	
	/**
	 * Sets design time filter for contacts from a given company based on the current contact
	 */
	function setPieceFilter() {
		// Phil 20070507 Dont set this filter unless viewing a company record
		if (!$this->showSubContactView($this->form_values['id'])) {
			return false;
		}
		if (!isset($this->db_functions)) {
			// $this->db_functions =& $this->mod->getDatabaseTable('item');
			$this->db_functions =& $this->getContactModuleTable();
		}
		if ((isset($this->db_functions) && is_object($this->db_functions)) || (isset($this->mod) && is_object($this->mod))) {
			$this->db_functions->where_statement = false; // Phil 20050708, caching whereStatement does not work here
			$this->db_functions->resetDesignTimeFilters();
			$this->db_functions->setDesignTimeFilters( 
				array(
					'contact_company' => array('where'=> ' (Contact__item.id <> ' . $this->form_values['id'] . ') AND (Contact__item.contact_company = "' . $this->form_values['id']. '")') // Phil 20050708
				) 
				// array( // Phil 20051115 see WebMATDatabaseTable::getDesignTimeFiltersWhere
					// '__special__' => array('where'=> ' (Contact__item.id <> ' . $this->form_values['id'] . ') AND (Contact__item.contact_company = "' . $this->form_values['id']. '")') // Phil 20051115
				// )
			);
		}
	}
	
	function getNewLink() {
		$current = $this->edit_link;
		$this->edit_link = 'new_sub_contact';
		$link = parent::getNewLink();
		$this->edit_link = $current;
		return $link;
	}
	
	/**
	 * 
	 * @param string $data_option the data_option is used to set the data option
	 */
	function setDataDefinition($data_option) {
		// print '<div>' . get_class($this) . '::setDataDefinition(' . $data_option . ')</div>'; print '<hr>';	
		if (is_object($this->db_functions)) {
			global $SERVICES;
			$dd_service =& $SERVICES->getService('DataDefinition');
			$data_definition['fields'] = $dd_service->getFields('Contact__item');
			$data_definition['data_options'] = $dd_service->getOptions();
			$this->db_functions->setDataDefinition($data_definition);
			$this->db_functions->setDataOption($data_option);
		}
		parent::setDataDefinition($data_option);
	}
	
	/**
	 * Whether to show list page 'links' (edit link and menus for each item)
	 *
	 * @return boolean false. Overridden to disable.
	 * @author Phil Calder
	 */
	function showListLinks() {
		return false;
	}
	
	/**
	 * Whether to show list page 'flags' (unseen, edited, alerts)
	 *
	 * @return boolean false. Overridden to disable.
	 * @author Phil Calder
	 */
	function showListFlags() {
		return false;
	}
	
	/**
	 * Disabled
	 *
	 * @return boolean false
	 * @author Phil Calder
	 */
	function getBrowseQueryDataOption() {
		return false;
	}
}
?>