<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 */

/**
 * The Contact module edit class
 * 
 * @package Contact
 * @author Phil Calder
 */
class ContactItemWebMATEdit extends WebMATEdit {
	
	/**
	 * @access private
	 */
	var $contact_permissions = array();
	
	var $container_name = 'contact-edit-container';
	
	/**
	 * Calls parent init
	 * Calls custom add second menu links function for new person and new company
	 */
	function init($module, $data_option, $post = false, $get = false, $table_name = false, $language_overwrite = false) {
		global $SERVICES;
		$modules =& $SERVICES->getService('Modules');
		$this->mod =& $modules->getModule('Contact');
		
		$id = $get['id'];
		if (!$id) { $id = $get['item']['id']; }
		if (!$id) { $id = $post['id']; }
		if (!$id) { $id = $post['item']['id']; }
		if (!$id) { $id = false; }
		
		$contact_company = $get['contact_company'];
		if (!$contact_company) { $contact_company = $get['item']['contact_company']; }
		if (!$contact_company) { $contact_company = $post['contact_company']; }
		if (!$contact_company) { $contact_company = $post['item']['contact_company']; }
		if (!$contact_company) { $contact_company = false; }
		
		$item_arg = false;
		if ($contact_company) {
			// Pass to Contact module methods
			$item_arg = array('contact_company' => $contact_company);
			// Pass to WebMAT object to populate form field
			$get['item']['contact_company'] = $contact_company;
		}
		
		$contact_type = $get['contact_type'];
		$this->mod->default_contact_type = ($contact_type ? 1 : 0);
		
		if ($id && ($data_option == 'custom')) { // see inc/custom_edit.inc
			$language_overwrite['title'] = 'contactAdminCustomFields';
		} else {
			
			if ($contact_type || $this->mod->isCompany($id, $item_arg)) {
				
				$data_option = 'company_edit';
				// TODO: set title text to lang key: contactCompanyTitleNew_itemedit
				$language_overwrite['title'] = ($id ? 'contactCompanyTitle_itemedit' : 'contactCompanyTitleNew_itemedit');
			} else {
				$data_option = 'person_edit';
				// TODO: set title text to lang key: contactPersonTitleNew_itemedit
				$language_overwrite['title'] = ($id ? 'contactPersonTitle_itemedit' : 'contactPersonTitleNew_itemedit');
			}
		}		
		parent::init($module, $data_option, $post, $get, $table_name, $language_overwrite);
	}
	
	/**
	 * Extended to return new company and new person links
	 *
	 * @author Phil Calder
	 * @todo Extend to include file set specific new links for adding an item and a Contact on one form
	 */
	function getNewLinks() {
		$contact_company = false;
		if (isset($this->form_values['id']) && $this->form_values['id']) {
			if ($this->mod->isCompany($this->form_values['id'])) {
				$contact_company = $this->form_values['id'];
			}
		}
		return array(
			'newCompany'   => $this->mod->getNewCompanyLink(), 
			'newPerson' => $this->mod->getNewPersonLink(false, $contact_company)
		);
	}
	
	/**
	 * Bosco 20051129 | Overridden to set default selected Value
	 * gets the next page row 
	 * @return string the next page row for the form
	 * @author Syed
	 */
	function getNextPageRow() {
		return $this->output->getNextPageRow('View');
	}
	
	/**
	 * Redirects to the parent contact view page
	 * @author Phil Calder
	 */
	function redirectToContactView() {
		if (isset($this->form_values['item']['contact_company']) && $this->form_values['item']['contact_company']) {
			global $SERVICES;
			$SERVICES->getService('Modules');
			$SERVICES->getService('Config');
			$contact_mod =& $SERVICES->modules->getModule('Contact');
			$link_href = $SERVICES->config->lookup('modules.Contact.file_set.item.functions.contact_view');
			$link_href = $contact_mod->getModulePath() . $link_href;
			$link_href = str_replace("%id%",$this->form_values['item']['contact_company'],$link_href);
			$link_href = str_replace("%contact_id%",'',$link_href);
			// print $link_href;
			header("location:" . $link_href);
			exit;
		}
	}
	
	/**
	 * Calls parent, then changes data definition based on contact type
	 */
	function setDataDefinition($data_option) {
		parent::setDataDefinition($data_option);
		
		$this->is_person = $this->mod->isPerson($this->form_values['id']);
		$this->is_company = $this->mod->isCompany($this->form_values['id']);
		$this->is_account = $this->mod->isAccount($this->form_values['id']);
		
		// --
		if (!$this->form_values['id']) {
			if (isset($this->form_values['contact_type'])) {
				$this->is_company = ($this->form_values['contact_type'] ? '1' : '0');
			} else {
				global $SERVICES;
				$SERVICES->getService('Preferences');
				$this->is_company = $SERVICES->preferences->getUserPref('contactNewContactType', '0');
			}
			$this->is_person = ($this->is_company ? '0' : '1');
		}
		
		$exclude_fields = $exclude_contact_fields = $exclude_accounts_fields = $exclude_company_fields = array();
		$exclude_contact_fields[] = 'display_name'; // Will either use Company or Person name
		
		if ($this->is_company) {
			$exclude_contact_fields[] = 'contact_company'; // Phil - edit page, DONT allow user to change company
			if (!isset($this->include_person_for_company) || !$this->include_person_for_company) {
				$exclude_person_fields = $this->dd->getFieldsBeginningWith('Person__item'); // ie Person__item.* and Person__item_custom.*
			}
			$exclude_fields = bcrm_array_merge($exclude_contact_fields, $exclude_person_fields);
			
			// Phil 20051221 Contact selector will filter on Company sub contacts
			if ($this->dd->fieldExists('Company__item.contact_person')) {
				$this->dd->changeAttributes(
					array(
						'field_key' => 'Company__item.contact_person', 
						'contact_company' => ($this->form_values['contact_id'] ? $this->form_values['contact_id'] : $this->form_values['id']), 
						'search_filter' => 'off'
					)
				);
			}
			
		} else if ($this->is_person) {
			
			if (!$this->is_account) {
				$exclude_accounts_fields = $this->dd->getFieldsBeginningWith('acc_'); // ie Accounts__item.* and Acounts__item_custom.*
			} else {
				// $exclude_contact_fields[] = 'contact_company'; // Phil - edit page, allow user to change company
			}
			$exclude_company_fields = $this->dd->getFieldsBeginningWith('Company__item'); // ie Company__item.* and Company__item_custom.*
			$exclude_accounts_fields[] = 'contact_account'; // Not used (20051027) but might be in future
			$exclude_fields = bcrm_array_merge($exclude_contact_fields, $exclude_accounts_fields, $exclude_company_fields);
			
			// Phil 20060405 Contact selector will filter on Companies
			if ($this->dd->fieldExists('contact_company')) {
				$this->dd->changeAttributes(array('field_key' => 'contact_company', 'is_person' => '1'));
			}
		}
		$this->dd->setDictionaryExcludeFields($exclude_fields);
	}
	
	/**
	 * Stores the current permissions for the item (to be used later)
	 *
	 * @param array $item
	 * @param array $fields
	 * @param array $error_msg 
	 */
	function beforeDBWrite(&$item, &$fields, &$error_msg) { 
		// Phil - Stores existing permissions for this item 
		// so we can check whether cascadePermissions needs to be run later
		$db_item = $this->db_functions->getItem($item['id']);
		$this->contact_permissions = array(
			'view_group_id'  => $db_item['view_group_id'], 
			'edit_group_id'  => $db_item['edit_group_id'], 
			'owner_group_id' => $db_item['owner_group_id'], 
			'admin_group_id' => $db_item['admin_group_id'], 
			'lock_permissions' => $db_item['lock_permissions']
		);
		$this->pre_submit_item = $item; // includes foreign fields
	}
	
	/**
	 * any processing required after adding to the db
	 *
	 * Calls functions to: 
	 * - Updates display name
	 * - Updates address display names
	 * - processes foreign fields
	 * - check and runs cascading permissions
	 *
	 * @param array $item
	 * @param array $fields
	 * @param array $error_msg 
	 */
	function afterDBWrite(&$item, &$fields, &$error_msg, $id = false) { 
		$this->processForeignFields($this->pre_submit_item);
		$this->processLinkedFields($this->pre_submit_item);
		
		// Phil 20060428
		$id_field = ((isset($item['id']) && $item['id']) ? $item['id'] : $id);
		
		if ($this->form_values['contact_type']) {
			$this->mod->updateContactCompany($id_field, $id_field);
		}
		// print '<div>afterDbWrite id: ' . $id_field . '</div>'; print_r($item); print '<hr>';
		$this->mod->updateContactDisplayName($id_field);
		$this->mod->updatePostalAddressDisplayName($id_field);
		$this->mod->updatePhysicalAddressDisplayName($id_field);
		$this->mod->updateContactPerson($id_field);
		// print '<div>updated display names ' . $id_field . '</div>'; exit();
		if ($this->checkCascadingPermissions($item, $fields, $error_msg, $id)) {
			$this->runCascadingPermissions($item, $fields, $error_msg, $id);
		}
		
		parent::afterDbWrite($item, $fields, $error_msg, $id);
	}
	
	/**
	 * Sets optional deault contact type for new contacts
	 *
	 * @param array $item the item array
	 * @param boolean $default_type Person: false, Company: true
	 * @return array containing the item id and the error message(language key) if an error message is present
	 */
	function setItem($item = false) {
		if (isset($this->form_values['contact_type'])) {
			$this->mod->default_contact_type = ($this->form_values['contact_type'] ? 1 : 0);
		}
		return parent::setItem($item);
	}
	
	/**
	 * Processes the item array for foreign fields
	 *
	 * @param array item - The item array (passed by reference)
	 *
	 */
	function processForeignFields(&$item) {
		if (!isset($item['id']) || !$item['id']) {
			global $SERVICES;
			$db =& $SERVICES->getService('Database');
			$item['id'] = $id = $this->db_functions->last_insert_id;
		}
		
		// Phil - to handle foreign fields
		if (isset($item['id']) && $item['id']) {
			// --
			$data_dictionary = $this->dd->data_dictionary;
			
			$main_contact_id = $item['id']; // Only because this is a contact (ie for other modules, we would probably use item[contact_id])
			
			if (is_array($data_dictionary)) {
				
				global $SERVICES;
				$SERVICES->getService('Config');
				
				$foreign_objects = array(); // Array for foreign objects and items to add
				
				foreach ($data_dictionary as $k => $v) {
					$tmp_object = $tmp_item = '';
					
					// if (isset($v['foreign']) && $v['foreign'] && isset($item[$v['field_key']]) ) {
					// Phil 20050815 - using array_key_exists so we still process blank entries
					if (isset($v['foreign']) && $v['foreign'] && array_key_exists($v['field_key'], $item) ) {
						
						$tmp_table = ( (($pos = strpos($v['field_key'], '.')) !== false) ? substr($v['field_key'], 0, $pos) : $tmp_table);
						
						// Phil 20051029 Remove the custom table suffix if it is there
						if (isset($v['field_is_custom']) && $v['field_is_custom'] == 1) {
							$tmp_table = ( (($pos = strpos($tmp_table, "_custom")) !== false) ? substr($tmp_table, 0, $pos) : $tmp_table);
							$v['field_key'] = str_replace($tmp_table . '.', $tmp_table . '_custom.', $v['field_key']);
						}
						
						$options = $SERVICES->config->lookup('modules.Contact.foreign_fields_edit. ' . $tmp_table);
						
						if (is_array($options)) {
							$module = $options['module'];
							// Phil - Either add this field to an existing object/item, or create a new object and item
							if (isset($foreign_objects[$module]['item']) && $foreign_objects[$module]['item']) {
								//existing (ie more than one field for the same object)
								$foreign_objects[$module]['item'][$v['field_key']] = $item[$v['field_key']];
							} else {
								// new
								$tmp_item['contact_id'] = $main_contact_id;
								$tmp_item[$v['field_key']] = $item[$v['field_key']];
								$class = 'WebMATEdit';
								if ($options['class']) {
									$class = $options['class'];
								}
								
								$args = array();
								if (is_array($options['args'])) {
									$args = $options['args'];
								}
								$args['post'] = $tmp_item; 
								$args['get'] = array();
								if (isset($this->form_values['incl_read_only']) && $this->form_values['incl_read_only']) {
									$args['get']['incl_read_only'] = 1;	
								}
								$tmp_object = getWebMATObject(
									$class, 
									$module, 
									$args
								);
								
								if (is_object($tmp_object)) {
									$foreign_objects[$module] = array(
										'object' => $tmp_object, 
										'item' => $tmp_item
									);
								}
							}
						}
					}
				}
			}
			
			foreach ($foreign_objects as $k => $v) {
				$primary_item = array();
				$primary_item = $v['object']->getPrimaryRecordFor($v['item']['contact_id']);
				if ($primary_item['id']) {
					$v['item']['id'] = $primary_item['id'];
				} else {
					$v['item']['primary_record'] = 1; // Phil 20051007
				}
				// $default_item = $v['object']->getDefaultItem();
				// print_r($default_item); print '<hr>';
				
				$has_content = false;
				foreach ($v['item'] as $k1 => $v1) {
					if ( ($k1 != 'contact_id') && ($k1 != 'primary_record')) {
						if (trim($v1)) {
							$has_content = true;
							break;
						}
					}
				}
				if ($has_content) { // need at least contact id and one other field
					if (method_exists($v['object'], 'setItem')) {
						$v['object']->setItem($v['item']);
					}
				}
			}
		}		
	}
	
	/**
	 * Processes the item array for linked fields
	 *
	 * A linked field is similar to the foreign field, except there is no primary record requirement (or check)
	 * @param array item - The item array (passed by reference)
	 */
	function processLinkedFields(&$item) {
		if (!isset($item['id']) || !$item['id']) {
			global $SERVICES;
			$db =& $SERVICES->getService('Database');
			$item['id'] = $id = $this->db_functions->last_insert_id;
		}
		
		$current_contact_id = $item['id'];
		
		if (isset($item['id']) && $item['id']) {
			$data_dictionary = $this->dd->data_dictionary;
			$main_contact_id = $item['id']; // Only because this is a contact (ie for other modules, we would probably use item[contact_id])
			
			if (is_array($data_dictionary)) {
				global $SERVICES;
				$SERVICES->getService('Config');
				$linked_objects = array(); // Array for linked objects and items to add
				
				foreach ($data_dictionary as $k => $v) {
					$tmp_object = $tmp_item = '';
					
					// Phil 20050815 - using array_key_exists so we still process blank entries
					if (isset($v['linked']) && $v['linked'] && array_key_exists($v['field_key'], $item) ) {
						$tmp_table = substr($v['field_key'], 0, strpos($v['field_key'], '.'));
						if (isset($v['field_is_custom']) && $v['field_is_custom'] == 1) {
							$tmp_table = substr($tmp_table, 0, strpos($tmp_table, "_custom"));
						}
						
						$options = $SERVICES->config->lookup('modules.Contact.foreign_fields_edit. ' . $tmp_table);
						if (is_array($options)) {
							$module = $options['module'];
							// Phil - Either add this field to an existing object/item, or create a new object and item
							if (isset($linked_objects[$module]['item']) && $linked_objects[$module]['item']) {
								//existing (ie more than one field for the same object)
								$linked_objects[$module]['item'][$v['field_key']] = $item[$v['field_key']];
							} else {
								// new
								$tmp_item['contact_id'] = $main_contact_id;
								$tmp_item[$v['field_key']] = $item[$v['field_key']];
								$class = 'WebMATEdit';
								if ($options['class']) {
									$class = $options['class'];
								}
								
								$args = array();
								if (is_array($options['args'])) {
									$args = $options['args'];
								}
								$args['post'] = $tmp_item; 
								$args['get'] = array();
								if (isset($this->form_values['incl_read_only']) && $this->form_values['incl_read_only']) {
									$args['get']['incl_read_only'] = 1;	
								}	
								$tmp_object = getWebMATObject(
									$class, 
									$module, 
									$args
								);
								
								if (is_object($tmp_object)) {
									$linked_objects[$module] = array(
										'object' => $tmp_object, 
										'item' => $tmp_item
									);
								}
							}
						}
					}
				}
			}
			
			foreach ($linked_objects as $k => $v) {
				// $primary_item = array();
				// $primary_item = $v['object']->getPrimaryRecordFor($v['item']['contact_id']);
				// if ($primary_item['id']) {
					// $v['item']['id'] = $primary_item['id'];
				// }
				$has_content = false;
				foreach ($v['item'] as $k1 => $v1) {
					if ($k1 != 'contact_id') {
						if (trim($v1)) {
							$has_content = true;
							break;
						}
					}
				}
				
				if ($has_content) { // need at least contact id and one other field
					if (method_exists($v['object'], 'setItem')) {
						if (strtolower($k) == 'company') {
							// add a new contact
							if ( 
								($new_contact_id = $this->addNewContactForLinkedItem())
								&& (is_object($contact_db_functions =& $this->getModuleDatabaseTable('Contact', 'item'))) 
							) {
								// set new contact contact_company to existing contact_id
								$contact_db_functions->setItem(
									array(
										'id' => $new_contact_id, 
										'contact_company' => $new_contact_id, 
										'contact_account' => $new_contact_id
									)
								);
								// set existing contacts contact_company to new contact_id
								$contact_db_functions->setItem(
									array(
										'id' => $current_contact_id,  // $v['contact_id'], 
										'contact_company' => $new_contact_id
									)
								);
								// change Company contact_id to new contact id
								$v['item']['contact_id'] = $new_contact_id; 
							}
						} else if (strtolower($k) == 'person') {
							if ( 
								($new_contact_id = $this->addNewContactForLinkedItem())
								&& (is_object($contact_db_functions =& $this->getModuleDatabaseTable('Contact', 'item'))) 
							) {
								// set new contact contact_company to existing contact_id
								$contact_db_functions->setItem(
									array(
										'id' => $new_contact_id, 
										'contact_company' => $current_contact_id, 
										'contact_account' => $current_contact_id
									)
								);
								// change Person contact_id to new contact id
								$v['item']['contact_id'] = $new_contact_id; 
							}
						}
						$v['object']->setItem($v['item']);
					}
				} 
			}
		}
	}
	
	/**
	 * Used for linked fields attached to a contacts company or person
	 * @return int $id The new contacts id (evaluates to false / 0 if unsuccessful)
	 */
	function addNewContactForLinkedItem() {
		// return new contact id
		global $SERVICES;
		$db_functions =& $this->getModuleDatabaseTable('Contact', 'item');
		$id = false;
		if (is_object($db_functions)) {
			$default_item = $this->dd->getDefaultItem();
			// $id = $db_functions->setItem(array());
			$id = $db_functions->setItem($default_item);
		}
		return $id;
	}
	
	/**
	 * @deprecated 20051021
	 * @see addNewContactForLinkedItem
	 */
	function addNewContactForCompany() {
		return addNewContactForLinkedItem();
	}
	
	function &getModuleDatabaseTable($module, $table) {
		global $SERVICES;
		$SERVICES->getService('Modules');
		if ($mod =& $SERVICES->modules->getModule($module)) {
			$db_functions =& $mod->getDatabaseTable($table);
			if (is_object($db_functions)) {
				return $db_functions;
			}
		}
		return false;		
	}
	
	/**
	 * Compares the edited item with the original permissions for the item.
	 *
	 * @return boolean Whether permissions have changed and we need to {@link runCascadingPermissions}
	 */
	function checkCascadingPermissions(&$item, &$fields, &$error_msg, $id = false) { 
		// Apply cascading permissions where applicable
		global $SERVICES;
		$SERVICES->getService('Config');
		
		$this->new_permissions = array(
			'view_group_id'  => $item['view_group_id'], 
			'edit_group_id'  => $item['edit_group_id'], 
			'owner_group_id' => $item['owner_group_id'], 
			'admin_group_id' => $item['admin_group_id'], 
		);
		
		if (is_array($this->contact_permissions)) {
			$old_lock_permissions = $this->contact_permissions['lock_permissions'];
			bcrm_unset($this->contact_permissions, 'lock_permissions');
			
			$db_item = $this->db_functions->getItem($item['id']);
			$this->contact_lock_permissions = $db_item['lock_permissions'];
			
			if (!$this->contact_lock_permissions) {
				return false;
			} else if ($this->contact_lock_permissions && !$old_lock_permissions) {
				// The edited contact has had the lock_permissions field set
				return true;
			}
			
			foreach ($this->contact_permissions as $k => $v) {
				if ( isset($this->new_permissions[$k]) && ($this->new_permissions[$k] != $v) ) {
					// The permissions have been changed
					return true;
				}
			}
		}	
		return false;
	}
	
	/** 
	 * Will recurse through other modules updating permissions for items locked to this contact
	 */
	function runCascadingPermissions(&$item, &$fields, &$error_msg, $id = false) {
		global $SERVICES;
		$SERVICES->getService('Log');
		$SERVICES->log->logAction('Contact: Run cascade permissions for contact item ' . $item['id'] . '');
		
		$modules = $SERVICES->config->lookup('modules');
		if (is_array($modules)) {
			foreach ($modules as $k => $v) {
				if (is_object($mod =& $SERVICES->modules->getModule($k))) {
					$file_sets = $v['file_set'];
					// $file_sets = $SERVICES->config->lookup('modules'.$k.'.file_set');
					if (is_array($file_sets)) {
						foreach($file_sets as $fk => $fv) {
							if (
								method_exists($mod, 'getDatabaseTable') 
								&& ($db_functions =& $mod->getDatabaseTable($fk))
							) {
								if (
									$this->contact_lock_permissions 
									|| isset($fv['display_by_group']) && $fv['display_by_group']
								) {
									$db_functions->cascadePermissions($this->new_permissions, $item['id']);
								}
								unset($db_functions);
							}
							
						}
					}
				}
				unset($mod);
			}
		}		
	}
	
	/**
	 * Overridden to do nothing for this class (ie read only fields are allowed)
	 */
	function disableReadOnly() {}
	
	/**
	 * Gets content for form fields
	 *
	 * @param array $item the item array
	 * @param array $error_msg the error message array
	 * @return mixed string or array content
	 * @author Phil Calder, Brenton Bartel
	 */
	function getFieldRows($item, $error_msg) {
		global $L;
		// todo - Phil 2007 - add a method to Output to load styles (similar to adding js files)
		//$incl_css = '<link rel="stylesheet" type="text/css"  href="' . $this->output->getSkinWebPath() . 'css/contact.css">';
		//$this->output->addToHead($incl_css);
		
		$dd_item = $this->dd->getDataStructureWithData($item, $this->getPriorityEdit());
		$dd_item = $this->handleMetaFields($dd_item, $item['id']);
		$dd_item = $this->handleUserFields($dd_item, $item['id']);
		$dd_item = $this->handleOtherFields($dd_item, $item['id']);
		
		$sections = array(
			'contact' => $this->getContactSectionFields(),
			'addresses' => $this->getAddressSectionFields(), 
			'postal' => $this->getPostalAddressSectionFields(), 
			'info' => $this->getMetaSectionFields(), 
		);
		
		$sections['misc'] = $this->getOtherSectionFields($sections); // All fields in data definition not in any other section
		
		$section_content = array();
		if (is_array($dd_item)) {
			foreach ($dd_item as $k => $v) {
				foreach ($sections as $sk => $sv) {
					if (isset($sv[$k])) {
						$split_key = false;
						if (isset($sv['_split_'])) {
							$split_key = $sv['_split_'];
						}
						// if ($k == '_split_') { // Phil 20070718 
						if ($k == $split_key) { // Phil 20070718
							$section_content[$sk]['_split_'] = '
									</table>
								</td>
								<td width="50%">
								<!-- split -->
								<table class="viewTbl-split" border="0" cellpadding="0" cellspacing="0">
							';
						}
						
						$section_content[$sk][$k] = $this->getFieldRow($v, $error_msg[$v['field_key']], $dd_item);
					}
				}
			}
		}
		
		// $content = '';
		// foreach ($content_rows as $k => $v) {
			// $content .= $v;
		// }
		// return $content;
		
		// --- Add content to tabsets ---
		
		// print "\n=============================\n";
		// print_r($sections['contact']);
		// print "\n=============================\n";
		
		$table_content =  '';	
		$table_content .= '
		
		<div id="contactEditTopContainer">
		
		';
		
		$main_tab_title = ($this->getTitleText() ? $this->getTitleText() : 'Contact');
		
		$main_tab = array(
			'id' => 'tabMainContact', 
			'title' => $main_tab_title, 
			'section' => 'contact'
		);
		$main_tabset_name = 'contactEditTopLeftTabs';
		//$this->output->initExt();
		$this->output->initTabs($main_tabset_name);
		$this->output->addTab($main_tab, $main_tabset_name);
		$this->output->setFirstTab($main_tab['id'], $main_tabset_name);
		
		if (is_array($section_content['contact'])) {
			$main_section_content_str = implode("\n", $section_content['contact']);
		} else {
			$main_section_content_str = '';
		}
		
		//viewTbl
		$main_tab_content = '
			<div id="' . $main_tab['id'] . '" class="tab-content">
				<table id="tbl-' . $main_tab['id'] . '" class="editTbl" border="0" cellpadding="0" cellspacing="0">
					' . $main_section_content_str . '
				</table>
			</div>
		';
		$table_content .= '
			<div id="' . $main_tabset_name . '">
				' . 
				$main_tab_content . 
				'
			</div>
		';
		
		// ---
		$tabs = array(
			'addresses' => array('id' => 'tabAddresses', 'title' => $L['contactAddressAddresses']), 
			'postal' => array('id' => 'tabPostalAddresses', 'title' => $L['contactAddressPostalAddresses']), 
			'info' => array('id' => 'tabOther', 'title' => $L['contactOtherFields']), 
			'misc' => array('id' => 'tabMisc', 'title' => $L['contactMiscFields']), 
		);
		
		$tabset_name = 'contactEditTopTabs';
		reset($tabs);
		$current_tab = current($tabs);
		$first_tab = $current_tab['id'];
		$active_tab = false;
		//$this->output->initExt();
		$this->output->initTabs($tabset_name);
		$tabs_content = '';
		foreach ($tabs as $k => $tab) {
			if (isset($section_content[$k]) && is_array($section_content[$k]) && count($section_content[$k])) {
				// print_r(array_keys($section_content[$k])); print '<hr>';
				$tab_content = implode("\n", $section_content[$k]);
				
				if (array_key_exists('_split_', $section_content[$k])) {
					$tab_content = '
					<tr>
						<td width="50%">
							<table class="viewTbl-split" border="0" cellpadding="0" cellspacing="0">
							' . $tab_content . '
							</table>
						</td>
					</tr>
					';
				}
				
				// print '<div>adding tab ' . $k . ':</div>'; print_r($tab); print_r($sections[$k]); print '<hr>';
				if (!$active_tab && (!isset($tab['disable']) || !$tab['disable'])) {
					$active_tab = $tab['id'];
				}
				$this->output->addTab($tab, $tabset_name);
				
				$tabs_content .= '
				<div id="' . $tab['id'] . '" class="tab-content">
					<table id="tbl-' . $tab['id'] . '" class="editTbl" border="0" cellpadding="0" cellspacing="0">
					' . 
					$tab_content . 
					'
					</table>
				</div>
				';
				// --
			}
			
		}
		$this->output->setFirstTab($active_tab ? $active_tab : $first_tab, $tabset_name);
		
		// Phil 20070501 table_content already contains individual div for each tab.
		// We just need to add them to the tabset
		$table_content .= '
		<div id="' . $tabset_name . '">
			' . $tabs_content . '
		</div>
		';
		
		$table_content .= '
		
		</div>
		
		'; //end topContainer
		
		
		// -----------------------------
		
		return $table_content;
	}
	
	/**
	 * Wrapper function for {@link Output::getHtmlFieldRow}
	 * @author Phil Calder
	 */
	function getFieldRow($field, $error_msg, $dd_item) {
		// add to address field label, eg:
		// for email1_content, append the email1_label form field to the email1_content form field
		// dont display email1_label
		
		// 20070816 - just adding label at this stage, width/wrapping, and labelling of additional fields to be resolved
		$map = array(
			'email1_content' => array('email1_label'), //, , 'email1_subscribed'), //, 'email1_valid'), // Dont include this checkbox field here (valid / bounced)
			'email2_content' => array('email2_label'), //, , 'email2_subscribed'), //, 'email2_valid'),
			'email3_content' => array('email3_label'), //, , 'email3_subscribed'), //, 'email3_valid'),
			'phone1_content' => array('phone1_label'),
			'phone2_content' => array('phone2_label'),
			'phone3_content' => array('phone3_label'), 
			'messenger1_username' => array('messenger1_type'), 
			'messenger2_username' => array('messenger2_type')
		);
		
		if (array_key_exists($field['field_key'], $map)) {
			// print '<div>getFieldRow in map: ' . $field['field_key'] . '</div>';
			// if (isset($dd_item[$map[$field['field_key']]]) && $label_field = $this->output->getHtmlField($dd_item[$map[$field['field_key']]])) {
				// // print '<hr><code>' . $label_field . '</code>';
				// $field['extra_field'] = $label_field; // . ' ' . $dd_item[$map[$field['field_key']]]['label'];
			// }
			if (is_array($map[$field['field_key']])) {
				foreach ($map[$field['field_key']] as $k => $v) {
					if (isset($dd_item[$v]) && $extra_field = $this->output->getHtmlField($dd_item[$v])) {
						$field['extra_field'] .= $extra_field;
						// print '<div>extra_field ' . $k . ' = <code>' . $field['extra_field'] . '</code></div>';
					}
				}
			}
			
			
		// } // else if (in_array($field['field_key'], $map)) {
			// print '<div>getFieldRow not in map: ' . $field['field_key'] . '</div>';
			// Dont display the label field separately
			// return;
		// }
		} else {
			foreach ($map as $k => $v) {
				if (is_array($v)) {
					if (in_array($field['field_key'], $v)) {
						// print '<div>getFieldRow dont display: ' . $field['field_key'] . '</div>';
						// Dont display the label field separately
						return;
					}
				}
			}
		}
			
		
		// Otherwise, proceed as normal
		return parent::getFieldRow($field, $error_msg);
	}
	
	function getAddressSectionFields() {
		// Look for fields: email1_content, phone1_content, fax1_content, url1_content
		// TODO: Possibly update dd_item to use [field]_label if it exists
		
		$address_fields = (array)$this->dd->getFieldsBeginningWith('contactby');
		$address_fields = array_merge($address_fields, (array)$this->dd->getFieldsBeginningWith('email'));
		$address_fields = array_merge($address_fields, (array)$this->dd->getFieldsBeginningWith('phone'));
		$address_fields = array_merge($address_fields, (array)$this->dd->getFieldsBeginningWith('fax'));
		$address_fields = array_merge($address_fields, (array)$this->dd->getFieldsBeginningWith('url'));
		// $address_fields = array_merge($address_fields, (array)$this->dd->getFieldsBeginningWith('addrpost'));
		// $address_fields = array_merge($address_fields, (array)$this->dd->getFieldsBeginningWith('addrstreet'));
		$address_fields = array_merge($address_fields, (array)$this->dd->getFieldsBeginningWith('skype'));
		$address_fields = array_merge($address_fields, (array)$this->dd->getFieldsBeginningWith('messenger'));
		
		
		$section_fields = array_flip($address_fields);
		return $section_fields;
	}
	
	function getPostalAddressSectionFields() {
		// Look for fields: addrpost%, addrstreet%
		// TODO: Possibly update dd_item to use [field]_l if it exists
		
		$address_fields = (array)$this->dd->getFieldsBeginningWith('addrpost');
		// $address_fields = array_merge($address_fields, (array)'_split_');
		$address_fields = array_merge($address_fields, array('addrstreet1_f' => '_split_'));
		$address_fields = array_merge($address_fields, (array)$this->dd->getFieldsBeginningWith('addrstreet'));
		
		$section_fields = array_flip($address_fields);
		return $section_fields;
	}
	
	function getMetaSectionFields() {
		// Exclude Permissions fields -- included elsewhere
		$permissions_fields = $this->dd->getPermissionsFields();
		if (sizeof($permissions_fields)) {
			$permissions_fields[5] = '_permission_controls';
		}
		$other_meta_fields = array(
			'id',
			'creator_id',
			'legacy_id', 
			'unique_id', 
			'import_id'
		);
		$meta_fields = array_merge($other_meta_fields, (array)$this->dd->getMetaFields()); 
		
		$section_fields = bcrm_array_merge(
			array_flip($meta_fields), 
			array_flip($permissions_fields)
		);
		return $section_fields;
	}
	
	function getContactSectionFields() {
		$account_fields = $this->dd->getFieldsBeginningWith('acc_');
		
		if ($this->is_person) {
			$contact_fields = (array)$this->dd->getFieldsBeginningWith('Person__item');
			$contact_fields = array_merge($contact_fields, (array)$this->dd->getFieldsBeginningWith('contact_company'));
		} else {
			$contact_fields = (array)$this->dd->getFieldsBeginningWith('Company__item');
		}
		
		$custom_fields = $this->dd->getCustomFields();
		
		// Adding permissions fields
		// $permissions_fields = $this->dd->getPermissionsFields();
		// if (sizeof($permissions_fields)) {
			// $permissions_fields[5] = '_permission_controls';
		// }
		// print_r($permissions_fields);
		$section_fields = bcrm_array_merge(
			array_flip($contact_fields), 
			array_flip($account_fields), 
			array_flip($custom_fields)//, 
			// array_flip($permissions_fields)
		);
		// print_r($section_fields);
		
		return $section_fields;
	}
	
	function getOtherSectionFields($exclude = array()) {
		$all_fields = (array)$this->dd->getFieldsBeginningWith();
		$section_fields = array_flip($all_fields);
		// print '<div>all_fields:</div>'; print_r($all_fields); print '<hr>';
		// print_r($exclude); print '<hr>';
		if (is_array($exclude)) {
			foreach ($exclude as $k => $v) {
				if (is_array($v)) {
					foreach ($v as $kk => $vv) {
						if (isset($section_fields[$kk])) {
							// print '<div>removing field: ' . $kk . '</div>';
							unset($section_fields[$kk]);
						}
					}
				}
			}
		}
		// print '<div>section_fields:</div>'; print_r($section_fields); print '<hr>';
		return (array)$section_fields;
	}
	
	/**
	 * Wraps content in a container.
	 *
	 * @param array $item An item array (optional for view pages)
	 * @param array $error_msg error message (optional for view pages)
	 * @return string Content
	 * @author Phil Calder
	 */
	function getDataContent($item, $error_msg) {
		$this->container_not_table = true;
		$content = '';
		//$content .= $this->getBeginContainer();
		$content .= parent::getDataContent($item, $error_msg);
		//$content .= $this->getEndContainer();
		// print '<hr><pre>' . $content . '</pre><hr>';
		// print $this->container_name . '<hr>';
		return $content;
	}
	
	/**
	 * Adds duplicates and download vCard to setup menu in the toolbar to save space
	 *
	 * Adds a Company Depth link based on user preference
	 *
	 * @param array $item the item array
	 * @return array an array of links
	 */
	function getTableLinks($item) {
		$links = (array)WebMATEdit::getTableLinks($item);
		
		if (isset($links['contactNestedDropdownValue']) && is_array($links['contactNestedDropdownValue'])) {
			$links['contactNestedDropdownValue']['icon'] = 'page_misc.png';
			$links['contactNestedDropdownValue']['menu'] = array();
		}
		
		if (isset($links['contactNestedDropdownValue']['menu']) && is_array($links['contactNestedDropdownValue']['menu'])) {
			if (isset($links['contactCheckForDuplicates']) && is_array($links['contactCheckForDuplicates'])) {
				$links['contactNestedDropdownValue']['menu']['contactCheckForDuplicates'] = $links['contactCheckForDuplicates'];
				bcrm_unset($links, 'contactCheckForDuplicates');
			}
			if (isset($links['contactLinkExportVcard']) && is_array($links['contactLinkExportVcard'])) {
				$links['contactNestedDropdownValue']['menu']['contactLinkExportVcard'] = $links['contactLinkExportVcard'];
				bcrm_unset($links, 'contactLinkExportVcard');
			}
		}
		return $links;
	}
}
?>