<?php
require_once("../../prepend.inc");
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 */

includeWebMAT('mass_edit.inc');

/**
 *  Handles mass edit and delete functionality for the Contact module
 * 
 * @package Contact
 * @author Phil Calder
 */
class ContactItemWebMATMassEdit extends WebMATMassEdit {
	var $contact_permissions = array();
	
	/**
	 * If this is a delete, checks that the user can delete contacts
	 *
	 * See Contact module user preferences
	 */
	function run() {
		global $SERVICES;
		$SERVICES->getService('Preferences');
		if (isset($this->form_values['delete']) && !($allow_delete = $SERVICES->preferences->getUserPref('contactAllowContactDelete'))) {
			$this->redirectBack();
			exit();
		}
		parent::run();
	}
	
	/**
	 * Stores permissions field for the item
	 *
	 * @see checkCascadingPermissions
	 *
	 * @param array $item
	 * @param array $fields
	 * @param array $error_msg 
	 */
	function beforeDBWrite(&$item, &$fields, &$error_msg) {  //passed by reference
		$db_item = $this->db_functions->getItem($item['id']);
		$this->contact_permissions = array(
			'view_group_id'  => $db_item['view_group_id'], 
			'edit_group_id'  => $db_item['edit_group_id'], 
			'owner_group_id' => $db_item['owner_group_id'], 
			'admin_group_id' => $db_item['admin_group_id'], 
			'lock_permissions' => $db_item['lock_permissions']
		);
	}
	
	/**
	 * any processing required after adding to the db
	 *
	 * @param array $item
	 * @param array $fields
	 * @param array $error_msg 
	 */
	function afterDBWrite(&$item, &$fields, &$error_msg, $id = false) {  //passed by reference
		$this->mod->updateContactDisplayName($item['id']);
		if ($this->checkCascadingPermissions($item, $fields, $error_msg, $id)) {
			$this->runCascadingPermissions($item, $fields, $error_msg, $id);
		}
		parent::afterDbWrite($item, $fields, $error_msg, $id);
	}
	
	function checkCascadingPermissions(&$item, &$fields, &$error_msg, $id = false) {  //passed by reference
		// Apply cascading permissions where applicable
		global $SERVICES;
		$SERVICES->getService('Config');
		
		$this->new_permissions = array(
			'view_group_id'  => $item['view_group_id'], 
			'edit_group_id'  => $item['edit_group_id'], 
			'owner_group_id' => $item['owner_group_id'], 
			'admin_group_id' => $item['admin_group_id'], 
		);
		
		if (is_array($this->contact_permissions)) {
				
			$old_lock_permissions = $this->contact_permissions['lock_permissions'];
			bcrm_unset($this->contact_permissions, 'lock_permissions');
			
			$db_item = $this->db_functions->getItem($item['id']);
			$this->contact_lock_permissions = $db_item['lock_permissions'];
			
			if (!$this->contact_lock_permissions) {
				return false;
			} else if ($this->contact_lock_permissions && !$old_lock_permissions) {
				// The edited contact has had the lock_permissions field set
				return true;
			}
			
			foreach ($this->contact_permissions as $k => $v) {
				if ( isset($this->new_permissions[$k]) && ($this->new_permissions[$k] != $v) ) {
					// The permissions have been changed
					return true;
				}
			}
		}	
		return false;
	}
	
	function runCascadingPermissions(&$item, &$fields, &$error_msg, $id = false) {
		global $SERVICES;
		
		$SERVICES->getService('Log');
		$SERVICES->log->logAction('Contact: Run cascade permissions for contact item ' . $item['id'] . '');
		
		$modules = $SERVICES->config->lookup('modules');
		if (is_array($modules)) {
			foreach ($modules as $k => $v) {
				if (is_object($mod =& $SERVICES->modules->getModule($k))) {
					$file_sets = $v['file_set'];
					if (is_array($file_sets)) {
						foreach($file_sets as $fk => $fv) {
							if (
								method_exists($mod, 'getDatabaseTable') 
								&& ($db_functions =& $mod->getDatabaseTable($fk))
							) {
								if (
									$this->contact_lock_permissions 
									|| isset($fv['display_by_group']) && $fv['display_by_group']
								) {
									$db_functions->cascadePermissions($this->new_permissions, $item['id']);
								}
								unset($db_functions);
							}
							
						}
					}
				}
				unset($mod);
			}
		}		
	}
}
?>