<?php
/**
 * The import file browser/upload form
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package ContactImport
 * @author Phil Calder
 */
class ContactImportLoadWebMATEdit extends WebMATEdit {
	
	function init($module, $data_option, $post = false, $get = false, $table_name = false, $language_overwrite = false) {
		
		// Phil 20060606
		if (!is_array($language_overwrite)) {
			$language_overwrite = array('title' => 'contactImportTitle_itemload');
		} else if (!array_key_exists('title', $language_overwrite)) {
			$language_overwrite['title'] = 'contactImportTitle_itemload';
		}
		
		parent::init($module, $data_option, $post, $get, $table_name, $language_overwrite);
		
		$this->alterDataDefinition();
		$this->addFileUploadField();
	}
	
	// Gurdeep Singh [20050828] overrided a function to remove "Next Page" field from this page
	function getNextPageRow() {
		return '';
	}
	
	// Gurdeep Singh [20050828] overrided a function to remove "Make thisitem private" field from this page
	function addMakePermissionsPrivate() {
		return '';
	}

	
	function alterDataDefinition() {
		global $L;
		
		$read_only_fields = array(
			'view_group_id', 'edit_group_id', 'owner_group_id', 'admin_group_id'
		);
		foreach($this->dd->data_dictionary as $k => $v) {
			
			if (in_array($this->dd->data_dictionary[$k]['field_key'], $read_only_fields)) {
				$this->dd->data_dictionary[$k]['read_only'] = 1;
			}
			
			// Bosco 20060219 | Replaced lang keys with values
			if ($v['type'] == 'dropdown' && is_array($v['dropdown_values']) && !empty($v['dropdown_values'])) {
                                foreach ($v['dropdown_values'] as $key => $val) {
                                        if (!empty($L[$val])) {
                                                $this->dd->data_dictionary[$k]['dropdown_values'][$key] = $L[$val];
                                        }
                                }
                        }

			
			// Gurdeep [20050823] written a code to remove standalone_companies from the form
			bcrm_unset($this->dd->data_dictionary, 'standalone_companies');
			// print_r($this->dd->data_dictionary[$k]); print '<hr>';
		}

		//MCR 20060710 (for patch only)
		$this->dd->changeAttributes(array('field_key' => 'title', 'explanation' => $L['contactImportFieldTitleExplanation']));
		
	}
	
	function run() {
		if (!isset($this->form_values['submit']) || !$this->form_values['submit']) {
			
			$this->store = new SessionStore(); // new, not reloaded
			$this->store->hardDeleteStore();
			
			// print '<p>run - empty store</p>';
			// print_r($_SESSION);
		}
		
		parent::run();
	}
	
	function doSubmitProcess() {
		$this->loadData();
	}
	
	/**
	 * Adds the non-database file upload field to our data definition
	 */
	function addFileUploadField() {
		
		global $L;
		$this->dd->data_dictionary[] = array(
			'field_key' => 'import_file', 
			'type' => 'file_upload', 
			'field_priority' => 999, 
			'explanation' => $L['contactImportFileUploadExplanation'], 
			'label' => $L['contactImportFileUpload'],
			
			//MCR 20060710 tried to add notnull but validation fails when a file IS selected
			//'notnull' => 1
			
			// 'get_content' => 1
		);
	
		$get_mapping = false;
		
		$count = $this->getCountTemplates();
		if ($count > 0) {
			$get_mapping = true;	
		}
		
		$this->dd->data_dictionary[] = array(
			'field_key' => 'get_mapping', 
			'type' => 'checkbox',
			'field_priority' => 786,
			'priority_edit' => 786,
			'explanation' => 'Select existing mapping template',
			'label' => $L['contactImportUseSavedMapping'],
			'checked' => $get_mapping
			// 'get_content' => 1
		);

	}
	
	function getCountTemplates() {
		global $SERVICES;
		$db =& $SERVICES->getService('Database');
		$id = $this->form_values['id'];
		
		$count = $db->quickGet('count(*)', "ContactImport__field_mapping", "template_id = " . $id);
		
		return $count;
	}
	
	/**
         * Gets the Char array from module.config and returns the array
         * @param String $array_name Name of the char array
         * @return Array $chars Array of Characters
         * @author Bosco
         */
        function getChars($array_name = '') {
                global $SERVICES;
                $chars = array();
                if (!empty($array_name)) {
                        $chars = $SERVICES->config->lookup('modules.ContactImport.' . $array_name);
                }
                return $chars;
        }
	
	function loadData() {
		global $L;
		$item = $this->getItemVariable();
		// Bosco 20060219 | Commented since we take values from module.config
		/*switch ($item['file_type']) { // temporary - should lookup dropdown table values
			case '3': 
			$line_terminator = '\r';
			break;
			case '2': 
			$line_terminator = '\n';
			break;
			case '1': 
			default:
			$line_terminator = '\r\n';
			break;
			
		}*/
	
		// Bosco 20060219 | Values coming from module.config	
		// Get Line Terminator (File Type)	
		$line_terminator_array = $this->getChars('line_terminate_chars');
		if (!($file_type_index = $item['file_type'])) {
                        $file_type_index = $SERVICES->preferences->getUserPref('contactImportFileType');
                }
		$line_terminator = $line_terminator_array[$file_type_index];
		// Get Field Terminator
		$field_terminator_array = $this->getChars('field_terminate_chars');
		if (!($field_terminator_index = $item['field_terminator'])) {
                        $field_terminator_index = $SERVICES->preferences->getUserPref('contactImportFieldTerminator');
                }
		$field_terminator = $field_terminator_array[$field_terminator_index];

		// Get Field Enclosed By Char
		$field_enclosed_array = $this->getChars('field_enclose_chars');
		if (!($field_enclosed_index = $item['field_enclosed_by'])) {
                        $field_enclosed_index = $SERVICES->preferences->getUserPref('contactImportFieldEnclosedBy');
                }
		$field_enclosed_by = $field_enclosed_array[$field_enclosed_index];
	
		// Get Field Escaped By Char
		$field_escaped_array = $this->getChars('field_escape_chars');
		if (!($field_escaped_index = $item['field_escaped_by'])) {
						$field_escaped_index = $SERVICES->preferences->getUserPref('contactImportFieldEscapedBy');
		}
		$field_escaped_by = $field_escaped_array[$field_escaped_index];

		$file_loader = new FileLoader(
			$inStore, 
			$item['import_file']['location'], 
			$line_terminator, 
			$field_terminator, 
			$item['has_header_row'], 
			$field_enclosed_by, 
			$field_escaped_by,
			$item['use_load_data_infile']
		);
		
		//print '<p>fileloader created</p>';
		
		if (file_exists($item['import_file']['location']) 
		&& is_readable($item['import_file']['location'])) {
			
			$error_msg = $file_loader->runLoad($item['import_file']['location']);
			
		} else {
			$error_msg = $L['contactImportErrorNoImportFileSpecified'];
			
		}
		$id = $item['id'];
		$count = $this->getCountTemplates();
		if (!$error_msg) {
			
			// $table_name = $file_loader->getTableName();
			if (isset($id) && $count > 0 && isset($this->form_values['item']['get_mapping'])) {
				header('Location: map_fields.php?id=' . $id . '');
				//MCR 20060711, let's always go to mapping
				//header('Location: save_mapping.php?id=' . $id . '');
				exit();
			} else {
				header('Location: map_fields.php?id=' . $id . '');
				exit();
			}
			
		} else {
			
			quickPage($error_msg);
			exit();
			
		}
		
		
	}
	
	function getExtraItemLinks() {
		global $L;
		$links = parent::getExtraItemLinks();
		
		if ($this->getFileSetHref($this->edit_link)) {
			$links['edit1'] = $this->getEditLink();
		}
		
		// $links['loadFile'] = $this->output->getLinkHref(
			// array(
				// 'href' => str_replace('%id%', $id, $this->getFileSetHref($this->load_file_link)), 
				// 'text' => $L['contactImportLoadFileLink']
			// )
		// );
		
		return $links;
	}
	
}
?>