<?php
/**
 * Holds the current mapping.
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package ContactImport
 */

/**
 * Holds the current mapping.
 *
 * This is a list of the mapping, from their fields to ours
 * We also store related information, like date formats, date seperators, name
 * Also the list of columns mapping from (user fields) and to (our field names)
 *
 * @package ContactImport
 */
class Mapping {
	
	/**
	 * Constructor method
	 * Gets the current session storage
	 * Should be passed a reference to the storage object
	 */
    function Mapping(&$inStore, $inID = -1, $name = 'map', $inMapping = Array(), $inValueMapping = Array(), $inMapFrom = Array(), $inMapTo = Array(), $inDateFormat = 'default', $inDateSeperator = '/', $inAddressLabels = Array(), $table_name = '') {
    	global $SERVICES;	
		$this->db =& $SERVICES->getService('Database');
		
		if(!$inStore){
			$this->store = new SessionStore();
		} else {
			$this->store = $inStore;
		}
		
		if($inStore) {
			// IF we are passed a store, get the values from there
			$this->mapName = $this->store->getValue('mapping_name');
			$this->mapID = $this->store->getValue('mapping_id');
			$this->mapping = $this->store->getValue('mapping');
			$this->setValueMapping = $this->store->getValue('setValueMapping');
			$this->mapFrom = $this->store->getValue('mapping_from');
			$this->mapTo = $this->store->getValue('mapping_to');
			$this->dateFormat = $this->store->getValue('date_format');
			$this->dateSeperator = $this->store->getValue('date_seperator');
			$this->addressLabels = $this->store->getValue('address_labels');
			$this->table_name = $this->store->getValue('table_name');
		}
			
	    if(!$this->mapName){
	    	$this->mapName = $name;
	    	$this->store->addValue('mapping_name', $this->mapName);
	    }
	    if(!$this->mapID){
	    	$this->mapID = $inID;
	    	$this->store->addValue('mapping_id', $this->mapID);
	    }
	    if(!$this->mapping){
		    $this->mapping = $inMapping;
		    $this->store->addValue('mapping', $this->mapping);
			}
	    if(!$this->setValueMapping){
		    $this->setValueMapping = $inValueMapping;
		    $this->store->addValue('setValueMapping', $this->setValueMapping);
			}
			if(!$this->mapFrom){
				$this->mapFrom = $inMapFrom;
				// print '<div>map from</div>'; print_r($inMapFrom); print '<hr>';
				$this->store->addValue('mapping_from', $this->mapFrom);
			}
			if(!$this->mapTo){
				$this->mapTo = $inMapTo;
				// print '<div>map to</div>'; print_r($inMapTo); print '<hr>';
				$this->store->addValue('mapping_to', $this->mapTo);
			}
			if(!$this->dateFormat){
				if ($inDateFormat == 'default')
					global $SERVICES;
					$SERVICES->getService('Preferences');
					$inDateFormat = $SERVICES->preferences->getUserPref('cfgShortDateFormat');
				//echo $inDateFormat;
				$this->dateFormat = $inDateFormat;
				$this->store->addValue('date_format', $this->dateFormat);
			}
			if(!$this->dateSeperator){
				$this->dateSeperator = $inDateSeperator;
				$this->store->addValue('date_seperator', $this->dateSeperator);
			}
			if(!$this->addressLabels){
				$this->addressLabels = $inAddressLabels;
				$this->store->addValue('address_labels', $this->addressLabels);
			}
			if(!$this->table_name){
	    	$this->table_name = $table_name;
	    	$this->store->addValue('table_name', $this->table_name);
	    }
	    
    }
    
    /**
	 * TrashMapping method
	 * Saves the current session storage
	 * WARNING: Must be called if the session variables are to be saved
	 */
    function trashMapping() {
    	$this->store->trashStore();
    }
    
    /**
	 * Methods to set the required settings
	 */
    function setStore(&$inStore){
    	if(!$inStore)
			$inStore = new SessionStore();
				
		$this->store = $inStore;
	}
		
	function setMapName($name = 'map'){
		$this->mapName = $name;
		$this->store->addValue('mapping_name', $this->mapName);
	}
	
	function setMapID($inID = -1){
		$this->mapID = $inID;
		$this->store->addValue('mapping_id', $this->mapID);
	}
	
	function setMapping($inMap = Array()){
		$this->mapping = $inMap;
		$this->store->addValue('mapping', $this->mapping);
	}
	
	function setValueMapping($inMap = Array()){
		$this->setValueMapping = $inMap;
		$this->store->addValue('setValueMapping', $this->setValueMapping);
	}
	
	function setColMapFrom($inMap = Array()){
		$this->mapFrom = $inMap;
		// print '<div>map from</div>'; print_r($inMap); print '<hr>';
		$this->store->addValue('mapping_from', $this->mapFrom);
	}
	
	function setColMapTo($inMap = Array()){
		$this->mapTo = $inMap;
		// print '<div>map to</div>'; print_r($inMap); print '<hr>';
		$this->store->addValue('mapping_to', $this->mapTo);
	}
	
	function setDateFormat($inForm = 'dd mm yyyy'){
		$this->dateFormat = $inForm;
		$this->store->addValue('date_format', $this->dateFormat);
	}
	
	function setDateSeperator($inSep = '/'){
		$this->dateSeperator = $inSep;
		$this->store->addValue('date_seperator', $this->dateSeperator);
	}
	
	// function setAddressLabels($inAddressLabels = Array()){
		// $this->addressLabels = $inAddressLabels;
		// $this->store->addValue('address_labels', $this->addressLabels);
	// }
	
	function setTableName($table_name = ''){
		$this->table_name = $table_name;
		$this->store->addValue('table_name', $this->table_name);
	}
		
	/**
	 * Methods to get the required settings
	 */
	function getStore(){
		return $this->store;
	}
	
	function getMapName(){
		return $this->mapName;
	}
	
	function getMapID(){
		return $this->mapID;
	}
	
	function getMapping(){
		return $this->mapping;
	}
	
	function getValueMapping(){
		return $this->setValueMapping;
	}
	
	function getColMapFrom(){
		return $this->mapFrom;
	}
	
	function getColMapTo(){
		return $this->mapTo;
	}
	
	function getDateFormat(){
		return $this->dateFormat;
	}
	
	function getDateSeperator(){
		return $this->dateSeperator;
	}
	function getAddressLabels(){
		return $this->addressLabels;
	}
	function getTableName(){
		return $this->table_name;
	}
	
	/**
	 * GetUnmappedTo method
	 * Returns a list of the fields we have not mapped to
	 * Assumes we have a mapping and a list of fields we are mapping to
	 */
	function getUnmappedTo(){
		$unmapped = array();
		
		foreach($this->mapTo as $k => $map_to){
			if(!in_array($map_to, $this->mapping)) {
				// $unmapped[$map_to] = true;
				$unmapped[$k] = true;
			}
		}
		
		// print'<div>Mapping::getUnmappedTo: </div>'; print_r($unmapped); print '<hr>';
		
		return $unmapped;
	}
		
	/**
	 * DeleteMapping method
	 * Deletes the mapping from the database
	 * WARNING: THIS assumes certain variables are set in the store
	 * IF not, then the mapping will not be saved correctly
	 */
    function deleteMapping($newID = -1){
		
		if($newID > -1){
			$error = $this->setMapID($newID);
			if ($error) {
				return $error;
			}
		}
			
		$this->db->update('Import__template', array("active" => "0"), "id='$this->mapID'"); 
	    
	    // Return with an error message if necessary		
		if($this->db->hasError()) {
			return $this->db->getErrorString();
		}
	}
		
	/**
	 * DeleteAll method
	 * Deletes the mapping children from the database
	 * THIS means we delete all the field_mapping and set_values
	 * From the databases, where the template id is the same as our current one
	 */
	function deleteAll($newID = -1){
		if($newID > -1){
			$error = $this->setMapID($newID);
			if ($error) {
				return $error;
			}
		}
			
		$this->db->delete('Import__field_mapping', "template_id='$this->mapID'");
		// Return with an error message if necessary		
		if($this->db->hasError()) {
			return $this->db->getErrorString();
		}
				
		$this->db->delete('Import__set_values_mapping', "template_id='$this->mapID'");
	    // Return with an error message if necessary		
		if($this->db->hasError()) {
			return $this->db->getErrorString();
		}
	}
	
	/**
	 * LoadMapping method
	 * Loads the mapping from the database
	 */
	function loadMapping($newID = -1, $fileName = ''){
		
		// print '<div>load mapping</div>';

		if($newID > -1){
			$error = $this->setMapID($newID);
			if ($error)
				return $error;
		}
		
		// Get the name
		$row = $this->db->getRow($this->db->select('*','Import__template', "id='$this->mapID'"));
			
		// Save the mapping name
		$this->setMapName($row['name']);
			
		// Save the address labels
		// $error = $this->setAddressLabels(unserialize($row['address_labels']));
			
		// Load the From and To arrays
		$this->loadMapTo();
		$this->loadMapFrom();
	
		$result = $this->db->select('*','Import__field_mapping',"template_id=$this->mapID");
			
		// The mappings of our current template
		while ($row = $this->db->getRow($result))
			$map[$row['external_name']] = $this->mapTo[$row['crm_field_id']];
		
		$this->setMapping($map);
		
		// Construct the set values
		$result = $this->db->select('*','Import__set_values_mapping',"template_id=$this->mapID");
		
		// The columns of our CRM tables
		while ($row = $this->db->getRow($result))
			$set_values[$row['crm_field_id']] = $row['set_value'];
		
		$this->setValueMapping($set_values);
		
		// Restore the date_standard and date_seperator
		$template = $this->db->getRow($this->db->select('*','Import__template',"id=$this->mapID"));
		
		$error = $this->setDateFormat($template['date_format']);
		if ($error)
			return $error;
			
		$error = $this->setDateSeperator($template['date_seperator']);
		if ($error)
			return $error;
			
		// Restore the other parameters to our loader class
		$loader = new FileLoader($this->store);
		
		$error = $loader->setFilename($fileName);
		if ($error)
			return $error;

		$error = $loader->setHeaderRow($template['header_row']);
		if ($error)
			return $error;
		
		$error = $loader->setLineTerminator($template['line_terminator']);
		if ($error)
			return $error;
			
		$error = $loader->setEnclosed($template['tmp_enclosed']);
		if ($error)
			return $error;
		
		$error = $loader->setEscaped($template['tmp_escaped']);
		if ($error)
			return $error;
		
		$error = $loader->setFieldTerminator($template['field_terminator']);
		if ($error)
			return $error;
			
		// Lastly save the variables used
		$loader->trashFileLoader();
		$newStore = new SessionStore(1);
		$this->setStore($newStore);
	}
		
	/**
	 * LoadMapFrom method
	 * Loads the mappingFrom array from the database
	 */
	function loadMapFrom($tableName = ''){
		
		if($tableName == '')
			$tableName = $this->store->getValue('table_name');
		
		// The columns of the user input
		if($tableName != '')
			$mapping_from_columns = $this->db->getColumns($tableName);
		else
			$mapping_from_columns = array();
	
		$this->setColMapFrom($mapping_from_columns);
	}	 
	    
    /**
	 * LoadMapTo method
	 * Loads the mappingFrom array from the database
	 */
	function loadMapTo(){
		// print '<p>Mapping::loadMapTo</p>';
		
		$fields = new Fields();
		// $mapping_to_columns = $fields->getFieldDropdownValues(); // Phil 20041213
		$mapping_to_columns = array_keys($fields->getFieldDropdownValues()); // Phil 20041213
		
		$map_to = array(0=>'');
		foreach ($mapping_to_columns as $k => $v) {
			$map_to[] = $v;	
		}
		// $mapping_to_columns = array(0=>'') + $mapping_to_columns; // First value is blank
		
		$this->setColMapTo($map_to);
	}
		
	/**
	 * SaveMapping method
	 * Saves the mapping to the database
	 * WARNING: THIS assumes certain variables are set in the store
	 * IF not, then the mapping will not be saved correctly
	 */
    function saveMapping($newName = ''){
		
    	if ($newName != '') {
			$error = $this->setMapName($newName); 
			if ($error) { // Phil, Note: setMapName has no return value!
				return $error;
			}
		}
		
		// Set the settings
		$template_settings['name'] = $this->mapName;
		$template_settings['date_format'] = $this->dateFormat;
		$template_settings['date_seperator'] = $this->dateSeperator;
		
		/*
		$template_settings['header_row'] = $this->store->getValue('header_row');
		$template_settings['field_terminator'] = $this->store->getValue('field_terminator');
		$template_settings['tmp_enclosed'] = $this->store->getValue('enclosed');
		$template_settings['tmp_escaped'] = $this->store->getValue('escaped');
		$template_settings['line_terminator'] = $this->store->getValue('line_terminator');
		*/
		$template_settings['header_row'] = $_SESSION['importParams']['header_row'];
		$template_settings['field_terminator'] = stripslashes($_SESSION['importParams']['field_terminator']);
		$template_settings['tmp_enclosed'] = stripslashes($_SESSION['importParams']['enclosed']);
		$template_settings['tmp_escaped'] = stripslashes($_SESSION['importParams']['escaped']);
		$template_settings['line_terminator'] = $_SESSION['importParams']['line_terminator'];
		
		$template_settings['address_labels'] = serialize($this->store->getValue('address_labels'));
		$template_settings['editable'] = 1;
		$template_settings['active'] = 1;
			
		// Insert into templte table passing the name
		if($this->mapID > -1){
			// Delete all the data for the old mapping
			$error = $this->deleteAll();
			if ($error) {
				return $error;
			}
				
			// Now update the mapping data
			$this->db->update('Import__template', $template_settings, "id='$this->mapID'");
		} else {
			// Insert the new data
			$this->db->insert('Import__template', $template_settings, "id='$this->mapID'");
			$this->setMapID($this->db->getInsertId());
		}
		 	
		// Return with an error message if necessary		
		if($this->db->hasError()) {
			return $this->db->getErrorString();
		}

		// Insert all the mappings with the template_id given
		if(sizeof($this->mapping) > 0){
			foreach($this->mapping as $map_from => $map_to){
				
				// Only save the nun-null
				if($map_to && $map_to != '') {
					$entry = array();
					$entry['template_id'] = $this->mapID;
					$entry['external_name'] = $map_from;
					$entry['crm_field_id'] = array_search($map_to, $this->getColMapTo());
					
					// Now insert
					$this->db->insert('Import__field_mapping', $entry);
				
					// Return with an error message if necessary		
					if($this->db->hasError()) {
						return $this->db->getErrorString();
					}
				}
			}
		}

		// Insert all the set variables
		if(sizeof($this->setValueMapping) > 0){
			foreach($this->setValueMapping as $field_id => $value){
			
				// Only save the nun-null
				if($value && $value != '') {
					$entry = array();
					$entry['template_id'] = $this->mapID;
					$entry['set_value'] = $value;
					$entry['crm_field_id'] = $field_id;
					
					// Now insert
					$this->db->insert('Import__set_values_mapping', $entry);
				
					// Return with an error message if necessary		
					if($this->db->hasError())
						return $this->db->getErrorString();
				}
			}
		}
	}
	
}

?>