<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 */

includeModule('Front', 'inc/database_tables.inc');

/**
 * Front module class
 * 
 * @package Front
 * @author Phil Calder
 */
class Front extends WebMATModule {
	
	/**
	 * @var default_redirect
	 * @access private
	 */
	var $default_redirect = false;
	/**
	 * @var bad_passwords
	 * @access private
	 */
	var $bad_passwords = '';
	
	/**
	 * Initialise module and required services
	 */
	function Front() {
		global $USER, $SERVICES;
		$this->db = & $SERVICES->getService('Database');
		$SERVICES->getService('Config');
		
		$SERVICES->getService('Language');
		$SERVICES->language->getLanguage('Front');
		$SERVICES->language->getLanguage('Licence');
		
		$this->bad_passwords = array('password', 'admin', 'user', 'god');
		
		$this->module_name = 'Front';
		$this->WebMATModule();
	}
	
	/**
	 * Returns an absolute URL to redirect page 
	 * (ie beginning in site root '/')
	 * Note that this is only the page that is loaded into the workpane.
	 **/
	function getDefaultRedirect($check_licence = true) {
		if ($this->default_redirect) {
			return $this->default_redirect;
		}
		global $SERVICES, $BCRM_DOCUMENT_ROOT, $USER;
		
		//MCR 20070327
		// Phil 20061115
		// if ($check_licence && !$SERVICES->isLicenceFileExisting()) {
			// $link = 'Front/register.php';
		// } else if ($USER['welcome'] == 2) { // Phil - readded 20051010; MCR ==2 20060413
			// $link = 'Front/welcome.php';
		// } else 
		{
			$SERVICES->getService('modules');
			$system_bookmarks_mod =& $SERVICES->modules->getModule('SystemBookmarks');
			if (is_object($system_bookmarks_mod) && ($bookmark = $system_bookmarks_mod->getWelcomeBookmark()) ) {
				$bookmark_link = $system_bookmarks_mod->getBookmarkLinks($bookmark['id']);
				$link = 'SystemBookmarks/redirect.php?href=' . base64_encode($bookmark_link['href']);
			} else {
				//$link = $SERVICES->config->lookup('modules.Front.default_redirect');
				$link = $SERVICES->config->lookup('start_page');
			}
		}
		
		if (!$link) {
			$link = 'Contact/index.php'; // Dont includes '../../modules/' here
		}
		return $this->getAbsoluteLinkTo($link);
	}
	
	function getAbsoluteLinkTo($link) {
		global $SERVICES, $BCRM_DOCUMENT_ROOT;
		$this->default_redirect = str_replace( 
			$BCRM_DOCUMENT_ROOT, 
			($SERVICES->config->lookup('site.websubdir')=='' ? '' : '/' . $SERVICES->config->lookup('site.websubdir')), 
			$SERVICES->config->lookup('modules.basedir')
		) . '/' . $link;	
		// print '<p>default_redirect: ' . $this->default_redirect . '</p>';
		return $this->default_redirect;
	}
	
	function getDefaultPDARedirect() {
		if ($this->default_pda_redirect) {
			return $this->default_pda_redirect;
		}
		global $SERVICES, $DOCUMENT_ROOT;
		$ans = $SERVICES->config->lookup('modules.Front.default_pda_redirect');
		if (is_array($ans)) {
			$ans = $ans[0];
		}
		$ans = str_replace( $DOCUMENT_ROOT, '', $SERVICES->config->lookup('modules.basedir')) . '/modules/' . $ans;
		$this->default_redirect = $ans;
		return $ans;
	}
	
	// incl_module - Phil - 20040722 - optionally include key module in returned array
	function getLinksList($incl_module = false) { 
		global $USER, $SERVICES, $L, $errorlog;
		$list = $SERVICES->config->getModuleConfig('Front'); // 'Front' added 20050317
		$list = $list['modulelink'];   
		//if user doesn't need welcome page any more
		if (!$USER['welcome']) {
			bcrm_unset($list, 'welcome');
		}
			
		// $list is an array of module links
		// a module link is an array with a "priority" and an "href"
		$ua =& $SERVICES->getService('Authentication');
		$modules = $ua->getPrivileges();
		// print_r($modules); print '<hr>';
		$answer = array();
		if (is_array($list)) {
			foreach ($list as $k => $v) {
				if (is_array($v)) {
					$index = 100 + $v['priority'];
					// so that comparison is lexigraphical & indexes are unique 
					// (assumes less than 100 modules exist).
					$index .= $k;  
					$module = $v['module'];
					
					if (!empty($v['privilege'])) {
						switch ($v['privilege']) {
							case $ua->getReadLevel():
								$privilege = $ua->getReadLevel();
							break;
							
							case $ua->getWriteLevel():
								$privilege = $ua->getWriteLevel();
							break;
							
							case $ua->getAdminLevel():
								$privilege = $ua->getAdminLevel();
							break;
							
							default :
								$privilege = $ua->getReadLevel();
						}
					} else {
						$privilege = $ua->getReadLevel();
					}

					if ( !$module || ( (isset($modules[$module]) && $modules[$module]) & $privilege) ) {
						// user has access to the module, so return the link
						//$answer[ $index ] = ($pda ? $v['pda_href'] : $v['href']);
						$SERVICES->language->getLanguage($module);
						//get translation for module label if exists
						if ( isset($v['label']) && isset($L[$v['label']]) && $L[$v['label']] ) {
							$temp_label = $L[$v['label']];
						} else {	//allow use of label direct from module config file with no language file present
							$temp_label = $v['label'];
							if ( substr($temp_label, 0, strlen($module)) == strtolower($module) ) {
								$temp_label = substr($temp_label, strlen($module)); //strip module name from front
							}
						}
						//
						$answer[ $index ] = array('link'=>$v['link'], 'label'=>str_replace(' ', '&nbsp;', $temp_label));
						if (isset($incl_module) && $incl_module) { // Phil - 20040722
							$answer[$index]['module'] = $module;	
							$answer[$index]['name'] = $k;
						}
					}
				} else {
					// $errorlog->log( 'error occured while loading Front links, module link does not have "href" and "priority" attributes.' );
				}
			}
		}
		ksort($answer);
		return $answer;
	}
	
	function setSkin($skin) {
		global $SERVICES;
		$path = '/skins/' . $skin . '/'; 
		$screen =& $SERVICES->getService('Screen');
		$screen->setSkin($path);
	}
	
	function getScreenResOptions() {
		$result = $this->db->select(
			'*', 
			'System__prefs_screen_res', 
			'', 
			'width ASC'
		);
		while ($row = $this->db->getRow($result)) {  
			$answer[$row['id']] = $row['screen_res'];
		}
		return $answer;
	}
	
	// Similar to above, but returns entire rows, not just the res value
	function getScreenResRows() {
		$result = $this->db->select(
			'*', 
			'System__prefs_screen_res', 
			'', 
			'width ASC'
		);
		
		$answer = array();
		while ($row = $this->db->getRow($result)) { 
			$answer[$row['id']] = $row;
		}
		return $answer;	
	}
	
	function setScreenRes($id) {
		global $SERVICES;
		$screen =& $SERVICES->getService('Screen');
		$screen->setScreenRes($id);
	}
	
	// user must have UserAdmin administrator rights
	// return_type can be array or string
	// will return false, a string (list of links to change passwords) or an array of user names with bad passowrds.
	function getBadPasswordsWarning($return_type = 'string') {
		global $SERVICES, $USER, $L;
		// either return content or user_array
		$content = '';
		$user_arr = array();
		
		// Prompt Admin to change passwords if they havent already done so
		$ua =& $SERVICES->getService('Authentication');
		
		if ( $ua->adminAccess( $USER['user_id'], 'UserAdmin') ) {	
			$bad_passwords = array();
			foreach ($this->bad_passwords as $k => $v) {
				$bad_passwords[$k] = 'password = password("' . $v .'")';	
			}
			
			// username and password are the same or in our bad_passwords list
			// Phil 20060814 added status check
			$result = $this->db->select(
				'id, username, password', 
				'System__users',
				'status=1 AND (password = password(username) OR password = password(LOWER(username)) OR ' . implode(' OR ', $bad_passwords) . ')'
			);
			while ($row = $this->db->getRow($result)) {
				$user_arr[] = $row;
			}
			if (sizeof($user_arr)) {
				$content .= '<p><b>' . $L['frontDefaultPasswordWarning'] . '</b></p>';
				$SERVICES->language->getLanguage('Admin');
				foreach ($user_arr as $k=>$val) {
					$content .= '<p><li>' . $val['username'] . ' <b><a href="../../modules/UserAdmin/password.php?user_id=' . $val['id'] . '">' . $L['adminModuleLabel'] .' | ' . $L['frontChangePassword'] . '</a></b></li></p>';
				}
				$content .= '<br>';
			}
		} else {
			return false;
		}
		
		if ($return_type == 'string') {
			return $content;	
		} else {
			return $user_arr;
		}
	}
	
	function getModuleNameLower() {
		return get_class($this);	
	}
}
?>