<?php
require_once("../../prepend.inc");
/**
 * Add attachment
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Mail
 */

includeWebMAT('edit.inc');

global $SERVICES;
$SERVICES->getService('FileSystem');


/**
 * Add attachment
 *
 * @package Mail
 */
class CustomEdit extends WebMATEdit {
	/**
	 *
	 * @param int $id Passed by reference.
	 * @param array $item Passed by reference.
	 * @param string $error_msg
	 */
	function beforeForm(&$id, &$item, &$error_msg) {
		// Senthil 20050705 - Display the attached file information - Begin		
		$list_a = array();
		
		// Phil 20070306 Calculate number of attachments, 
		// add to item edit form and resize the curr_attachments text area
		$num_attachments = $num_docs = $num_files = $num_strings = 0; 
		
		if (is_array($_SESSION[$this->form_values['hash']]['documents'])) {
			$num_docs = sizeof($_SESSION[$this->form_values['hash']]['documents']);
			foreach ($_SESSION[$this->form_values['hash']]['documents'] as $k => $v) {
				$fsize = round($v['size']/1024,2); // Ranga 200500705  
				$list_a[] = $v['filename'] . ' ('. $fsize .'KB)';
			}
		}

		// MCR 20051011
		// String attachments
		if (is_array($_SESSION[$this->form_values['hash']]['strings'])) {
			$num_strings = sizeof($_SESSION[$this->form_values['hash']]['strings']);
			foreach ($_SESSION[$this->form_values['hash']]['strings'] as $k => $v) {
				$fsize = round($v['size']/1024,2);
				$list_a[] = $v['name'] . ' ('. $fsize .'KB)';
			}
		}
		
		if (is_array($_SESSION[$this->form_values['hash']]['files'])) {
			$num_files = sizeof($_SESSION[$this->form_values['hash']]['files']);
			foreach ($_SESSION[$this->form_values['hash']]['files'] as $k => $v) {
				$fsize = round($v['size']/1024,2); 
				$list_a[] = $v['name'] . ' ('. $fsize .'KB)';
			}
		}
		
		// Phil 20070306
		$num_attachments = ($num_docs + $num_files + $num_strings - 1);
		$num_attachments = max($num_attachments, 1);

		if (!empty($list_a)) {
			// $lists = implode(', ', $list_a);
			$lists = implode('\r\n', $list_a);
		} else {
			$lists = '';
		}
		
		// print '<p>lists: ' . $lists . '</p>'; exit();

		// Senthil 20050705 - Display the attached file information - End

		$this->output->addInitJS('" onunload="processClose()'); // onunload=""');

		// Could use: window.opener.document.getElementById("fld_curr_attachments").rows += 1;
		
		// Phil 20070306 Added resize based on number of attachments
		$js = '<script type="text/javascript" language="javascript">
		function processClose() {				
			window.opener.document.forms["mainform"].elements["item[curr_attachments]"].value="' . $lists . '";	
			window.opener.document.forms["mainform"].elements["item[curr_attachments]"].rows = ' . $num_attachments . ';
		}
		</script>
			';
		$this->output->addToHead($js);
		
		if ($this->form_values['hash']) {
			$item['hash'] = $this->form_values['hash'];
		} else {
			$this->form_values['hash'] = $item['hash'];
		}
		// set the multiselect
		$select = array();
		if (is_array($_SESSION[$this->form_values['hash']]['files'])) {
			
			foreach ($_SESSION[$this->form_values['hash']]['files'] as $k => $v) {
				$select[$k] = $v['name'];
			}
			$this->dd->changeAttributes(array('field_key' => 'attach_multi', 'dropdown_values' => $select)); //MCR 20060629 'notnull' => 1
		}
	}
	
	/**
	 *
	 * @param array $item
	 * @param string $error_msg
	 */
	function getFieldRows($item, $error_msg) {
		global $L;
		$content = '';
		
		$str_error_msg = $error_message;
		if (is_array($str_error_msg)) {
			$str_error_msg = implode('<br>', $str_error_msg);
		}
		
		// Phil, TODO: check that changes to WebMATEdit::getFieldRows dont need to be appllied here (handleMetaFields)
		foreach ($this->dd->getDataStructureWithData($item) as $k => $v) {
			if ($v['field_key'] == 'attach_multi') {
				if (empty($_SESSION[$this->form_values['hash']]['files'])) {
					continue;
				}
				// add remove submit button after
				$html_field = $this->output->getHtmlField($v);

				$content .= '
				<tr>
					<th>' . $v['label'] . (isset($v['notnull'] ) && $v['notnull'] ? '*' : '') .'</th>'
					. '<td>' . $html_field . ($error_msg ? '<br>' . $str_error_msg . '<br>&nbsp;' : '') . '<br>' . $this->output->getHtmlSubmit($L['mailRemove'], 'submit') . '</td>
				</tr>
				';
			} else if ($v['field_key'] == 'attachment') {
				$html_field = $this->output->getHtmlField($v);
				
				$content .= '
				<tr>
					<th>' . $v['label'] . (isset($v['notnull'] ) && $v['notnull'] ? '*' : '') .'</th>'
					. '<td>' . $html_field . ($error_msg ? '<br>' . $str_error_msg . '<br>&nbsp;' : '') . '<br>' . $this->output->getHtmlSubmit($L['mailAttachFile'], 'submit') . '</td>
				</tr>
				';
			} else {
				$content .= $this->output->getHtmlFieldRow($v, $error_msg[$v['field_key']]);
			}
		}
		return $content;
	}

	/**
	 *
	 * @param array $item
	 * @param array $fields
	 * @param string $error_msg
	 */
	function beforeDBWrite(&$item, &$fields, &$error_msg) {  //passed by reference
		//any other processing before posting to database
		
		// do the removal for any removed attachments
		if ($this->form_values['submit'] == 'Remove') {
			if (is_array($item['attach_multi'])) { // Phil 20070306
				foreach ($item['attach_multi'] as $k => $v) {
					if (isset($_SESSION[$item['hash']]['files'][$v])) {
						unset($_SESSION[$item['hash']]['files'][$v]);
					}
				}
			}
		}

		// reform the list of attachments for main email form
		$list_a = array();
		if (is_array($_SESSION[$item['hash']]['documents'])) {
			foreach ($_SESSION[$item['hash']]['documents'] as $k => $v) {
				$list_a[] = $v['filename'] . ' (' . $v['size'] . ')';
			}
		}
		//MCR 20051011
		// String attachments
		if (is_array($_SESSION[$item['hash']]['strings'])) {
			foreach ($_SESSION[$item['hash']]['strings'] as $k => $v) {
				$list_a[] = $v['name'] . ' (' . $v['size'] . ')';
			}
		}
		if (is_array($_SESSION[$item['hash']]['files'])) {
			foreach ($_SESSION[$item['hash']]['files'] as $k => $v) {
				$list_a[] = $v['name'] . ' (' . $v['size'] . ')';
			}
		}
		if ($this->form_values['submit'] != 'Remove') {
			$list_a[] = $item['attachment']['name'] . ' (' . $item['attachment']['size'] . ')';
		}
		if (!empty($list_a)) {
			$list = implode(', ', $list_a);
		} else {
			$list = '';
		}

		if ($this->form_values['submit'] != 'Remove' && isset($item['attachment'])) {
			$_SESSION[$item['hash']]['files'][] = $item['attachment'];
		}

		$this->main();
		exit();
	}
	
	/**
	 * This over-rides the parent version, because we want a slightly
	 * different setup, so we can close the window
	 */
	function getSubmitRow(){
		global $L;
		$content = '
		<tr>
			<td>&nbsp;</td>
			<td align="left">' . 
			$this->output->getHtmlSubmit($L['mailReturnToEmail'], 'submitReturnToEmail', 'onClick="processClose(); window.close();"') . '</td>
		</tr>';
		return $content;
	}
	
	/**
	 * Functionality disabled
	 *
	 * @deprecated version 4.153.00
	 * @see getPastLinksTable
	 */
	function getLastViewedRow() {
		return '';
	}
	
	/**
	 * Functionality disabled
	 */
	function getPastLinksTable() {
		return false;
	}
	
	/**
	 * Functionality disabled
	 *
	 * @deprecated version 4.153.00
	 * @see getActionLinksContent
	 */
	function editTableLinks(& $links, $id, $item) {
		$links = array();
	}
	
	/**
	 * Functionality disabled
	 */
	function getActionLinksContent() {
		return false;
	}
	
	/**
	 * returns the title row array
	 *
	 * @param string $contact_id the contact id if required
	 * @return string a string containing the menu items
	 */
	function getTitleRowLinks() {
		$title_links = array(
			'bookmark' => 'off',
			'refresh' => 'off',
			'back' => 'off',
			'help' => '-1'
		);
		return $title_links;
	}
}

$customEdit = new CustomEdit('Mail', 'upload', $_POST, $_GET); 
$customEdit->run();
$customEdit->main();
?>