<?php
/**
 * The edit class for the Mail module
 *
 * Used for new emails, and deleting existing emails. 
 * Replying, forwarding, and editing message settings are handled elsewhere
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Mail
 */

includeModule('Mail', 'inc/item_edit.inc');

class MailEditWebMATEdit extends MailItemWebMATEdit {

	/**
	 * any processing
	 */
	function getItem($id = false, $item = false) {
		// print '<div>getItem================</div>';
		if (!$item) {
			$item = parent::getItem($id, $item);
		
			if ($this->form_values['id'] && $item['id']) {
				$this->checkoutMessage($item);
			}
			
			// if editing, could also be new, so don't display previous actions
			if (array_key_exists('previous_actions', $this->dd->data_dictionary)) {
				bcrm_unset($this->dd->data_dictionary, 'previous_actions'); // get rid of it - no need if nothing to display
			}
		}
		return $item;
	}
	
	/**
	 * Set the recipients (to, cc, bcc)
	 *
	 * @param array $item
	 */
	function setRecipients(& $item) {
		global $SERVICES;
		$concat_recipients = false;
		// if we already have an id, then probably have other stuff, so just leave it alone
		if ( ($item['sent_status'] == 'mailTemplates') && $this->form_values['contact_id']) {
			// Phil 20060830
			$concat_recipients = true;
		} else if (isset($item['id'])) {
			return;
		}

		// if we have a hash (means, back from spellcheck) so don't do anything
		// have to check form_values and *not* item because hash gets set in the item
		// before this function is called
		if (!isset($this->form_values['hash'])) {
			$default_cc = $SERVICES->preferences->getUserPref('mailDefaultCcAddresses');
			if ($default_cc) {
				$item['cc'] = $default_cc;
			}
			$default_bcc = $SERVICES->preferences->getUserPref('mailDefaultBccAddresses');
			if ($default_bcc) {
				$item['bcc'] = $default_bcc;
			}
		}
		
		if ($this->form_values['address_id']) {
			// deprecated
			// $address = $this->mod->getEmailFromAddressEmail($this->form_values['address_id']);
		} else if ($this->form_values['contact_id']) {//TODO whats this actually called?
			$address = $this->mod->getEmailFromContact($this->form_values['contact_id']);
		} else if ($this->form_values['address']) {
			$address = $this->mod->getEmailFromAddress($this->form_values['address']);
		}
		if (!empty($address)) {
			$addr_string = '';
			if ($address['from']) {
				$addr_string .= $address['from'] . ' ';
			}
			$addr_string .= '<' . $address['address'] . '>';
			if ($concat_recipients) { // Phil 20060830
				$item['recipient'] .= (strlen($item['recipient']) ? ', ' . $addr_string : $addr_string);
			} else {
				$item['recipient'] = $addr_string;
			}
		}
	}

	/**
	 * sets the subject. intended to be overriden
	 *
	 * @param array $item
	 */
	function setSubject(& $item) {
		global $SERVICES;
		
		// if no id, and no hash, means it's a brand new email!
		// we need to check form_values for a hash, cause chance might be returning from a spellcheck
		if (!isset($item['id']) && !isset($this->form_values['hash'])) {
			$default_subject = $SERVICES->preferences->getUserPref('mailDefaultSubject');
			if ($default_subject) {
				$item['subject'] = $default_subject;
			}
		}
	}
	
	/**
	 * Sets up the body/content of the email, should it be required
	 *
	 * @param array $item
	 * @param string $decode mime decode content (optional)
	 */
	function setContent(& $item, $decoded_mail = false) {
		$content = $item['content']; // in case nothing happens below

		if ($decoded_mail) {
			// echo 'decode';
			//$content = $this->mod->getContent($decoded_mail);
			$display_part = $this->mod->getDisplayPart($decoded_mail);
			$content = $display_part['body'];
		}
		
		//MCR 20051025
		//TODO then should also set content type?
		
		$item['content'] = $content;
	}
	
	/**
	 * Sets up to get ready for sending an email
	 * @param array $item
	 * @return array Returns an array of emails that the message will be sent to
	 */
	function fillEmailMessage(& $item) {
		
		/* -------------- PROCESS ATTACHMENTS FROM CONTINUING A DRAFT -------- */
		if ($this->form_values['id']) {
			$decode = $this->mod->getDecodedMime($item['id']);			
			
			if ($decode->parts) {
				foreach ($decode->parts as $k => $v) {					
					if ($v->disposition == 'attachment' || $v->disposition == 'inline') {
						// if we have an attachment add it back						
						$this->mail->AddStringAttachment($v->body, $v->d_parameters['filename'], $v->headers['content-transfer-encoding'], 
						$v->ctype_primary . '/' . $v->ctype_secondary);
						$item['attachments'] = 1;
					} 
				}
			}
			$item['id'] = $this->form_values['id'];
			$item['checked_out'] = 0;
		}
	
		// and finally, continue on with the parent
		return parent::fillEmailMessage($item);
	}
	
	/**
	 * Create, edit, remove the links, as appropriate for the given item (overrides parent)
	 *
	 * @param array $links The array of links for the item
	 * @param int $id
	 * @param array $item
	 */
	function editTableLinks(& $links, $id, $item) {
		parent::editTableLinks($links, $id, $item);
	}
}
?>