<?php
/**
 * The main display page
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Mail
 */

global $SERVICES;
$SERVICES->getService('Preferences');

class MailItemWebMATList extends WebMATList {


	/**
	 * overridden from the list.inc class.
	 * this will allow us to restrict our search to other items
	 * like a parent_id
	 */
	function afterDataDictionary() {
		global $L, $SERVICES;
		
		// if we have a parent id, then let's filter on that.
		if ($this->form_values['parent_id']) {
			$this->filters->setDesignTimeFilter(array('parent_id' => array('where' => 'parent_id = '. $this->form_values['parent_id'])));
			
			// need to get the parent item
			$parent_item = $this->db_functions->getItem($this->form_values['parent_id']);
			$filter_value = '';
			if ($parent_item['id']) {
				$parent_subject = $parent_item['subject'];

				// set the subject length
				$subject_len = $SERVICES->preferences->getUserPref('mailSubjectLinkLength');		
				if ($subject_len && $subject_len != 'All') {
					$parent_subject = substr($parent_subject, 0, $subject_len) . '...';
				}
				
				$href =  str_replace('%id%', $parent_item['id'], $this->getFileSetHref('view'));
				$href .= '&parent=1';
				$filter_value = $SERVICES->output->getLinkHref(array('href' => $href, 'text' => $parent_subject)); //MCR 20060505 'target' => '_new'
			}

			// if we have a parent_id, need the link in disable row
			$this->filters->addExtraFilterToDisableRow($L['mailLabelParentEmail'], $filter_value, array('parent_id' => ''));
		}

		// check the filters to deal with the 'account_id' and dropdown options stuff.
		$filters = $this->filters->getFilterInfo();

		if (($filters['value'] == 'mailHidden') || ($filters['value'] == 'none')) {
			$this->addCheckboxMenuRowExtraItems(array('mark_unhide' => $L['mailMarkAsUnhide']));
		} 
		
		if ($filters['value'] != 'mailHidden') {
			$this->addCheckboxMenuRowExtraItems(array('mark_hide' => $L['mailMarkAsHide']));
			$this->addCheckboxMenuRowExtraItems(array('mark_read' => $L['mailMarkAsRead']));
		}
		
		
		if ($filters['value'] != 'mailInboxActionRequired') {
			$this->addCheckboxMenuRowExtraItems(array('mark_required' => $L['mailMarkAsRequired']));
		}

		$this->addCheckboxMenuRowExtraItems(array('mark_not_required' => $L['mailMarkAsNotRequired']));
		$this->addCheckboxMenuRowExtraItems(array('mark_checked_in' => $L['mailMarkAsCheckedIn']));
		
		// if we're viewing an inbox, then don't need the sending account id
		// and vice versa for sending stuff
		// plus since we're only viewing 1, alter the label to just 'Account'
		// if both are view (ex: 'all'), then keep labels as they should already be set properly 
		
		if (($filters['value'] == 'mailInboxActionRequired') || ($filters['value'] == 'mailInbox') || ($filters['value'] == 'mailInboxUnread')) {
			bcrm_unset($this->dd->data_dictionary, 'send_account_id');
			$this->dd->changeAttributes(array('field_key' => 'receive_account_id', 'label' => $L['mailAccount']));
		} else if (($filters['value'] == 'mailSentItems') || ($filters['value'] == 'mailDrafts')  || ($filters['value'] == 'mailTemplates')) {
			bcrm_unset($this->dd->data_dictionary, 'receive_account_id');
			$this->dd->changeAttributes(array('field_key' => 'send_account_id', 'label' => $L['mailAccount']));
		}
		
		// Phil 20051101 We dont want to trim various fields (some of them have links/images)
		// Mail subject has already been trimmed (above)
		$this->dd->setNoTrimOption();
	}
	
	/**
	 * Forces list to NOT use an iterator for getItems
	 *
	 * ie process and return the entire items array
	 * Added 200512xx. This modules list page relies on bulk processing 
	 * of items in the {@link afterGetItems} method
	 *
	 * @return boolean false
	 * @author Phil Calder
	 */
	function useGetItemsIterator() {
		return false;
	}
	
	/**
	 * After we get the items, do some of this
	 *
	 * @param array $items
	 */
	function afterGetItems(& $items) {
		global $L;
		global $SERVICES;

		if (is_array($items)) {

			$SERVICES->getService('Output');
			
			$this->dd->changeAttributes(array('field_key' => 'attachments', 'type' => 'text'));
			$this->dd->changeAttributes(array('field_key' => 'action_required', 'type' => 'text'));
			$this->dd->changeAttributes(array('field_key' => 'checked_out', 'type' => 'text'));
			$this->dd->changeAttributes(array('field_key' => 'is_replied', 'type' => 'text'));

			// get the subject length
			$subject_len = $SERVICES->preferences->getUserPref('mailSubjectLinkLength');
			
			// $mail_address =& $this->mod->getDatabaseTable('item_email'); - not used anymore
			foreach ($items as $k => $v) {
				// if we have an attachment, just make it into a gif
				if ($items[$k]['attachments']) {
					$items[$k]['attachments'] = $SERVICES->output->getImageTag('img/mailAttmt.gif', $L['mailAttachment']);
				} else {
					$items[$k]['attachments'] = '';
				}
				
				// if we have an action, just make it into a gif
				if ($items[$k]['action_required']) {
					$items[$k]['action_required'] = $SERVICES->output->getImageTag('img/mailActRq.gif', $L['mailActionRequired']);
				} else {
					$items[$k]['action_required'] = '';
				}
				
				// if we have it's checked out, just make it into a gif
				if ($items[$k]['checked_out']) {
					$items[$k]['checked_out'] = $SERVICES->output->getImageTag('img/mailChOut.gif', $L['mailCheckedOutLabel']);
				} else {
					$items[$k]['checked_out'] = '';
				}
				
				// if we have it's been replied to, just make it into a gif
				if ($items[$k]['is_replied']) {
					$href =  str_replace('%contact_id%', '', $this->getFileSetHref('list'));
					$href .= '&filter=fnone&parent_id='. $items[$k]['id'];
					$items[$k]['is_replied'] = $SERVICES->output->getLinkHref(array('href' => $href, 'title' => 'Replied', 'text' => $SERVICES->output->getImageTag('img/mailRepld.gif', $L['mailReplied'])));
				} else {
					$items[$k]['is_replied'] = '';
				}

				$items[$k]['recipient'] = $this->getAddressField($items[$k]['recipient']);//htmlspecialchars($items[$k]['recipient']);
				$items[$k]['sender'] = $this->getAddressField($items[$k]['sender']);//htmlspecialchars($items[$k]['sender']);
				$items[$k]['cc'] = $this->getAddressField($items[$k]['cc']);//htmlspecialchars($items[$k]['cc']);
				$items[$k]['bcc'] = $this->getAddressField($items[$k]['bcc']);//htmlspecialchars($items[$k]['bcc']);

				if ($subject_len && $subject_len != 'All') {
					$items[$k]['subject'] = substr($items[$k]['subject'], 0, $subject_len);
				}
				
				//check for blank subject
				if (!$items[$k]['subject']) {
					$items[$k]['subject'] = $L['mailSubjectNoSubject'];
				}
				
				// Phil 20060117 Removed -- see WebMATList::buildStringFieldRow
				// $href =  str_replace('%id%', $items[$k]['id'], $this->getFileSetHref('view'));
				// $items[$k]['subject'] = $this->output->getLinkHref( array('href' => $href, 'text' => $items[$k]['subject']) );

				// this will stop any errors in case the date is bad (ex: -1)
				if ($items[$k]['date'] < 0) {
					$items[$k]['date'] = 0;
				}
			}
		}
	}


	/**
	 * Create, edit, remove the links, as appropriate for the given item (overrides parent)
	 *
	 * @param array $links The array of links for the item
	 * @param int $id
	 * @param array $item
	 */
	function editTableLinks(& $links, $id, $item) {
		global $L, $SERVICES;

		$SERVICES->getService('Output');

		// here we'll do some checks on the item, to setup the proper links
		// firstly, deal with the sent_status
		if ($item['sent_status'] == 'mailDraft') {
			$links['mailReply'] = $SERVICES->output->getLinkHref(array('href' => $this->getFileSetHref($this->edit_link), 'text' => $L['mailContinue']));

		// template?
		} else if($item['sent_status'] == 'mailTemplates') {
			$links['mailReply'] = $SERVICES->output->getLinkHref(array('href' => $this->getFileSetHref($this->edit_link), 'text' => $L['mailUse']));
		}

		if ($item['is_trash']) {
			// need to also reset the $links['delete'] value, should it need it
			if (array_key_exists('delete', $links)) {
				$links['delete'] = $this->mod->getDeleteFromTrashLink($id, $item['is_trash']);
			}
		}

		bcrm_unset($links, 'edit');
		bcrm_unset($links, 'mailReplyAll'); // don't need this in the list view

		if (!$this->db_functions->isEditable($id, $item)) {
			// check if we can edit the properties
			if (array_key_exists('mailSettings', $links)) {
				$links['mailSettings'] = '';
			}
			
			// no edit, no reply
			if (array_key_exists('mailReply', $links)) {
				$links['mailReply'] = '';
			}
			
			if (array_key_exists('delete', $links)) {
				$links['delete'] = '';
			}
		}
		// Bosco 20060328 | Removed Action Required Links
		bcrm_unset($links, 'mailMarkAsRequired');
		bcrm_unset($links, 'mailMarkAsNotRequired');
		
	}

	/**
	 *
	 *
	 * @param string $address_string
	 */
	function getAddressField($address_string) {
		$addresses = $this->mod->splitAddresses($address_string);
		$return = array();
		foreach ($addresses as $k => $v) {
			if ($v['from']) {
				$return[] = $v['from'];
			} else {
				$return[] = $v['address'];
			}
		}
		return implode(', ', $return);
	}
	
	/**
	 * Returns the new link for the current module
	 *
	 * Adds a Check Email link
	 * @author Phil Calder
	 */
	function getNewLinks() {
		// Modules can override this method to include more than one new link
		$links =  array(
			'new'   => $this->getNewLink(), 
			'template' => $this->getNewFromTemplateLink(),
			'check' => $this->output->getLinkHref(array('href' => 'check_email.php', 'text' => 'mailCheckMail'))
		);
		
		if ($this->form_values['filter'] == 'fmailTrash') {
			global $SERVICES;
			$prefs =& $SERVICES->getService('Preferences');
			if ($prefs->getUserPref('mailAllowDeleteTrash')) {
				$links['empty_trash'] = $this->output->getLinkHref(array('href' => 'empty_trash.php', 'text' => 'mailEmptyTrash'));
			}
		}
		
		
		return $links;
	}
	
	/**
	 * @author Phil Calder
	 */
	function getNewFromTemplateLink() {
		global $L;
		$id = ($this->form_values['contact_id'] ? $this->form_values['contact_id'] : 'template');
		$contact_id = ($this->form_values['contact_id'] ? $this->form_values['contact_id'] : '');
		$mail_template_menu_link = '';
		// $this->output->req_contactMenu_js = true;
		// $mail_template_menu_link = $this->output->getLinkHref(
			// array(
				// 'href' => 'javascript:mailNewFromTemplateMenu(\'' . 'mtrMen' . $id . '\',\'' . $id . '\');',
				// 'img' => $this->output->getFlagMenu('mtrMen' . $id),
				// 'tag_extras' => ' onMouseOut="hM(\'contactMen\')" '
			// ),
			// false // no language text check
		// );
		$template_text_link = ' <a href="../../modules/Mail/template_menu.php?page=1&contact_id=' . $contact_id . '">' . $L['mailSentStatusDropdownTemplates'] . '</a>'; // TODO: use a function / config lookup here
		return ($mail_template_menu_link . $template_text_link);
	}
}

?>