<?php
/**
 * The main class for this module
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Mail
 */

includeModule('Mail', 'inc/database_tables.inc');

class Mail extends WebMATModule {

	/**
	 * @access private
	 * @var timeout_limit
	 */
	var $timeout_limit = 60;
	
	/**
	 * @access private
	 * @var empty_trash_result
	 */
	var $empty_trash_result = false;

	/**
	 * Default constructor
	 */
	function Mail() {
		$this->module_name = 'Mail';
		$this->WebMATModule();
	}


	/**
	 * Function that will check email.
	 */
	function checkMailOverdueAccounts() {
		$this->checkMail(false, false);
	}
	
	/**
	 * Tests a connection to a pop account
	 *
	 * @author Phil Calder
	 */
	function checkAccountSettings($account_id) {
		global $SERVICES;
		$retval = false;
		// Phil 20060714 First test whether the system is allowed to download email
		if (!$this->isCheckMailEnabled()) {
			
			if ($halt) {
				global $L;
				quickPage($L['mailErrorPopAccountsDisabled'] . '<br><br><a href="' . $_SESSION['return_page'] . '">' . $L['Back'] . '</a>');
				exit();
			}
			
			return false;
		}
		// Phil 20070214 Possibility of PHP Warnings using SSL
		$current_error_reporting = error_reporting(0);
		$pop = new POPMail($halt);
		
		// Phil 20070214 Get account to check
		$account = $pop->db_functions->getItem($account_id);
		if (is_array($account)) {
			// print '<div>account:</div>'; print_r($account); print '<hr>';
			
			//--
			// Phil 20070125 Accounts now have a port number
			$port = (int)$account['port'];
			if (!$port) {
				$port = $pop->default_port;
			}
			// Phil 20070125 Accounts can use SSL (requires php open ssl)
			// will fail to get messages if open ssl support not installed
			$ssl_host_prefix = 'ssl://';
			if ($account['use_ssl']) {
				$account['host'] = $ssl_host_prefix . str_replace($ssl_host_prefix, '', $account['host']);
			}
			
			// connect and check mail for each account
			// connect and login
			// print '<div>connect to host: ' . $account['host'] . ' port: ' . $port . '</div>'; // DEBUG
			if (!$pop->connect($account['host'], $port, $account['username'], base64_decode($account['password']))) {
				// print '<div>failed to connect</div>';
				$retval =  false;
			} else {
				// print '<div>connected</div>';
				$retval =  true;
			}
			//--
		}
		// print '<div>after get account item</div>';
		$current_error_reporting = error_reporting($current_error_reporting);
		return $retval;
	}
	
	
	/**
	 * Function that will check email for all accounts
	 *
	 * @param bool $force
	 * @param bool $halt
	 */
	function checkMail($force = true, $halt = true) {
		global $SERVICES;
		
		// Phil 20060714 First test whether the system is allowed to download email
		if (!$this->isCheckMailEnabled()) {
			
			if ($halt) {
				global $L;
				quickPage($L['mailErrorPopAccountsDisabled'] . '<br><br><a href="' . $_SESSION['return_page'] . '">' . $L['Back'] . '</a>');
				exit();
			}
			
			return false;
		}
		// Phil 20070125 Possibility of PHP Warnings using SSL
		$current_error_reporting = error_reporting(0);
		$pop = new POPMail($halt);
		
		if ($force) {		
			
			$pop->main();
			
		} else {
			$mail_check = $SERVICES->preferences->getUserPref('mailRefresh');
			if ($mail_check) {
				// check if we need to check mail for each account
				// Phil 20060523 Get the fields in order of importance (based on last_checked)
				$order_by = array('field' => 'last_checked', 'order' => 'ASC');
				
				// Phil 20070124 Get all accounts that we automatically check (ie not manual check only)
				$accounts = $pop->getAccounts($order_by, 'auto');
				// $accounts = $pop->getAccounts($order_by);
				if (is_array($accounts)) {
					// Phil 20060523 Adding 1 minute timeout
					// any accounts not checked this time will be picked up on the next reload
					$start_time = time(); // see $this->timeout_limit;
					// Phil 20051018 use =& for singleton instance classes		
					// Phil 20060323 Moved out of loop
					$db_functions =& $this->getDatabaseTable('pop_account'); 
					foreach ($accounts as $k => $v) {
						$account = $db_functions->getItem($v['id']);				
						if (strtotime("+$mail_check seconds", $account['last_checked']) < time()) {
							// check email for this account					
							$pop->getMailForAccount($v);
						}
						if ( $this->timeout_limit && ((time() - $start_time) > $this->timeout_limit) ) {
							break;
						}
					}
				}
			}
		}
		
		$current_error_reporting = error_reporting($current_error_reporting);
	}
	
	/**
	 * 
	 * @return boolean true or false whether or not the system is allowed to download emails
	 * @author Phil Calder
	 * @access private
	 */
	function isCheckMailEnabled() {
		global $SERVICES;
		if (is_object($conf =& $SERVICES->getService('Config'))) {
			$pop_enabled = $conf->lookup('modules.Mail.account_settings.pop');
			if ($pop_enabled === '') { 
				return true; // config setting not found, pop enabled by default
			} else {
				return ($pop_enabled ? true : false);
			}
		}
		return false;
	}
	
	/**
	 * 
	 * @param int $type The send account transport method (one of: 1, 2, 3, or 4)
	 * @return boolean true or false whether or not the system is allowed to send using the given transport method
	 * @author Phil Calder
	 * @access private
	 */
	function isTransportMethodEnabled($type) {
		$key = '';
		switch ($type) {
			case 1: // PHPMail
				$key = 'phpmail';
			break; 
			case 2: // sendmail
				$key = 'sendmail';
			break;
			case 3: // SMTP
				$key = 'smtp';
			break;
			case 4: // SMTP Authenticated
				$key = 'smtpauth';
			break;
			default:
				return false; // Unknown transport method
			break;	
		}
		
		global $SERVICES;
		if (is_object($conf =& $SERVICES->getService('Config'))) {
			$method_enabled = $conf->lookup('modules.Mail.account_settings.' . $key);
			// print '<div>lookup: modules.Mail.account_settings.' . $key . ': ' . ($method_enabled ? 'true' : 'false/blank') . '</div>';
			if ($method_enabled === '') { 
				return true; // config setting not found, method enabled by default
			} else {
				return ($method_enabled ? true : false);
			}
		}
		return false;
	}
	
	
	/**
	 * Get list of emails for a given contact
	 *
	 * @param int $contact_id The id of a contact for whose email we will get
	 *
	 * @return array The email associated with the contact
	 */
	function getEmailFromContact($contact_id) {
		// print '<div>TODO: ' . get_class($this) . '::getEmailFromContact(' . $contact_id . ')</div>';
		// print '<div>' . get_class($this) . '::getEmailFromContact(' . $contact_id . ')</div>';
		global $SERVICES;
		$contact_mod =& $SERVICES->modules->getModule('Contact');
		$primary_email = $contact_mod->getEmailAddressFor($contact_id);
		// print_r($primary_email); print '<hr>';
		if (is_array($primary_email) && isset($primary_email['content']) && $primary_email['content']) {
			return array('from' => $primary_email['name'], 'address' => $primary_email['content']);
		}
		return array();
	}
	
	/**
	 * Parses an email address to split the name from the actual address in conjunction with a record in the Contact module.
	 *
	 * @param string $address The entire address string
	 * @param boolean $one_only Whether to return one contact only
	 * @return array The name split from the address
	 */
	function getEmailFromAddress($address, $one_only = true) {
		// print '<div>' . get_class($this) . '::getEmailFromAddress(' . $address . ')</div>';
		global $SERVICES;
		$contact_mod =& $SERVICES->modules->getModule('Contact');
		$contacts = $contact_mod->getMatchingEmailAddressesDetails($address);
		// print_r($contacts); print '<hr>';
		if ($one_only && current($contacts)) {
			$contact_item = current($contacts);
			// print_r($contact_item); print '<hr>';
			return array(
				'from' => $contact_item['name'], 
				'address' => $contact_item['content']
			);
		} else {
			$results = array();
			foreach ($contacts as $k => $v) {
				// print '<div>contact: ' . $k . '</div>'; print_r($v); print '<hr>';
				$results[] = array(
					'contact_id' => $v['contact_id'],
					'from' => $v['name'], 
					'address' => $v['content']
				);
			}
			return $results;
		}
		return array('address' => $address);
	}
	
	
	/**
	 * Returns a list of attachments and their links
	 *
	 * @param object $message the message object file
	 * @param int $id The id of the message
	 * @return array The list of attachments with links
	 * Phil - 20050517 - added htmlentities and urlencode to filename(s)
	 */
	function getAttachmentLinks($message, $id) {
		global $L;
		$attachments = array();
		if ($message->parts) {
			foreach ($message->parts as $k => $v) {
				//echo '<p>' . $k . '</p>';
				if ($v->disposition == 'attachment' || $v->disposition == 'inline') {
					$current = '<tr><td>';
					if ($v->d_parameters['filename'] != '') {
						$current .= $v->d_parameters['filename'];
					} else {
						$current .= $L['mailUnnamedAttachment'] . ' (' . $v->ctype_primary . '/' . $v->ctype_secondary . ')';
					}
					//$current .= '</td><td> <a href="attachment.php?id=' . $id . '&filename=' . htmlentities(urlencode($v->d_parameters['filename'])) . '&save=1">'. $L['mailDownload'] . '</a>';
					$current .= '</td><td>';
					
					//$k + 1 because we dont want to pass '0' in the query string 
					$current .= ' &nbsp; <a href="attachment.php?id=' . $id . '&part=' . ($k + 1) . '&save=1">'. $L['mailDownload'] . '</a>';
					$primary_types = array('image');
					$secondary_types = array('plain', 'html', 'pdf');					
					if ((in_array($v->ctype_secondary, $secondary_types)) || (in_array($v->ctype_primary, $primary_types))) {
						//$current .= ' <a href="attachment.php?id=' . $id . '&filename=' . htmlentities(urlencode($v->d_parameters['filename'])) . '">' . $L['mailView'] . '</a></td></tr>';
						$current .= ' &nbsp; <a href="attachment.php?id=' . $id . '&part=' . ($k + 1) . '">' . $L['mailView'] . '</a>';
						$current .= ' <a href="attachment.php?id=' . $id . '&part=' . ($k + 1) . '" target="_blank" title="' . $L['LinkOpenInNewWindow'] . '">[+]</a>';
					}
					$current .= '</td></tr>';
					
					//TODO add link to copy attachment to Documents module
					
					$attachments[] = $current;
				} 
			}
		}
		return $attachments;
	}


	/**
	 * Gets the attachments for a message
	 *
	 * @param object $message The entire message to be parsed
	 *
	 * @param array An array of attachments (if applicable)
	 */
	function getAttachmentsWithinMessage($message) {
		global $L;
		
		$attachments = array();
		if ($message->parts) {
			foreach ($message->parts as $k => $v) {
				if ($v->disposition == 'attachment' || $v->disposition == 'inline') {
					if ($v->d_parameters['filename'] != ''){
						$current = array('filename' => $v->d_parameters['filename']); //'<tr><td>' . $v->d_parameters['filename'] . '</td><td> <a href="attachment.php?id=' . $id . '&filename=' . $v->d_parameters['filename'] . '&save=1">Save</a>';
					} else {
						$current = array('filename' => $L['mailUnnamedAttachment']);
					}
					$current['size'] = strlen($v->body) * 8;
					$current['body'] = $v->body;
					$current['content-type'] = $v->content-type;
					$current['content-transfer-encoding'] = $v->content-transfer-encoding;
					$attachments[] = $current;
				} 
				//echo '<p>part: ';print_r($current); echo '</p>';
			}
		}
		return $attachments;
	}


	/**
	 * checks if content has html part, basic.
	 *
	 * @param string $content The content to check.
	 * @param string $identifier A unique needle to find that will indicate if it has html.
	 *
	 * @return bool True or false for whether '<html>' was found.
	 */
	function isHtmlPart($content = '', $identifier = '<html'){
		
		$pos = strpos(strtolower($content), $identifier);
		
		if ($pos === false) {
			return false;
		} else {
			return true;
		}
		
	}


	/**
	 * Recursive function to get the part of a message to display
	 * @param object $message the message
	 * @param array $part with elements string $ctype_secondary and string $body 
	 */
	//function getContent($message) {
	function getDisplayPart($message) {
		
		//$function_debug = true;
		
		if ($function_debug) {
			//echo '<p>getDisplayPart()</p>';
			//echo '<p>message:</p>'; print_r($message);echo '<hr>';
		}
		
		$display_part = array();
		$text_types = array(	'plain' => 0, 
					'enriched' => 1, 
					'richtext' => 2, 
					'html' => 3
					);
		
		$current_part = array('priority' => -1);
		
		if ($message->parts) {
			foreach ($message->parts as $k => $v) {
				// if this part has parts itself
				if ($v->parts) {
					if ($function_debug) {
						echo 'searching parts<br>';
					}
					
					//$display_part = $this->getDisplayPart($v);
					return( $this->getDisplayPart($v) );
					
				} else if ($v->disposition == 'attachment') {
					//?
					if ($function_debug) {
						echo 'TODO attachment or inline<br>';
					}
				
				//TODO
				
				} else if ($message->ctype_secondary == 'alternative'  //TOD verify - should this be $v-ctype_secondary
//					|| $v->ctype_secondary == 'plain'  //20060504
					//MCR tried this casually, appears not necessary  || $message->ctype_secondary == 'mixed' 
					|| $v->disposition == 'inline') {
						
					if ($function_debug) {
						//echo 'ifblock ' ;
						//echo $v->ctype_secondary;
					}
					
					if (in_array($v->ctype_secondary, $text_types)) {
						// decodable text part, use priority of that type
						
						//echo 'found text part ' . $v->ctype_secondary . '<br>';
						//MCR 20060504
						if ($text_types[$v->ctype_secondary] > $current_part['priority']) {
							
							//echo 'found text part ' . $v->ctype_secondary . '<br>';
							if ($function_debug) {
								echo 'found text part ' . $v->ctype_secondary . '<br>';
							}
							$current_part = array('priority' => $text_types[$v->ctype_secondary], 'part' => $k);
						}
					}
				} else {
					//echo 'found text part ' . $v->ctype_secondary . '<br>' . $v->body;
					//mimeDecode will return a single space ' ' for a blank body
					if (trim($v->body)) {
						//echo 'YES ' . $v->ctype_secondary . '<br>*' . $v->body . '*';
						$display_part = array('ctype_secondary' => $v->ctype_secondary, 'body' => $v->body);
					}
					
					// if ($function_debug) {
						// echo '<h1>reached else block: ' . $v->ctype_secondary . '</h1>' . $v->body ;
						// echo '<hr>';
						// print_r($v);
						// echo '<hr>';
						// echo 'part: add body2<br> *' . $v->ctype_secondary . '*';
						// //echo '<h2>' . $v->ctype_secondary . '<h2>' . ': ' . $v->body;
					// }
					
				}
			}
			
			// see if displayable part was found (current part will be set to highest priority displayable part)
			if ($current_part['priority'] > -1) {
			
		
				//get correct alternate
				//$content .= $message->parts[$current_part['part']]->body;
				//print_r($current_part['part']);
				$display_part = array('ctype_secondary' => $message->parts[$current_part['part']]->ctype_secondary, 'body' => $message->parts[$current_part['part']]->body);
				
				if ($function_debug) {
					echo 'chosen part priority: ' . $display_part['priority'] . ', add body<br>';
					echo 'ctype_secondary: ' . $display_part['ctype_secondary'] . ', add body<br>';
					//echo $message->parts[$current_part['part']]->body;
					echo $message->parts[$current_part['part']]->body;
				}
			}
		} else {
			$display_part = array('ctype_secondary' => $message->ctype_secondary, 'body' => $message->body);
			
			if ($function_debug) {
				echo 'no parts: use body<br>';echo $message->body;
			}
		}
		
		
		if ($function_debug) {
			//echo '<p>FINAL: returning display part: '; print_r($display_part); echo '</p>';
		}
		//echo '<p>FINAL: returning display part: '; print_r($display_part); echo '</p>';
		//print_r($message);
		return $display_part;

	}


	/**
	 * Something to do with returning from the spell check
	 *
	 * @param array $item
	 * @param string $hash the session hash
	 */
	/*
	function spellcheckReturn(&$item, $hash) {
		$item['recipient'] = $_SESSION[$hash]['recipient'];
		$item['cc'] = $_SESSION[$hash]['cc'];
		$item['bcc'] = $_SESSION[$hash]['bcc'];
		$item['subject'] = stripslashes($_SESSION[$hash]['subject']);
		$item['content'] = stripslashes($_SESSION[$hash]['content']);
		$item['signature'] = stripslashes($_SESSION[$hash]['signature']);
		
		$md5 = $this->form_values['hash'];
		// set the initial attachment field value incase we had attachments when we spell checked
		$list_a = array();
		if (is_array($_SESSION[$md5]['files'])) {
			foreach ($_SESSION[$md5]['files'] as $k => $v) {
				$list_a[] = $v['name'] . ' (' . $v['size'] . ')';
			}
		}
		if (is_array($_SESSION[$md5]['documents'])) {
			foreach ($_SESSION[$md5]['documents'] as $k => $v) {
				$list_a[] = $v['name'] . ' (' . $v['size'] . ')';
			}
		}		
		if (!empty($list_a)) {
			$list = implode(', ', $list_a);
		} else {
			$list = '';
		}
		$item['curr_attachments'] = $list;
	}
	*/

	/**
	 * Takes a string of email addresses and splits them up into 'name' and email address
	 *
	 * @param string $addresses_string String of addresses with possible form of "Name Name" <email@address.com>
	 *
	 * @return array Key->Value array in the form of v['from'] = "Name", v['address'] = email@address.com
	 */
	function splitAddresses($addresses_string) {
		
		//Saravanakumar :: 20050627 :: To insert the space before '<' after '>'symbol in Email address. 
		$addresses_string = str_replace('<',' <',$addresses_string);
		$addresses_string = str_replace('>','> ',$addresses_string);
		$addresses_string = str_replace(',', ', ', $addresses_string);
		$addresses_string = str_replace(';', '; ', $addresses_string);
		$addresses_string = str_replace('  ', ' ', $addresses_string);
		$addresses_string = str_replace("\r\n", ' ', $addresses_string);
		$addresses_string = str_replace("\n", ' ', $addresses_string);
		
		$addresses_string = str_replace('"', '', $addresses_string);
		$addresses_string = str_replace('\'', '', $addresses_string);	//  2005/05/18
		
		//we are so far way from properly processing addresses
		//big hack to replace out these long flags
		if (strpos(strtolower($addresses_string), '=?iso-8859-1') !== false) {
			$addresses_string = str_replace('=?ISO-8859-1?Q?', '', $addresses_string);
			$addresses_string = str_replace('=?iso-8859-1?q?', '', $addresses_string);
			$addresses_string = str_replace('=?ISO-8859-1', '', $addresses_string);
			$addresses_string = str_replace('=?iso-8859-1', '', $addresses_string);
			$addresses_string = str_replace('?=', '', $addresses_string);
		}
		
		$addresses = explode(' ', $addresses_string);
		
		
		$return = array();
		$trim = array('<', '>', ',', ';');
		$replace = array('', '', '', '');
		$count = 0;
		if (is_array($addresses)) {
			foreach ($addresses as $k => $v) {
				if (ereg("[_a-zA-Z0-9-]+(\.[_a-zA-Z0-9-]+)*@[a-zA-Z0-9-]+(\.[a-zA-Z0-9-]+)+", $v) !== FALSE) {
					// we have an email address	
					//echo '<p>got: ' . htmlspecialchars($v);
					$return[$count]['address'] = str_replace($trim, $replace, $v);
					$temp = array();
					for ($i = 0; $i < $k; $i++) {
						if (isset($addresses[$i])) {
							$temp[] = $addresses[$i];
							bcrm_unset($addresses, $i);
						}
					}
					bcrm_unset($addresses, $k);
					$from = implode(' ', $temp);
					$from = stripslashes($from);
					$from = trim($from);
					$from = trim($from, ',');
					$return[$count]['from'] = $from;
					//echo '<p>got: ' . htmlspecialchars($from);
					$count++;
				}
			}
		} else {
			return false;
		}
		//print_r($return);die();
		
		// remove duplicate addresses
		if (is_array($return)) {
			for ($i = 0; $i <= count($return); $i++) {
				for ($j = $i+1; $j <= count($return); $j++) {
					if ($return[$i]['address'] == $return[$j]['address']) {
						bcrm_unset($return, $j);
					}
				}
			}
		}
		return $return;
	}


	/**
	 * Builds the Delete link
	 * 
	 * @param int $id Optional id variable (for overriding this functionality)
	 * @param bool $is_trash_status Optional, if we already know the status
	 *
	 * @return string containing the link
	 */
	function getDeleteFromTrashLink($id = false, $is_trash_status = false) {
		global $L, $SERVICES;

		$ret_val = '';

		// sometimes it might come in as the item array, so need to check that
		if (is_array($is_trash_status)) {
			if (array_key_exists('is_trash', $is_trash_status)) {
				$is_trash_status = $is_trash_status['is_trash'];
			}
		}

		// is_trash status not provided, need it.
		if (($is_trash_status === false) && ($id > 0)){
			$temp_item = $this->db_functions->getItem($id);
			if (is_array($temp_item)) {
				if (array_key_exists('is_trash', $temp_item)) {
					$is_trash_status = $temp_item['is_trash'];
				}
			}
			unset($temp_item);
		}
		
		// if it's already trashed, need the correct link
		if ($is_trash_status > 0) {
			// careful, check for permissions
			if ($SERVICES->preferences->getUserPref('mailAllowDeleteTrash')) {
				
				// Phil 20050712 - This code did not work. 
				// Has obviously been copied from an 'edit' class to the 'module' class at some point.
				// Need to set the following 3 variables to use getFileSetHref: module_name, table_name, edit_link
				// todo: tidy this up 
				if (empty($this->module_name)) {
					$this->module_name = 'Mail';
				}
				if (empty($this->table_name)) {
					$this->table_name = 'Mail__item';
				}
				if (empty($this->edit_link)) {
					$this->edit_link = 'edit';
				}
				// print '<p>edit_link: ' . $this->edit_link . '</p>';
				// --
				$link_href = $this->getFileSetHref($this->edit_link .'_delete');
				// print '<p>delete from trash url: <b>' . $link_href . '</b></p>';
				// return $SERVICES->output->getLinkHref(array('href' => $link_href, 'text' => $L['mailDeleteTrash']));
				// Phil 20070305 Use Output function - includes client side confirmation
				return $SERVICES->output->getLinkDelete($link_href, $L['mailDeleteTrash']);
			}
		}
		
		return $ret_val;
	}
	
	
	/**
	 * Decode a iso8859-1 header
	 * @param $string
	 * @return string decoded info
	 * @source http://nz2.php.net/en/imap_mime_header_decode Pawel Tomicki <pafka at klub dot chip dot pl> 23-Apr-2004 06:04
	 */
	 function decode_iso88591($string) {
		if (strpos(strtolower($string), '=?iso-8859-1') === false) {
			return $string;
		}
		$string = explode('?', $string);
		
		return strtolower($string[2]) == 'q' ? quoted_printable_decode($string[3]) : base64_decode($string[3]);
	}
	
	/**
	 * Gets the decoded mime info.
	 * @param int $id  id of the message
	 * @return object  the decoded mime info
	 */
	function getDecodedMime($id) {
		global $SERVICES;

		$ret_val = '';
		
		// must have an id to get a file
		if ($id) {
			$SERVICES->getService('FileSystem');
			
			//$path = $SERVICES->filesystem->getDirectoryPath('mail');
			$file = $SERVICES->filesystem->readFileContents($id, 'mail');
			
			if (is_array($file)) {
				$file = implode('', $file);
			}
			
			//$mime = new CustomMimeDecode($file);
			//$ret_val = CustomMimeDecode::decode($file);
			
			$mime = new CustomMimeDecode($file);
			//$mime->_decode_bodies = true;
			$ret_val = $mime->decode(array('decode_bodies' => true, 'include_bodies' => true));//, 'input'=>$file));
			
			//echo '888<br>'; print_r($ret_val); echo '<hr>';
			//MCR 20051011 additional parsing for charset encoded headers
			//check $ret_val['headers']['from']
			// [from] => =?ISO-8859-1?Q?Gonzalo_De_la_Pe=F1a_Andreu?= <gnz@amorfo.com>
			//print_r( $ret_val->headers );
			//echo $this->decode_iso88591($ret_val->headers['from']);
			$ret_val->headers['from'] = $this->decode_iso88591($ret_val->headers['from']);
			
			// done, free the space
			unset($mime);
			unset($file);
		}
		
		return $ret_val;
	}
	
	
	/**
	 * If there are any children for a given mail item, this will create a short string saying so
	 *
	 * @param int $id
	 *
	 * @return string A short message if applicable, otherwise blank
	 */
	function getPreviousActionStatement($id) {
		global $SERVICES, $L;

		// spiffy feature: previous actions:
		// here, we'll check if there are any previous actions, or child items.
		$db_functions =& $this->getDatabaseTable('item');
		$child_items = $db_functions->getItems(array('date'), false, 'parent_id = \''. $id .'\' AND sent_by_user_id > 0');
		
		if (count($child_items) <= 0) {
			return '';
		}
		
		$ua =& $SERVICES->getService('Authentication');
		$date =& $SERVICES->getService('DateLib');
		//$ret_val = '';
		$actions = array();
		
		//$comma = '';
		
		foreach ($child_items as $ci => $c_item) {
			// Phil 20070305 Ensure these items are active
			if (is_array($c_item) && isset($c_item['id']) && $c_item['id']) {
			
				$action = '';
				
				$action .= $ua->users->getUsername($c_item['sent_by_user_id']);
				
				if ($c_item['message_send_type'] == 'reply') {
					$action .= $L['mailPreviousActionReply'];
				} else if ($c_item['message_send_type'] == 'forward') {
					$action .= $L['mailPreviousActionForward'];
				}
	
				$action .= $date->getUserPrefDateFromTimestamp($SERVICES->offsetTimeZoneForRead($c_item['date'])) . ' '. $date->getUserPrefTimeFromTimestamp($SERVICES->offsetTimeZoneForRead($c_item['date']));
				
				$actions[] = $SERVICES->output->getLinkHref(array('href' => 'view.php?id=' . $c_item['id'], 'text' => $action));
	
				//$comma = '<br />'; // for the next loop
			}
		}
		
		unset($child_items);	
		
		return implode('<br />', $actions);
	}
	
	/*
	* Saravanakumar :: 20050704 :: To get the information of original message
	* @param object $decode
	* @return string 
	*/
	function getWroteOnLabel($decode) {
		global $SERVICES, $L;
		
		if ($decode->headers['date']) {
			
			$date = substr($decode->headers['date'],0,strpos($decode->headers['date'],'+')); 
			
			/* BBartel 20050830. Also check for '-' since dates can be negative timezones (ideally this should be all changed */
			if (trim($date) == '') {
				$date = substr($decode->headers['date'],0,strpos($decode->headers['date'],'-'));
				if (trim($date) == '') {
					$date = $decode->headers['date'];
				}
			}
			
		}
		if ($decode->headers['from']) {
			$from = substr($decode->headers['from'],0,strpos($decode->headers['from'],'<')); 
		}
		
		$wroteOnLabel = (trim($date) == '' ? '' : $L['mailReplyHeaderOn'] . " " . trim($date).", "). ucfirst(trim($from))." " . $L['mailReplyHeaderWrote'] . ":";
		return $wroteOnLabel;
	
	}
	
	function getTemplateToolbarMenuItems($default_menu_item) {
		$db_functions =& $this->getDatabaseTable('item');
		// TODO: config lookup on Templates filter
		$dtf = array('sent_status'=>array('where'=>"Mail__item.sent_status = 'mailTemplates' AND Mail__item.is_trash = 0 AND active = 1 AND (seen <> 3 OR seen IS NULL)"));
		$db_functions->setDesignTimeFilters($dtf);
		$items = $db_functions->getItems();
		// print '<div>templates:</div>'; print_r($items); print '<hr>';
		if (is_array($items) && count($items)) {
			$template_menu_items = array();
			foreach ($items as $k => $v) {
				$template_menu_items[] = array(
					'href' => $default_menu_item['href'] . '&template=' . $v['id'], 
					'text' => addslashes($v['subject']), 
					'icon' => $default_menu_item['icon'],
					'title' => $v['subject']
				);
			}
			return $template_menu_items;
		}
		return false;
	}
	
	/**
	 * @see emptyTrash
	 * @return integer Either the number of items deleted, or a status message 
	 * 0 = unknown, 
	 * 1 = no write access to module, 
	 * 2 = Not allowed to delete emails, 
	 * 3 = sql error
	 * @author Phil Calder
	 */
	function getEmptyTrashResult() {
		return $this->empty_trash_result;
	}
	
	/**
	 * Redirects to the Mail inbox, or the users saved mail index filter (pref), provided that filter displays inbox items
	 *
	 * Populates 
	 * @return boolean Whether the operation was successful
	 * @author Phil Calder
	 */
	function emptyTrash() {
		// Satisfy several requirements
		// Write permission to module
		global $SERVICES, $USER;
		$ua =& $SERVICES->getService('Authentication');
		if (!$ua->writeAccess(false, $module)) {
			$this->empty_trash_result = 1;
			return false;
		}
		
		// Delete pref for module
		$prefs =& $SERVICES->getService('Preferences');
		if (!$prefs->getUserPref('mailAllowDeleteTrash')) {
			$this->empty_trash_result = 2;
			return false;
		}
		
		// Empty Trash pref for module
		// TODO
		
		// Empty Trash
		$db_functions =& $this->getDatabaseTable('item');
		
		$where_array = array("is_trash='1'");
		// $where_array = $db_functions->getDisplayByGroupWhere($where_array);//  No, this will allow viewbale items to be deleted
		$where_array[] = "
		(
			edit_group_id IN (" . $USER['group_string'] . ") OR 
			owner_group_id IN (" . $USER['group_string'] . ") OR 
			admin_group_id IN (" . $USER['group_string'] . ")
		) ";
		
		$where = implode(" AND ", $where_array);
		// print '<p>
		// table: ' . $db_functions->getTableName() . '<br />
		// [set active = 0]<br />
		// where ' . $where . '
		// </p>'; exit();
		$result = $db_functions->db->update(
			$db_functions->getTableName(), 
			array('active' => '0'), 
			$where
		); 
		// print '<p>emptyTrash, initial result: ' . $result . ' [' . ($result ? 'true' : 'false') . ']</p>';
		if ($db_functions->db->hasError()) {
			$db_functions->db->printError();
			$this->empty_trash_result = 3;
			return false;
		} else {
			$this->empty_trash_result = $db_functions->db->affectedRows();
			// print '<p>emptyTrash, empty_trash_result: ' . $this->empty_trash_result . '</p>';
		
			$db_functions->setLog('delete','rows: ' . $num_rows); 
		}
		return $result;
		
	}
	
	/**
	 * Redirects to the Mail inbox, or the users saved mail index filter (pref), provided that filter displays inbox items
	 *
	 * @author Phil Calder
	 */
	function redirectToUsersInboxList() {
		global $SERVICES;
		$prefs =& $SERVICES->getService('Preferences');
		if (is_object($prefs)) {
			$mail_index_filter = $prefs->getUserPref('Mail.index.Filter');
			// $this->printDebug('mail index filter: ' . $mail_index_filter);
			
			// Mail list filters that include Inbox items
			// Could possibly allow the 'All' option here (fnone)
			$inbox_filters = array(
				'fmailInbox', 
				'fmailInboxActionRequired', 
				// 'fmailInboxUnread', // Phil: removed because downloaded emails are currently still marked as read
				'fmailVisible'
			);
			
			if (!$mail_index_filter || (!in_array($mail_index_filter, $inbox_filters)) ) {
				$mail_index_filter = 'fmailInbox';
			}
			
		}
		header('Location: ../../modules/Mail/index.php?search_filter=off&filter=' . $mail_index_filter);
		exit();
	}
	
	/**
	 * @deprecated version 5
	 */
	function getEmailFromAddressEmail($address_id) {
		// global $SERVICES;
		// $SERVICES->getService('Log');
		// $SERVICES->log->logError('Call to deprecated function getEmailFromAddressEmail', 'Mail');
		return array();
	}
	
}
?>