<?php
/**
 * A collection of classes extended from WebMATDatabaseTable
 * 
 * You need an extended class for each table in you module not including the 
 * dropdown tables.
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Memo
 */

/**
 * This class handles the database functions for the item table
 * 
 * This module is an extend verison of WebMATDatabaseTable with the table_name 
 * set and the alert_where set.
 * 
 * @author Matthew Corich, Minggang Chen
 * @package Memo
 */
class itemMemoDatabaseTable extends WebMATDatabaseTable {
	
	/**
	 * The constructor class for itemMemoDatabaseTable
	 *
	 * @param string $module_name the module name
	 * @param array $data_definition the data definition
	 */
	function itemMemoDatabaseTable($module_name, $data_definition) {
		$this->table_name = $module_name . '__item';
		$this->table_name_group = $module_name . '__item_group';
		$this->join_tables = '' . $this->table_name . ' LEFT JOIN ' . $this->table_name_group . ' ON ' . $this->table_name . '.id = ' . $this->table_name_group . '.item_id';
		$this->WebMATDatabaseTable($module_name, $data_definition); //calls the constructor for WebMATDatabaseTable
	}
	
	/**
	 * set the table name for the module
	 * 
	 * @param string $table_name the table name
	 */
	function setTable($table_name) {
		$this->table_name = $table_name;
		$this->columns =  $this->db->getColumns($table_name);
		$this->columns = bcrm_array_merge($this->columns, array('user_id')); // This is from the Memo__item_seen table
	}
	
	/**
	 * Processes active memos only, based on seen flag
	 * @author Phil Calder
	 */
	function getFixedWhere($where) {
		// $this->processActive($where); // Phil 20060221
		$where[] = " Memo__item_seen.seen < 4";
		return $where;
	}
	
	/** 
	 * Checks the viewable permission for user
	 * @param integer $id the item id
     * @param array $item Optional item array
     * @return bool
	 * @author Syed, Bosco 20050718
	 */
	function isViewable($id,$item=false) {
		$id = $this->cleanId($id); // Phil 20070305
		global $USER;
		$seen_table = $this->getTableName().'_seen';
		$where = 'item_id = "' . $id . '" AND  user_id = "' . $USER['user_id'] . '"';
		if ($result = $this->db->quickGet('*',$seen_table,$where)) {
			return parent::isViewable($id, $item);
		} else {
			return false;
		}
	}

	/**
	 * Checks the editable permission for user
	 * @param integer $id the item id
	 * @param array $item Optional item array
	 * @return bool
	 * @author Syed Bosco 20050718
	 */
	function isEditable($id, $item=false) {
		$id = $this->cleanId($id); // Phil 20070305
		global $SERVICES,$USER,$L;
		$seen_table = $this->getTableName().'_seen';
		$where = 'item_id = "' . $id . '" AND  user_id = "' . $USER['user_id'] . '"';
		if ($result = $this->db->quickGet('*',$seen_table,$where)) {
			return parent::isEditable($id, $item);
		} 
		return false;
	}
	
	/**
	 * Returns order by statement without 'ORDER BY'
	 * 
	 * @param string $order_by the order by array from the sorting functions
	 * @return string the order by statement without 'ORDER BY'
	 */
	function getOrderBy($order_by = '') {
		$sort='';
		if ($order_by == 'none') {
			$sort = '';
		}
		// Bosco 20050624 | To handle excluded column names
		if (($order_by) && (is_array($order_by)) && in_array($order_by['field'], $this->columns)) {
			$sort = $this->table_name . '.' .  $order_by['field'] . ' ' .$order_by['order'];
		}
		return $sort;
	}
	
	function getItemSeenTable() {
		global $SERVICES;
		$SERVICES->getService('Modules');
		$this->mod =& $SERVICES->modules->getModule('Memo');
		return $this->mod->getDatabaseTable('item_seen');	
	}
	
	// 2005/05/25 Cecil
	function isArchived($id) {
		global $USER;
		$item = $this->getItem($id);
		if ($id) { // Phil 20050819 - added id check
			$result = $this->db->select('seen', 'Memo__item_seen', "item_id='$id' and user_id='" . $USER['user_id'] . "'", '', '', '');
		}
		if (!$result) {
			return false;
		}
		
		if ($this->db->hasError()) {
			$this->db->printError();
		}
		
		while ($row = $this->db->getRow($result)) {
			if ($row['seen'] == 3) {
				return true;
			}
		}
			
		return false;
	}
	
	function isActive($id) {
		// Bosco | Added if Condition
		if (!empty($id)) {
			$item = $this->getItem($id);	
			if ($this->db->hasError()) {
				$this->db->printError();
			}
			// 2005/06/01 CECIL
			// Bosco 20060327 | Changed if condition for active
			if ($item['seen'] < 4) {
				return true;
			}
		}
		return false;
	}
	
	/**
	 * Overridden to change table name
	 *
	 * @param integer $id The item id
	 * @return mixed string display name for the item, or false
	 * @author Phil Calder
	 */
	function getItemDisplayName($id) {
		if ( ($field = $this->getDisplayNameField()) != '__none__') {
			// Phil - Use cached value if possible (see function getItem)
			// unlikey that its cached - this functionality generally called before getItem/getItems
			if ($id && isset($this->item_cache[$this->module_name][$this->getTableName()][$id][$field]) ) {
				return $this->item_cache[$this->module_name][$this->getTableName()][$id][$field];
			} else {
				// $table = $this->getTableName();
				// Phil 20060224
				global $USER;
				$table = "Memo__item LEFT JOIN Memo__item_seen ON Memo__item.id = Memo__item_seen.item_id
				AND Memo__item_seen.user_id = '" . $USER['user_id'] . "'";
				$where = $this->getFixedWhere(array('id' => 'id=' . $id));
				$where = implode(' AND ', $where);
				if ($result = $this->db->quickGet(
					$field, 
					$table,
					$where
				)) {
					return $result;
				}
			}
		}
		return false; // Cant determine what the display name is, or there isn one
	}
	
	/**
	 * Creates/Updates an item in the DB
	 * 
	 * @param array $item the item as an array
	 */
	function setItem($item) {
		global $SERVICES, $USER, $L;
		// can't let webmat allocate value to these two field because
		// created and creator_id in item_data_definition are not hidden
		// So, we have to exclude them in data_definition and set the value 
		// here.
		$item['created'] = time();
		$item['creator_id'] = $USER['id'];
		$private_groups = $item['groups'];
		bcrm_unset($item, 'groups'); //removes the groups multi select
		
		$this->db->insert($this->table_name, $item);
		$id = $this->db->getInsertId();
		
		if ($this->db->hasError()) {
			$this->db->printError();
		} else {
			$this->setLog($L['logCreate'], 'ID=' . $id);
		}
		
		$this->setGroupItem($id, $private_groups);
		// print '<div>call setUserSeenItem id: ' . $id . ', private: groups:</div>'; print_r($private_groups); print '<hr>';
		$this->setUserSeenItem($id, $private_groups);
		$this->setItemSeenStatus($id, 2);
		return $id;
	}
	
	/**
	 * Add the items to the item_group table
	 * 
	 * @param integer $item_id the current item id
	 * @param array $user_ids an array or user id's
	 */
	function setGroupItem($item_id, $group_ids) {
		foreach ($group_ids as $k => $v) {
			$item = array('group_id' => $v, 'item_id' => $item_id);
			$this->db->insert($this->table_name_group, $item);
			if ($this->db->hasError()) {
				$this->db->printError();
			}
		}
	}
	
	/**
	 * Add the items to the item_group table
	 * 
	 * @param integer $item_id the current item id
	 * @param array $user_ids an array or user id's
	 */
	function setUserSeenItem($item_id, $group_ids) {
		global $SERVICES;
		$groups =& $SERVICES->getService('Groups');
		$user_ids = $groups->getUsers($group_ids);
		// print '<div>Memo setUserSeenItem</div>'; print_r($user_ids); print '<hr>'; // DEBUG
		if (is_array($user_ids) && sizeof($user_ids)) {
			// insert into seen
			// item_id, user_id, seen=0
			$db_functions = $this->getItemSeenTable();
			foreach ($user_ids as $id => $user) {
				$item_seen = array (
					'item_id' => $item_id,
					'user_id' => $id,
					'seen' => 0
				);
				$db_functions->setItem($item_seen);
				// $db_functions->db->printError(); // DEBUG
			}
		}
	}
	
	/**
	 * Soft deletes an item
	 * 
	 * Updated to deal with joined table
	 * 
	 * @param intger $id the item id
	 */
	function deleteItem($id) {
		global $SERVICES, $L;
		//Soft Delete Item - set seen status to 4
		$this->setItemSeenStatus($id, 4);
		if ($this->db->hasError()) {
			$this->db->printError();
		} else {
			$this->setLog($L['logDelete'], 'id=' . $id);
		}
	}
	
	/**
	 * Archives an item
	 * 
	 * @param intger $id the item id
	 */
	function archiveItem($id) {
		global $SERVICES, $L,$USER;
		//Archive Item - set seen status to 3
		$this->setItemSeenStatus($id, 3); 
		if ($this->db->hasError()) {
			$this->db->printError();
		} else {
			$this->setLog($L['logDelete'], 'id=' . $id);
		}
	}
	
	/**
	 * Returns the number of items in the database can filter by contact id
	 * 
	 * @param integer $contact_id the contact id
	 * @param integer $active the deleted status 1 = undeleted, 0 = deleted
	 * @return integer the number of items
	 */
	function countItems() {  //count all
		global $SERVICES, $USER;
		$tables = $fields = '';
		$tables .= $this->table_name;
		
		$this->processGetItemsContactFields($tables, $fields); // Phil 20050419, Alters the where statement for contact fields
		$this->processGetItemsForeignFields($tables, $fields); // Phil 20050419, Phil (not necessary) Alters the where statement for foreign fields	
		$this->processGetItemsItemUserTable($tables, $fields); // Phil 20050419
		$this->processGetItemsItemGroupTable($tables, $fields);
		
		$where = $this->getWhereStatement();	
		// Phil 20060221 removed // $this->processActive($where);
		$retval = $this->db->quickGet('count(DISTINCT Memo__item.id)', $tables, $where);
		return $retval;
	}
	
	/**
	 * returns the seen where
	 * 
	 * @param the current where statement
	 * @return the current were statement with the new where statement added to the end
	 */
	function processGetItemsItemGroupTable(&$tables, &$fields) {
		global $SERVICES;
		if ($SERVICES->config->lookup($this->getFilesetConfigPath() . 'group_table')) {
			global $USER;
			$group_table = $this->table_name . '_group';
			$tables .= ' LEFT JOIN ' . $group_table . ' ON ' .
				$this->table_name . '.id = ' . $group_table . '.item_id ' . 
				' AND ' . $group_table . '.group_id IN (' . $USER['group_string'] . ')';
			
			// $fields .= ', '. $group_table . '.seen AS seen';
			//$dtf['_seen'] = array('where' => $seen_table . '.user_id = \'' . $USER['user_id'] . '\'');
			//$this->setDesignTimeFilters($dtf);
			return true;
		}
		return false;
	}
	
	/**
	 * Returns the alert where statement
	 */
	// Bosco | 20050630 | Replaced '__DEFAULT__' with 1 in return string  to remove '__DEFAULT__' from query string
	function getAlertWhere() {
		global $USER;
		if (!($alert_where = parent::getAlertWhere())) {
			// Bosco 20050722 | Changed alert where
			$alert_where = 'Memo__item_seen.seen = 0 AND Memo__item_seen.user_id = ' . $USER['user_id'];//'__DEFAULT__'
		}
		return $alert_where;
	}	
	
	/**
	 * Custom string for this module
	 *
	 * @see getUnseenWhere
	 * @author Phil Calder
	 */
	function getUnseenWhereString() {
		global $USER;
		return 'Memo__item_seen.seen = 0 AND Memo__item_seen.user_id = ' . $USER['user_id'];
	}
	
	/**
	 * Custom string for this module
	 *
	 * @see getModifiedWhere
	 * @author Phil Calder
	 */
	function getModifiedWhereString() {
		global $USER;
		return '( ( (Memo__item_seen.seen <> 1) OR (Memo__item_seen.seen IS NULL) ) AND (Memo__item_seen.user_id = ' . $USER['user_id'] . ') ) ';
	}
	
	/**
	 * 
	 * @param string $limit the paging limit
	 * @return array an array of alert ids
	 */
	function getAlertIds($limit = '', $item_ids = false) { // passing in ids
		// Bosco | 20050630 | Replaced '__DEFAULT__' with 1
		// Bosco | 20050809 | Changed the if condition
		if (!(parent::getAlertWhere())) {//'__DEFAULT__'
			global $USER;
			$fields = 'DISTINCT Memo__item.id AS id';
			$tables = "Memo__item
				LEFT JOIN Memo__item_group ON Memo__item.id = Memo__item_group.item_id
				LEFT JOIN Memo__item_seen ON Memo__item.id = Memo__item_seen.item_id
				AND Memo__item_seen.user_id = '" . $USER['user_id'] . "'";
			
			$where = '
				Memo__item_group.group_id
				IN ( ' . $USER['group_string'] . ' )
				AND Memo__item_seen.seen < 1';
				// AND Memo__item_group.active =1
				//AND Memo__item_group.is_opened =0';
	
			$result = $this->db->select($fields, $tables, $where, '', '',$limit);
			if ($this->db->hasError())	$this->db->printError();
			while ($row = $this->db->getRow($result)) {
				$items[] = $row['id'];
			}
			return $items;
		} else {
			return parent::getAlertIds($limit, $item_ids);
		}
	}
	
	// Phil 20060221 See getFixedWhere
	//2005/06/01 Cecil
	// function processActive(&$where) {
		// $where .= (strlen($where) ? " AND " : '' ) . " Memo__item_seen.seen < 4"; // Phil 20051111 added strlen check
	// }
}

class item_seenMemoDatabaseTable extends WebMATDatabaseDetailTable {
	
	/**
	 * The constructor class for item_seenMemoDatabaseTable
	 *
	 * @param string $module_name the module name
	 * @param array $data_definition the data definition
	 */
	function item_seenMemoDatabaseTable($module_name, $data_definition) {
		global $USER;
		$this->table_name = 'item_seen';
		$this->WebMATDatabaseTable($module_name, $data_definition); //calls the constructor for WebMATDatabaseDetailTable
	}

	function setItem($item) {
		// print '<p>seen table - setItem</p>'; print_r($item); print '<hr>'; exit();
		global $SERVICES, $USER, $L;
		$item = $this->setPermissionsFields($item);
		//insert new record
		$this->db->insert($this->getTableName(), $item);
		$id = $this->db->getInsertId();
		
		if ($this->db->hasError()) {
			$this->db->printError();
		} else {
			$this->setLog($L['logCreate'], 'ID=' . $id);
			$this->setItemSeenStatus($id, 1);
		}
	}
	
	function getItems($order_by = false, $limit = false, $where = false) {  //get all
		global $SERVICES, $USER;
		$contact_fields = array();
			
		$tables = $fields = '';	// set in processGetItemsContactFields and processGetItemsForeignFields
		$contact_fields = $this->processGetItemsContactFields($tables, $fields);
		$foreign_fields = $this->processGetItemsForeignFields($tables, $fields);
		$this->processGetItemsItemUserTable($tables, $fields);
		
		$sort = $this->getOrderBy($order_by);
		if ($where) {
			$where = $where;
		} else {
			$where = $this->getWhereStatement();
		}
		
		
		// If the hidden/unhidden is to be more system wide, then probably
		// will need to add some more where statements in here!
		// Otherwise, it's currently used as a design time filter.
		 
		$result = $this->db->select($fields, $tables, $where, $sort, '',$limit);
		if ($this->db->hasError())	$this->db->printError();
		while ($row = $this->db->getRow($result)) {
			$items[] = $row;
		}
		return $items;
	}
}

class item_groupMemoDatabaseTable extends WebMATDatabaseDetailTable {
	
	/**
	 * The constructor class for item_seenMemoDatabaseTable
	 *
	 * @param string $module_name the module name
	 * @param array $data_definition the data definition
	 */
	function item_groupMemoDatabaseTable($module_name, $data_definition) {
		global $USER;
		$this->table_name = 'item_group';
		$this->WebMATDatabaseTable($module_name, $data_definition); //calls the constructor for WebMATDatabaseDetailTable
	}
	
	function getItems($order_by = false, $limit = false, $where = false) {  //get all
		global $SERVICES, $USER;
		$contact_fields = array();
		$tables = $fields = '';	// set in processGetItemsContactFields and processGetItemsForeignFields
		$contact_fields = $this->processGetItemsContactFields($tables, $fields);
		$foreign_fields = $this->processGetItemsForeignFields($tables, $fields);
		$this->processGetItemsItemUserTable($tables, $fields);
		
		$sort = $this->getOrderBy($order_by);
		if ($where) {
			$where = $where;
		} else {
			$where = $this->getWhereStatement();
		}
		
		// If the hidden/unhidden is to be more system wide, then probably
		// will need to add some more where statements in here!
		// Otherwise, it's currently used as a design time filter.
		// print '<div>fields:</div>'; print_r($fields); print '<hr>';
		$result = $this->db->select($fields, $tables, $where, $sort, '',$limit);
		if ($this->db->hasError())	$this->db->printError();
		while ($row = $this->db->getRow($result)) {
			$items[] = $row;
		}
		return $items;
	}
}
?>