<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 */

/**
 * The edit class
 * 
 * @package Person
 * @author Phil Calder
 */
class PersonItemWebMATEdit extends WebMATEdit {
	
	/**
	 * any processing required before adding to the form is created
	 */
	function beforeForm(&$id, &$item, &$error_msg) {
		//any other processing before the Form is created (ie. adding extra items)
		// if ($this->form_values['contact_id']) {
			// Phil 20050510 - disable contact selector whether we have a contact id or not
			$this->output->setShowContactSelector(false);
		// }
	}
	
	/**
	 * Updates display name by calling module function
	 *
	 * @param array $item
	 * @param array $fields
	 * @param array $error_msg 
	 */
	function beforeDBWrite(&$item, &$fields, &$error_msg) { 
		if (is_object($this->mod) && method_exists($this->mod, 'calculateDisplayName') ) {
			$item['display_name'] = $this->mod->calculateDisplayName($item);
		}
		//print_r($item);
		//print_r($fields);
	}	
	
	/**
	 * Updates Contact module records
	 *
	 * @param array $item
	 * @param array $fields
	 * @param array $error_msg 
	 */
	function afterDBWrite(&$item, &$fields, &$error_msg, $id = false) {  //passed by reference
		global $SERVICES;
		$SERVICES->getService('Modules');
		$contact_mod =& $SERVICES->modules->getModule('Contact');	
		$contact_id = ( (isset($item['contact_id']) && $item['contact_id']) ? $item['contact_id'] : false);
		$contact_mod->updateContactDisplayName($contact_id, '', $item['display_name']);
	}	
	
	/**
	 * Checks if for a delete or update runs the submit function
	 */
	function run() {
		if (!$this->form_values['contact_view'] && !isset($this->form_values['delete'])) {
			if ( !isset($this->form_values['id']) || !$this->form_values['id']) {
				if ($new_contact_link = $this->mod->getNewPersonLink()) {
					header('Location: ' . $new_contact_link);
					exit();
				}
				
			}
		}
		parent::run();
	}
	
	/**
	 * deletes the contact for this item.
	 */
	function doSubmitDelete() {
		global $SERVICES;
		$SERVICES->getService('Modules');
		if ( 
			($contact =& $SERVICES->modules->getModule('Contact')) && 
			($contact_id = $this->getContactForPerson($this->form_values['delete'])) 
		) {
			
			// $contact_delete_url = '../../modules/Contact/edit.php?delete=' . $contact_id;
			$contact_delete_url = $contact->getContactDeleteLink(true);
			$contact_delete_url = str_replace('%id%', $contact_id, $contact_delete_url);
			header('Location: ' . $contact_delete_url);
			exit();
			
		} else {
			return parent::doSubmitDelete();
		}
	}
	
	// 
	function getContactForPerson($id) {
		$item = $this->getItem($id);
		return $item['contact_id'];
	}
	
}
?>