<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Search
 **/

includeWebMAT('view.inc');

/**
 * @package Search
 */
class Search extends WebMATView {

	/**
	 * Keyword(s) to search: array of strings.
	 */
	var $search_keywords;

	/**
	 * Keyword(s) that have errors (ex: too short, bad words): array of strings.
	 */
	var $error_keywords;

	/**
	 * Type of search match: exact, all or any, default is 'any'
	 */
	var $search_type;

	/**
	 * Modules we will be searching
	 */
	var $search_modules;
	
	/**
	 * Number of results to display per module
	 */
	var $limit_paging_overwrite;

	/**
	 * Number of results found
	 */
	var $num_found;

	/**
	 * Array of modules to skip
	 */
	var $excluded_modules;

	/**
	 * Default constructor.
	 */
	function Search($keywords = '', $type = '', $modules = '') {
		$this->init();
	}

	/**
	 * Initialize this class. Is a stripped version of WebMat
	 */
	function init() {
		global $SERVICES;

		$this->error = array();
		$this->db =& $SERVICES->getService('Database');
		$SERVICES->getService('Config');
		$this->config =& $SERVICES->config->getModuleConfig('Search');
		$this->output =& $SERVICES->getService('Output');
		$this->result = array();
		$this->module_name = 'Search';
		$this->module_name_lower = 'search';
		$this->search_keywords = trim($this->getFormValue('search_keywords'));
		$this->limit_paging_overwrite = $this->getFormValue('limit_paging_overwrite');
		if ($this->limit_paging_overwrite == '') {
			$this->limit_paging_overwrite = $this->getSearchPref('limit_paging_overwrite');
		}
		
		$this->search_type = $this->getFormValue('search_type');
		if ($this->search_type == '') {
			$this->search_type = $this->getSearchPref('search_type');
		}
		
		$this->search_modules = $this->getFormValue('search_modules');
		if (($this->search_modules == '') && (!$this->getFormValue('submit'))) {
			$SERVICES->getService('Preferences');
			$this->search_modules = $this->getSearchPref('search_modules');
		}
		$this->num_found = 0;
	}

	/**
	 * Runs the search
	 */
	function main () {
		$this->sidebar = false;
		if (isset($_GET['sidebar']) && $_GET['sidebar']) {
			$this->sidebar = true;
			$this->output->setFooterBlockSwitch(false); // Dont display footer
		}
		
		$content = array();
		// ---------------
		//$content['beginMain']           = $this->getBeginMainContent();
		// Title (and links)
		
		$this->output->has_toolbar = false;
		$this->output->setDefaultLinkTarget('contentframe');
		$content['title']               = $this->getTitleContent();
		// Description (optional)
		//$content['description']         = $this->getDescriptionContent();
		$content['beginDataCell']       = $this->getBeginDataCell();
		$content['data']                = $this->getSearchForm(true, true, $this->sidebar);
		$content['result']              = $this->performSearch();
		//$content['endDataCell']         = $this->getEndDataCell();
		
		// Browse / paging
		// $content['browsePaging_bottom'] = $content['browsePaging_top']; // $this->getBrowsePagingContent();
		// ---------------
		//$content['endMain']             = $this->getEndMainContent();
		
		return $this->display($content);
	}
	
	/* TODO - in sidebar mode still need to add validation and style sheets (output handles these)
	 * Either print the given content (custom), or call parent display method
	 *
	 * Calls parent when sidebar variable is false
	 * @param array $custom This is passed to {@link buildPage}
	 */
	// function display($custom = array()) {
		// if ($this->sidebar) {
			// print implode('', $custom);
			// return;
		// }
		// return parent::display($custom);
	// }

	/**
	 * Retrieves the value from the form
	 * @param string $var Variable to retrieve
	 * @return string Form value
	 */
	function getFormValue($var) {
		// global $_GET, $_POST;
		$retVal = '';
		if (isset($_POST['item'][$var])) {
			$retVal = trim($_POST['item'][$var]);
		} else if (isset($_GET['item'][$var])) {
			$retVal = trim($_GET['item'][$var]);
		} else if (isset($_POST[$var])) {
			$retVal = trim($_POST[$var]);
		} else if (isset($_GET[$var])) {
			$retVal = trim($_GET[$var]);
		}
		return $retVal;
	}
	
	function getSearchPref($type = 'search_type') {
		global $SERVICES;
		$prefs =& $SERVICES->getService('Preferences');
		switch ($type) {
			case 'search_type':
				$search_type = $prefs->getUserPref('searchType');
				if (!$search_type) {
					$search_type = 'initial';	
				}
				return $search_type;
			break;
			case 'search_modules':
				$search_modules = $SERVICES->preferences->getUserPref('searchScope');
				if (!$search_modules) {
					$search_modules = 'ContactOnly';	
				}
				return $search_modules;
			break;
			case 'limit_paging_overwrite':
				$limit_paging_overwrite = $SERVICES->preferences->getUserPref('searchMaxResultsPerModule');
				// if (!$limit_paging_overwrite) {
					// $limit_paging_overwrite = '';	
				// }
				return $limit_paging_overwrite;
			break;
			default:
			break;
		}
		return '';
	}

	/**
	 * Performs the search and returns the output.
	 *
	 * @return string Whatever output is generated.
	 */
	function performSearch($keywords = '', $type = '', $modules = '') {
		global $L;
		// no keywords? no search
		if ($this->search_keywords == ''){
			return '';
		}

		$content = '';

		// we have a minimum we have to keep.
		if (strlen($this->search_keywords) < $this->config['minimum_string_length']) {
			$msg = $L['searchYourSearchMustBeMinimumOf'];
			$msg .= ' ' . $this->config['minimum_string_length'] . ' ';
			if ($this->config['minimum_string_length'] == 1) {
				$msg .= $L['searchMinimumCharacter'];
			} else {
				$msg .= $L['searchMinimumCharacters'];
			}
			$content .= $this->getMessage($msg);
		} else {
			$content .= $this->getSearchPieces();
			if ($this->num_found == 0) {
				$content .= $this->getMessage($L['searchYourSearchReturnedNoResults']);
			}
		}
		return $content;
	}

	/**
	 * Explicitly sets the keywords.
	 * @param string $new_keywords New keywords to search
	 */
	function setSearchKeywords($new_keywords_to_search) {
		$this->keywords = $new_keywords_to_search;
	}
	
	/**
	 * @return string The combined output of all searched pieces where a result was returned
	 */
	function getSearchPieces() {
		global $BCRM_TIME_START, $L;

		$output =& $this->getOutput();
		$error_msg = '';

		$module_list = $this->getModules();

		$content = $content_other = ''; // build the contact view content	
		$num_mod_found = 0;

		// Build the display content
		$this->output->req_contact_js = true;
		
		// todo - Phil 20060130 - add a method to Output to load styles (similar to adding js files)
		//$incl_css = '<link rel="stylesheet" type="text/css"  href="' . $this->output->getSkinWebPath() . '/css/contact.css">';
		//$this->output->addToHead($incl_css);
		
		
		// ----------
		$tabs_content = '';
		$tabs = array();
		$active_tab = false;
		
		// Phil 20070524 Splitting preferences into 2 tabsets
		$tabs_1_content = '';
		$tabs_1 = array();
		$active_tab_1 = false;
		
		// if (count($module_list) > 10) {
		if (1) { // Forces splitting into 2 tabsets - 1 for contacts, 1 for the rest
			$tabs_1_modules = array(
				'Contact',
				'Company', 
				'Person',
				'ContactRelationship', 
				'ClientLoginManager'
			);
			foreach ($tabs_1_modules as $k => $v) {
				$tabs_1[$v] = false;
			}
		} else {
			$tabs_1_modules = array();
		}
		
		// ----------
		foreach ($module_list as $mod_name) {
			$module = $mod_name;
			$search_piece_object = getWebMATObject( 
				'WebMATSearch', 
				$mod_name, 
				array('data_option' => 'search_results')
			);

			if (is_object($search_piece_object) && isset($search_piece_object->db_functions)) {
				// setPieceFilter will return true or false, depending on if any fields were searched.
				if ($search_piece_object->setPieceFilter(false, $this->search_keywords, $this->search_type)) {
					// Phil 20060719
					if ($this->limit_paging_overwrite) {
						// print $this->limit_paging_overwrite . '<hr>';
						if ($this->limit_paging_overwrite == 'All') {
							$this->limit_paging_overwrite = $this->output->paging_max_list_size;
						}
						$search_piece_object->setPagingOverwrite($this->limit_paging_overwrite);
					}
	
					$num_mod_found = $search_piece_object->db_functions->countItems();
					// print '<div>num_mod_found: ' . $num_mod_found . ' (' . $mod_name . ')</div>';

					if ($num_mod_found > 0) {
						$this->num_found += $num_mod_found;
						$piece_identifier = $search_piece_object->getPieceId();
						// print '<div>piece identifier: ' . $piece_identifier . ' class: ' . get_class($search_piece_object) . '</div>';
						// Tabbed Content
						// todo: use skin web path for images
						// todo: use title text instead of identifier as label
						
						// --
						// Phil 20051121 Only display row if there is content for this piece
						if (strlen($contact_piece_content = $search_piece_object->main(true))) {
							$module_label = $search_piece_object->getTitleText() . ' ('. $num_mod_found .')';
							$current_tab_content = '
							<div id="tab' . $piece_identifier . '" class="tab-content">';
							$current_tab_content .= $contact_piece_content;
							$current_tab_content .= '
							</div>';
							
							
							if (in_array($module, $tabs_1_modules)) {
								$tabs_1_content .= $current_tab_content;
								if (!$active_tab_1) {
									$active_tab_1 = 'tab' . $module;
								}
								$tabs_1[$module] = array('id' => 'tab' . $module, 'title' => $module_label);
								
							} else {
								$tabs_content .= $current_tab_content;
								if (!$active_tab) {
									$active_tab = 'tab' . $module;
								}
								$tabs[$module] = array('id' => 'tab' . $module, 'title' => $module_label);
								
							}
							
						}
					}
				}
				unset($search_piece_object);
				$search_piece_object = NULL;
			}

			// time check: if the search takes over a certain amount of time,
			// we break from the loop, and throw out a message saying so, and
			// that perhaps they should narrow their search down.
			$time_end = microtimeFloat();
			$time = $time_end - $BCRM_TIME_START;

			if ($time > $this->config['maximum_run_time']) {
				$error_msg .= $L['searchSearchNotCompletedExceededMaxRunTime'];
				break;
			}

			// another check is the number of results.
			if ($this->num_found > $this->config['maximum_num_results']) {
				$error_msg .= $L['searchSearchNotCompletedExceededMaxNumResults'];
				break;
			}
		}
		
		// only return if there were any found
		if ($this->num_found > 0) {
			if (count($tabs_1)) {
				$content .= '
			<div id="searchTabs_1" class="search-results-container">
			' . $tabs_1_content . '
			</div>
			';
			}
			
			if (count($tabs)) {
				$content .= '
			<div id="searchTabs" class="search-results-container">
			' . $tabs_content . '
			</div>
			';
			}
			
			$output->initExt();
			
			if (count($tabs_1)) {
				$output->initTabs('searchTabs_1');
				foreach ($tabs_1 as $tab) {
					if ($tab) {
						$output->addTab($tab, 'searchTabs_1');
					}
				}
				$output->setFirstTab($active_tab_1, 'searchTabs_1');
			}
			if (count($tabs)) {
				$output->initTabs('searchTabs');
				foreach ($tabs as $tab) {
					if ($tab) {
						$output->addTab($tab, 'searchTabs');
					}
				}
				$output->setFirstTab($active_tab, 'searchTabs');
			}
		} else {
			$content = '';
		}

		if ($error_msg != '') {
			$content = $this->getMessage($error_msg) . $content;
		}
		$content .= $set_first_js; 
		return $content;
	}

	/**
	 * @access private
	 * @deprecated
	 * @return string The combined output of all searched pieces where a result was returned
	 */
	function getSearchPieces_version4() {
		global $BCRM_TIME_START, $L;

		$error_msg = '';

		$module_list = $this->getModules();

		$content = $content_other = ''; // build the contact view content	
		$num_mod_found = 0;

		// Build the display content
		$this->output->req_contact_js = true;
		$first = true;
		// initialise tabs string (table)
		$tabs_content = '';
		$tabs_content .= $this->output->getBeginRow();
		$tabs_content .= $this->output->getBeginCell(3, '', 'th');
		
		// todo - Phil 20060130 - add a method to Output to load styles (similar to adding js files)
		//?$incl_css = '<link rel="stylesheet" type="text/css"  href="' . $this->output->getSkinWebPath() . '/css/contact.css">';
		//$this->output->addToHead($incl_css);
		
		$viewAll_flag = true;
		$first_flag = true;
		$first_tab_in_list = '';
		foreach ($module_list as $mod_name) {
		
			//Adds tab in for View All - David
			if($viewAll_flag) {
				$piece_identifier = "ViewAll";
				$tabs_content .= '<div onClick="chngActv(\'' . $piece_identifier . '\'); viewAllState();" id="tab' . $piece_identifier . '" class="tab"> 
				<table border="0" cellspacing="0" cellpadding="0">
				<tr>
				<td rowspan="2"><img src="' . $this->output->getSkinWebPath() .'/img/Curve-TL.gif" width="6" height="6" alt=""></td>
				<th><th>
				<td rowspan="2"><img src="' . $this->output->getSkinWebPath() .'/img/Curve-TR.gif" width="6" height="6" alt=""></td>
				</tr>
				<tr>
				<th><div onClick="toggleViewAll();" id="tabUpDn" class="tabDn"></div></th>
				</tr>
				</table>
				</div>
				<div id="' . $piece_identifier . '"></div>';
				$viewAll_flag = false;
			}	
		
		
			$search_piece_object = getWebMATObject( 
				'WebMATSearch', 
				$mod_name, 
				array('data_option' => 'search_results')
			);

			if (is_object($search_piece_object) && isset($search_piece_object->db_functions)) {
				// setPieceFilter will return true or false, depending on if any fields were searched.
				if ($search_piece_object->setPieceFilter(false, $this->search_keywords, $this->search_type)) {
					// Phil 20060719
					if ($this->limit_paging_overwrite) {
						// print $this->limit_paging_overwrite . '<hr>';
						if ($this->limit_paging_overwrite == 'All') {
							$this->limit_paging_overwrite = $this->output->paging_max_list_size;
						}
						$search_piece_object->setPagingOverwrite($this->limit_paging_overwrite);
					}
	
					$num_mod_found = $search_piece_object->db_functions->countItems();

					if ($num_mod_found > 0) {
						$this->num_found += $num_mod_found;
						$piece_identifier = $search_piece_object->getPieceId();
						// print '<div>piece identifier: ' . $piece_identifier . ' class: ' . get_class($search_piece_object) . '</div>';
						// Tabbed Content
						// todo: use skin web path for images
						// todo: use title text instead of identifier as label
						$first = false;
						
			
				if (!$first_tab_in_list) {
					$first_tab_in_list = $piece_identifier;
				}
		
						
				//note first tab with items 20070124
				if ($first_flag && $num_mod_found) {
					//$set_first_js = '<script type="text/javascript">alert(\'' . $piece_identifier . '\'); chngActv(\'' . $piece_identifier . '\');</script>';
					$set_first_piece = $piece_identifier;
					$first_flag = false;
				}
				
						
						$current_tab =
						'<div onClick="chngActv(\'' . $piece_identifier . '\'); chngClass(\'single\');" id="tab' . $piece_identifier . '" class="tab' . ($firstTab ? ' tabF' : '') . '">
						<table border="0" cellspacing="0" cellpadding="0">
						<tr>
						<td rowspan="2"><img src="../../skins/blue/img/Curve-TL.gif" width="6" height="6"></td>
						<th><th>
						<td rowspan="2"><img src="../../skins/blue/img/Curve-TR.gif" width="6" height="6"></td>
						</tr>
						<tr>
						<th>' . $search_piece_object->getTitleText() . ' ('. $num_mod_found .')</th>
						</tr>
						</table>
						</div>
						';
					 
						// --
						// Phil 20051121 Only display row if there is content for this piece
						if (strlen($contact_piece_content = $search_piece_object->main(true))) {
							$tabs_content .= $current_tab;
							$content .= '
							<div id="' . $piece_identifier . '">';
							
							// $content .= $search_piece_object->main(true);
							$content .= $contact_piece_content;
							
							$content .= '
							<table width="100%" class="tabContentBottom">
							<tr>
								<td>
								</td>
							</tr>
							</table>
							<div class="spacer">&nbsp;</div>
							</div>';
						}
					}
				}
				unset($search_piece_object);
				$search_piece_object = NULL;
			}

			// time check: if the search takes over a certain amount of time,
			// we break from the loop, and throw out a message saying so, and
			// that perhaps they should narrow their search down.
			$time_end = microtimeFloat();
			$time = $time_end - $BCRM_TIME_START;

			if ($time > $this->config['maximum_run_time']) {
				$error_msg .= $L['searchSearchNotCompletedExceededMaxRunTime'];
				break;
			}

			// another check is the number of results.
			if ($this->num_found > $this->config['maximum_num_results']) {
				$error_msg .= $L['searchSearchNotCompletedExceededMaxNumResults'];
				break;
			}
		}
		
		if (!$set_first_piece) {
			$set_first_piece = $first_tab_in_list;
		}	
		
		$set_first_js = '<script type="text/javascript">chngActv(\'ViewAll\'); chngClass(\'all\');</script>
						<script type="text/javascript">chngActv(\'' . $set_first_piece . '\');chngClass(\'single\');</script>';

		// end tabs string (table)
		$tabs_content .= $this->output->getEndCell('th');
		$tabs_content .= $this->output->getEndRow();
		
		// only return if there were any found
		if ($this->num_found > 0) {
			$content = $this->output->getBeginTable('contactPiecesTbl') . 
			$tabs_content . 
			$this->output->getBeginRow() . 
			$this->output->getBeginCell(1,'contactPiecesCell') . 
			'<div id="activeArea"></div>' . 
			'<div class="lSpacer">&nbsp;</div>' . 
			'<div id="nonActiveArea" class="single">' . 
			$content . 
			'</div>' . 
			$this->output->getEndCell() . 
			$this->output->getEndRow() . 
			$this->output->getEndTable();
		} else {
			$content = '';
		}

		if ($error_msg != '') {
			$content = $this->getMessage($error_msg) . $content;
		}
		$content .= $set_first_js; 
		return $content;
	}

	/**
	 * Retrieves an array of modules that we'll search, filters out modules the user doesn't have access to.
	 *
	 * @param boolean $show_all Optional, default false. Set to true to return all the modules the user has access to
	 * Use this param only to populate the seach scope dropdown (Phil 20060719)
	 */
	function getModules($show_all = false) {
		global $SERVICES, $USER;

		if (!$show_all && $this->search_modules == 'ContactOnly') {
			$module_list = $this->config['contact_modules'];
		} else {
			$ua =& $SERVICES->getService('Authentication');
			$mod =& $SERVICES->getService('Modules');
			$module_list = $mod->getModules(2); // 2 - denotes we want all.
	
			// narrow the scope, if applicable - these are user prefs.
			if ($this->search_modules == 'NotMail') {
				bcrm_unset($module_list, array_search('Mail', $module_list));
			}

			// loop through the modules
			foreach ($module_list as $k => $v) {
				// remove the ones we don't have access to or the ones that are specific meant to be excluded
				if (!$ua->readAccess($USER['user_id'], $v)
				|| in_array($v, $this->config['exclude_modules'])
				|| in_array($v, $this->config['exclude_system_modules'])) {
					bcrm_unset($module_list, $k);
				} else if (!$show_all && ($this->search_modules == $v)) { // Phil 20060719
					// print '<div>Search modules:</div>'; print_r(array($v)); print '<hr>';
					return array($v);
				}
			}
		}
		// print '<div>Search modules:</div>'; print_r($module_list); print '<hr>';
		return $module_list;
	}

	/**
	 * Returns a formatted message for display under the form.
	 */
	function getMessage($msg = '') {
		if ($msg == ''){
			return '';
		}

		$content = '';
		$content .= $this->output->getBeginDataCell();
		$content .= '<p align="center">&nbsp;<br>' . $msg . '<br>&nbsp;</p>';
		$content .= $this->output->getEndDataCell();

		return $content;
	}
	
	/**
	 * Creates the form for doing a search
	 * @param bool $needFormToggle Whether or not we actually need the toggle button.
	 * @param bool $defaultShowForm If we're using the toggle, if the form is shown by default.
	 */
	function getSearchForm($needFormToggle=false, $defaultShowForm=false, $sidebar = false){
		global $L, $SERVICES;

		if ($needFormToggle) {
			$jsearch_filter = "
			<script type='text/javascript' language='javascript1.2'>
				/**
				 * Toggles showing the search form
				 */
				function toggleSearchForm(obj,show,hide){
					if (obj.innerHTML == show) {
						obj.innerHTML = hide;
						status = 'block';
					} else if( obj.innerHTML == hide ) {
						obj.innerHTML = show;
						status = 'none';
					}
	
					if (document.getElementById) { // DOM3 = IE5, NS6
						document.getElementById('searchform').style.display=status;
					} else if (document.layers){ // Netscape 4
						document.searchform. display = status;
					} else { // IE 4
						document.all.searchform.style. display = status;
					}
				}
			</script>";
			// $this->output->addToHead($jsearch_filter);
		}
	
		$keyword_field = array(
			'label' => $L['searchKeywords'],
			'notnull' => 1,
			'field_key' => 'search_keywords',
			'type' => 'text',
			'value' => $this->search_keywords);
	
		$search_type = array(
			'label' => $L['searchType'],
			'notnull' => 1,
			'field_key' => 'search_type',
			'type' => 'dropdown',
			'value' => $this->search_type,
			'dropdown_values' => array(
				'initial' => $L['searchInitialMatch'],
				'contains' => $L['searchContaining'])
		);

		$search_modules = array(
			'label' => $L['searchModules'],
			
			'field_key' => 'search_modules',
			'notnull' => 1,
			'type' => 'dropdown',
			'value' => $this->search_modules,
			'dropdown_values' => array(
				'ContactOnly' => $L['searchScopeContact'],
				'NotMail' => $L['searchScopeNotMail'],
				'All' => $L['searchScopeAll'])
		);
		
		$this->addIndividualModules($search_modules['dropdown_values']);
		
		$num_results_per_module = array(
			'label' => $L['searchMaxResultsPerModule'],
			'field_key' => 'limit_paging_overwrite',
			'notnull' => 0,
			'type' => 'dropdown',
			'value' => $this->limit_paging_overwrite, 
			'dropdown_values' => array(
				'5'  => '5',
				'10' => '10', 
				'15' => '15', 
				'20' => '20', 
				'25' => '25', 
				'30' => '30', 
				'40' => '40', 
				'50' => '50', 
				'75' => '75', 
				'100'=> '100', 
				'All' => $L['ListAll']
			)
		);
		

		// $content .= '<!-- Start search form -->';
		
		// if($needFormToggle){
			// $content .= $this->getSearchFormShowHideLink($defaultShowForm) ."<br />";
			// $content .= "<div id=\"searchform\" style=\"display:". ($defaultShowForm ? "block" : "none") .";\">\n";
		// }
		// print '<div>sidebar ' . ($sidebar ? 'yes' : 'no') . '</div>';
		if ($sidebar) {
			
			// Replace labels with explanations
			$keyword_field['explanation'] = $keyword_field['label'];
			$keyword_field['label'] = '';
			$search_type['explanation'] = $search_type['label'];
			$search_type['label'] = '';
			$search_modules['explanation'] = $search_modules['label'];
			$search_modules['label'] = '';
			$num_results_per_module['explanation'] = $num_results_per_module['label'];
			$num_results_per_module['label'] = '';
			
			$action = 'modules/Search/index.php';
			$content .= $this->output->getBeginForm($action, 0, '', false, 'searchform', 'contentframe');
		} else {
			$action = $SERVICES->getGlobal('php_self') . '?' . $SERVICES->getGlobal('query_string');
			$content .= $this->output->getBeginForm($action);
		}
		
		//$content .= $this->output->getBeginDataCell();

		// we don't want the search form stretched, so we empty the width
	//	$this->output->setTagContent('viewTbl', 'width', '');
	//	$content .= $this->output->getBeginTable('viewTbl');
		// since we emptied the width, we need to reset it
	//	$this->output->resetTagContent('viewTbl');
	
		$content .= '<div class="search-container">';
		$content .= $this->output->getBeginTable('viewTbl');
	
		$content .= $this->output->getHtmlFieldRow($keyword_field);
		$content .= $this->output->getHtmlFieldRow($search_type);
		$content .= $this->output->getHtmlFieldRow($search_modules);
		$content .= $this->output->getHtmlFieldRow($num_results_per_module);
		
		$content .= $this->output->getSubmitRow();

		$content .= $this->output->getEndTable();
		$content .= '</div>';
		
		//$content .= $this->output->getEndDataCell();
		$content .= $this->output->getEndForm();
		
		// if($needFormToggle){
			// $content .= "</div>\n";
		// }
		
		// $content .= '<!-- End search form -->';

		return $content;
	}
	
	function addIndividualModules(&$dropdown_values) {
		global $SERVICES;
		// Phil 20060719
		$all_mods = $this->getModules(true); 
		// print_r($all_mods); print '<hr>';
		if (is_array($all_mods)) {
			$lang =& $SERVICES->getService('Language');
			foreach ($all_mods as $k => $v) {
				if (strlen($v)) {
					$prefix = strtolower( substr($v, 0, 1) ) . substr($v, 1);
					$label = $lang->getLanguageText($prefix . 'EventNamePlural', $v);
					$dropdown_values[$v] = ($label ? $label : $v);
				}
			}
		}
	}
	
	
	/**
	 * Returns the show/hide form link
	 * @param bool $defaultShowForm If we're using the toggle, if the form is shown by default.
	 */
	function getSearchFormShowHideLink($defaultShowForm=true){
		global $SERVICES, $L;
	
		$content = "";
		$output =& $SERVICES->getService('Output');
		$tag_extras = ' onClick="javascript: toggleSearchForm(this,\''.$L['labelSearchFilterShow'].'\',\''.$L['labelSearchFilterHide'].'\');"';
	
		if ($defaultShowForm) {
			$link .= $output->getLinkHref(array(
				'tag_extras' => $tag_extras,
				'href' => 'javascript: ;',
				'text' => $L['labelSearchFilterHide']));
		} else {
			$link .= $output->getLinkHref(array(
				'tag_extras' => $tag_extras,
				'href' => 'javascript: ;',
				'text' => $L['labelSearchFilterShow']));
		}
	
		return $link;
	}


	/**
	 * @deprecated version 4.5.
	 */
	function reIndex() {}
	/**
	 * @deprecated version 4.5.
	 */
	function addIndex() {}
}


/**
 * @deprecated version 4.5.
 */
class SearchMatch {
	function SearchMatch () {}
}

?>