<?php
/**
 * A collection of classes extended from WebMATDatabaseTable
 * 
 * You need an extended class for each table in you module not including the 
 * dropdown tables.
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package SystemBookmarks
 */

/**
 * This class handles the database functions for the item table
 * 
 * This module is an extend verison of WebMATDatabaseTable with the table_name 
 * set and the alert_where set.
 * 
 */
class itemSystemBookmarksDatabaseTable extends WebMATDatabaseTable {
	
	/**
	 * The constructor class for itemSystemBookmarksDatabaseTable
	 *
	 * @param string $module_name the module name
	 * @param array $data_definition the data definition
	 */
	function itemSystemBookmarksDatabaseTable($module_name, $data_definition) {
		$this->table_name = 'item';
		$this->WebMATDatabaseTable($module_name, $data_definition); //calls the constructor for WebMATDatabaseTable
	}
	
	/*
	 * Adds the design time filters to the where array
	 * 
	 * changed to add the user_id design time filter it not there
	 * 
	 * @param the current where statement
	 * @return the current were statement with the new where statement added to the end
	 */
	// function getDesignTimeFiltersWhere($where_array) {
		// if (!isset($this->design_time_filters['user_id'])) {
			// global $USER;
			// //Syed 20050714 filters without userid
			// //$this->setDesignTimeFilters(array('user_id' => $USER['id'])); 
		// }
		// return parent::getDesignTimeFiltersWhere($where_array);
	// }
	
	/**
	 * Calculates a calculated value
	 * 
	 * @param array $dd_item the items with dd structure
	 */
	function calculateValues(& $dd_item, $item) {
		global $SERVICES;
		if (!$this->bookmark_links) {
			$SERVICES->getService('modules');
			$system_bookmarks_mod =& $SERVICES->modules->getModule('SystemBookmarks');
			$this->bookmark_links = $system_bookmarks_mod->getBookmarkLinks(false, false, false, false, false); // subscribed=true
		}
		$display_name = $this->getDisplayNameField();
		$output =& $SERVICES->getService('Output');
		$dd_item[$display_name]['value'] = $output->getLinkHref($this->bookmark_links[$item['id']]);
		$dd_item[$display_name]['no_link'] = true;
		// print '<div>' . get_class($this) . ' calculate values ' . $item['id'] . ' display name: ' . $dd_item[$display_name]['value'] . '</div>';
	}
	
	function setItem($item) {
		$id = parent::setItem($item);
		if (is_numeric($id)) {
			$this->resetValues($id, $item);
		}
		return $id;
	}
	
	/**
	 * @access private
	 */
	function resetValues($id = false, $item = false, $reset = false) {
		global $USER;
		$where = array();
		
		$user_id = $USER['user_id'];
		$where[] = 'user_id=\'' . $user_id . '\'';
		
		if (!$reset) {
			if ($item) {
				if (!$item['id']) {
					$item['id'] = $id;
				}
			} else {
				$item = $this->getItem($id);
			}
			$where[] = 'id<>\'' . $id . '\'';
		} else if (!$item) {
			return;
		}
		
		if ($item['is_alert']) {
			$where['option'] = 'is_alert=1';
			$this->db->update($this->getTableName(), array('is_alert' => '0'), implode(' AND ', $where));
			bcrm_unset($where, 'option');
		}
		if ($item['is_dashboard']) {
			$where['option'] = 'is_dashboard=1';
			$this->db->update($this->getTableName(), array('is_dashboard' => '0'), implode(' AND ', $where));
			bcrm_unset($where, 'option');
		}
		if ($item['is_welcome']) {
			$where['option'] = 'is_welcome=1';
			$this->db->update($this->getTableName(), array('is_welcome' => '0'), implode(' AND ', $where));
			bcrm_unset($where, 'option');
		}
	}
	
	/**
	 * Returns true and updates tables and fields parameters, or returns false if there is no seen table
	 * 
	 * @param string The tables string (gets updated if there is a seen table)
	 * @param string The fields string (gets updated if there is a seen table)
	 * @return boolean Whether or not there is a seen table
	 * @author Phil Calder
	 */
	function processGetItemsItemUserTable(&$tables, &$fields) {
		if (parent::processGetItemsItemUserTable($tables, $fields)) {
			$seen_table = $this->table_name . '_seen';
			$fields .= ', '. $seen_table . '.subscribed AS subscribed';
			return true;
		}
		return false;
	}
	
}
?>