<?php
/**
 * The main class for this module
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package SystemBookmarks
 */

includeModule('SystemBookmarks', 'inc/database_tables.inc');

class SystemBookmarks extends WebMATModule {
	function SystemBookmarks() {
		$this->module_name = 'SystemBookmarks';
		$this->WebMATModule();
	}
	
	/**
	 * returns a array of bookmark links to display on the Front menu
	 *
	 * @return array an array of links
	 * @author Phil Calder
	 */
	function getMenuBookmarkLinks() {
		return $this->getBookmarkLinks(false,true,false);
	}
	
	/**
	 * returns a array of bookmark links
	 * 
	 * @param string $module module name (optional)
	 * @param integer $id the bookmark id (optional)
	 * @param string $dashboard what list to display in the dashboard (optional)
	 * @return array an array of links
	 */
	function getBookmarkLinks($id = false, $dashboard = false, $module=false, $exclude_module = false, $subscribed = true) {
		global $USER, $SERVICES;
		$db_functions =& $this->getDatabaseTable('item'); // Phil 20050116 Added ampersand to assignment
		$db_functions->resetDesignTimeFilters(); // Phil 20060201 This method can be called multiple times with different parameters
		$db_functions->resetWhereStatement(); // Phil 20060201 This method can be called multiple times with different parameters
		
		//Syed 20050726 | filter by modulewise
		if ($module || $dashboard || !$id) {
			// Phil 20060223 See db functions calculate values
			if ($subscribed) {
				// Phil 20050804 gets subscribed bookmarks by default
				$db_functions->setDesignTimeFilters(
					array(
						'__special__' => array(
							'where' => " subscribed = 1 "
						 )
					)
				);
			}
			if ($module) {
				$db_functions->setQueryType('items'); // Phil 20050824 -- why is this here?
				$db_functions->setDesignTimeFilters(array('module' => $module));
			}
		}
		if ($id) {
			$db_functions->setDesignTimeFilters(array('id' => $id));
		} 
		
		// Phil 20060201
		if ($exclude_module) {
			$db_functions->setDesignTimeFilters(
				array(
					'module' => array(
						'where' => ' module <> "' . $exclude_module . '"'
					 )
				)
			);
		}
		$bookmarks = array();
		if ($dashboard) {
			$SERVICES->getService('Preferences');
			//Syed 20050725 | removed is_int function from the condtion
			if (!($dashboard_count = $SERVICES->preferences->getUserPref('systemBookmarksDashboardCount', 10))) {
				$dashboard_count = 10;
			}
			$limit = 'LIMIT ';
			if ($dashboard == 'list') {
				$limit .= $dashboard_count;
			} else {
				$limit .= "0 , 1000"; //syed 20050705
			}
		} else {
			$limit = false;
		}
		$items = $db_functions->getItems(array('field' => 'priority'), $limit);
		
		if (is_array($items)) {
			foreach ($items as $k => $v) {
				if ($bm = $this->getBookmarkLink($v)) {
					if ($id) {
						return $bm;
					} else {
						if ($bm) {
							$bookmarks[$v['id']] = $bm;
						}
					}
				}
			}
		}
		$db_functions->resetDesignTimeFilters();
		$db_functions->resetWhereStatement();
		return $bookmarks;
	}
	
	/**
	 * builds the link based on the bookmark item
	 * 
	 * @param array the bookmark item
	 * @return array with the keys: 
	 *	- 'text' the bookmark title
	 * 	- 'href' the bookmark link
	 */
	function getBookmarkLink($item) {
		global $SERVICES;
		$SERVICES->getService('Config');
		
		// Incorrect - replaced with call to new Module Service function
		// $modules_dir = '';
		// if ($SERVICES->config->lookup('site.websubdir')) {
			// $modules_dir .= '/' . $SERVICES->config->lookup('site.websubdir');
		// } 
		// $modules_dir .= '/modules';
		
		$modules_dir = $SERVICES->modules->getModulePath($item['module']);
		if (!$modules_dir) {
			return array('text' => $item['name'], 'href' => '#');
		}
		
		if ($query_string = $item['querystring']) {
			$query_string = '?' . $query_string;
		}
		switch ($item['page_type']) {
			case 'list':
				// $href = $modules_dir . '/' . $item['module'] . '/' . $item['filename'] . '?bookmark=' . $item['id'];
				$href = $modules_dir . '/' . $item['filename'] . '?bookmark=' . $item['id'];
				$bm = array('id'=>$item['id'],'text' => $item['name'], 'href' => $href); //Syed 20050728 | included the item id 
			break;
			case 'view':
			case 'edit':
			case 'multi_edit':
				// $href = $modules_dir . '/' . $item['module'] . '/' . $item['filename'] . $query_string;
				$href = $modules_dir . '/' . $item['filename'] . $query_string;
				$bm = array('text' => $item['name'], 'href' => $href);
			break;
			case 'url':
				$href = $item['module'] . '/' . $item['filename'] . $query_string;
				$bm = array('text' => $item['name'], 'href' => $href);
			break;
			default:
				$bm = false;
			break;
		}
		return $bm;
	}
	
	/**
	 * Returns a bookmark based on the design time filter
	 * 
	 * Checks for both the user_id on the design time filter and for a user id of
	 * zero
	 * 
	 * @param array $dtf the design time filter
	 * @param string boolean $search_default check to see if a default 
	 *  bookmark is set (optional default value is true)
	 * @return array a bookmark item
	 */
	function getSpecialBookmark($dtf, $search_default = true) {
		global $USER;
		$db_functions =& $this->getDatabaseTable('item');
		$db_functions->setDesignTimeFilters($dtf);
		if ($bookmark = $db_functions->getItem(false, 'priority DESC')) {
			return $this->unserializeExtraValues($bookmark);
		} else if ($search_default) {
			$db_functions->setDesignTimeFilters(array('user_id' => 0));
			if ($bookmark = $db_functions->getItem(false, 'priority DESC')) {
				return $this->unserializeExtraValues($bookmark);
			}
		}
		return array();
	}
	
	/**
	 * returns the alert bookmark for the dashbord
	 * 
	 * @param string $module the module name
	 * @param string $table the table name
	 * @return array the bookmark item arry
	 */
	function getAlertBookmark($module, $table) {
		global $USER;
		$dtf = array(
				'module' => $module,
				'table_name' => $table,
				'is_alert' => 1,
				'user_id' => $USER['id']
			);
		return $this->getSpecialBookmark($dtf);
	}
	
	/**
	 * returns the desktop bookmark for the dashbord
	 * 
	 * @param string $module the module name
	 * @param string $table the table name
	 * @return array the bookmark item arry
	 */
	function getDesktopBookmark($module, $table) {
		global $USER;
		$dtf = array(
				'module' => $module,
				'table_name' => $table,
				'is_dashboard' => 1,
				'user_id' => $USER['id']
			);
		return $this->getSpecialBookmark($dtf);
	}
	
	/**
	 * returns the welcome bookmark for the dashbord
	 * 
	 * @return array the bookmark item arry
	 */
	function getWelcomeBookmark() {
		global $USER;
		$dtf = array(
				'is_welcome' => 1,
				'user_id' => $USER['id']
			);
		return $this->getSpecialBookmark($dtf);
	}
	
	/**
	 * returns a bookmark item based on id
	 * 
	 * @return array the bookmark item arry
	 */
	function getBookmark($id = false) {
		$db_functions =& $this->getDatabaseTable('item');
		if ($bookmark = $db_functions->getItem($id, 'priority DESC')) {
			return $this->unserializeExtraValues($bookmark);
		}
		return false;
	}
	
	/**
	 * returns the desktop bookmark for the dashbord (ordered on the priority)
	 * 
	 * @param string $module the module name
	 * @param string $table the table name
	 */
	function getFirstBookmarkForModule($module) {
		global $USER;
		$dtf = array(
				'module' => $module,
				'user_id' => $USER['id']
			);
		return $this->getBookmarkLink($this->getSpecialBookmark($dtf, false));
	}
	
	/**
	 * unserializes the extra values
	 * 
	 * @param array the item array
	 * @return array the item array with a unserialized extra_values
	 */
	function unserializeExtraValues($item) {
		if (isset($item['extra_values'])) {
			$item['extra_values'] = unserialize($item['extra_values']);
		}
		return $item;
	}
}
?>