<?php
require_once("../../prepend.inc");
/**
 * Create a new BrowserCRM user.
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package UserAdmin
 * @author Phil Calder
 */

$ua =& $SERVICES->getService('Authentication');

$ua->adminRestrict();
 
$SERVICES->getService('Language');

$SERVICES->language->getLanguage('UserAdmin');
$SERVICES->language->getLanguage('Front');
$SERVICES->language->getLanguage('Licence');

$output =& $SERVICES->getService('Output');

$content = '';

function getFormFields($item) {
	global $L, $ua, $SERVICES;
	
	$skins = array();
	
	$screen =& $SERVICES->getService('Screen');
	$detailed_skins = $screen->getSkins();
	
	foreach ($detailed_skins as $k => $v) {
		$skin_label = trim(strip_tags($v['name']));
		$skin_label = (isset($L[$skin_label]) && $L[$skin_label] ? $L[$skin_label] : $skin_label);
		$skins[$v['dir_name'] . '/'] = $skin_label;
	}
	
	if (empty($item['privs_groups'])) {
		$everyone_group = $ua->groups->getGroupID('Everyone');
		$item['privs_groups'] = array(); 
		if (!empty($everyone_group)) {
			$item['privs_groups'][] = $everyone_group;
		}
	}
	
	$groups = array('-1'=>$L['useradminNewUsersPrivateGroup']);
	$groups = $groups + $ua->groups->getGroupList();
	
	//field definitions
	$fields =  array( 
		array('label'=>'', 'field_key'=>'id', 'type'=>'hidden'),
		array('label'=>'', 'field_key'=>'returnpage', 'type'=>'hidden'),
		
		array('label'=>$L['useradminFirstName'], 'field_key'=>'firstname', 'type'=>'text'), // , 'notnull'=>1), 
		array('label'=>$L['useradminLastName'], 'field_key'=>'lastname', 'type'=>'text'), // , 'notnull'=>1), 
		
		array('label'=>$L['useradminAddContactRecord'], 'field_key'=>'create_contact', 'type'=>'checkbox'), 
		
		array('label'=>$L['useradminProductUsername'] . ' ' . $L['useradminUsuallyJustFirstName'], 'field_key'=>'username', 'type'=>'text', 'notnull'=>1), 
		array('label'=>$L['useradminPassword'], 'field_key'=>'password1', 'type'=>'password', 'notnull'=>1), 
		array('label'=>$L['useradminPasswordAgain'], 'field_key'=>'password2', 'type'=>'password', 'notnull'=>1), 
		
		array('label'=>$L['useradminSkin'], 'field_key'=>'skin', 'type'=>'dropdown', 'dropdown_values'=>$skins, 'notnull'=>1),
		
		array('label'=>$L['useradminUserPrivsGroups'],    'field_key'=>'privs_groups', 'type'=>'multi_select', 'dropdown_values'=>'__PRIVSGROUPS__', 'notnull'=>1), 
		array('label'=>$L['useradminUserTeamGroups'],     'field_key'=>'team_groups', 'type'=>'multi_select', 'dropdown_values'=>'__TEAMGROUPS__'), 
		
		array('label'=>$L['useradminDefaultViewGroup'],   'field_key'=>'default_view_group', 'type'=>'dropdown', 'dropdown_values'=>$groups), 
		array('label'=>$L['useradminDefaultEditGroup'],   'field_key'=>'default_edit_group', 'type'=>'dropdown', 'dropdown_values'=>$groups),
		array('label'=>$L['useradminDefaultOwnerGroup'],  'field_key'=>'default_owner_group', 'type'=>'dropdown', 'dropdown_values'=>$groups),
		array('label'=>$L['useradminDefaultAdminGroup'],  'field_key'=>'default_admin_group', 'type'=>'dropdown', 'dropdown_values'=>$groups),
		
		array('label'=>$L['useradminPersonalViewGroup'],  'field_key'=>'personal_view_group', 'type'=>'dropdown', 'dropdown_values'=>$groups), 
		array('label'=>$L['useradminPersonalEditGroup'],  'field_key'=>'personal_edit_group', 'type'=>'dropdown', 'dropdown_values'=>$groups),
		array('label'=>$L['useradminPersonalOwnerGroup'], 'field_key'=>'personal_owner_group', 'type'=>'dropdown', 'dropdown_values'=>$groups),
		array('label'=>$L['useradminPersonalAdminGroup'], 'field_key'=>'personal_admin_group', 'type'=>'dropdown', 'dropdown_values'=>$groups),
	
		array('label'=>$L['useradminPublicViewGroup'],    'field_key'=>'public_view_group', 'type'=>'dropdown', 'dropdown_values'=>$groups), 
		array('label'=>$L['useradminPublicEditGroup'],    'field_key'=>'public_edit_group', 'type'=>'dropdown', 'dropdown_values'=>$groups),
		array('label'=>$L['useradminPublicOwnerGroup'],   'field_key'=>'public_owner_group', 'type'=>'dropdown', 'dropdown_values'=>$groups),
		array('label'=>$L['useradminPublicAdminGroup'],   'field_key'=>'public_admin_group', 'type'=>'dropdown', 'dropdown_values'=>$groups)
	
		
		
	);
	
	foreach ($fields as $k => $v) {
		$fields[$k]['value'] = (isset($item[$v['field_key']]) ? $item[$v['field_key']] : '');
	}
	
	return $fields;
	
}

function doSubmit() {
	global $SERVICES, $PHP_SELF, $L, $mod, $ua, $output, $returnpage, $USER;
	
	$item = $_POST['item'];
	
	if ($_POST['submit']) {
		
		//server-side data validation goes here
		$fields = getFormFields($item);	//get field definitions
		$output->validateFields($item, $fields, $error_msg); //validate $item based on the definitions in $fields; $item values will get updated
		
		if (is_array($error_msg)) {		//if server-side validation errors, return to form
			main( $item['id'], $item, $error_msg);
			exit();
		} else {
			// Custom validation for this page
			if ($item['password1'] != $item['password2']) {
				$error_msg = array();
				$error_msg['password1'] = $L['useradminPasswordsDidNotMatch'];
				
				bcrm_unset($item, 'password1');
				bcrm_unset($item, 'password2');
				
				main( $item['id'], $item, $error_msg);
				exit();
			}
		}

		// -----------------------
		$SERVICES->getService('Preferences');
		$SERVICES->getService('Modules');
		
		$contact_id = '';
		$url = 'users.php';
		
		if ($ua->writeAccess($USER['user_id'], 'Contact')) {
			
			$contact =& $SERVICES->modules->getModule('Contact');
		
			// Only create a contact if they wanted us to
			if (!empty($item['create_contact'])) {
				
				$db_functions =& $contact->getDatabaseTable('item');
				$new_contact = array();
				$contact_id = $db_functions->setItem($new_contact);
				
				if ($contact_id) {
				// Bosco 20050831 | Changed to redirect to Person page with first and last names	
					if ($ua->writeAccess($USER['user_id'], 'Person')) {
						$person =& $SERVICES->modules->getModule('Person');
						$db_functions =& $person->getDatabaseTable('item');
						$new_person = array(
							'first_name' => $item['firstname'], 
							'last_name' => $item['lastname'], 
							'contact_id' => $contact_id, 
							'primary_record' => 1
						);
						$person_id = $db_functions->setItem($new_person);
					}
					//$person_info = '&new[Person][first_name]=' . $item['firstname'] . '&new[Person][last_name]=' . $item['lastname'] . '&exclude_modules[0]=Company';
					// Bosco 20050901 | Removed $person_info
					$url = $SERVICES->config->getModuleHref("Contact", "new") . '&id=' . $contact_id;	
					$url = str_replace('%contact_type%', '0', $url);
						
				} else { // Otherwise set the url, so we dont break
					$url = 'users.php';
				}
			
			} else { // Otherwise set the url, so we dont break
				$url = 'users.php';
			}
		}
	
		$user_id = $ua->users->addUser( 
			$item['username'], 
			$item['password1'], 
			$item['skin'], 
			$contact_id, 
			$item['privs_groups'], 
			$item['team_groups'] 
		);
		
		if (empty($user_id)) { // User not added
			
			$error_msg = array();
			bcrm_unset($item, 'username');
			$error_msg['username'] = $L['useradminUsernameAlreadyInUse'];
			main( $item['id'], $item, $error_msg);
			exit();			
			
		} else {
			
			$prefs =& $SERVICES->getService('Preferences');
			$new_users_private_group = $ua->users->getPrivateGroup($user_id);
			$permissions_keys = array(
				'frontDefaultGroupView'  => 'default_view_group', 
				'frontDefaultGroupEdit'  => 'default_edit_group', 
				'frontDefaultGroupOwner' => 'default_owner_group', 
				'frontDefaultGroupadmin' => 'default_admin_group', 
				
				'frontPrivateGroupView'  => 'personal_view_group', 
				'frontPrivateGroupEdit'  => 'personal_edit_group', 
				'frontPrivateGroupOwner' => 'personal_owner_group', 
				'frontPrivateGroupAdmin' => 'personal_admin_group', 
				
				'frontPublicGroupView'   => 'public_view_group', 
				'frontPublicGroupEdit'   => 'public_edit_group', 
				'frontPublicGroupOwner'  => 'public_owner_group', 
				'frontPublicGroupAdmin'  => 'public_admin_group', 
				
				'frontDefaultGroupAssignedTo' => 'default_owner_group', 
				'frontPrivateGroupAssignedTo' => 'personal_owner_group', 
				'frontPublicGroupAssignedTo'  => 'public_owner_group', 
				
			);
			
			foreach ($permissions_keys as $k => $v) {
				if (isset($item[$v])) {
					if ($item[$v] == -1) {
						$item[$v] = $new_users_private_group;
					}
					if (!empty($item[$v])) {
						// print '<div>setUserPref(' . $k . ', ' . $item[$v] . ', ' . $user_id . ')</div>';
						$prefs->setUserPref($k, $item[$v], $user_id);
					}
					// print '<div>' . $v . ': [' . $item[$v] . ']</div>';
				}
			}
		}
	
		header("Location: $url");
		exit();

	}
	
}

function doRedirect($returnpage, $contact_id) {
	global $SERVICES;
	
	$SERVICES->getService('Config');
	//this event page may be called from the module, the contact_view page, or 
	//within the contact_view event frame, so reflect that in the return location
	if ($returnpage == 'contact_view') {
		header('Location: ' . $SERVICES->config->getModuleHref( 'Contact', 'view' ) .'?id=' . $contact_id);
		exit();
	} else {
		header('Location: index.php?contact_id=' . $contact_id);
		exit();
	}	
	exit();
}


function main($id = '', $item = '', $error_msg = '') {
	global $SERVICES, $USER, $PHP_SELF, $L, $mod, $output, $ua, $contact_id; //, $returnpage;
	
	//$id is the id of a particular item record in this module
	//$contact_id is the id of the contact passed to this paage, if any
	$SERVICES->getService('Language');
	$SERVICES->language->getLanguage('Licence');
	
	if (!$item) {  //item could be passed back from server-side data validation failure
		
		global $SERVICES;
		$prefs =& $SERVICES->getService('Preferences');
		
		// -- set any defaults here
		$item = array(
			'create_contact' => 1, 
			
			// 'default_view_group' => -1, // new users private group
			'default_view_group'   => ( (($default_view_group   = $prefs->getCompanyPref('frontDefaultGroupView'))  !== false) ? $default_view_group   : '-1'), 
			'default_edit_group'   => ( (($default_edit_group   = $prefs->getCompanyPref('frontDefaultGroupEdit'))  !== false) ? $default_edit_group   : '-1'), 
			'default_owner_group'  => ( (($default_owner_group  = $prefs->getCompanyPref('frontDefaultGroupOwner')) !== false) ? $default_owner_group  : '-1'), 
			'default_admin_group'  => ( (($default_admin_group  = $prefs->getCompanyPref('frontDefaultGroupAdmin')) !== false) ? $default_admin_group  : '-1'), 
			
			'personal_view_group'  => ( (($personal_view_group  = $prefs->getCompanyPref('frontPrivateGroupView'))  !== false) ? $personal_view_group  : '-1'), 
			'personal_edit_group'  => ( (($personal_edit_group  = $prefs->getCompanyPref('frontPrivateGroupEdit'))  !== false) ? $personal_edit_group  : '-1'), 
			'personal_owner_group' => ( (($personal_owner_group = $prefs->getCompanyPref('frontPrivateGroupOwner')) !== false) ? $personal_owner_group : '-1'), 
			'personal_admin_group' => ( (($personal_admin_group = $prefs->getCompanyPref('frontPrivateGroupAdmin')) !== false) ? $personal_admin_group : '-1'), 
			
			'public_view_group'    => ( (($public_view_group    = $prefs->getCompanyPref('frontPublicGroupView'))   !== false) ? $public_view_group    : '-1'), 
			'public_edit_group'    => ( (($public_edit_group    = $prefs->getCompanyPref('frontPublicGroupEdit'))   !== false) ? $public_edit_group    : '-1'), 
			'public_owner_group'   => ( (($public_owner_group   = $prefs->getCompanyPref('frontPublicGroupOwner'))  !== false) ? $public_owner_group   : '-1'), 
			'public_admin_group'   => ( (($public_admin_group   = $prefs->getCompanyPref('frontPublicGroupAdmin'))  !== false) ? $public_admin_group   : '-1'), 
			
		);	
		foreach ($item as $k => $v) {
			if ($v == 'private_group_id') {
				$item[$k] = '-1';
			}
		}
		// -- 
	}
	
	//add any data elements not in item to the item array
	
	//if page was called from events, preserve this for return
	if (!isset($item['returnpage'])) {
		if (isset($_GET['returnpage'])) {
			$item['returnpage'] = $_GET['returnpage'];
		}
	}
	
	$licensed_users = $SERVICES->getNumLicensedUsers();
	
	//user count reporting
	$users = $ua->users->getUsersList();
	$current_users = sizeof($users) -1; // -1 for the Admin user
	
	//it is a violation of the terms of the licence agreement to alter or modify the 
	//user count reporting
	$warning = array();
	$halt = false;
	
	if ($current_users >= $licensed_users) {
		if ($current_users > $licensed_users) {  
			$warning[] = $L['licenceLimitExceeded'];
			$param_key = 'user_count_exceeded';
		} elseif ($current_users == $licensed_users) {
			$warning[] = $L['licenceLimitReached'];
			$param_key = 'upgrade';
		}
		// $warning[] = $output->getAlert($L['useradminLicenceUpgrade']);
		
		// Do not change the line below
		$show_upgrade_warning = 1;
		if ($show_upgrade_warning) {
			$warning[] = '<a href="' . $SERVICES->getContactUrl($SERVICES->isLicenceFileExisting() ? $param_key : 'buy') . '" target="_blank">' . ($SERVICES->isLicenceFileExisting() ? $L['licenceGetUpgrade'] : $L['licenceGetNewKey']) . '</a>';
			$warning[] = '<br>&nbsp;<br><a href="../../licence" target="_blank">' . $L['licenceUploadLicence'] . '</a>';
		}
		
		$halt = true;
	}	
	
	// ---
	$action_links = array(
		'<a href="users.php">' . $L['useradminAdminUsersListAndMailAccounts'] . '</a>', 
	);
	$output->toolbar_link_mode = true;
	$output->setActionLinksMenu((array)$action_links);
	$output->toolbar_link_mode = false;
	
	//initialise $content variable for the page
	$content = '';
	
	$content .= $output->getBeginTable();
	
	// $content .= $output->getTitleRow($L['useradminCreateNewUser'], array('help' => 'useradminNew_user'));
	// $content .= $output->getFieldSpacerRow();
	// $content .= $output->getBeginNestedViewTable();
	// 
	// $content .= $output->getTextRow($L['licUsers'] . ': ' . $licensed_users); // . ' ' . $L['useradminLCUsers']);
	// $content .= $output->getTextRow($L['useradminCurrentUsers'] . ': ' . $current_users . '<br>&nbsp;');
	// 
	// if (sizeof($warning)) {
		// foreach($warning as $msg) {
			// $content .= $output->getTextRow($msg);	
		// }
	// }
	// 
	// $content .= $output->getEndNestedTable();
	// $content .= $output->getFieldSpacerRow();
	
	//nested data table ------------------
	$content .= $output->getBeginNestedEditTable();
	
	//form
	$content .= $output->getBeginForm($PHP_SELF);  
	//or, to exclude javascript input validation for your form, call it like this:   $content .= $output->getBeginForm($PHP_SELF, 0);
	
	$content .= $output->getTextRow($L['licUsers'] . ': ' . $licensed_users); // . ' ' . $L['useradminLCUsers']);
	$content .= $output->getTextRow($L['useradminCurrentUsers'] . ': ' . $current_users . '<br>&nbsp;');
	
	if (sizeof($warning)) {
		foreach($warning as $msg) {
			$content .= $output->getTextRow($msg);	
		}
	}
	
	// ============ Halt here ===============================
	if ($halt) { // Cannot add a new user
		$content .= $output->getEndNestedTable();
		$content .= $output->getEndTable();
		displayNewUserScreen($content);
	} // ====================================================
	
	
	foreach (getFormFields($item) as $k => $v) {
		//$v['value'] = $item[$v['field_key']];
		$content .= $output->getHtmlFieldRow($v, $error_msg[$v['field_key']]);
	}
	
	// $content .= $output->getFieldSpacerRow();
	$content .= $output->getSubmitRow();
	// $content .= $output->getFieldSpacerRow();
	$content .= $output->getEndForm();
	//end form
	
	$content .= $output->getEndNestedTable();
	//end nested data table ------------------
	$content .= $output->getEndTable();
	
	displayNewUserScreen($content);

}

function displayNewUserScreen($content = '') {
	global $L, $output;
	$padding = true;
	$window_layout = $output->getBeginTable('mainTbl', true);
	$window_layout .= $output->getTitleRow($L['useradminCreateNewUser'], array('help' => 'useradminNew_user', 'bookmark'=>'off'));
	$window_layout .= $output->getBeginRow('', true) . '<td id="bodyCell"' . ($padding ? 'style="padding: 10px"' : '') . '>';
	$window_layout .= $content;
	$window_layout .= '</td>' . $output->getEndRow(true);
	$window_layout .= $output->getEndTable(true);
	
	$output->addToBody($window_layout);
	$output->display();
	exit();
	
}



if (isset($_POST['submit'])) {
	doSubmit();
}

main();

?>