<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package BrowserCRMDefault
 */

// Phil 20060615 Use output buffering. 
// Not necessary if encoded or zlib.output_compression is On (see php.ini)
if (!isset($_REQUEST['_no_gz'])) {
	ob_start("ob_gzhandler");
} else {
	ob_start();
}


// Temporary - error_reporting not usually set here
// error_reporting(E_ALL^E_NOTICE);
// error_reporting(E_ALL);

/**
 * @access private
 */
function microtimeFloat() {
	if (function_exists('microtime')) {
		list($usec, $sec) = explode(" ", microtime());
	}
	return ((float)$usec + (float)$sec);
}

$BCRM_TIME_START = microtimeFloat();

if (!empty($_REQUEST)) extract($_REQUEST);
if (!empty($_FILES)) extract($_FILES);
if (empty($PHP_SELF)) $PHP_SELF = $_SERVER['PHP_SELF'];

$_prepend_paths = array(); // Phil 20060529

/**
 * Attempts to include the file for the specified Service
 *
 * @param string Service name
 * @return integer (boolean) 0 or 1
 */
function includeService($name) {
	global $BCRM_DOCUMENT_ROOT, $_prepend_paths;
	$service_file = $BCRM_DOCUMENT_ROOT . '/services/inc/' . $name . '.inc';
	if (in_array($service_file, $_prepend_paths)) {
	// if (in_array($name, $_prepend_paths)) {
		// print "<div>cached includeService($name)</div>";
		return 1;
	}
	
	$retval = 0;
		
	if (file_exists($service_file) && is_readable($service_file)) {
		$_prepend_paths[] = $service_file; // Phil 20060529
		// $_prepend_paths[] = $name; // Phil 20060916
		require_once($service_file);
		// print "<div>caching - includeService($name)</div>";
		$retval = 1;
	} 
	// print '<div>name: ' . $name .' prepend paths:</div>'; print_r($_prepend_paths); print '<hr>';
	return $retval;
}

/**
 * Attempts to include the specified file for the Module
 *
 * This function handles custom modules and (standard or custom) module extensions.
 * Files are loaded in order:
 * - The 'standard' or 'custom' module class_file (if it exists)
 * ie relative the the 'standard' or 'custom' module directory
 * - The 'extended' module class_file (if it exists)
 * ie relative to the 'module extension' directory
 *
 * @param string Module name
 * @param string $class_file The filename (and path relative to the module), eg 'inc/classes.inc'
 * @return integer (boolean) 0 or 1
 */ 
function includeModule($name, $class_file) {
	global $BCRM_DOCUMENT_ROOT, $_prepend_paths;
	
	$retval = 0;

	// Load the requested standard OR custom module file
	$standard_class_file = $BCRM_DOCUMENT_ROOT . '/modules/' . $name . '/' . $class_file;
	
	// Phil 20060529 This will mean that the standard, ext, or custom file has been loaded
	if (in_array($standard_class_file, $_prepend_paths)) {
		// print "<div>cached includeModule($name, $class_file)</div>";
		return 1;
	}
	
	if (file_exists($standard_class_file) && is_readable($standard_class_file)) {
		include_once($standard_class_file);
		$retval = 1;
	} else if (getExtDirExists('modules_custom')) {
		$custom_class_file = $BCRM_DOCUMENT_ROOT . '/modules_custom/' . $name . '/' . $class_file;
		if (file_exists($custom_class_file) && is_readable($custom_class_file)) {
			include_once($custom_class_file);
			$retval = 1;
		}
	}

	// Load the (optional) extension to the requested standard OR custom module file
	if (getExtDirExists('modules')) {
		$ext_class_file = $BCRM_DOCUMENT_ROOT . '/modules_ext/' . $name . '/' . $class_file;
		if (file_exists($ext_class_file) && is_readable($ext_class_file)) {
			include_once($ext_class_file);
			$retval = 1;
		} 
	}
	
	if ($retval) {
		$_prepend_paths[] = $standard_class_file;
	}
	
	return $retval;
}

/**
 * Attempts to include the specified WebMAT file
 *
 * This function handles WebMAT extensions. WebMAT extensions allow for custom code to be
 * placed in the WebMAT extension directory to extend standard WebMAT functionality.
 * Files are loaded in order: (eg for list.inc (the WebMATList class file)
 *
 * - The 'standard' webmat file (if it exists),
 * eg 'webmat_list.inc'. Contains the class WebMATStandardList.
 * This is the parent class for WebMATList.
 * This will be in the 'webmat' directory.
 *
 * EITHER:
 * - The 'extended' webmat class_file (if it exists)
 * eg 'list.inc'. Should contain the class WebMATList, which extends WebMATStandardList
 * This will be in the 'webmat extension' directory - This is for end-user custom code.
 * OR:
 * - The 'webmat' class_file (if it exists),
 * eg 'list.inc'. Contains the class WebMATList, which extends WebMATStandardList
 * This will be in the 'webmat' directory.
 *
 * @param string $class_file_basename The filename, eg 'list.inc'
 * @return integer (boolean) 0 or 1
 */
function includeWebMAT($class_file_basename) {
	global $BCRM_DOCUMENT_ROOT, $_prepend_paths;
	// global $SERVICES;
	// $SERVICES->servicesPrint('includeWebMAT(' . $class_file_basename . ')');
	$retval = 0;
	
	$class_file = $BCRM_DOCUMENT_ROOT . '/webmat/webmat_' . $class_file_basename;
	
	// Phil 20060529 This will mean that the ext file, and webmat_* file will have been loaded too
	if (in_array($class_file, $_prepend_paths)) {
		// print "<div>cached includeWebMAT($class_file_basename)</div>";
		return 1;
	}
	
	if (file_exists($class_file) && is_readable($class_file)) {
		// $SERVICES->servicesPrint('includeWebMAT - include_once(' . $class_file . ')');
		$_prepend_paths[] = $class_file; // Phil 20060529
		include_once($class_file);
		// $SERVICES->servicesPrint('includeWebMAT - included ' . $class_file . '');
	}

	$class_file = $BCRM_DOCUMENT_ROOT . '/webmat/' . $class_file_basename;
	$ext_class_file = $BCRM_DOCUMENT_ROOT . '/webmat_ext/' . $class_file_basename;

	if (getExtDirExists('webmat') && file_exists($ext_class_file) && is_readable($ext_class_file)) {
		// $SERVICES->servicesPrint('includeWebMAT - include_once(' . $ext_class_file . ')');
		include_once($ext_class_file);
		// $SERVICES->servicesPrint('includeWebMAT - included ' . $ext_class_file . '');
		$retval = 1;
	} else if (file_exists($class_file) && is_readable($class_file)) {
		// $SERVICES->servicesPrint('includeWebMAT - include_once(' . $class_file . ')');
		include_once($class_file);
		// $SERVICES->servicesPrint('includeWebMAT - included ' . $class_file . '');
		$retval = 1;
	}
	// $SERVICES->servicesPrint('includeWebMAT(' . $class_file_basename . ') returning');
	return $retval;
}

/**
 * Absolute path to the 'external' directory
 *
 * @return string
 */
function getExternalDir() {
	global $BCRM_DOCUMENT_ROOT;
	return $BCRM_DOCUMENT_ROOT . '/external';
}

/**
 * Absolute path to the 'licence' directory
 *
 * @return string
 */
function getLicenceDir() {
	global $BCRM_DOCUMENT_ROOT;
	return $BCRM_DOCUMENT_ROOT . '/licence/txt';
}

/**
 * Attempts to include the file from the 'external' directory
 *
 * @param string $package_name The name of the package directory
 * @param string $class_file The name of the class file to load
 * @return integer (boolean) 0 or 1
 */
function includeExternal($package_name, $class_file) {
	// global $BCRM_DOCUMENT_ROOT;
	// $class_file = $BCRM_DOCUMENT_ROOT . '/external' . '/' . $package_name . '/' . $class_file;
	$class_file = getExternalDir() . '/' . $package_name . '/' . $class_file;
	
	$retval = 0;

	if (file_exists($class_file) && is_readable($class_file)) {
		include_once($class_file);
		$retval = 1;
	} 
	
	return $retval;
}

/**
 * Attempts to include the file from the 'Smarty' directory
 *
 * @param string $class_file The name of the 'Smarty' file to load
 * (relative to the 'Smarty' directory)
 * @return integer (boolean) 0 or 1
 */
function includeSmarty($class_file) {
	return includeExternal('smarty', $class_file);
}

/**
 * Attempts to include a template file for the specific module
 *
 * DISABLED
 *
 * @param string $module_name The name of the module we're currently dealing with
 * @param string $template_file The filename (and path relative to the module), eg 'template/item_edit.tpl'
 * @return integer (boolean) 0 or 1
 *
 * @author Brenton Bartel
 */
function includeTemplate($module_name, $template_file) {
	global $BCRM_DOCUMENT_ROOT, $SERVICES;
	return 0; // Phil 20060529 Temporaily disabled
	// print "<div>includeTemplate($module_name, $template_file)</div>";

	// Find the requested standard OR custom module file
	$template_check = $BCRM_DOCUMENT_ROOT . '/modules/' . $module_name . '/'. $SERVICES->config->lookup('templates.tpl_dir') .'/' . $template_file .'.tpl';

	$retVal = 0;
	$has_file = false;

	if (file_exists($template_check) && is_readable($template_check)) {
		$has_file = true;
	} else {
		// might be custom, so check that too.
		$template_check = $BCRM_DOCUMENT_ROOT . '/modules_custom/' . $module_name . '/'. $SERVICES->config->lookup('templates.tpl_dir') .'/' . $template_file .'.tpl';
		if (file_exists($template_check) && is_readable($template_check)) {
			$has_file = true;
		}
	}

	if ($has_file) {
		// we don't actually need to include this, cause the TemplateEngine will do it for us
		// include_once($template_file);
		//
		$template =& $SERVICES->getService('TemplateEngine');
		$template->setTemplatePath($template_check);

		$retVal = 1;
	}

	return $retVal;
}

/**
 * This function makes it easier for classes/pages to access WebMAT objects without needing to know
 * whether the class has been extended.
 *
 * By specifying the required WebMAT class, Module name, and other parameters, this function
 * will load required files and return the necessary object. This allows end users to extend
 * functionality on a per-module basis.
 *
 * @param string $base_class The base WebMAT class (eg WebMATList)
 * @param string $module The module this object will be used for
 * @param array $args An array or args to pass to the class constructor.
 * Keys for the $args array vary depending on the $base_class.
 * Standard keys (for many WebMAT classes) are:
 * 'module' Optional, defaults to the $module parameter already specified)
 * 'data_option' Optional, defaults to 'list'
 * 'post' Optional, defaults to $_POST
 * 'get' Optional, defaults to $_GET
 * 'table' Optional, defaults to 'item'
 * 'language_overwrite' Optional, defaults to false (should be an array of language keys and text)
 * 'piece_identifier' Optional, neccessary when you want two separate classes for the same Module
 * with the same data definition. Defaults to whatever the 'table' arg is.
 * This arg is used along with the 'module' arg and 'base_class' to determine the class name
 * to look for.
 *
 * Example:
 * You want the WebMATList object for the Template module. By default, this function would look for:
 * i) The WebMATList class (by calling {@link includeWebMAT})
 * ii) The TemplateItemWebMATList class
 * (based on the $module, default 'table' arg (item) and $base_class)
 * This class shoul be in a file returned by calling {@link includeModule}:
 * includeModule($module, $class_name), where $class_name is:
 * inc/(piece_identifier)_(base_class_file).inc
 * There are various other args for certain WebMAT classes. Consult the WebMAT documentation.
 *
 * @return Object|false The WebMAT object, or false
 */
function getWebMATObject($base_class, $module, $initial_args = array()) {
	global $BCRM_DOCUMENT_ROOT;

	$default_args = array(
		'module'=> $module,
		'data_option'=> 'list', // default data option
		'post'=> $_POST,
		'get'=> $_GET,
		'table'=> 'item', // default table
		'language_overwrite' => false
	);

	if (!is_array($initial_args) || !sizeof($initial_args)) { // default args for new object
		$args = $default_args;
	} else {
		$args = array_merge((array)$default_args, (array)$initial_args);
	}
	
	$file_prefix = $args['table'];
	$file_prefix = ( (isset($args['piece_identifier']) && $args['piece_identifier']) ? $args['piece_identifier'] : $file_prefix);

	switch ($base_class) { // TODO - replace with WebMAT.config array
		case 'WebMAT':
			$base_name = 'webmat';
		break;
		case 'WebMATList':
			$base_name = 'list';
		break;
		case 'WebMATPublicList':
			$base_name = 'public_list';
		break;
		case 'WebMATEdit':
			$base_name = 'edit';
		break;
		case 'WebMATPublicEdit':
			$base_name = 'public_edit';
		break;
		case 'WebMATView':
			$base_name = 'view';
		break;
		case 'WebMATPublicView':
			$base_name = 'public_view';
		break;
		case 'WebMATMassEdit':
			$base_name = 'mass_edit';
		break;
		case 'WebMATMultiEdit':
			$base_name = 'multi_edit';
		break;
		case 'WebMATMassCreate':
			$base_name = 'mass_create';
		break;
		case 'WebMATProcessMulti':
			$base_name = 'process_multi';
		break;
		case 'WebMATExport':
			$base_name = 'export';
		break;
		case 'WebMATDropdown':
			$base_name = 'dropdown';
			if (!isset($args['field_name'])) {
				$args['field_name'] = false;
			}
			if (!isset($args['where'])) {
				$args['where'] = false;
			}
		break;
		case 'WebMATAdminDropdown':
			$base_name = 'admin_dropdown';
		break;
		case 'WebMATAdminNestedDropdown':
			$base_name = 'admin_nested_dropdown';
		break;
		case 'WebMATSelector':
			$base_name = 'selector';
		break;
		case 'WebMATShortView':
			if (isset($args['piece_identifier'])) {
				$args['get']['piece_identifier'] = $args['piece_identifier'];
			}
			$base_name = 'short_view';
		break;
		case 'WebMATShortList':
			if (isset($args['piece_identifier'])) {
				$args['get']['piece_identifier'] = $args['piece_identifier'];
			}
			$base_name = 'short_list';
		break;
		case 'WebMATContactView':
			if (isset($args['piece_identifier'])) {
				$args['get']['piece_identifier'] = $args['piece_identifier'];
			}
			$base_name = 'contact_view';
		break;
		case 'WebMATContactViewEventList':
			if (isset($args['piece_identifier'])) {
				$args['get']['piece_identifier'] = $args['piece_identifier'];
			}
			$base_name = 'contact_view_event_list';
		break;
		case 'WebMATSearch':
			$base_name = 'search';
		break;
		case 'WebMATDesktop':
			// if (isset($args['bookmark'])) {
				// $args['get'] = array_merge((array)$args['get'], array('bookmark'=>$args['bookmark']));
			// }
			$base_name = 'desktop';
		break;
		case 'WebMATDashboard':
		case 'WebMATBookmarkDashboard':
			$base_name = 'dashboard';
		break;
		case 'WebMATDataDictionary':
			$base_name = 'data_dictionary';
			if (!isset($args['data_definition'])) {
				$args['data_definition'] = false;
			}
		break;
		case 'WebMATFilters':
			$base_name = 'filters';
			// $args['module'],
			// $args['form_values'],
			// $args['table'],
			// $args['dd'],
			// $args['db_functions']
		break;
		case 'WebMATAdvancedFilters':
			$base_name = 'advanced_filters';
			if (!isset($initial_args['data_option']) || !$initial_args['data_option']) {
				// default_args would have set this to 'list'
				$args['data_option'] = '__all__';
			}
		break;
		case 'WebMATModule':
			$base_name = 'module';
		break;
		case 'WebMATOpenFile':
			if (!isset($args['master_dir'])) {
				$args['master_dir'] = false;
			}
			if (!isset($args['sub_dir'])) {
				$args['sub_dir'] = false;
			}
			$base_name = 'open_file';
		break;
		case 'WebMATMenu':
			$base_name = 'menu';
			if (isset($args['post'])) {
				unset($args['post']);
			}
			if (isset($args['get'])) {
				unset($args['get']);
			}
			if (isset($args['table'])) {
				unset($args['table']);
			}
		break;

		default:
			$base_name = 'webmat';
		break;

		// TODO
		// (some of these could be "abstract" eg pieces_view, some may be "final")
		// menu, filters,
		// database_detail_table, database_table
		// open_file, pieces_view, ruleset_edit
	}
	
	if (includeWebMAT($base_name . '.inc')) {
		if (class_exists($base_class)) {
			$class_name = $base_class;
		}
	}

	if (includeModule($module, 'inc/' . $file_prefix . '_' . $base_name . '.inc')) {
		if (class_exists($module . $file_prefix . $base_class)) {
			$class_name = $module . $file_prefix . $base_class;
		}
		if (class_exists('Custom' . $module . $file_prefix . $base_class)) {
			$class_name = 'Custom' . $module . $file_prefix . $base_class;
		}
	}

	// template additions - BB - 20050811
	// we also need to check if a module-specific template exists
	// This function will get the 'TemplateEngine' service, which takes care of the template.
	// When the Output class gets instantiated, it'll check if the TemplateEngine
	// service has been loaded (which correctly assumes a template exists). From then on,
	// the Output and whatever other functions can use it.
	includeTemplate($module, $file_prefix . '_' . $base_name);

	if (isset($class_name) && class_exists($class_name)) {
		switch ($base_class) { // TODO - replace with WebMAT.config array
			case 'WebMATDropdown':
				return new $class_name($args['module'], $args['table'], $args['field_name'], $args['where']);
			break;
			case 'WebMATAdminDropdown':
			case 'WebMATAdminNestedDropdown':
				return new $class_name($args['post'], $args['get'], $args['mini_data_definition'], $args['access_level']);
			break;
			case 'WebMATDataDictionary':
				return new $class_name($args['data_definition'], $args['data_option']);
			break;
			case 'WebMATFilters':
				return new $class_name($args['module'], $args['form_values'], $args['table'], $args['dd'], $args['db_functions']);
			break;
			case 'WebMATOpenFile':
				return new $class_name($args['module'], $args['table'], $args['post'], $args['get'],  $args['use_database'],  $args['sub_dir']);
			break;
			default:
				return new $class_name($args['module'], $args['data_option'], $args['post'], $args['get'], $args['table'], $args['language_overwrite']);
			break;
		}
	}

	return false;
}

function getExtDirExists($type = 'webmat') {
	global $BCRM_DOCUMENT_ROOT, $SERVICES;
	// print '<div>getExtDirExists(' . $type . ')</div>';
	if (isset($SERVICES->_extdirs[$type])) {
		// $SERVICES->servicesPrint('getExtDirExists(' . $type . ') returning cached value ' . $SERVICES->_extdirs[$type] . '');
		// print '<div>getExtDirExists(' . $type . ') returning cached value ' . $SERVICES->_extdirs[$type] . '</div>';
		return $SERVICES->_extdirs[$type];
	}
	
	$directory_name = '';
	switch ($type) {
		case 'webmat':	
			$directory_name = $BCRM_DOCUMENT_ROOT . '/webmat_ext';
		break;
		case 'modules':	
			$directory_name = $BCRM_DOCUMENT_ROOT . '/modules_ext';
		break;
		case 'modules_custom':	
			$directory_name = $BCRM_DOCUMENT_ROOT . '/modules_custom';
		break;
		default:
			// $SERVICES->servicesPrint('getExtDirExists(' . $type . ') returning false [1]');
			// print '<div>getExtDirExists(' . $type . ') returning false</div>';
			return false;
		break;
	}
	if (file_exists($directory_name) && is_readable($directory_name)) {
		$SERVICES->_extdirs[$type] = $directory_name;
		// $SERVICES->servicesPrint('getExtDirExists(' . $type . ') returning calculated value ' . $SERVICES->_extdirs[$type] . '');
		// print '<div>getExtDirExists(' . $type . ') returning calculated value ' . $SERVICES->_extdirs[$type] . '</div>';
		return $SERVICES->_extdirs[$type];
	}
	$SERVICES->_extdirs[$type] = 0;
	// $SERVICES->servicesPrint('getExtDirExists(' . $type . ') returning false [2]');
	// print '<div>getExtDirExists(' . $type . ') returning 0</div>';
	return false;
}

// use BCRM_DOCUMENT_ROOT instead of DOCUMENT_ROOT; no trailing slash
// this gives more control; eg for subdir installation; for Windows
if (empty($BCRM_DOCUMENT_ROOT)) {
	$BCRM_DOCUMENT_ROOT = dirname(__FILE__);
}

$BCRM_DOCUMENT_ROOT = str_replace('\\', '/', $BCRM_DOCUMENT_ROOT); // for Windows
$BCRM_DOCUMENT_ROOT = str_replace('//', '/', $BCRM_DOCUMENT_ROOT); // for Windows
$BCRM_DOCUMENT_ROOT = rtrim($BCRM_DOCUMENT_ROOT, '/');

session_start();

// Phil - These cookie parameters need to match Authentication Service code
if (isset($forget) && $forget) {
	
	setcookie('remember_me', '', time()-10, '/');
	session_unset(); // Logout as well
	
	header('Location: ./');
	exit();
} else if (isset($logout)) {
	// setcookie('remember_me', '', time()-10, '/');
	if (isset($_COOKIE['remember_me']) && isset($_SESSION['USER']) && is_array($_SESSION['USER']) && isset($_SESSION['USER']['username']) && $_SESSION['USER']['username']) {
		setcookie('remember_me', '~#~' . $_SESSION['USER']['username'], time() + 60 * 60 * 24 * 30, '/');
	}
	session_unset();
	if ($logout == 1) {
		//header('Location: index.php');  
		//MCR 20060110 for clean urls
		header('Location: ./');
	} else {
		header('Location: ' . base64_decode($logout));
	}
	exit();
}

if (!isset($_SESSION['USER'])) {
	$_SESSION['USER'] = 0; // to resolve session.bug_compat_42 and bug_compat_warn warning
}

$USER =& $_SESSION['USER'];

global $SERVICES;
// Necessary for the $SERVICES object
include_once($BCRM_DOCUMENT_ROOT . '/services/Services.inc');
includeService('Service');

if (empty($SERVICES)) {
	if (!file_exists($BCRM_DOCUMENT_ROOT . '/config.inc.php')) {
		header('Location: setup/index.php?ver=5.001.00');
		exit();
	}
	$SERVICES = new Services($BCRM_DOCUMENT_ROOT . '/config.inc.php');
}
// --------------------------------------------

$SERVICES->init();

$ua =& $SERVICES->getService('Authentication');
$ua->init();

$log =& $SERVICES->getService('Log');
$log->logUsage();

if (!isset($login['webform']) || !$login['webform']) {
	$ua->readRestrict();
}

?>