<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Services
 */

/**
 * Handles retrieval of configuration options.
 * 
 * @author Matthew Ross, Phil Calder
 * @package Services
 */
class Config extends Service {
	
	/**
	 * @var string $config_file
	 * @access private
	 */
	var $config_file = false;
	
	/**
	 * Stores given config file and calls {@link init}
	 * @author Phil Calder
	 */
	function Config($config_file) {
		$this->config_file = $config_file;
		$this->init();
	}
	
	/**
	 * Loads or rebuild the config structure
	 * @author Phil Calder
	 */
	function init() {
		$config_file = $this->config_file;
		// $this->printDebug('Config service constructor: ' . $config_file);
		$this->root = array();
		// print '<p>Config constructor</p>';
		if ($this->configNeedsRebuilding($config_file)) {
			// print '<p>Config constructor calling initConfig, addConfig, rebuildConfig</p>';
			$this->initConfig($config_file);
			$this->addConfig();
			$this->rebuildConfig();
		} else {
			// print '<p>Config constructor calling addConfig</p>';
			global $BCRM_DOCUMENT_ROOT;
			$this->root['includes'] = array($BCRM_DOCUMENT_ROOT . '/cache/compiled.config.inc.php');
			$this->addConfig();
		}
		$this->setConfigBackstop();
		$this->cache = array();
	}
	
	/**
	 * @author Phil Calder
	 * @access private
	 */
	function configNeedsRebuilding($config_file) {
		global $BCRM_DOCUMENT_ROOT;
		// print '<p>BCRM_DOCUMENT_ROOT: ' . $BCRM_DOCUMENT_ROOT . '</p>';
		// print '<p>config_file: ' . $config_file . '</p>';
		// TODO: check created time etc...
		
		// $front_config = $BCRM_DOCUMENT_ROOT . '/modules/Front/module.config';
		// $config = $this->readConfig($front_config);
		// print '<div>front config: </div>'; print_r($config); print '<hr>';
		// $this->version = $config['app']['version'];
		
		if (file_exists($BCRM_DOCUMENT_ROOT . '/cache/compiled.config.inc.php') && is_readable($BCRM_DOCUMENT_ROOT . '/cache/compiled.config.inc.php')) {
			// print '<p>compiled config file exists - no need to rebuild</p>';
			return false;
		}
		// print '<p>compiled config file does not exist - need to rebuild</p>';
		return true;
	}
	
	/**
	 * @author Phil Calder
	 * @access private
	 */
	function rebuildConfig() {
		if (function_exists('var_export') && is_array($this->root) && sizeof($this->root)) {
			global $BCRM_DOCUMENT_ROOT;
			$config_content = '<?php' . "\n" . '$config=';
			$config_content .= var_export($this->root, true);
			$config_content .= "\n" . '?>';
			$newfile = $BCRM_DOCUMENT_ROOT . '/cache/compiled.config.inc.php';
			if ($file = @fopen($newfile, "w")) {
				@fwrite($file, $config_content);
				@fclose ($file);
			}
		}
	}
	
	/**
	 * @author Phil Calder
	 * @access private
	 */
	function removeCachedConfig() {
		global $BCRM_DOCUMENT_ROOT;
		$config_file = $BCRM_DOCUMENT_ROOT . '/cache/compiled.config.inc.php';
		
		if (@file_exists($config_file)) {
			$result = @unlink($config_file);
			// print '<div>result: ' . ($result ? 'true' : 'false') . '</div>'; exit();
		} // else {
			// print '<div>no config</div>';
		// }
		$this->root = array(
			'includes' => array($BCRM_DOCUMENT_ROOT . '/cache/compiled.config.inc.php')
		);
		// print '<hr>'; exit();
	}
	
	/**
	 * Sets any default config items which may not appear in the loaded config files from earlier version installations.
	 * @author Matthew Ross, 20040506
	 * @access private
	 */
	function setConfigBackstop() { 
		//addition for old config files with no charset
		if (!isset($this->root['charset'])) {
			$this->root['charset'] = 'iso-8859-1';
		}
	}
	
	/**
	 * Returns the array or element specified by 'key'.
	 *
	 * @param $key The config item to search for.
	 * @param boolean $log_error Whether to log non-existant lookups
	 * @return The config item for the given key. Can be an array, a value, or empty.
	 * @author Matthew Ross, edited 20040506
	 * @author Phil Calder, added error logging using the Log service.
	 */
	function lookup($key, $log_error = false) {
		if (array_key_exists($key, $this->cache)) {
			// $this->printDebug('Config::lookup("' . $key . '") cached');
			return $this->cache[$key];
		}
		// $this->printDebug('Config::lookup("' . $key . '")');
		$answer = &$this->root;
		$tokens = explode('.',$key);
		$last_key = '<root>';
		
		foreach ($tokens as $k => $tok) {
			$tok = trim($tok);
			if (!$tok) {
				if ($log_error) {
					$this->logError('Config: searching for a key "' . $key . '"created an empty key token (' . $k . '), last key: ' . $last_key . '');
					}
				continue;
			} else if (!isset($answer[$tok])) {
				return '';
			} else if (!is_array($answer)) {
				if ($log_error) {
					$this->logError('Config: searching for a key "' . $key . '" but config tree branch ends at lask key "' . $last_key . '"');
				}
				return '';
			} else {
				$answer = &$answer[$tok];
			}
			$last_key = $tok;
		}
		$this->cache[$key] = $answer;
		return $answer;
	}
	
	/**
	 * @param $arg Optional module name which defaults to the current module.
	 * @return The config item for the given argument. Can be an array, a value, or empty.
	 */
	function getModuleConfig($arg = false) {
		if (empty($arg)) {
			global $SERVICES;
			$SERVICES->getService('Modules');
			$arg = $SERVICES->modules->getCurrent();
		}
		$cfg = $this->lookup('modules.' . $arg);
		// $this->printDebug('Config::getModuleConfig for ' . $arg);
		return $cfg;
	}
	
	/**
	 * Soon to be Deprecated
	 *
	 * Use Config::lookup to retrieve a particular page
	 **/
	function getModuleHref($module, $page = 'default', $level = 0, $file_set = false) {
		global $BCRM_DOCUMENT_ROOT, $SERVICES, $USER;
		if (isset($level) && $level)  {
			$ua =& $SERVICES->getService('Authentication');
			if (! $ua->access($USER['user_id'], $module, $level)) {
				return 0;
			}
		}
		
		if ($file_set) {
			$temp = $this->lookup('modules.' . $module . '.file_set.' . $file_set . '.functions.' . $page);
		} else {
			$temp = $this->lookup("modules.$module.functions.$page");
		}
		
		$href = $this->lookup('modules.basedir') . "/$module/" . $temp;
		$href = str_replace($BCRM_DOCUMENT_ROOT,'',$href);
		
		if ( strlen($subdir = $this->lookup('site.websubdir')) > 0 ) {
			$href = '/' . $subdir . '/' . trim($href, '/'); // Phil 20051031 Remove double separator
		}
		
		return $href;
	}
	
	/**
	 * Build an array: $config['includes'] containing paths to the module config files
	 *
	 * @param $config_file The initial config file.
	 * @author Matthew Ross, Phil Calder 
	 * @access private
	 */
	function initConfig($config_file) {
		// $this->printDebug('start initConfig: ' . $config_file);
		global $BCRM_DOCUMENT_ROOT;  //is required within the main config file that will be included
		include_once($config_file);
		
		// Standard Modules
		if ($handle = opendir($config['modules']['basedir'])) {
			if (!isset($config['includes']) || !is_array($config['includes']) ) {
				$config['includes'] = array();
			}
			// $this->printDebug('reading dir: ' . $config['modules']['basedir'] . '');
			while ($dir = readdir($handle)) {
				// print '<div>reading subdir: ' . $dir . '</div>';
				if (($dir != '.') && ($dir != '..') && is_dir($config['modules']['basedir'] . '/' . $dir)) {
					// $this->printDebug('subdir: ' . $dir . '');
					if (file_exists($config['modules']['basedir'] . '/' . $dir . '/module.config')) {
						// $this->printDebug('file exists: ' . $config['modules']['basedir'] . '/' . $dir . '/' . $config['modules']['configfile'] . '');
						$config['includes'][] = $config['modules']['basedir'] . '/' . $dir . '/module.config';
					}
				}
			}
		}
		
		// Custom Modules
		if (
			file_exists($config['modules']['custom_basedir']) && 
			is_dir($config['modules']['custom_basedir']) && 
			($handle = opendir($config['modules']['custom_basedir']))
		) {
			if (!isset($config['includes']) || !is_array($config['includes']) ) {
				$config['includes'] = array();
			}
			
			// $this->printDebug('reading dir: ' . $config['modules']['basedir'] . '');
			while ($dir = readdir($handle)) {
				// print '<div>reading subdir: ' . $dir . '</div>';
				if (($dir != '.') && ($dir != '..') && is_dir($config['modules']['custom_basedir'] . '/' . $dir)) {
					// $this->printDebug('subdir: ' . $dir . '');
					if (file_exists($config['modules']['custom_basedir'] . '/' . $dir . '/module.config')) {
						// $this->printDebug('file exists: ' . $config['modules']['basedir'] . '/' . $dir . '/' . $config['modules']['configfile'] . '');
						$config['includes'][] = $config['modules']['custom_basedir'] . '/' . $dir . '/module.config';
					}
				}
			}
		}
		
		// Module Extensions
		if (
			file_exists($config['modules']['extension_basedir']) && 
			is_dir($config['modules']['extension_basedir']) && 
			($handle = opendir($config['modules']['extension_basedir']))
		) {
			if (!isset($config['includes']) || !is_array($config['includes']) ) {
				$config['includes'] = array();
			}
			
			// $this->printDebug('reading dir: ' . $config['modules']['basedir'] . '');
			while ($dir = readdir($handle)) {
				// print '<div>reading subdir: ' . $dir . '</div>';
				if (($dir != '.') && ($dir != '..') && is_dir($config['modules']['extension_basedir'] . '/' . $dir)) {
					// $this->printDebug('subdir: ' . $dir . '');
					if (file_exists($config['modules']['extension_basedir'] . '/' . $dir . '/module.config')) {
						// $this->printDebug('file exists: ' . $config['modules']['basedir'] . '/' . $dir . '/' . $config['modules']['configfile'] . '');
						$config['includes'][] = $config['modules']['extension_basedir'] . '/' . $dir . '/module.config';
					}
				}
			}
		}
		// $this->printDebug($config);
		// $this->root = array_merge_recursive($this->root, $config);
		// print '<p>arrayMergeReplace - 1</p>';
		$this->root = $this->arrayMergeReplace($this->root, $config);
		// $this->printDebug('end initConfig: ' . $config_file);
		// $this->printDebug($config['includes']);
	}
	
	/** 
	 * Includes the specified file. 
	 * Merges the $config array with existing values (in $this->root).
	 * unsets $config
	 *
	 * @author Matthew Ross, Phil Calder
	 */
	function addConfig() {
		global $BCRM_DOCUMENT_ROOT;  //may be required within the config file that will be included
		$config_files = $this->root['includes'];
		if (isset($config_files) && is_array($config_files) && (sizeof($config_files)>0)) {
			if (sizeof($config_files) == 1) {
				$config_file = current($config_files);
				$this->root = $this->readConfig($config_file);
			} else {
				foreach($config_files as $k => $config_file) {
					// $this->root = array_merge_recursive($this->root,  $this->readConfig($config_file));
					// print '<p>arrayMergeReplace - 2</p>';
					// print '<p>addConfig file: ' .$config_file . '</p>';
					$this->root = $this->arrayMergeReplace($this->root, $this->readConfig($config_file));
				}
			}
		}
	}
	
	/**
	 * @author Phil Calder
	 * @access private
	 */
	function readConfig($config_file) {
		global $BCRM_DOCUMENT_ROOT;  //may be required within the config file that will be included
		if (file_exists($config_file)) {
			$config = array();
			include_once($config_file);
		 	return $config;
		}
	}
	
	/**
	 * Use this instead of array_merge_recursive
	 * @see WebMAT::arrayMergeReplace
	 */
	function arrayMergeReplace($old_array, $new_array) {
		if (!is_array($new_array)) {
			$new_array = array($new_array);	
		}
		
		foreach($new_array as $k => $v) {
		   if (is_array($v)) {
			   if (!isset($old_array[$k])) {
				   $old_array[$k] = array();
			   }
			   $old_array[$k] = $this->arrayMergeReplace($old_array[$k], $v);
		   } else {
			   $old_array[$k] = $v;
		   }
		}
		return $old_array;
	}
}
?>