<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Services
 */
 
//These variables are shared by all database objects
// Mimics a class 'static' variable
$GLOBALS['_database']['connection'] = '';
$GLOBALS['_database']['db_is_selected'] = false;
$GLOBALS['_database']['show_tables'] = '';
 
/**
 * This Database class handles single database functionality for tables with 'custom fields'.
 *
 * Table and column caching is used. The connection is retained regardless of which instance is used.
 * @author Phil Calder, Matthew Ross
 * @package Services
 */
class Database extends Service {
	
	/**
	 * @access private
	 */
	var $username = '';
	/**
	 * @access private
	 */
	var $password = '';
	/**
	 * @access private
	 */
	var $database = '';
	/**
	 * @access private
	 */
	var $host = '';
	/**
	 * @access private
	 */
	var $connection = '';
	/**
	 * @access private
	 */
	var $db_is_selected = '';
	/**
	 * @access private
	 */
	var $is_standalone = false;
	/**
	 * @access private
	 */
	var $show_tables = '';
	/**
	 * @access private
	 */
	var $error = '';
	/**
	 * @access private
	 */
	var $error_status = '';
	/**
	 * @access private
	 */
	var $field_checking_status = '';
	
	/**
	 * @access private
	 */
	var $tmp_count = 0; // Comment this line
	
	/**
	 * Constructor Database
	 * @param $args - array containing connection information:
	 *
	 * 'username' - MySQL username
	 * 'password' - MySQL password
	 * 'database' - name of the BrowserCRM database
	 * 'host'     - host name used for connecting to the database (defaults to localhost)
	 * 'port'     - port number used for connecting to the database (optional)
	 *
	 */
	function Database(
		$args = array(
			'username' => 'bcrm', 
			'password' => '', 
			'database' => 'browsercrm', 
			'host' => 'localhost', 
			'port' => '', 
			'standalone' => false
		)) {
			
		parent::Service(); // parent class constructor
	
		$this->username = $args['username'];
		$this->password = $args['password'];
		$this->database = $args['database'];
		$this->host = $args['host'] . (isset($args['port']) && $args['port'] ? ':' . $args['port'] : '');
		$this->error = $this->resetError();
		
		// Phil 20070514 In most cases we reuse the same database object. 
		// Setting this flag enables us to create a standalone Database object (eg to connect to a different database)
		// When this flag is not set, we know whether we are already connected
		$this->is_standalone = (isset($args['standalone']) && ($args['standalone'] === true) ? true : false);
		// print '<div>standalone: ' . ($this->is_standalone ? 'yes' : 'no') . '</div>';
		if (!$this->is_standalone) {
			// $this->connection = & $GLOBALS['_database']['connection'];
			// $this->db_is_selected = & $GLOBALS['_database']['db_is_selected'];
		} else {
			$this->connection = false;
			$this->db_is_selected = false;
		}
		// $this->show_tables = & $GLOBALS['_database']['show_tables']; // Phil 20050516 - not used
		$this->fn_prefix = '%sqlfn_%';
		$this->str_prefix = '%strfn_%';
		$this->field_checking_status = false;
		$this->errorStatus(true);
	}
	
	/**
	 * Not in use yet
	 *
	 * @access private
	 * @author Phil Calder
	 * 20051102
	 */
	function setDefaultVars() {}
	
	/** 
	 * @return string The Database version string
	 */
	function getVersion() {
		return @mysql_get_server_info();
	}
	
	/**
	 * Establish a connection to this database, or display a Message (service) and exit
	 */
	function dbConnect() {
		global $L;
		
		if (!$this->connection) {
			$this->connection = @mysql_connect($this->host, $this->username, $this->password, true);
		}
		
		if (!$this->connection) {
			// Should initialise a Fatal Error here instead of directly printing error message
			$msg = array();
			$msg[] = '&nbsp;<br>';
			//$msg[] = $L['dbErrCouldNotConnect']; //'BrowserCRM could not connect to MySQL.';
			$msg[] = 'BrowserCRM could not connect to MySQL.';
			$msg[] = '&nbsp;<br>';
			//$msg[] = $L['dbErrCouldNotConnectNew']; //'If this is a new installation of BrowserCRM, you need to correctly configure your mySQL connection parameters in config.inc.php';
			$msg[] = 'If this is a new installation of BrowserCRM, you need to correctly configure your mySQL connection parameters in config.inc.php';
			$msg[] = '&nbsp;<br>';
			//$msg[] = $L['dbErrCouldNotConnectEstablished']; //'If this is an established installation of BrowserCRM, you need to check that your mySQL server is up and look for any recent configuration changes.';
			$msg[] = 'If this is an established installation of BrowserCRM, you need to check that your mySQL server is up and look for any recent configuration changes.';
			$msg[] = '&nbsp;<br>';
			
			global $SERVICES;
			$message =& $SERVICES->getService('Message');
			$message->displayMessage($msg);
			exit();
		}
		
		//MCR 20061122
		if ( $this->getVersion() >= '5.0' ) {
			//echo 'run strict mode Session off';
			//if (!$result = @mysql_query("SET SESSION @@sql_mode=''")) {  //ok too I think
			if (!$result = @mysql_query("SET @@session.sql_mode=''")) {
				//error msg as done above? or allow to continue for now since may not be required
				//echo 'error';
			}
		} else {
			//echo 'pre 5.0';
		}
		
		$this->cache = $this->getCache();
	}
	
	/**
	 * Database call to select the current database with the current connection.
	 * Attempts to establish a connection is there is no current connection.
	 */
	function selectDb() {
		global $L;
		
		if (!$this->db_is_selected) {
			if (!$this->connection) {
				$this->dbConnect();
			}
			
			$this->setDefaultVars(); // Phil 20051102
			if (@mysql_select_db($this->database, $this->connection)) {
				$this->db_is_selected = true;
			}
			
			if ($error_num = $this->getErrorNumber()) {
				$this->connection = 0;
				$this->db_is_selected = false;
				$this->setErrorNumber($error_num);
				$this->setErrorString();
				$error = $this->getError();
				if ($error['no'] == 1049) {
					
					$msg = array();
					$msg[] = '&nbsp;<br>';
					//$msg[] = $L['dbErrCouldNotConnect']; //'BrowserCRM could not connect to MySQL.';
					$msg[] = $L['dbErrCouldNotConnect']; //'BrowserCRM could not connect to the database.';
					$msg[] = 'BrowserCRM could not connect to the database.';
					$msg[] = '&nbsp;<br>';
					$msg[] = 'Please report this problem to your system administrator';
					$msg[] = '&nbsp;<br>';
					
					if (!$this->hidePrintErrors()) {
						// $msg[] = $error['no'];
						$msg[] = $error['str'];
					}
					
					$subdir = '';
					global $SERVICES;
					if (is_object($SERVICES) && is_object($config =& $SERVICES->getService('Config'))) {
						$subdir = $config->lookup('site.websubdir');
					}
					$setup_href = ($subdir ? '/' . $subdir : '') . '/setup/index.php';
					
					$msg[] = '&nbsp;<br>';
					$msg[] = 'Alternatively, try running <a href="' . $setup_href . '">setup</a>';
					$msg[] = '&nbsp;<br>';
					
					global $SERVICES;
					$message =& $SERVICES->getService('Message');
					$message->displayMessage($msg);
					exit();
				}
			}
		}
	}
	
	/**
	 * Executes a select query and returns a result set identifier.
	 *
	 * The query is bulit from the following parameters:
	 * @param string $fields
	 * @param string $tables
	 * @param string $where - optional
	 * @param string $order_by - optional
	 * @param string $extras1 - optional, 'extras1' go after 'where', but before 'order by'
	 * @param string $extras2 - optional, 'extras2' go after 'order by', ie. at end of query.
	 * @return string result set identifier
	 */
	function select($fields, $tables, $where = '', $order = '', $extras1 = '', $extras2 = '') {
		$this->selectDb();
		if (empty($where)) {
			$where = '1';
		}
		
		$sql = 'SELECT ' . $fields . ' FROM ' . $tables . ' WHERE ' . $where;
		if (!empty($extras1)) {
			$sql .= ' ' . $extras1;
		}
		
		if (!empty($order)) {
			$sql .= ' ORDER BY ' . $order;
		}
		
		if (!empty($extras2)) {
			$sql .= ' ' . $extras2;
		}
		//echo $sql;
		return $this->sqlQuery($sql);
	}
	
	/**
	 * quickGet
	 *
	 * @param field
	 * @param table
	 * @param where - optional
	 * @param order_by - optional
	 * @param extras1 - optional
	 * @param extras2 - optional
	 * executes a select query and returns the first row as an array, 
	 * or the first field of the first row if only one field is selected.
	 */
	function quickGet($field, $table, $where = '', $order_by = '', $extras1 = '', $extras2 = '') {
		$this->selectDb();
		$result = $this->select($field, $table, $where, $order_by, $extras1, $extras2);
		
		$row = $this->getRow($result);
		if (is_array($row)) {
			if (sizeof($row) == 1) {
				$v = array_values($row);
				return stripslashes($v[0]);
			} else {
				foreach ($row as $k => $v) {
					$row[$k] = stripslashes($v);
				}
			}
		} else if ($row !== false) {
			$row = stripslashes($row);
		}
		return $row;
	}
	
	/**
	 * quickGetAllRows
	 *
	 * @param field
	 * @param table
	 * @param where - optional
	 * @param order_by - optional
	 * @param extras1 - optional
	 * @param extras2 - optional
	 * executes a select query and returns the rows as an array, 
	 */
	function quickGetAllRows($field, $table, $where = '', $order_by = '', $extras1 = '', $extras2 = '') {
		$this->selectDb();
		$result = $this->select($field, $table, $where, $order_by, $extras1, $extras2);
		$rows = array();
		while ($row = $this->getRow($result)) {
			$rows[] = (sizeof(array_keys($row)) == 1 ? current($row) : $row);
		}
		return $rows;
	}
	
	/** 
	 * insert
	 *
	 * @param table - the table to insert into
	 * @param fields - array of fields and values to insert
	 * @return the insert id or result of the database insert
	 */
	function insert($table, $fields) {
		$this->selectDb();
		$field_list = '';
		$value_list = '';
		foreach ($fields as $k => $v) {
			if (
				$this->setFieldChecking() && 
				(
					(substr($v, 0, strlen($this->fn_prefix)) == $this->fn_prefix) || 
					(substr($v, 0, strlen($this->str_prefix)) == $this->str_prefix)
				)
			) {
				if ((substr($v, 0, strlen($this->fn_prefix)) == $this->fn_prefix)) {
					$fields[$k] = substr($v, strlen($this->fn_prefix)); // Assumes fn_prefix and str_prefix are the same length
				} else {
					$fields[$k] = '\'' . addslashes(substr($v, strlen($this->str_prefix))) . '\'';
				}
			} else {
				if (is_array($v)) { // Phil - 20050509 - for multi select fields
					$item[$k] = implode(',', $v);
				}
				if (empty($v) || (!is_numeric($v) && is_string($v))) {
					$fields[$k] = '\'' . addslashes($v) . '\'';
				}
			}
		}
		
		$field_list = implode(', ', array_keys($fields));
		$value_list = implode(', ', array_values($fields));
		
		$sql = 'INSERT INTO ' . $table . ' (' . $field_list . ') VALUES (' . $value_list .')';
		$result = $this->sqlQuery($sql);
		if ($result && !@mysql_error($this->connection)) {
			$insert_id = $this->getInsertId();
			return $insert_id;
		}
		return $result;
	}
	
	/**
	 * getInsertId
	 * @return the auto increment id last updated by this connection.
	 */
	function getInsertId() {
		$this->selectDb();
		return @mysql_insert_id($this->connection);
	}
	
	/**
	 * isGood
	 *
	 * @param $result The result 
	 * @return true or false, whether this result is good and contains rows.
	 */
	function isGood($result) {
		$this->selectDb();
		return ($result && @mysql_num_rows($result));
	}
	
	/**
	 * updates a record or records in a table
	 *
	 * @param $table
	 * @param $fields - array
	 * @param $where
	 * @param $limit - int
	 * @return the database query result
	 *
	 */
	function update($table, $fields, $where = '', $limit = '') {
		$this->selectDb();
		foreach ($fields as $k => $v) {
			if (
				$this->setFieldChecking() && 
				(
					(substr($v, 0, strlen($this->fn_prefix)) == $this->fn_prefix) || 
					(substr($v, 0, strlen($this->str_prefix)) == $this->str_prefix) 
				)
			) {
				
				if ((substr($v, 0, strlen($this->fn_prefix)) == $this->fn_prefix)) {
					$fields[$k] = $k . '= ' . substr($v, strlen($this->fn_prefix)); // Assumes fn_prefix and str_prefix are the same length
				} else {
					$fields[$k] = $k . '= \'' . addslashes(substr($v, strlen($this->str_prefix))) . '\'';
				}
			} else {
				if (empty($v) || (!is_numeric($v) && is_string($v))) {
					$fields[$k] = $k . '= \'' . addslashes($v) . '\'';
				} else {
					$fields[$k] = $k . '=' . $v . '';
				}
			}
		}
		
		$update_str = implode(', ', $fields);
		if (empty($where)) {
			$where = '1';
		}
		
		$sql = 'UPDATE ' . $table . ' SET ' . $update_str . ' WHERE ' . $where;
		if ($limit && (substr(trim($limit), 0, strlen('LIMIT')) == 'LIMIT')) {
			$sql .= ' ' . $limit;
		}
		return $this->sqlQuery($sql);
	}
	
	/**
	 * deletes a row/rows from a table
	 *
	 * @param $table
	 * @param $where - string
	 * @return database query result
	 */
	function delete($table, $where = '') {
		$this->selectDb();
		if (empty($where)) {
			$where = 1;	
		}
		$sql = 'DELETE FROM ' . $table . ' WHERE ' . $where . '';
		// $this->logAction('(test) Database class: ' . $sql);
		return $this->sqlQuery($sql);
	}
	
	/**
	 * Get the number of affected rows from a database query.
	 *
	 * @param $result is optional, and is being deprecated. The current database connection will be used instead.
	 * @return the number of rows affected by a delete, insert, or update command.
	 */
	function affectedRows($result = '') {
		if (isset($this->affected_rows)) {
			$num = $this->affected_rows;
			unset($this->affected_rows);
			return $num;
		}
		if (empty($result)) {
			return @mysql_affected_rows($this->connection);
		}
		return @mysql_affected_rows($result);
	}
	
	function setAffectedRows($num) {
		$this->affected_rows = $num;
	}
	
	/**
	 * It is possible to override the affectedRows function. This function, called before every query, 
	 * resets the stored value so that the true value will be returned from affectedRows.
	 *
	 * @see setAffectedRows
	 * @see affectedRows
	 */
	function resetAffectedRows() {
		if (isset($this->affected_rows)) {
			unset($this->affected_rows);
		}
	}
	
	/**
	 * @param $table The table name to query on.
	 * @return MySQL resource identifier for information about columns
	 * Each row returned contains information about the column in the table
	 */
	function showColumns($table) {
		$this->selectDb();
		// print '<div>show columns from ' . $table . '</div>';
		return $this->sqlQuery('SHOW COLUMNS FROM ' . $table . '');
	}
	
	/**
	 *
	 */
	function showTables($prefix = '') {
		$this->selectDb();
		$sql = 'SHOW TABLES';
		if (!empty($prefix)) {
			$sql .= ' LIKE "' . str_replace('_', '\_', $prefix) . '%"';
		}
		// print '<div>showTables ' . $sql . '</div>';
		return @$this->sqlQuery($sql);
	}
	
	/**
	 * Checks the database to see if a given table exists
	 *
	 * As of version 4.135 search is case insensitive
	 *
	 * @param string $table The name of the table.
	 * @param boolean $custom_only Whether to only look for custom table (faster)
	 * @return boolean true or false
	 */
	function tableExists($table, $custom_only = false) {
		// Phil 20051029in_array is case sensitive, which may not work on some systems (eg MySQL on Windows)
		if ($custom_only) {
			$table_array = $this->getCustomTables();
			// print '<div>custom tables:</div>'; print_r($table_array); print '<hr>';
		} else {
			$table_array = $this->getTables();
			// print '<div>tables:</div>'; print_r($table_array); print '<hr>';
		}
		$lc_table = strtolower($table);
		foreach ($table_array as $k => $v) {
			if (strtolower($v) == $lc_table) {
				return true;
			}
		}
		return false;
	}
	
	function getCustomTables() {
		$this->selectDb();
		if (!isset($this->cache['custom_tables']) || !is_array($this->cache['custom_tables'])) {
			$suffix = $this->custom_suffix;
			$sql = 'SHOW TABLES LIKE "%' . str_replace('_', '\_', $suffix) . '"';
			// print '<div>getCustomTables ' . $sql . '</div>';
			$result = @$this->sqlQuery($sql);
			$this->cache['custom_tables'] = array();
			while ( $row = $this->getRow($result) ) {
				if ( count(array_keys($row)) == 1 ) {
					$this->cache['custom_tables'][] = current($row);
				} else {
					$this->cache['custom_tables'][] = $row;
				} 
			}
			$this->writeCache();
		}
		return $this->cache['custom_tables'];
	}
	
	/**
	 * @author Phil Calder
	 * @access private
	 */
	function getCacheFilename() {
		if (!$this->is_standalone) {
			global $BCRM_DOCUMENT_ROOT; 
			return $BCRM_DOCUMENT_ROOT . '/cache/compiled.db.inc.php';
		}
	}
		 
	
	/**
	 * @author Phil Calder
	 * @access private
	 */
	function getCache() {
		// return false;
		if (!$this->is_standalone) {
			$this->cache_file = $this->getCacheFilename();
			// print '<div>cache file: ' . $this->cache_file . '</div>';
			if (file_exists($this->cache_file)) {
				$db_cache = array();
				include_once($this->cache_file);
				// print '<div>sizeof  db_tables = ' . count($db_tables) . '</div>';
				return $db_cache;
			}
		}
	}
	
	/**
	 * @author Phil Calder
	 * @access private
	 */
	function writeCache() {
		// return false;
		// print '<div>=== writeCache ===</div>';
		if (!$this->is_standalone) {
			if (function_exists('var_export')) {
				global $BCRM_DOCUMENT_ROOT;
				$content = '<?php' . "\n" . '$db_cache=';
				$content .= var_export($this->cache, true);
				$content .= "\n" . '?>';
				$newfile = $this->getCacheFilename();
				if ($file = @fopen($newfile, "w")) {
					@fwrite($file, $content);
					@fclose ($file);
				}
			}
		}
	}
	
	/**
	 * @author Phil Calder
	 * @access private
	 */
	function emptyCache() {
		if (!$this->is_standalone) {
			$cache_file = $this->getCacheFilename();
			if (file_exists($cache_file)) {
				$result = unlink($cache_file);
			}
			$this->cache = false;
		}
	}
	
	/**
	 * Called by Manage Installation - Rebuild Cache
	 * @author Phil Calder
	 */
	function rebuildCache() {
		if (!$this->is_standalone) {
			// print '<div>=== rebuildCache ===</div>';
			if (function_exists('var_export')) {
				$this->emptyCache();
				$custom_tables = $this->getCustomTables();
				$all_tables = $this->getTables();
				if (is_array($all_tables)) {
					foreach ($all_tables as $k => $v) {
						$columns = $this->getColumns($v);
					}
				}
			}
		}
	}
	
	/**
	 * Gets an array of tables in the database
	 *
	 * @param string $prefix Option table name filter
	 * @return array or tables, value is the table name
	 */
	function getTables($prefix = '') {
		$this->selectDb();
		
		// print '<div>number of tables: ' . count($this->tables) . '</div>';
		if (!isset($this->cache['tables']) || !is_array($this->cache['tables']) || !count($this->cache['tables'])) {
			// print '<div>need to get tables</div>';
			// $result = $this->showTables($prefix);
			$result = $this->showTables();
			$this->cache['tables'] = array();
			while ( $row = $this->getRow($result) ) {
				if ( count(array_keys($row)) == 1 ) {
					$this->cache['tables'][] = current($row);
				} else {
					$this->cache['tables'][] = $row;
				} 
			}
			$this->writeCache();
		}
		
		if (!empty($prefix)) {
			$prefixed_tables = array();
			$lower_prefix = strtolower($prefix);
			foreach ($this->cache['tables'] as $k => $v) {
				// if (substr($v, 0, strlen($prefix))==$prefix) {
					// $prefixed_tables[] = $v;
				// }
				if (substr(strtolower($v), 0, strlen($prefix))==$lower_prefix) {
					$prefixed_tables[] = $v;
				}
			}
			return $prefixed_tables;
		}
		return $this->cache['tables'];
	}
	
	/**
	 * Get the column names for a given table.
	 *
	 * @param $table The table to query on.
	 * @return An array containing the table column names
	 */
	function getColumns($table) {
		$this->selectDb();
		if ( isset($this->cache['columns'][$table]) && is_array($this->cache['columns'][$table])) {
			// print '<div>returning class var cached columns for ' . $table . '</div>';
			return $this->cache['columns'][$table];
		} else {
			
			// TODO remove
			// if (isset($this->cache['tables'][$table]) && is_array($this->cache['tables'][$table])) {
				// // print '<div>returning class var cached *table* columns for ' . $table . '</div>';
				// return $this->tables[$table];
			// }
			
			$result = $this->showColumns($table);
			$this->cache['columns'][$table] = array(); // Phil 20050523
			while ($row = $this->getRow($result)) {
				$this->cache['columns'][$table][] = $row['Field'];
			}
			$this->cache['columns'][$table] = $this->cache['columns'][$table];
			$this->writeCache();
		}
		return $this->cache['columns'][$table];
	}
	
	/**
	 * Get the next row, or the specified row, from a result set
	 * 
	 * @param mixed $result - the result set
	 * @param integer $row_number - optional row number (defaults to the next row)
	 * @param boolean $stripslashes - optional, whether to stripslashes before retuning data (default true, should only be false in exceptional circumstances)
	 * @return mixed Either an array - row, from the result set - or false
	 */
	function getRow($result, $row_number = -1, $strip_slashes = true) {
		$this->selectDb();
		if (!$this->isGood($result)) {
			return false;
		}
		if ($row_number >= 0) {
			@mysql_data_seek($result, $row_number);
		}	
		$row = @mysql_fetch_assoc($result);
		if ($strip_slashes && is_array($row)) {
			foreach ($row as $k => $v) {
				$row[$k] = stripslashes($v);	
			}
		}
		return $row;
	}
	
	/**
	 * Gets the number of rows available from the specified result set
	 * 
	 * @param $result - result set 
	 * @return The number of rows.
	 */
	function getNumRows($result = '') {
		$this->selectDb();
		return @mysql_num_rows($result);
	}
	
	/**
	 * Returns the sql needed to create the given table
	 */
	function getShowCreateTable($table) {
		$result = $this->sqlQuery('SHOW CREATE TABLE `' . $table . '`');
		$row = @mysql_fetch_array($result,MYSQL_ASSOC);
		$sql = $row['Create Table'];
		return $sql;
	}
	
	/**
	 * The insert and update functions will parse field values to treat values 
	 * beginning with this->fn_prefix differently when this is true.
	 *
	 * @param boolean $status - Turn field checking on or off
	 * @return boolean $current_status - The staus prior to the current change
	 */
	function setFieldChecking($status = '') {
		$current_status = $this->field_checking_status;
		if ($status === true || $status === false) {
			$this->field_checking_status = $status;
		}
		return $current_status;
	}
	
	function prefixSpecialFieldValue($value, $as_string = false) {
		if ($as_string) {
			return $this->str_prefix . str_replace($this->str_prefix, '', $value);
		} 
		return $this->fn_prefix . str_replace($this->fn_prefix, '', $value);
	} 
	
	/**
	 * getErrorNumber
	 *
	 * @return The last recorded MySQL error number (defaults to 0)
	 */
	function getErrorNumber() {
		if (!isset($this->error['no']) || !$this->error['no']) {
			return @mysql_errno($this->connection);
		}
		return $this->error['no'];
	}
	
	/**
	 * getErrorString
	 *
	 * @return The last recorded MySQL error number (defaults to 0)
	 */
	function getErrorString() {
		if (!$this->error['str']) {
			// return @mysql_error($this->connection);
			 $this->error['str'] = @mysql_error($this->connection);
		}
		return $this->error['str'];
	}
	
	/**
	 * Sets the 'error status' (ie whether to track errors) on or off
	 *
	 * @param boolean $status Set the staus to true or false (optional)
	 * @return boolean The current status - prior to this update
	 */
	function errorStatus($status = '') {
		$current = $this->error_status;
		if (($status === false) || ($status === true)) {
			$this->error_status = $status;
		}
		return $current;
	}
	
	/**
	 * Resets the stored error values: number, string, and sql
	 *
	 * @param $sql Optional string to set as the error sql value
	 */
	function resetError($sql = '') {
		$this->error = array(
			'no' => '', 
			'str' => '', 
			'sql' => $sql 
		);
	}
	
	/**
	 * Set the error number
	 *
	 * @param $num Optional error number, will default to the last known error number.
	 */
	function setErrorNumber($num = '') {
		if (empty($num)) {
			$num = @mysql_errno($this->connection);
		}
		$this->error['no'] = $num;
	}
	
	/**
	 * Set the error string
	 *
	 * @param $str Optional error number, will default to the last known error string.
	 */
	function setErrorString($str = '') {
		if (empty($str)) {
			$str = @mysql_error();
		}
		$this->error['str'] = $str;
	}
	
	/**
	 * Will return the stored error number which defaults to 0 (ie no error)
	 */
	function hasError() {
		return ($this->getErrorNumber() > 0 ? true : false);
	}
	
	/**
	 * @return The current error (array)
	 */
	function getError() {
		return $this->error;
	}
	
	function hidePrintErrors() {
		if (!isset($this->hide_print_errors)) {
			global $SERVICES;
			if ($config =& $SERVICES->getService('Config')) {
				$this->hide_print_errors = $config->lookup('database.hide_print_errors');
			}
		}
		return $this->hide_print_errors;
	}
	
	/**
	 * Displays an error message (for the user) if there is an error to display. Uses the Message service.
	 *
	 * @return boolean true of false - if there is an error to display
	 */
	function printError() {
		if (!$this->hidePrintErrors() && $this->hasError()) {
			$error = $this->getError();
			$num = $error['no'];
			$str = $error['str'];
			$sql = $error['sql'];
			
			$content = '';
			$content .=  '
			<p><font face="Verdana,Arial,Helvetica,Sans-serif" size="2" color="#ff0000">
			';
			
			$content .= '
			<strong>Database Error ' . $num . ':</strong> ' . $str . '
			<br>
			';
			$content .= '</font></p>';
			
			
			/*
			$sql = trim($sql);
			$sql = ereg_replace('(^| )(SELECT|FROM|WHERE|ORDER BY|GROUP BY|DELETE|UPDATE|CREATE) ',"<br>\n&nbsp;<br><b>\\2</b> ",$sql);
			$sql = ereg_replace('(^|[^[:alnum:]])(AND|OR|AS|ASC|DESC)([^[:alnum:]])'," <i>\\2</i>\\3",$sql);
			
			
			if ($sql) { 
				$content .=  '
				<strong>SQL</strong>: <font color="#cc0000">' . $sql . '</font>
				<br>&nbsp;
				<br>
				';
			}
			
			$content .= '
			<strong>Host</strong>: ' . $this->host . '
			<br>
			</font>';
			
			$error_message = '';
			$error_message .= "Database Error: $num: $str\n";
			if ($sql) { $error_message .= "SQL: $sql\n"; }
			$error_message .= "&nbsp;\n";
			$error_message .= "Host: " . $this->host . "\n";
			
			// form to send error report back to BrowserCRM 
			// can't use output class since this will probably be nested inside another Page instance
			global $L, $SERVICES;
			// prepend the subdir
			if ($config =& $SERVICES->getService('Config')) {
				$path = $config->lookup('site.websubdir');
				if ($path != '') {
					$path = '/'. $path;
				}
			}

			$content .=  '
			<div>
			<table border="0" cellpadding="0" cellspacing="0" class="outerTable" width="440"><tr><td>
			<form name="mainform" action="'. $path .'/error_report.php" method="post"><table border="0" cellpadding="3" cellspacing="0" align="left" width="100%">
			<tr><td class="titleCell" width="34%" nowrap>' . $L['errErrorReport'] . '</td><td nowrap class="titleMenuCell" width="66%">&nbsp;&nbsp;&nbsp;&nbsp;</td></tr>
					<tr><td colspan="2">
					<table border="0" cellpadding="3" cellspacing="0" align="left" width="100%">
					<tr><td valign="top" align="left" nowrap><b>' . $L['errYourEmail'] . '</b>&nbsp;</td><td></td><td valign="top" align="left"><input type="text" style="width: 220px" name="item[email]" maxlength="128"  value=""></td></tr>
					<tr><td valign="top" align="left" nowrap><b>' . $L['errServerName'] . '</b>&nbsp;</td><td></td><td valign="top" align="left"><input type="text" style="width: 220px" name="item[server_name]" maxlength="128"  value="' . $_SERVER['SERVER_NAME'] . '"></td></tr>
					<tr><td valign="top" align="left" nowrap><b>' . $L['errCurrentPage'] . '</b>&nbsp;</td><td></td><td valign="top" align="left"><input type="text" style="width: 220px" name="item[current_page]" maxlength="128"  value="' . bcrm_htmlspecialchars($_SERVER['PHP_SELF'] . '?' . $_SERVER['QUERY_STRING'])  . '"></td></tr>
					<tr><td valign="top" align="left" nowrap><b>' . $L['errSubject'] . '</b>&nbsp;</td><td></td><td valign="top" align="left"><input type="text" style="width: 220px" name="item[subject]" maxlength="128"  value="Database error"></td></tr>
					<tr><td valign="top" align="left" nowrap><b>' . $L['errMessage'] . '</b>&nbsp;</td><td></td><td valign="top" align="left"><textarea style="width: 220px" name="item[message]" 
								rows="10">' . strip_tags($error_message) . '</textarea></td></tr></table></td></tr>
					<tr><td></td><td><input type="submit" name="submit" value="' . $L['ButtonSubmit'] . '"></td></tr></table></form>
			</td></tr>
			</table>
			</div>';
	
			*/
			
			global $SERVICES;
			$message =& $SERVICES->getService('Message');
			$message->display($content);
			return true;
		}
		return false;
	}
	
	/**
	 * accepts sql which is passed straight to sqlQuery. Resets the cached tables array
	 */
	function createTable($sql) {
		unset($this->cache['tables']);
		return $this->sqlQuery($sql);
	}
	
	/** 
	 * Executes the given query. Logs an error (using the Log service) if the query does not execute properly.
	 *
	 * @param $sql The query to execute.
	 * @return The result of the query
	 */
	function sqlQuery($sql) {
		$this->selectDb();
		// print '<div>db: ' . $this->database . ' connection: ' . $this->connection . ' query: ' . $sql . '</div>'; 
		if ($this->errorStatus()) {
			$this->resetError($sql);
		}
		$this->resetAffectedRows();
		
		if (isset($this->log_sql) && $this->log_sql) $this->logSql($sql);
		// $this->tmp_count++; // Comment this line
		// $this->printDebug('[' . $this->tmp_count .']<br />' . $sql . ''); // Comment this line
		// print '<p>[' . $this->tmp_count .' - ' . (round($this->getPageExecutionTime(), 3)) . '] ' . $sql . '</p>'; // Comment this line
		
		$result = @mysql_query($sql, $this->connection);
		if (!$result && $this->errorStatus()) {
			$this->setErrorNumber();
			$this->setErrorString();
			$this->logError('Database error ' . $this->getErrorString());
			$this->logError('SQL error ' . $sql);
		}
		// $this->printDebug('[' . $this->tmp_count .']<br /> finished'); // Comment this line
		return $result;
	}
		
	function setSqlLogStatus($status = true, $facility = '') {
		$this->log_sql = $status;
		$this->log_sql_facility = trim('ModuleEditor ' . $facility);
	}
	
	function getSqlLogFacility() {
		return ($this->log_sql_facility ? $this->log_sql_facility : 'ModuleEditor');
	}
	
	function logSql($sql) {
		// $this->log_sql = false;
		$facility = $this->getSqlLogFacility();
		$this->setSqlLogStatus(false);
		$this->logAction($sql, $facility);
	}
} // class Database

/**
 * The custom database splits queries for tables with 'custom' tables. 
 *
 * If there is a custom table for one of the tables being queried, 
 * this class will join the standard and custom queries by editing the query 'on the fly'. 
 * Can result in 2 inserts or updates, which the user need not be aware of.
 * @author Phil Calder
 * @package Services
 */
class CustomDatabase extends Database {
	/**
	 * @access private
	 */
	var $custom_suffix = '';
	
	/**
	 * Database object constructor. Sets custom table suffix variable and calls parent
	 */
	function CustomDatabase($args) {
		$this->custom_suffix = '_custom';
		parent::Database($args);
	}
	
	/**
	 * Handles inserts for queries containing custom fields
	 
	 * Parses the insert query for custom fields. Performs a 'standard' insert, 
	 * then inserts custom fields into the custom table. 
	 * @param string $table - Same as {@link Database::insert()}
	 * @param array $fields - Same as {@link Database::insert()}
	 * @return mixed the insert id from the standard insert (usually an int)
	 */
	function insert($table, $fields) {
		$custom_table = $table . $this->custom_suffix;
		
		if ($this->tableExists($custom_table, true)) {
			$standard_fields = parent::getColumns($table);
			$standard_field_array = array();
			$custom_field_array = array();
			// Split the fields into standard and custom
			foreach ($fields as $k => $v) {
				// Phil 20050808 Added second level field name checking - supplied name and (optionally) name without table prefix
				$field_name = false;
				if (strpos($k, '.')) {
					$field_name = substr($k, (strpos($k, '.')+1));
				}
				if ( in_array($k, $standard_fields) || ($field_name && in_array($field_name, $standard_fields)) ) {
					$standard_field_array[$k] = $v;
				} else { 
					// could check: in_array($k, $custom_fields) here, but that leads to sloppy code elsewhere 
					// (ie not unsetting unwanted variables before posting to db)
					$custom_field_array[$k] = $v;
				}
			}
			
			if (!sizeof($standard_field_array)) {
				$standard_field_array['id'] = '';
			}
			// perform the standard insert
			$standard_insert_result = parent::insert($table, $standard_field_array);
			
			if ($standard_insert_result && !$this->hasError() && ($insert_id = $this->getInsertId())) {
				// Need an id number
				$custom_field_array['custom_id'] = $standard_insert_result;
				// perform the custom insert
				$custom_insert_result = parent::insert($custom_table, $custom_field_array);
				return $insert_id;
			}
			return $standard_insert_result;
		}	
		return parent::insert($table, $fields);
	}
	
	/**
	 * Handles updates for queries containing custom fields
	 
	 * Parses the update query for custom fields. Performs a 'standard' update, 
	 * then updates (or inserts) custom fields into the custom table. 
	 * @param string $table - Same as {@link Database::update()}
	 * @param array $fields - Same as {@link Database::update()}
	 * @param string $where - Same as {@link Database::update()}
	 * @param string $limit - Same as {@link Database::update()}
	 * @return mixed the standard update result (MySQL update query return value)
	 */
	function update($table, $fields, $where = "", $limit = '') {
		$custom_table = $table . $this->custom_suffix;
		if ($this->tableExists($custom_table, true)) {
			$standard_fields = parent::getColumns($table);
			$standard_field_array = array();
			$custom_field_array = array();
			
			// Split the fields into standard and custom
			foreach ($fields as $k => $v) {
				// Phil 20050808 Added second level field name checking - supplied name and (optionally) name without table prefix
				$field_name = false;
				if (strpos($k, '.')) {
					$field_name = substr($k, (strpos($k, '.')+1));
				}
				if ( in_array($k, $standard_fields) || ($field_name && in_array($field_name, $standard_fields)) ) {
					$standard_field_array[$k] = $v;
				} else { 
					// could check: in_array($k, $custom_fields) here, but that leads to sloppy code elsewhere 
					// (ie not unsetting unwanted variables before posting to db)
					$custom_field_array[$k] = $v;
				}
			}
			// perform the standard update
			$standard_update_result = parent::update($table, $standard_field_array, $where, $limit);
			$standard_affected_rows = $this->affectedRows();
			if (!$this->hasError() && isset($standard_field_array['id'])) {
				// May need to insert first
				if (!$this->quickGet('count(*)', $custom_table, 'custom_id = ' . $standard_field_array['id'])) {
					$custom_insert_id = parent::insert($custom_table, array('custom_id' => $standard_field_array['id']));
				}
				if (sizeof($custom_field_array)) {
					parent::update($custom_table, $custom_field_array, 'custom_id = ' . $standard_field_array['id'], $limit);
					$custom_affected_rows = $this->affectedRows();
					$this->setAffectedRows(max($standard_affected_rows, $custom_affected_rows));
				}
			}
			return $standard_update_result;
		}
		return parent::update($table, $fields, $where, $limit);
	}
	
	/**
	 * Call this rather the select to force bypassing the custom field processing
	 *
	 * @author Phil Calder
	 * @access private
	 */
	function standardSelect($fields, $tables, $where = '', $order = '', $extras1 = '', $extras2 = '') {
		return parent::select($fields, $tables, $where, $order, $extras1, $extras2);
	}
	
	/**
	 * Modifies the select query to include fields from a custom table, if it exists.
	 *
	 * All parameters are the same as for {@link Database::select()}
	 * @return mixed The return value for a standard {@link Database::select()}
	 */
	function select($fields, $tables, $where = '', $order = '', $extras1 = '', $extras2 = '') {
		$first_table = (strpos($tables, ',') ? substr($tables, 0, strpos($tables, ',')) : $tables);
		$first_table = trim((strpos($first_table, ' ') ? substr($first_table, 0, strpos($first_table, ' ')) : $first_table));
		$custom_table = $first_table . $this->custom_suffix;
		
		if ($this->tableExists($custom_table, true) ) {
			$tables .= ' LEFT JOIN ' . $custom_table . ' ON ' . $first_table . '.id=' . $custom_table . '.custom_id';
			// Query must be looking for * from at least one table
			if (($fields != '*') && (strpos($fields, '*'))) {
				// Add custom_table.* to the field list
				// This is only necessary because the field list may not contain *
				// Exclude for min max and count queries, 
				// and queries looking for a fixed list of fields (eg SELECT id FROM table) where we dont want to change the result set
				$group_by_functions = array('min', 'max', 'count');
				$incl_custom_field_list = true;
				foreach ($group_by_functions as $k => $gb) {
					if (stristr($fields, $gb . '(')) {
						$incl_custom_field_list = false;
						break;
					}
				}
				if ($incl_custom_field_list) {
					$fields .= ', ' . $custom_table . '.*';
				}
			}
		}
		return parent::select($fields, $tables, $where, $order, $extras1, $extras2);
	}
	
	/**
	 * Combines the columns from the given table with any custom fields.
	 *
	 * @param string $table The table to get columns for
	 * @return array An array containing standard and custom fields 
	 */
	function getColumns($table) {
		$standard = parent::getColumns($table);
		if ($this->tableExists($table . $this->custom_suffix, true)) {
			$custom = parent::getColumns($table . $this->custom_suffix);
			// Get rid of custom_id field
			if (isset($custom[0]) && ($custom[0] = 'custom_id')) {
				bcrm_unset($custom, 0); // custom_id
			}
			return bcrm_array_merge($standard, $custom);
		}
		return $standard;
	}
	
	/** 
	 * Bypass custom field processing
	 *
	 * @author Phil Calder
	 * access private
	 */
	function standardGetRow($result, $row_number = -1, $strip_slashes = true) {
		return parent::getRow($result, $row_number, $strip_slashes);
	}
	
	/**
	 * Removes the 'custom_id' fields from the row returned
	 *
	 * @param mixed $result - the result set
	 * @param integer $row_number - optional row number (defaults to the next row)
	 * @param boolean $stripslashes - optional, whether to stripslashes before retuning data (default true, should only be false in exceptional circumstances)
	 * @return mixed Either an array, row - from the result set - or false 
	 */
	function getRow($result, $row_number = -1, $strip_slashes = true) {
		$row = parent::getRow($result, $row_number, $strip_slashes);
		// Get rid of custom_id field
		if (isset($row['custom_id'])) {
			bcrm_unset($row, 'custom_id'); // custom_id
		} 
		return $row;
	}
}
?>