<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package Services
 */
 
/**
 * Screen handles skins and screen resolution.
 * 
 * @author Matthew Ross, Phil Calder
 * @package Services
 */
class Screen extends Service {
	
	function Screen() {
		global $SERVICES;
		$this->db =& $SERVICES->getService('Database');
	}
	
	// ----- Skins functions -----
	
	/**
	 * Gets the skins allowed for the current user.
	 */
	function getSkins() {
		global $SERVICES;
		
		$exclude_list = array(
			'.', 
			'..', 
			'CVS',   // Phil 20060207 - just incase!
			'metal', // Phil 20060207 need to upgrade this to UI for version 4.5+
			'kiwi',  // Phil 20060207 need to upgrade this to UI for version 4.5+
			'print',  // Phil 20060823 used for printing only
			
			'_base',  // new base - not a selectable skin
			'_template', //dev
			'blue',
			'BlueIce',
			'GreyBlue',
			
		);
		
		$SERVICES->getService('Config');
		$options = array();
		$skins_dir = $SERVICES->config->lookup('site.root') . '/skins';
		$dir = opendir( $skins_dir );
		$skin_list = array();
		
		while ($dir_name = readdir($dir)) {
			if (!in_array($dir_name, $exclude_list) && is_dir($skins_dir . '/' . $dir_name)) {
				$name = $dir_name;
				$description = '';
				$screen_shot = '';
				
				if (file_exists($desc_file = $SERVICES->config->lookup('site.root') . '/skins/' . $dir_name . '/screenshot.png')) {
					$screenshot = '/skins/' . $dir_name . '/screenshot.png';
				}
				
				if (file_exists($desc_file = $SERVICES->config->lookup('site.root') . '/skins/' . $dir_name . '/description.inc')) {
					$temp_str = implode(' ', file( $SERVICES->config->lookup('site.root') . '/skins/' . $dir_name . '/description.inc'));
					$temp_ar = explode('<br>', $temp_str);
					$name = $temp_ar[0];
					$description = $temp_ar[1];
				}
				
				$skin_list[] = array( 
					'name' => $name, 
					'dir_name' => '/skins/' . $dir_name, 
					'description' => $description, 
					'screenshot' => $screenshot
				);
			}
		}
		return $skin_list;
	}
	
	function getSkinExtTheme($skin)
	{
		global $SERVICES;
		$SERVICES->getService('Config');
		
		//$skin should be in format: '/skin/plain/'
		$skin = str_replace('/skins/', '', $skin);
		$skin = str_replace('/', '', $skin);
		//now $skin is in format: 'plain'
		
		if (file_exists($desc_file = $SERVICES->config->lookup('site.root') . '/skins/' . $skin . '/skin.config')) {
			include($desc_file); //has $config['skin'] array
		}
		
		return $config['skin']['ext_theme'];
	}
	
	/**
	 * Returns the number of users who use this skin.
	 */
	function getSkinUserCount($skin) {
		global $USER;
		return $this->db->quickget(
			'count(*)',
			'System__users',
			'status = 1 AND skin="' . $skin . '"'
		);
	}
	
	/**
	 * Sets the skin for the current user.
	 */
	function setSkin($skin) {
		global $USER, $SERVICES;
		if (!empty($skin) && isset($USER['user_id'])) {
			$this->db->update( 
				'System__users', 
				array('skin' => $skin), 
				'id=' . $USER['user_id'] 
			);
			
			//$this->setSkinForSession($skin);
			$SERVICES->getService('Users');
			$SERVICES->users->setSessionSkin($skin);
			
		}
	}
	
	// function setSkinForSession($skin) {
		// global $USER, $SERVICES, $BCRM_DOCUMENT_ROOT;
		// $SERVICES->getService('Config');
		// //adjust skin path for subdir install
		// //replace the prepend_file path with blank to get the subdir path
		// $websubdir = $SERVICES->config->lookup('site.websubdir');
		// $USER['skin_web_path'] = ($websubdir ? '/' . $websubdir : '') . $skin; 
		// $USER['skin_file_path'] = $BCRM_DOCUMENT_ROOT . $skin;
	// }
	// 
	// function setScreenRes($id) {
		// global $USER;
		// if (!empty($id)) {
			// $this->db->update( 
				// 'System__users', 
				// array('screen_res_id' => $id), 
				// 'id=' . $USER['user_id'] 
			// );
			// $USER['screen_res_id'] = $id;
		// }
//	}
} // class Screen
?>