<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 *
**/

@set_time_limit(900); 
@ini_set('memory_limit', '512M');

class BrowserCRMV4toV5Upgrade {
	
	var $print_count_at = 2000; // overridden by constructor
	
	function BrowserCRMV4toV5Upgrade($args = array()) {	
		$args = (array)$args;
		
		// $this->printLog('BrowserCRMV4toV5Upgrade args:'); $this->printLog($args, true); 
		
		$config_file = '../config.inc.php';
		$service_class_file = '../services/inc/Service.inc';
		$log_class_file = '../services/inc/Log.inc';
		$database_class_file = '../services/inc/Database.inc';
		
		
		if (file_exists($service_class_file) && is_readable($service_class_file)) {
			if (file_exists($log_class_file) && is_readable($log_class_file)) {
				if (file_exists($database_class_file) && is_readable($database_class_file)) {
					
					if (file_exists($config_file) && is_readable($config_file)) {
						global $config;
						include($config_file);
						$db_user = $config['database']['username'];
						$db_pass = $config['database']['password'];
						$db_name = $config['database']['dbname'];
						$db_host = $config['database']['host'];
					} else {
						global $item; // From setup/index.php
						$db_user = ($item['mysql_username'] ? $item['mysql_username'] : $item['mysql_root_username']);
						$db_pass = ($item['mysql_password1'] ? $item['mysql_password1'] : $item['mysql_root_password']);
						$db_name = $item['mysql_dbname'];
						$db_host = $item['mysql_host'];
					}
					
					include($service_class_file);
					include($log_class_file);
					include($database_class_file);
					
					$db_args = array(
						'username'   => $db_user, 
						'password'   => $db_pass, 
						'database'   => $db_name, 
						'host'       => $db_host, 
						'port'       => '', 
						'standalone' => true
					);
					
					if (class_exists('Database')) {
						$this->db = new Database($db_args);
						//print '<div>Database: ' . get_class($this->db) . '</div>';
					}
				}
			}
		}
		
		
		$this->print_log = ( (isset($args['print_log']) && $args['print_log']) ? true : false);
		$this->print_error_results = ( (isset($args['print_error_results']) && $args['print_error_results']) ? true : false);
		$this->print_count_at = ( (isset($args['print_count_at']) && $args['print_count_at']) ? (int)$args['print_count_at'] : false);
		$this->version = ( (isset($args['version']) && $args['version']) ? (float)$args['version'] : false);
		
	}
	
	function run() {
		if ($this->needsImport()) {
			$this->runImport();
		} else {
			// $this->printLog('class: ' . get_class($this) . '::run. Import not required.', true);
		}
	}
	
	function needsImport() {
		// TODO: check current version - must be less than 4.690.00 ?
		return true; 
	}
	
	function runImport() {
		$this->printLog('class: ' . get_class($this) . '::runImport');
		
		$print_error_results = (boolean)$this->print_error_results;
		
		// ------------------------------------------------------------------
		
		$this->printLog('upgradeContacts');
		$results = $this->upgradeContacts();
		$this->printLog('Results.<br />Updated: ' . count($results['updated']) . '.<br />Errors: ' . count($results['error']) );
		if ($print_error_results && count($results['error'])) {
			$this->printLog('Errors encountered:'); $this->printLog($results['error']);
		}
		$this->printLogSpacer();
		
	}
	
	function doExit($do_exit = false) {
		global $USER;
		$USER['no_log'] = 0;
		
		if ($this->version) {
			$this->db->update(
				'System__version', 
				array(
					'version' => $this->version
				)
			);
		}
		
		if ($do_exit) {
			exit();
		}
		
	}
	
	function getContactsIterator() {
		$sql = 'SELECT id FROM `Contact__item` WHERE `active`=1 AND `status_id`=0 ORDER BY `id` ASC '; 
		$result = $this->db->sqlQuery($sql);
		return $result;
	}
	
	function getNextContact($resource) {
		if ($row = $this->db->getRow($resource)) {
			return $row;
		}
		return false;
	}
	
	/**
	 * @param resource The mysql query result
	 * @return integer The number of rows returned for the given query result
	 */
	function getNumRows($result) {
		return $this->db->getNumRows($result);
	}
	
	function getContactAccount($contact_id) {
		if (!$contact_id) {
			return false;
		}
		$sql = 'SELECT industry,source,relationship,is_alive,type_id FROM `Accounts__item` WHERE `contact_id` = "' . $contact_id . '" AND `primary_record`=1 AND `active`=1 LIMIT 1';
		$result = $this->db->sqlQuery($sql);
		if (is_resource($result)) {
			$result = $this->db->getRow($result);
		}
		// print_r($result); print '<hr>';
		// $this->db->printError();
		return $result;
	}
	
	function getContactEmailAddresses($contact_id) {
		if (!$contact_id) {
			return false;
		}
		$sql = 'SELECT content,subscribed FROM `AddressEmail__item` WHERE `contact_id` = "' . $contact_id . '" AND `active`=1 ORDER BY `primary_record` DESC, `subscribed` DESC, `priority` ASC';
		$result = $this->db->sqlQuery($sql);
		$items = false;
		if (is_resource($result)) {
			$items = array();
			$num = 0;
			while ($row = $this->db->getRow($result)) {
				$num++;
				if ($num <= 3) {
					$items['email' . $num . '_label'] = 0;
					$items['email' . $num . '_content'] = $row['content'];
					$items['email' . $num . '_subscribed'] = $row['subscribed'];
				} else {
					if (!isset($items['note_content'])) {
						$items['note_content'] = '';
					}
					$items['note_content'] .= $row['content'] . "\n";
				}
			}
			
			if (isset($items['note_content']) && strlen($items['note_content'])) {
				$items['note_title'] = 'Additional email addresses';
			}
			
		}
		
		return $items;
	}
	
	function getContactFaxAddresses($contact_id) {
		if (!$contact_id) {
			return false;
		}
		$sql = 'SELECT content,subscribed FROM `AddressFax__item` WHERE `contact_id` = "' . $contact_id . '" AND `active`=1 ORDER BY `primary_record` DESC, `subscribed` DESC, `priority` ASC';
		$result = $this->db->sqlQuery($sql);
		$items = false;
		if (is_resource($result)) {
			$items = array();
			$num = 0;
			while ($row = $this->db->getRow($result)) {
				$num++;
				if ($num <= 1) {
					$items['fax' . $num . '_content'] = $row['content'];
				} else {
					if (!isset($items['note_content'])) {
						$items['note_content'] = '';
					}
					$items['note_content'] .= $row['content'] . "\n";
				}
			}
			
			if (isset($items['note_content']) && strlen($items['note_content'])) {
				$items['note_title'] = 'Additional fax numbers';
			}
			
		}
		
		return $items;
	}
	
	function getContactPhoneAddresses($contact_id) {
		if (!$contact_id) {
			return false;
		}
		$sql = 'SELECT content,subscribed FROM `AddressPhone__item` WHERE `contact_id` = "' . $contact_id . '" AND `active`=1 ORDER BY `primary_record` DESC, `subscribed` DESC, `priority` ASC';
		$result = $this->db->sqlQuery($sql);
		$items = false;
		if (is_resource($result)) {
			$items = array();
			$num = 0;
			while ($row = $this->db->getRow($result)) {
				$num++;
				if ($num <= 3) {
					$items['phone' . $num . '_label'] = 0;
					$items['phone' . $num . '_content'] = $row['content'];
				} else {
					if (!isset($items['note_content'])) {
						$items['note_content'] = '';
					}
					$items['note_content'] .= $row['content'] . "\n";
				}
			}
			
			if (isset($items['note_content']) && strlen($items['note_content'])) {
				$items['note_title'] = 'Additional phone numbers';
			}
			
		}
		
		return $items;
	}
	
	function getContactPhysicalAddresses($contact_id) {
		if (!$contact_id) {
			return false;
		}
		$sql = 'SELECT * FROM `AddressPhysical__item` WHERE `contact_id` = "' . $contact_id . '" AND `active`=1 ORDER BY `primary_record` DESC, `subscribed` DESC, `priority` ASC';
		$result = $this->db->sqlQuery($sql);
		$items = false;
		if (is_resource($result)) {
			$items = array();
			$num = 0;
			while ($row = $this->db->getRow($result)) {
				$num++;
				if ($num <= 1) {
					$items['addrstreet' . $num . '_displayname'] = $row['display_name'];
					$items['addrstreet' . $num . '_format']      = $row['format'];
					$items['addrstreet' . $num . '_name']        = $row['name'];
					$items['addrstreet' . $num . '_houseno']     = $row['houseno'];
					$items['addrstreet' . $num . '_housesuffix'] = $row['housesuffix'];
					$items['addrstreet' . $num . '_street']      = $row['street'];
					$items['addrstreet' . $num . '_line2']       = $row['line2'];
					$items['addrstreet' . $num . '_line3']       = $row['line3'];
					$items['addrstreet' . $num . '_city']        = $row['city'];
					$items['addrstreet' . $num . '_state']       = $row['state'];
					$items['addrstreet' . $num . '_postcode']    = $row['postcode'];
					$items['addrstreet' . $num . '_country']     = $row['country'];
					
				} else {
					if (!isset($items['note_content'])) {
						$items['note_content'] = '';
					}
					$items['note_content'] .= $row['display_name'] . "\n";
				}
			}
			
			if (isset($items['note_content']) && strlen($items['note_content'])) {
				$items['note_title'] = 'Additional street addresses';
			}
			
		}
		
		return $items;
	}
	
	function getContactPostalAddresses($contact_id) {
		if (!$contact_id) {
			return false;
		}
		$sql = 'SELECT * FROM `AddressPostal__item` WHERE `contact_id` = "' . $contact_id . '" AND `active`=1 ORDER BY `primary_record` DESC, `subscribed` DESC, `priority` ASC';
		$result = $this->db->sqlQuery($sql);
		$items = false;
		if (is_resource($result)) {
			$items = array();
			$num = 0;
			while ($row = $this->db->getRow($result)) {
				$num++;
				if ($num <= 1) {
					$items['addrpost' . $num . '_displayname'] = $row['display_name'];
					$items['addrpost' . $num . '_format']      = $row['format'];
					$items['addrpost' . $num . '_name']        = $row['name'];
					$items['addrpost' . $num . '_houseno']     = $row['houseno'];
					$items['addrpost' . $num . '_housesuffix'] = $row['housesuffix'];
					$items['addrpost' . $num . '_street']      = $row['street'];
					$items['addrpost' . $num . '_line2']       = $row['line2'];
					$items['addrpost' . $num . '_line3']       = $row['line3'];
					$items['addrpost' . $num . '_city']        = $row['city'];
					$items['addrpost' . $num . '_state']       = $row['state'];
					$items['addrpost' . $num . '_postcode']    = $row['postcode'];
					$items['addrpost' . $num . '_country']     = $row['country'];
					
				} else {
					if (!isset($items['note_content'])) {
						$items['note_content'] = '';
					}
					$items['note_content'] .= $row['display_name'] . "\n";
				}
			}
			
			if (isset($items['note_content']) && strlen($items['note_content'])) {
				$items['note_title'] = 'Additional postal addresses';
			}
			
		}
		
		return $items;
	}
	
	function getContactURLAddresses($contact_id) {
		if (!$contact_id) {
			return false;
		}
		$sql = 'SELECT content,subscribed FROM `AddressURL__item` WHERE `contact_id` = "' . $contact_id . '" AND `active`=1 ORDER BY `primary_record` DESC, `subscribed` DESC, `priority` ASC';
		$result = $this->db->sqlQuery($sql);
		$items = false;
		if (is_resource($result)) {
			$items = array();
			$num = 0;
			while ($row = $this->db->getRow($result)) {
				$num++;
				if ($num <= 2) {
					$items['url' . $num . '_content'] = $row['content'];
				} else {
					if (!isset($items['note_content'])) {
						$items['note_content'] = '';
					}
					$items['note_content'] .= $row['content'] . "\n";
				}
			}
			
			if (isset($items['note_content']) && strlen($items['note_content'])) {
				$items['note_title'] = 'Additional URLs';
			}
			
		}
		
		return $items;
	}
	
	
	function updateBcrmContact($contact_id, $item) {
		$contact_id = (int)$contact_id;
		if (!$contact_id) {
			return false;
		}
		
		$contact = array();
		//$contact['id'] = (int)$contact_id;
		$contact['status_id'] = -1; // Flag this contact as updated
		
		// Account
		$contact_account = $this->getContactAccount($contact_id);
		if ( is_array($contact_account) && isset($contact_account['type_id']) ) {
			$contact['acc_industry']     = $contact_account['industry'];
			$contact['acc_source']       = $contact_account['source'];
			$contact['acc_relationship'] = $contact_account['relationship'];
			$contact['acc_is_alive']     = $contact_account['is_alive'];
			$contact['acc_type_id']      = $contact_account['type_id'];
		}
		
		// Email addresses
		$contact_emails = $this->getContactEmailAddresses($contact_id);
		if ( is_array($contact_emails) ) {
			if ( isset($contact_emails['email1_content']) ) {
				$contact['email1_label']       = $contact_emails['email1_label'];
				$contact['email1_content']     = $contact_emails['email1_content'];
				$contact['email1_subscribed']  = $contact_emails['email1_subscribed'];
			}
			if ( isset($contact_emails['email2_content']) ) {
				$contact['email2_label']       = $contact_emails['email2_label'];
				$contact['email2_content']     = $contact_emails['email2_content'];
				$contact['email2_subscribed']  = $contact_emails['email2_subscribed'];
			}
			if ( isset($contact_emails['email3_content']) ) {
				$contact['email3_label']       = $contact_emails['email3_label'];
				$contact['email3_content']     = $contact_emails['email3_content'];
				$contact['email3_subscribed']  = $contact_emails['email3_subscribed'];
			}
			if (isset($contact_emails['note_content'])) {
				$notes[] = array(
					'title' => $contact_emails['note_title'], 
					'content' => $contact_emails['note_content']
				);
			}
		}
		
		// Fax numbers
		$contact_fax = $this->getContactFaxAddresses($contact_id);
		if ( is_array($contact_fax) ) {
			if ( isset($contact_fax['fax1_content']) ) {
				$contact['fax1_content']       = $contact_fax['fax1_content'];
			}
			
			if (isset($contact_fax['note_content'])) {
				$notes[] = array(
					'title' => $contact_fax['note_title'], 
					'content' => $contact_fax['note_content']
				);
			}
		}
		
		// Phone numbers
		$contact_ph = $this->getContactPhoneAddresses($contact_id);
		if ( is_array($contact_ph) ) {
			if ( isset($contact_ph['phone1_content']) ) {
				$contact['phone1_label']       = $contact_ph['phone1_label'];
				$contact['phone1_content']     = $contact_ph['phone1_content'];
			}
			if ( isset($contact_ph['phone2_content']) ) {
				$contact['phone2_label']       = $contact_ph['phone2_label'];
				$contact['phone2_content']     = $contact_ph['phone2_content'];
			}
			if ( isset($contact_ph['phone3_content']) ) {
				$contact['phone3_label']       = $contact_ph['phone3_label'];
				$contact['phone3_content']     = $contact_ph['phone3_content'];
			}
			if (isset($contact_ph['note_content'])) {
				$notes[] = array(
					'title' => $contact_ph['note_title'], 
					'content' => $contact_ph['note_content']
				);
			}
		}
		
		// Street addresses
		$contact_street = $this->getContactPhysicalAddresses($contact_id);
		if ( is_array($contact_street) ) {
			if ( isset($contact_street['addrstreet1_displayname']) ) {
				$contact['addrstreet1_displayname'] = $contact_street['addrstreet1_displayname'];
				$contact['addrstreet1_format']      = $contact_street['addrstreet1_format'];
				$contact['addrstreet1_name']        = $contact_street['addrstreet1_name'];
				$contact['addrstreet1_houseno']     = $contact_street['addrstreet1_houseno'];
				$contact['addrstreet1_housesuffix'] = $contact_street['addrstreet1_housesuffix'];
				$contact['addrstreet1_street']      = $contact_street['addrstreet1_street'];
				$contact['addrstreet1_line2']       = $contact_street['addrstreet1_line2'];
				$contact['addrstreet1_line3']       = $contact_street['addrstreet1_line3'];
				$contact['addrstreet1_city']        = $contact_street['addrstreet1_city'];
				$contact['addrstreet1_state']       = $contact_street['addrstreet1_state'];
				$contact['addrstreet1_postcode']    = $contact_street['addrstreet1_postcode'];
				$contact['addrstreet1_country']     = $contact_street['addrstreet1_country'];
				
			}
			
			if (isset($contact_street['note_content'])) {
				$notes[] = array(
					'title' => $contact_street['note_title'], 
					'content' => $contact_street['note_content']
				);
			}
		}
		
		// Street addresses
		$contact_post = $this->getContactPostalAddresses($contact_id);
		if ( is_array($contact_post) ) {
			if ( isset($contact_post['addrpost1_displayname']) ) {
				$contact['addrpost1_displayname'] = $contact_post['addrpost1_displayname'];
				$contact['addrpost1_format']      = $contact_post['addrpost1_format'];
				$contact['addrpost1_name']        = $contact_post['addrpost1_name'];
				$contact['addrpost1_houseno']     = $contact_post['addrpost1_houseno'];
				$contact['addrpost1_housesuffix'] = $contact_post['addrpost1_housesuffix'];
				$contact['addrpost1_street']      = $contact_post['addrpost1_street'];
				$contact['addrpost1_line2']       = $contact_post['addrpost1_line2'];
				$contact['addrpost1_line3']       = $contact_post['addrpost1_line3'];
				$contact['addrpost1_city']        = $contact_post['addrpost1_city'];
				$contact['addrpost1_state']       = $contact_post['addrpost1_state'];
				$contact['addrpost1_postcode']    = $contact_post['addrpost1_postcode'];
				$contact['addrpost1_country']     = $contact_post['addrpost1_country'];
				
			}
			
			if (isset($contact_post['note_content'])) {
				$notes[] = array(
					'title' => $contact_post['note_title'], 
					'content' => $contact_post['note_content']
				);
			}
		}
		
		// URLs
		$contact_url = $this->getContactURLAddresses($contact_id);
		if ( is_array($contact_url) ) {
			if ( isset($contact_url['url1_content']) ) {
				$contact['url1_content']       = $contact_url['url1_content'];
				$contact['url2_content']       = $contact_url['url2_content'];
			}
			
			if (isset($contact_url['note_content'])) {
				$notes[] = array(
					'title' => $contact_url['note_title'], 
					'content' => $contact_url['note_content']
				);
			}
		}
		
		// print '<div>contact_id: ' . $contact_id . '</div>'; print_r($contact); print '<hr>';
		
		$update_result = $this->db->update(
			'Contact__item', 
			$contact, 
			'`id`="' . $contact_id . '"', 
			'LIMIT 1'
		);
		
		// Add notes if we have any
		if ($update_result && isset($notes) && is_array($notes) && sizeof($notes)) {
			// print '<div>NOTES</div>'; print_r($notes); print '<hr>';
			foreach ($notes as $k => $v) {
				$insert_result = $this->db->insert(
					'Note__item', 
					array(
						'title' => $v['title'], 
						'content' => $v['content'], 
						'contact_id' => $contact_id, 
						'active' => 1, 
						'created' => time()
					)
				);
				if (!$insert_result) {
					// print '<div>error:</div>'; $this->db->printError();
				}
			}
		}
		
		if (!$update_result) {
			// $this->printLog('Unable to update contact for ' . $contact_id . '. Error: ' . $this->db->getErrorString());
			return false;
		} 
		return $contact_id;
	}
	
	function upgradeContacts() {
		$results = array(
			'updated' => array(), 
			'error' => array()
		);
		$contacts_iterator = $this->getContactsIterator();
		// print '<div>contacts_iterator: ' . $contacts_iterator . '</div>'; $this->doExit(1);
		
		$num_rows = $this->getNumRows($contacts_iterator);
		// $this->printLog('upgradeContacts... number of rows: ' . $num_rows);
		$total_count = $count = 0;
		while ($contact = $this->getNextContact($contacts_iterator)) {
			// print '<div>contact:</div>'; print_r($contact); print '<hr>';
			if ($bcrm_contact_id = $contact['id']) {
				$update_result = $this->updateBcrmContact($bcrm_contact_id, $contact);
				if ($update_result) {
					$results['updated'][] = $contact['id'];
				} else {
					$results['error'][] = $contact['id'];
				}
			} 
			$total_count++; $count++;
			if ($count >= $this->print_count_at) {
				$this->printLog('upgradeContacts... ' . $contact['id'] . ' total count: ' . $total_count);
				$count = 0;
			}
		}
		unset($contacts_iterator);
		
		return $results;
	}
	
	function printLog($msg, $spacer = false) {
		if (!$this->print_log) {
			return;
		}
		if (is_array($msg)) {
			ob_start();
			print_r($msg);
			$msg = nl2br(ob_get_contents());
			ob_end_clean();
		}
		
		$usage = $this->db->getMemoryUsage(true);
		// or just print to screen
		print '<p>' . round($this->db->getPageExecutionTime(), 2) . ': ' . $msg . '' . (strlen($usage) ? '<br>' . $usage : '') . '</p>';
		// @ob_end_flush();
		while (@ob_end_flush());
		@flush();
		ob_start();
		if ($spacer) {
			$this->printLogSpacer();
		}
	}
	
	function printLogSpacer() {
		if (!$this->print_log) {
			return;
		}
		print '<hr />
		';
	}
}

$v4tov5_args = array(
	'print_log' => false, 
	'print_error_results' => false, 
	'print_count_at' => 10000, 
	'version' => '4.999'
);

$upgrader = new BrowserCRMV4toV5Upgrade($v4tov5_args);
// print '<div>upgrader ' . get_class($upgrader) . '</div>';

$upgrader->run();
// print '<div>before doExit</div>';
$upgrader->doExit();
// print '<div>upgrade finished</div>';
?>