<?php
/**
 * Creates a dropdown admin page for a nested dropdown
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
**/

includeWebMAT('admin_dropdown.inc');

/**
 * @abstract
 */
class WebMATStandardAdminNestedDropdown extends WebMATAdminDropdown {
	
	function WebMATAdminNestedDropdown($post, $get, $mini_data_definition = false, $access_level = '') {
		$this->WebMATAdminDropdown($post, $get, $mini_data_definition, $access_level);
	}

	/**
	 * Creates a cut down Data Dictionary
	 *
	 * @param array $item the current item from the dropdown table.
	 * @return array an array of items with formating infromation
	 */
	function getFormFields($item) {
		global $L;
		
		if (!$this->mini_data_definition) {
			
			$fields =  array(
				array('label'=>'', 'field_key'=>'id', 'type'=>'hidden'),
				array('label'=>'Parent', 'field_key'=>'parent_id', 'type'=>'dropdown', 'dropdown_values' => $this->dropdown->getValues('-', $this->form_values['id'])),
				array('label'=>$L[$this->module_name_lower . 'DropdownValueName'], 'field_key'=>$this->dropdown_config['field'], 'type'=>'text', 'notnull' => 1),
				array('label'=>$L['DisplayOrder'], 'field_key'=>'priority', 'type'=>'integer', 'min' => 0, 'max' => 999, 'default_value' => 100, 'notnull' => 1)
			);
			
		} else {
			
			$fields = $this->mini_data_definition;
			
		}
		
		foreach ($fields as $k => $v) {
			
			if ($item[$v['field_key']]) {
				$fields[$k]['value'] = $item[$v['field_key']];
			} else {
				$fields[$k]['value'] = $fields[$k]['default_value'];
			}
		}
		
		return $fields;
		
	}
	
	/**
	 * Builds the nested table for the list view
	 *
	 * @return string the table rows for the nested table. If there are no values for this dropdown, the string will be empty.
	 */
	function getNestedTable(){
		
		$table_rows = ''; // Phil - initialised our return value.
		
		$values = $this->dropdown->getValues('&nbsp;&nbsp;&nbsp;');
		
		if (is_array($values)) {
			
			$items = array();
			foreach ($values as $k => $v) {
				$items[] = Array( 'id' => $k, $this->dropdown_config['field'] => $v);
			}
			
			foreach ($items as $item) {
				
				$table_row = array();
				
				foreach ($this->getFormFields($item) as $field) {
					
					if ($field['field_key'] == $this->dropdown_config['field']) {
						// $table_row[] = $this->output->getStringField($field); // Phil
						//$table_row[] = $this->getStringField($field);
						$table_row[] = array (
							'content' => $this->output->getStringField($field),
							'tag' => 'th'
						);
					}
					
				}
				
				$table_row = $this->getEditDeleteLinks($table_row,$item['id']);
				$table_rows .= $this->output->getTableRow($table_row);
				
			}
			
			// return $table_rows; // Phil - returns at end of function.
			
		}
		
		return $table_rows;
		
	}
	
	
	/**
	 * Deprecated version 4.0 @see $this->getItemLinks()
	 *
	 * @param array $link_array the current row_array
	 * @param string $id the current item id
	 * @return array the row_array with the links appended
	 */
	function getEditDeleteLinks($row_array, $id) {
		
		return $this->getItemLinks($row_array, $id);
		
	}
	
	/**
	 * Displays a tabluar list of data for a dropdown.
	 *
	 * @param array $link_array the current row_array
	 * @param string $id the current item id
	 * @return array the row_array with the links appended
	 */
	function getItemLinks($row_array, $id) {
		global $L, $SERVICES;
		
		$php_self = $SERVICES->getGlobal('php_self');
		
		$row_array[] = $this->output->getLinkHref(array('href' => $php_self . '?id=-1&parent_id=' .
			$id, 'text' => $L['LinkNew']) ) . '&nbsp;&nbsp;';
		
		$row_array[] = $this->output->getLinkHref( array('href' => $php_self . '?id=' . $id, 'text' => $L['LinkEdit']) );
		$row_array[] = $this->output->getLinkDelete($php_self . '?delete=' . $id);
		
		return $row_array;
		
	}
	
	function getExtraDefaults($item) {
		
		if ($this->form_values['parent_id']) {
			return array('parent_id' => $this->form_values['parent_id']);
		}
		
	}
	
}
?>