<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 */

includeWebMAT('pieces_view.inc');

/**
 * Class for displaying contact view content
 * 
 * @package WebMAT
 * @author Phil Calder
 * @abstract
 */
class WebMATStandardContactView  extends WebMATPieces {
	
	var $begin_table_arg = 'contactPieceTable'; // Phil 20050728, added to skins.config
	
	// WebMATList functions
	function showCheckboxMenuRow(){ return false; }
	function showSearchFilterForm() { return false; }
	function getDescriptionContent() { return ''; }
	function getPagingRow() { return ''; }
	function beginSelectItemForm() { return ''; }
	function getCheckboxMenuRow() { return ''; }
	function getTitleRowLinks() { return ''; }
	function getSecondMenuRow() { return ''; }
	function showConfigFilters() { return false; }
	function showFilteredOnRow() { return false; }
	function getItemCheckbox() { return array(); }
	function getNewMenuContent() { return ''; }
	
	/**
	 * Need to call setShowContent to show by default
	 *
	 * @author Phil Calder
	 */
	function init($module, $data_option, $post = false, $get = false, $table_name = false, $language_overwrite = false) {
		parent::init($module, $data_option, $post, $get, $table_name, $language_overwrite);
		$this->setSortField(); // Phil 20050510 - Consistent with Dashboard. ets default order by (created DESC)
		$this->setShowContent(true);
		// Phil - Could change this depending on isStandardDisplay (?)
		$this->setTagHeaderRow('cpHeaderRow');  //css class for contact piece header row
		//$this->setContentDisplayMode(1);
		$this->addContactField();
	}
	
	/**
	 * Set display options
	 *
	 * @author Phil Calder
	 */
	function setOptions($do_count_items = true, $do_count_alerts = true, $do_show_items = 20) {
		// print '<div>setOptions for module ' . $this->getModuleName() . '</div>';
		$this->do_count_items = $do_count_items; 
		$this->do_count_alerts = $do_count_alerts;
		$this->do_show_items = $do_show_items;
		if ($this->do_show_items) {
			$this->paging_overwrite['limit'] = intVal($this->do_show_items);
		} else {
			$this->setContentDisplayMode(2); // Dont show items
		}
	}
	
	/**
	 * sets the content display mode
	 * 
	 * @param string $mode the display mode see below:
	 * 	- 0 = display content in hidden div
	 * 	- 1 = display content in shown div
	 * 	- 2 = do not display content
	 */
	function setContentDisplayMode($mode = false) {
		if ($mode) {
			$this->content_display_mode = $mode;
		} else {
			// global $SERVICES;
			//$SERVICES->preferences->getUserPref($this->getPrefName() . 'ContentDisplay', 0);
			$this->content_display_mode = 1; 
		}
	}
	
	/** 
	 * Does not query user pref for limit. See {@link setOptions} instead
	 * 
	 * @author Phil Calder
	 */
	function setLimit($number = 0, $start = 0) {
		// print '<div>[NOT] setLimit for module ' . $this->getModuleName() . '</div>';
		if (isset($this->form_values['limit'][$this->getPieceId()]) && ($this->form_values['limit'][$this->getPieceId()])) {
			$this->limit = array('number'=>$this->output->paging_list_size);
			return;
		}
		
		// Phil, Removed: will use result of $this->setOptions(...) called later
		// if (empty($number)) {
			// global $SERVICES;
			// $SERVICES->getService('Preferences');
			// $number = $SERVICES->preferences->getUserPref($this->getPrefName() . 'ListSize');
			// if (!$number) {
				// $number = $SERVICES->preferences->getUserPref('frontDashboardListSize');
			// }
		// }
		
		// $this->limit = array();
		// $this->limit['number'] = $number;
		// $this->limit['start'] = $start;
	}
	
	/**
	 * Adds the contact field to the data definition (if applicable)
	 *
	 * Only for modules which have a contact field and when contact company depth is on
	 * @author Phil Calder
	 */
	function addContactField() {
		global $SERVICES;
		$prefs =& $SERVICES->getService('Preferences');
		if ($prefs->getUserPref('contactShowCompanyDepth', '0')) {
			$dd =& $this->getDataDefinition();
			$fields = $this->db_functions->getDataDefinitionFields();
			$contact_id_field = $this->db_functions->getContactIdField();
			if (isset($fields[$contact_id_field]) && $fields[$contact_id_field] ) { 
				// print '<div>module: ' . $this->getModuleName() . ' include contact field: ' . $contact_id_field . '</div>'; print_r($fields[$contact_id_field]); print '<hr>';
				$dd->setDictionaryIncludeField($fields[$contact_id_field], true);
			}
		}		
	}
	
	/**
	 * Sets a contact id filter
	 *
	 * @author Phil Calder
	 */
	function setPieceFilter() {	
		// print '<div>setPieceFilter ++++++++==</div>';
		if (isset($this->db_functions) || (isset($this->mod) && is_object($this->mod))) {
			if (!isset($this->db_functions)) {
				$this->db_functions =& $this->mod->getDatabaseTable('item');
			}
			$this->db_functions->resetDesignTimeFilters();
			$this->db_functions->setDesignTimeFilters( array('contact_id' => $this->getContactId()) );
			
			// Phil 20050818 - Enabling full company depth
			global $SERVICES;
			$prefs =& $SERVICES->getService('Preferences');
			if (
			(isset($this->form_values['contact_depth']) && $this->form_values['contact_depth']) 
			|| 
			($prefs->getUserPref('contactShowCompanyDepth', '0'))
			) {
				
				// $contact_ids_string = $this->getContactId() . ', 2679';
				if ($this->contact_ids_string = $this->getSubContactIdsString(true)) {
					// TODO: use $contact_id_field = $this->getContactIdField();
					$this->db_functions->resetDesignTimeFilters();
					$this->db_functions->setDesignTimeFilters( 
						array(
							'contact_id' => array(
								'where' => 'contact_id IN (' . $this->contact_ids_string . ')'
							)
						) 
					);
				}
			}
		}
	}
	
	/**
	 * Overridden - functionality disabled in contact view
	 * 
	 * @return string an empty string
	 * @author Phil Calder
	 * 20050517
	 */
	function getBookmarkLinkLink() {
		return '';
	}
	
	/**
	 * For use in a MySQL 'IN' query, eg IN ( id1 [, id2, id3])
	 *
	 * @param boolean $include_parent Whether to include the current contact in the returned string. Defaults to false.
	 * @return mixed false or a comma separated string of ids (possibly blank)
	 * @author Phil Calder
	 */
	function getSubContactIdsString($include_parent = false) {
		$sub_contacts = $this->getSubContactIds();
		if (is_array($sub_contacts)) {
			if ($include_parent) {
				$sub_contacts = $this->arrayMergeReplace((array)$this->getContactId(), $sub_contacts);	
			}
			return implode(', ', $sub_contacts);
		}
		return false;
	}
	
	/**
	 * Applies only to companies
	 *
	 * @return mixed false or an array of ids
	 * @author Phil Calder
	 */
	function getSubContactIds() {
		global $SERVICES;
		$SERVICES->getService('Modules');
		$contact =& $SERVICES->modules->getModule('Contact');
		$contact_id = $this->getContactId();
		return $contact->getSubContacts($contact_id);
	}
	
	/**
	 * @return array with 'field' and 'order' keys, used to build an order by clause
	 * @author Phil Calder
	 */
	function getDefaultSort() {
		if (isset($this->sort_field) && is_array($this->sort_field)) {
			return $this->sort_field;
		}
		return array(
			'field' => 'id', 
			'order' => 'DESC'
		);
	}
	
	/**
	 * Set to 'false'. Can be overridden.
	 * @return boolean Whether to only show a 'View' link
	 * @author Phil Calder
	 */
	function showViewLinkOnly() {
		return false;
	}
	
	/**
	 * Returns an array of standard links
	 * 
	 * Will only show (at most) view link based on {@link showViewLinkOnly}
	 *
	 * @param integer $id the current item id
	 * @param array the item array
	 * @return array the array of standard links
	 * @author Phil Calder
	 */
	function getStandardLinks($id, $item) {
		if ($this->showViewLinkOnly()) {
			global $SERVICES;
			$links = array();
			
			if ($this->getFileSetHref('view')) {
				$links['view'] = $this->getViewLink($id);
			}
			
		} else {
			$links = parent::getStandardLinks($id, $item);
		}
		
		return $links;
	}
	
	/**
	 * @return string A link to edit this contact view piece
	 * @author Phil Calder
	 */
	function getShowItemsLink() {
		global $L;
		// print '<div>module: ' . $this->getModuleName() . ' piece id: ' .$this->getPieceId() . '</div>';
		$module = $this->getModuleName();
		$piece = $this->getPieceId();
		return $this->output->getLinkHref(
			array(
				'href' => '../../modules/Contact/admin_contact_view_setup.php?action=edit&module=' . $module . '&piece=' . $piece . '', 
				'text' => $L['LinkSettings']
			)
		);
	}
}
?>