<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 */

includeWebMAT('pieces_view.inc');

/**
 * Class for displaying contact view events
 * 
 * @package WebMAT
 * @author Phil Calder
 * @abstract
 */
class WebMATStandardContactViewEventList  extends WebMATPieces {
	
	/**
	 * @access private
	 */
	var $begin_table_arg = 'contactPieceTable'; // Phil 20050728, added to skins.config
	
	/**
	 * @access private
	 */
	var $detail_length = 50; // Phil 20060615, could use a pref for this instead
	
	// WebMATList functions
	function showCheckboxMenuRow() { return false; }
	function showSearchFilterForm() { return false; }
	function getDescriptionContent() { return ''; }
	function beginSelectItemForm() { return ''; }
	function getCheckboxMenuRow() { return ''; }
	function getTitleRowLinks() { return ''; }
	function getSecondMenuRow() { return ''; }
	function showConfigFilters() { return false; }
	function showFilteredOnRow() { return false; }
	function getItemCheckbox() { return array(); }
	function getNewMenuContent() { return ''; }
	
	/**
	 * Need to call setShowContent to show by default
	 */
	function init($module, $data_option, $post = false, $get = false, $table_name = false, $language_overwrite = false) {
		$this->setEventType($get['event_type']);
		parent::init($module, $data_option, $post, $get, $table_name, $language_overwrite);
		// $this->setSortField();
		$this->setShowContent(true);
		$this->setTagHeaderRow('cpHeaderRow');
		
		// $this->addContactField();
	}
	
	function setEventType($type = 'history') {
		// print '<div>' . $this->getModuleName() . ' setEventType: ' . $type . '</div>';
		if ($type == 'history') {
			$this->setEventPrefName('event_history_search_values');
		} else {
			$this->setEventPrefName('event_list_search_values');
		}
		$this->event_type = $type;
	}
	
	function getEventType() {
		return $this->event_type;
	}
	
	function setEventPrefName($pref_name = 'event_history_search_values') {
		// print '<div>' . $this->getModuleName() . ' ssetEventPrefName: ' . $pref_name . '</div>';
		$this->event_pref_name = $pref_name;
	}
	
	function getEventPrefName() {
		// print '<div>' . $this->getModuleName() . ' getEventPrefName: ' .  $this->event_pref_name . '</div>';
		return $this->event_pref_name;
	}
	
	/**
	 * Builds the expandable header row and content piece (list page).
	 *
	 * The content piece (list page) is often empty, unless overridden.
	 * Alternatively, call {@link getExpandableHeader} and {@link getDisplayContent} which both return content. 
	 * You can format this content your own way.
	 *
	 * @return mixed Either string content, or sends content to Output and exits.
	 */
	function main($return_string = true) {
		$this->loadContactViewEventSettings();
		$content = array();
		$this->setReturnString($return_string);
		if ($this->item_count) {
			$content = $this->getDataContent();
		}
		return (array)$content;
	}
	
	/**
	 * @return array Content
	 * @author Phil Calder
	 */
	function getDataContent() {
		
		$this->beforeGetItems($sort, $limit);
		$items = (array)$this->getItems($sort, false); // No Limit
		$this->afterGetItems($items);
		
		$module_name = $this->getTitleText();
		$events = array();
		foreach ($items as $k => $v) {
			if (is_array($v)) {
				$events[$k]['_module'] = $this->getModuleLink($v, $module_name);
				$events[$k]['_date'] = $this->getEventDate($v);
				$events[$k]['_label'] = $this->getEventLabel($v);
				$events[$k]['_detail'] = $this->getEventDetail($v);
				$events[$k]['_contact_id'] = (isset($v['contact_id']) ? $v['contact_id'] : '');
				$events[$k]['_contact_id_display_name'] = (isset($v['_contact_id_display_name']) ? $v['_contact_id_display_name'] : '');
			}
		}
		return $events;
	}
	
	function loadContactViewEventSettings() {
		global $SERVICES;
		$type = $this->getEventType();
		$this->event_list_settings = (array)$SERVICES->config->lookup('modules.Front.contactevent' . $type . 'config.' . $this->getModuleName() . '.settings');
	}
	
	function getTitleText($prefix = '') {
		global $SERVICES;
		$title_text = $this->getModuleNameLower() . 'EventName';
		return $SERVICES->language->getLanguageText($title_text, $this->module_name);	
	}
	
	function getModuleLink($item, $label) {
		global $L;
		$contact_id = ($item['contact_id'] ? $item['contact_id'] : $this->form_values['id']);
		if ($link_href = $this->getFileSetHref('list')) {
			if ($contact_id) {
				if (!strstr($link_href, '%contact_id%')) {
					$link_href .= '&contact_id=%contact_id%';
				}
			}
			$link_href = str_replace(array('%id%', '%contact_id%'), array('', $contact_id) , $link_href);
			$link = array(
				'href' => ($this->getModulePath() . $link_href), 
				'text' => ($label ? $label : $L['LinkList'])
			);
			return $this->output->getLinkHref($link);
		}
		return $label;
	}
	
	function getEventDateFieldName(&$alt_date_field_name) {
		$alt_date_field_name = false;
		$date_field = false;
		if (isset($this->event_list_settings['date']) && isset($this->event_list_settings['alt_date'])) {
			$alt_date_field_name = $this->event_list_settings['alt_date'];
			// print '<div>' . $this->getModuleName() . ' getEventDateFieldName, date: ' . $this->event_list_settings['date'] . ', alt_date: ' . $this->event_list_settings['alt_date'] . '</div>';
			return $this->event_list_settings['date'];
		}
		
		$dd = $this->dd->getDataDictionary();
		if (array_key_exists('last_update', $dd)) {
			if (!isset($this->event_list_settings['date'])) {
				$this->event_list_settings['date'] = 'last_update';
			}
			if (array_key_exists('created', $dd)) {
				$this->event_list_settings['alt_date'] = $alt_date_field_name = 'created';
			}
		} else if (array_key_exists('created', $dd)) {
			if (!isset($this->event_list_settings['date'])) {
				$this->event_list_settings['date'] = 'created';
			}
			$this->event_list_settings['alt_date'] = $alt_date_field_name = false;
		}
		// print '<div>' . $this->getModuleName() . ' getEventDateFieldName, date: ' . $this->event_list_settings['date'] . ', alt_date: ' . $this->event_list_settings['alt_date'] . '</div>';
		return $this->event_list_settings['date'];
	}
	
	function getEventDate($item) {
		// Returned value is NOT offset for users timezone
		$date_field_name = $this->getEventDateFieldName($alt_date_field_name);
		// return ($item[$date_field_name] ? $item[$date_field_name] : $item['created']);
		$date = ($item[$date_field_name] ? $item[$date_field_name] : $item['created']);
		if ($date && !is_numeric($date)) {
			// print '<div>' . $this->getModuleName() . ' date field value before: ' . $date . '</div>';
			if ($date == '0000-00-00') {
				// print '<div>' . $this->getModuleName() . ' date field value is zero, use created: ' . $item['created'] . '</div>';
				return $item['created'];
			}
			global $SERVICES;
			$date = $SERVICES->datelib->getTimestampFromMysqlDate($date);
			// print '<div>' . $this->getModuleName() . ' date field value after: ' . $date . '</div>';
		}
		if (!$date) {
			// print '<div>' . $this->getModuleName() . ' no date field value, use created: ' . $item['created'] . '</div>';
			return $item['created'];
		}
		// print '<div>getEventDate returning: ' . $date . '</div>';
		return $date;
	}
	
	function getEventLabel($item) {
		if (isset($this->event_list_settings['label']) && (isset($item[$this->event_list_settings['label']])) && strlen(trim($item[$this->event_list_settings['label']]))) {
			$label = $item[$this->event_list_settings['label']];
		} else {
			global $L;
			// Look for default label field
			$label = ($item['label'] ? $item['label'] : ($item['title'] ? $item['title'] : ($item['content'] ? $item['content'] : $L['noName'])));
		}
		$label = strip_tags($label);
		if (strlen($label) > $this->detail_length) {
			if (strlen($label) > $this->detail_length) {
				$label = substr($label, 0, $this->detail_length) . '...';
			}
		}
		return $this->getViewLink($item['id'], $label);
	}
	
	/**
	 * Builds the View link
	 * 
	 * @param integer $id Optional id variable (for overriding this functionality)
	 * @return string containing the link
	 */
	function getViewLink($id = false, $label = '') {
		global $L;
		$link_href = $this->getFileSetHref('view');
		if ($this->form_values['contact_id']) {
			if (!strstr($link_href, '%contact_id%')) {
				$link_href .= '&contact_id=%contact_id%';
			}
		}
		$link_href = str_replace(array('%id%', '%contact_id%'), array($id, '') , $link_href);
		$link = array(
			'href' => ($this->getModulePath() . $link_href), 
			'text' => ($label ? $label : $L['LinkView'])
		);
		return $this->output->getLinkHref($link);
	}
	
	function getEventDetail($item) {
		if (isset($this->event_list_settings['detail'])) {
			$dd_item = $this->dd->getDataStructureWithData($item);
			$detail_fields = explode(',', ($this->event_list_settings['detail']));
			$detail = '';
			$multiple_fields = (count($detail_fields) > 1);
			foreach ($detail_fields as $k => $v) {
				if (array_key_exists($v, $dd_item)) {
					$dd_item[$v]['list_detail_length'] = $this->detail_length;
					if (strlen($field_value = $this->getStringField($dd_item[$v]))) {
						$detail .= ($multiple_fields ? $dd_item[$v]['label'] . ': ' : '');
						$detail .= $this->getStringField($dd_item[$v]);
						$detail .= ($multiple_fields ? '<br />' : '');
					}
				}
			}
		} 
		if (!strlen($detail)) {
			$detail = '&nbsp;';
		}
		return $detail;
	}
	
	function getUsersSearchDateRange($pref_name) {
		global $USER;
		$after = $before = $values = false;
		if (isset($USER['contact_event'][$pref_name]['after'])) {
			$after = $USER['contact_event'][$pref_name]['after'];
		}
		if (isset($USER['contact_event'][$pref_name]['before'])) {
			$before = $USER['contact_event'][$pref_name]['before'];
		}
		if ($before || $after) {
			$values = array('after' => $after, 'before' => $before);
		}
		return $values;
	}
	
	/**
	 * Sets a contact id filter
	 */
	function setPieceFilter() {	
		// print '<div>[setPieceFilter, ' . $this->getModuleName() .']</div>';
		if (isset($this->db_functions) || (isset($this->mod) && is_object($this->mod))) {
			if (!isset($this->db_functions)) {
				$this->db_functions =& $this->mod->getDatabaseTable('item');
			}
			$this->db_functions->resetDesignTimeFilters();
			$this->db_functions->setDesignTimeFilters( array('contact_id' => $this->getContactId()) );
			
			// Phil 20050818 - Enabling full company depth
			global $SERVICES;
			$prefs =& $SERVICES->getService('Preferences');
			if (
			(isset($this->form_values['contact_depth']) && $this->form_values['contact_depth']) 
			|| 
			($prefs->getUserPref('contactShowCompanyDepth', '0'))
			) {
				if ($contact_ids_string = $this->getSubContactIdsString(true)) {
					$this->db_functions->resetDesignTimeFilters();
					$this->db_functions->setDesignTimeFilters( 
						array(
							'contact_id' => array(
								'where' => 'contact_id IN (' . $contact_ids_string . ')'
							)
						) 
					);
				}
				
				// Date filter
				if ($date_field_name = $this->getEventDateFieldName($alt_date_field_name)) {
					global $SERVICES, $L;
					// print '<div>' . get_class($this) . ' mod: ' . $this->getModuleName() . ' form_values[item]:</div>'; print_r($this->form_values['item']); print '<hr>';
					// print '<div>date_field_name: ' . $date_field_name . ' alt_date_field_name: ' . $alt_date_field_name . '</div>'; print '<hr>';
					if (!$this->form_values['item']['before'] && !$this->form_values['item']['after']) {
						if (!isset($this->form_values['submit'])) {
							// $values = $SERVICES->preferences->getUserPref($this->getEventPrefName());
							// if (($values !== false) && strlen($values)) {
								// $values = unserialize($values);
							// }
							// print '<div>' . get_class($this) . ' mod: ' . $this->getModuleName() . ' calling getUsersSearchDateRange</div>';
							$values = $this->getUsersSearchDateRange($this->getEventPrefName());
							// print '<div>mod: ' . $this->getModuleName() . ' webmat.. values for pref ' . $this->getEventPrefName() . ':</div>'; print_r($values); print '<hr>';
							if (is_array($values)) {
								$this->form_values['item']['after'] = $values['after'];
								$this->form_values['item']['before'] = $values['before'];
							}
						}
					}
					
					$before = $this->getFilterDateRangeBefore(); 
					$after = $this->getFilterDateRangeAfter(); 
					if ($after || $before) {
						$prefix = $this->getTableName();
						$original_date_field_name = $date_field_name;
						$has_alt_date_field_name = $alt_date_field_name;
						$date_field_name = $prefix . '.' . $date_field_name;
						$alt_date_field_name = $prefix . '.' . $alt_date_field_name;
						// print '<div>module: ' . $this->getModuleName() . ', date_field_name: ' . $date_field_name . ', alt_date_field_name: ' . $alt_date_field_name . ', before: ' . $before . ', after: ' . $after . '</div>';
						
						$standard_before_where = $alt_before_where = $standard_after_where = $alt_after_where;
						if ($before) {
							$standard_before_where = ' AND (' . $date_field_name . ' < "' . $before . '")';
							$alt_before_where = ' AND (' . $alt_date_field_name . ' < "' . $before . '")';
						}
						if ($after) {
							$standard_after_where = ' AND (' . $date_field_name . ' > "' . $after. '")';
							$alt_after_where = ' AND (' . $alt_date_field_name . ' > "' . $after. '")';
						}
						// $date_where = '((' . $date_field_name . ' <> "0") AND (' . $date_field_name . ' < "' . $before . '") AND (' . $date_field_name . ' > "' . $after. '"))';
						$date_where = '((' . $date_field_name . ' <> "0") ' . $standard_before_where . $standard_after_where . ')';
						// print '<div>date_where 1: ' . $date_where . '</div>';
						if ($has_alt_date_field_name) {
							
							$date_where = '( ' . $date_where . ' OR ';
							// $date_where .= '((' . $date_field_name . ' = "0") AND (' . $alt_date_field_name . ' < "' . $before . '") AND (' . $alt_date_field_name . ' > "' . $after. '"))';
							$date_where .= '((' . $date_field_name . ' = "0")' . $alt_before_where . $alt_after_where . ')';
							
							$date_where .= ' )';
						}
						// print '<p>setPieceFilter, ' . $this->getModuleName() .' setDesignTimeFilters for: ' . $original_date_field_name . ': ' . $date_where . '</p>';
						$this->db_functions->setDesignTimeFilters( 
							array(
								$original_date_field_name => array(
									'where' => $date_where
								)
							) 
						);
					}
				}
			}
		}
	}
	
	function getFilterDateRangeAfter() {
		// print '<div>getFilterDateRangeAfter</div>'; print_r($this->form_values['item']); print '<hr>';
		if (isset($this->form_values['item']['after']) && $this->form_values['item']['after']) {
			if ($this->form_values['item']['after'] && !is_numeric($this->form_values['item']['after'])) {
				// print '<div>' . $this->getModuleName() . ' date field value before: ' . $this->form_values['item']['after'] . '</div>';
				if ($date == '0000-00-00') {
					return false;
				}
				global $SERVICES;
				$this->form_values['item']['after'] = $SERVICES->datelib->getTimestampFromMysqlDate($this->form_values['item']['after']);
				// print '<div>' . $this->getModuleName() . ' date field value after: ' . $this->form_values['item']['after'] . '</div>';
			}
			// print '<div>' . get_class($this) . '::getFilterDateRangeAfter mod: ' . $this->getModuleName() . ' returning: ' . $this->form_values['item']['after'] . '</div>'; 
			return $this->form_values['item']['after'];
		}
		return false;
	}
	
	function getFilterDateRangeBefore() {
		// print '<div>getFilterDateRangeBefore</div>'; print_r($this->form_values['item']); print '<hr>';
		if (isset($this->form_values['item']['before']) && $this->form_values['item']['before']) {
			if ($this->form_values['item']['before'] && !is_numeric($this->form_values['item']['before'])) {
				// print '<div>' . $this->getModuleName() . ' date field value before: ' . $this->form_values['item']['before'] . '</div>';
				if ($date == '0000-00-00') {
					return false;
				}
				global $SERVICES;
				$this->form_values['item']['before'] = $SERVICES->datelib->getTimestampFromMysqlDate($this->form_values['item']['before']);
				// print '<div>' . $this->getModuleName() . ' date field value after: ' . $this->form_values['item']['before'] . '</div>';
			}
			return $this->form_values['item']['before'];
		}
		return false;
	}
	
	/**
	 * Overridden - functionality disabled in contact view
	 * 
	 * @return string an empty string
	 * @author Phil Calder
	 */
	function getBookmarkLinkLink() {
		return '';
	}
	
	/**
	 * For use in a MySQL 'IN' query, eg IN ( id1 [, id2, id3])
	 *
	 * @param boolean $include_parent Whether to include the current contact in the returned string. Defaults to false.
	 * @return mixed false or a comma separated string of ids (possibly blank)
	 * @author Phil Calder
	 */
	function getSubContactIdsString($include_parent = false) {
		$sub_contacts = $this->getSubContactIds();
		if (is_array($sub_contacts)) {
			if ($include_parent) {
				$sub_contacts = $this->arrayMergeReplace((array)$this->getContactId(), $sub_contacts);	
			}
			return implode(', ', $sub_contacts);
		}
		return false;
	}
	
	/**
	 * Applies only to companies
	 *
	 * @return mixed false or an array of ids
	 * @author Phil Calder
	 */
	function getSubContactIds() {
		global $SERVICES;
		$SERVICES->getService('Modules');
		$contact =& $SERVICES->modules->getModule('Contact');
		$contact_id = $this->getContactId();
		return $contact->getSubContacts($contact_id);
	}
	
	/**
	 * @return array with 'field' and 'order' keys, used to build an order by clause
	 * @author Phil Calder
	 */
	function getDefaultSort() {
		if (isset($this->sort_field) && is_array($this->sort_field)) {
			return $this->sort_field;
		}
		return array(
			'field' => 'id', 
			'order' => 'DESC'
		);
	}
	
	function setBrowseQueryDataOption() {}
	function clearBrowseQuery() {}
	
}

?>