<?php
/** 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 */

includeWebMAT('expandable_list.inc');

/**
 * Class for displaying dashboard content
 *
 * @abstract
 * @package WebMAT
 * @author Phil Calder
 */
class WebMATStandardDashboard extends WebMATExpandableList {
	
	/**
	 * Calls parent and sets default link target, sort field, tag header row css
	 */
	function init($module, $data_option) {
		
		parent::init($module, $data_option);
		
		$this->output->setDefaultLinkTarget('contentframe');
		
		$this->setSortField();
		$this->setTagHeaderRow('dHeaderRow');  //css class for dash piece header row
	}
	
	
	/** 
	 * @see getContentDisplayMode
	 */
	function setContentDisplayMode($mode = false) {
		if (strstr($this->getPrefName(), 'ContactView') && is_object($this->mod)) {
			$this->setPrefName($this->mod->getModuleNameLower() . 'Dashboard');
		}
		parent::setContentDisplayMode($mode);
	}
	
	/**
	 * Dashboard pieces return a table with only a header rowwhen $return_tstring is true
	 * @return string Header row
	 */
	function main($return_string = true) {	
		$this->setReturnString($return_string);
		
		// Phil - was testing only returning a row (ie no table)
		// if ($return_string) {
			// $content = '';
			// $content .= $this->getHeaderRow();
			// return $content;
		// }
		// 
		// parent::main($return_string);
		// -----
		
		$content = '';
		
		$content .= $this->output->getBeginTable('pieceTable'); // - header
		
		$content .= $this->getExpandableHeader(true); // true = as a row
		
		$content .= $this->output->getEndTable();
		
		if ($return_string) {
			
			return $content;
			
		}
		
		$this->output->addToBody($content);
		
		$this->output->display();
		
	}
	
	/**
 	 * Removes "All'link (list view) from standard header
	 * @return string content
	 */
	function getHeaderLinks() {
		
		if (isset($this->header_links) && $this->header_links) {
			
			$content = $this->header_links;
			
		} else {
		
			$content = array();
			
			if ($bookmark = $this->getBookmarkLinkLink()) {
				$content[] = $bookmark;
			}
			
			//mcr 2005-05-18
			// if ($all = $this->getAllLink()) {
				// $content[] = $all;
			// }
			
			if ($new = $this->getNewLink()) {
				$content[] = $new;
			}
			
			
		}
		
		if (is_array($content)) {
		
			//return implode($this->getMenuSeparator(), $content);
			return implode('', $content);
			
		} 
		
		return $content;
		
	}	
	
	/** 
	 * Content display is disabled in a dashboard view
	 */
	function getContentDisplayMode() {
		//MCR TEMP switch off piece content
		return 2;
	}
	
	/**
	 * Resets design filters and calls parent
	 */
	function setPieceFilter($type = 'items') {
		if (isset($this->db_functions) && is_object($this->db_functions)) {
			global $USER;
			$this->db_functions->resetDesignTimeFilters();
		}
		parent::setPieceFilter($type);
	}
	
	/**
	 * @return string An empty string
	 * @todo Phil: remove style - This should be in a css file
	 */
	function getMenuLink() {
		return ''; // Phil 20050615 - Removed dropdown arrow link (for menu) - yet to be implemented
		// Phil - took this link out of getMenuCellContent
		// Would return something like this in a 'Dashboard' piece:
		return '<span style="vertical-align:top"><a href="" target="contentframe">' . $this->output->getImageTag('icons_dash/drop_menu.gif', false, 'align="absmiddle"') . '</a></span>';
	}
	
	/**
	 * Item counts on the dashboard disabled here
	 * Can override per installation using webmat_ext directory
	 * by overriding getShowCountItems
	 * @return false
	 * @access private
	 */
	function getShowCountItems() {
		return false;
	}
	
}

includeWebMAT('bookmark_dashboard.inc'); // Phil 20050523

?>