<?php
/**
 * Handles the DataDictionary functionality for WebMAT
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 */

includeWebMAT('webmat.inc');

/**
 * this class handles the DataDictionary functionality for WebMAT
 *
 * @author Matthew Corich, Phil Calder, Nathan Humphries
 * @abstract
 */
class WebMATStandardDataDictionary extends WebMAT {
	
	/**
	 * @var array the data definition
	 * @access private
	 */
	var $data_dictionary = array();
	/**
	 * @var array options for processing the data definition
	 * @access private
	 */
	var $data_options;
	/**
	 * @var array $fields a list on fields for the current option
	 * @access private
	 */
	var $fields = false;
	/**
	 * @var array $removed_fields a list on fields not in the current option
	 * @access private
	 */
	var $removed_fields = array();
	
	/**
	* Constructor function sets all the class variables and runs the data_options
	*
	* @param  array  $data_definition  the current module name. Must contain a key 'fields' (an array) or the constructor will print an error and exit.
	* @param  array  $option  any extra options
	*/
	function WebMATStandardDataDictionary($data_definition, $option = '') {
		//NLH 20041206 - Array of values that the rankofArray should exlcude from its depth checking
		//eg... Dropdown values
		//echo '<div><br>initialising...</div>';
		//print_r(array_keys($data_definition));
		
		$this->rank_array_exclusions = array('dropdown_values');
		if (is_array($data_definition['fields'])) { // Phil 20050523 - is_array 
			// print '<div>data definition option: ' . $option . '</div>'; 
			// print '<div>data definition field keys:</div>'; print_r(array_keys($data_definition['fields'])); print '<hr>';
			$this->data_dictionary = $data_definition['fields'];
			// Phil 20051031 see getDataOption, getNoTrim
			if ($option) {
				// Phil 20061003 Added other special cases: __visible__, __noperms__
				switch ($option) {
					case '__all__':
						// __all__ is a special case: includes all defined fields
						$data_definition['data_options'][$option] = array( 
							'method' => 'exclude',  
							'values' => array(), 
							'options' => array( 
								'exclude_hidden' => 0,  
								'exclude_calculated' => 0,  
								'exclude_foreign' => 0,  
								'exclude_custom' => 0,  
								'dropdown_values' => 1 
							)
						);
					break;
					case '__altlist__':
						// __altlist__ is a special case: includes all defined fields except hidden. 
						// Adds no trim.
						// Useful for printing
						$data_definition['data_options'][$option] = array( 
							'method' => 'exclude',  
							'values' => array(), 
							'options' => array( 
								'exclude_hidden' => 1,  
								'exclude_calculated' => 0,  
								'exclude_foreign' => 0,  
								'exclude_custom' => 0,  
								'dropdown_values' => 1, 
								'no_trim' => 1
							)
						);
					break;
					case '__altlistnoperms__':
						// __altlist__ is a special case: includes all defined fields except hidden, view, edit, and admin groups. 
						// Adds no trim.
						// Useful for printing
						$data_definition['data_options'][$option] = array( 
							'method' => 'exclude',  
							'values' => array('view_group_id','edit_group_id','admin_group_id'), 
							'options' => array( 
								'exclude_hidden' => 1,  
								'exclude_calculated' => 0,  
								'exclude_foreign' => 0,  
								'exclude_custom' => 0,  
								'dropdown_values' => 1, 
								'no_trim' => 1
							)
						);
					break;
					case '__visible__':
						// __visible__ is a special case: includes all defined fields except hidden
						$data_definition['data_options'][$option] = array( 
							'method' => 'exclude',  
							'values' => array(), 
							'options' => array( 
								'exclude_hidden' => 1,  
								'exclude_calculated' => 0,  
								'exclude_foreign' => 0,  
								'exclude_custom' => 0,  
								'dropdown_values' => 1
							)
						);
					break;
					case '__shortlist__':
						// __shortlist__ is a special case: includes only the (one) display name field
						$display_name_field = false;
						$backup_display_name_field = false;
						foreach ($data_definition['fields'] as $k => $v) {
							if ($v['display_name']) {
								$display_name_field = $v['field_key'];
								break;
							} else if ($v['type'] == 'text') {
								$backup_display_name_field = $v['field_key'];
							}
						}
						if (!$display_name_field) {
							$display_name_field = $backup_display_name_field;
						}
						$method = 'include';
						if (!$display_name_field) {
							$method = 'exclude';
						}
						$data_definition['data_options'][$option] = array( 
							'method' => $method,  
							'values' => array($display_name_field), 
							'options' => array( 
								'exclude_hidden' => 1,  
								'exclude_calculated' => 0,  
								'exclude_foreign' => 0,  
								'exclude_custom' => 0,  
								'dropdown_values' => 1 
							)
						);
					break;
				}
				// if ($option == '__all__') {
					// // __all__ is a special case: includes all defined fields
					// $data_definition['data_options'][$option] = array( 
						// 'method' => 'exclude',  
						// 'values' => array(), 
						// 'options' => array( 
							// 'exclude_hidden' => 0,  
							// 'exclude_calculated' => 0,  
							// 'exclude_foreign' => 0,  
							// 'exclude_custom' => 0,  
							// 'dropdown_values' => 1 
						// )
					// );
				// }
				if (is_array($data_definition['data_options']) && array_key_exists($option, $data_definition['data_options'])) {
					$this->data_options = $data_definition['data_options'][$option]; 
				} else {
					$this->data_options = $data_definition['data_options']['list']; 
					// print '<div>module: using list instead of: ' . $option . '</div>'; print_r($data_definition['data_options']['list']); print '<hr>';
				}
				// $this->data_options = $data_definition['data_options'][$option]; 
				// print '<div>option = ' . $option . '</div>'; print_r($data_definition['data_options'][$option]); print '<hr>';
			} // else {
				// print '<div>using list: option = ' . $option . '</div>'; print_r($data_definition['data_options'][$option]); print '<hr>';
				// $option = 'list';
				// $this->data_options = $data_definition['data_options'][$option];
			// }
		
		} else {
			global $SERVICES, $L;
			$SERVICES->getService('Log');
			$title = '<strong>' . $L['errDataDictionary'] . ':</strong>';
			$msg = $L['errNoFields'];
			$message = $title . ': ' . $msg;
			$SERVICES->log->logAction($message);
			
			quickPage($message);
			exit();
		}
		
		$this->option = $option; // Phil 20051031
		$data_options = (isset($data_definition['data_options'][$option]) ? $data_definition['data_options'][$option] : false);
		
		if (is_array($data_options)) {
			
			switch ($data_options['method']) {
				case 'exclude':
					$this->setDictionaryExcludeFields($data_options['values']);
				break;
				
				case 'include':
					$this->setDictionaryIncludeFields($data_options['values']);
				break;
				
				default:
				break;
			}
			
			if (is_array($data_options['options'])) {
				foreach ($data_options['options'] as $k => $v) {
					//echo '<div>now:</div>'; print_r(array_keys($this->data_dictionary));
					//echo '<br>' . $k;
					
					switch ($k) {
						case 'change_attributes':
							$this->changeAttributes($v);
						break;
						
						case 'exclude_hidden':
							if ($v == 1) {
								$this->setDictionaryExcludeHidden();
							}
						break;
						
						case 'exclude_calculated':
							if ($v == 1) {
								$this->setDictionaryExcludeCalculated();
							}
						break;
						
						case 'exclude_custom': // Phil 20051029
							if ($v == 1) {
								$this->setDictionaryExcludeCustom();
							}
						break;
						
						case 'exclude_foreign': // Phil 20051029
							if ($v == 1) {
								$this->setDictionaryExcludeForeign();
							}
						break;
						
						case 'dropdown_values':
							if ($v == 1) {
								$this->getDropdownValues();
							}
						break;
						
						case 'no_trim':
							if ($v == 1) {
								$this->setNoTrim();
							}
						break;
						
						default:
						break;	
					}
				}
			}
			//echo '<div>after processing (data_dictionary)</div>'; print_r(array_keys($this->data_dictionary));
		}
	}
	
	/**
	 * Returns an array containing the data dictionary
	 *
	 * @return array the processed data dictionary
	 */
	function getDataDictionary() {
		return $this->data_dictionary;
	}
	
	/**
	 * @return array The current data options, keys may include 'method', 'values', 'options'.
	 * @author Phil Calder
	 */
	function getDataOption() {
		// print_r($this->data_options);
		return $this->data_options;
	}
	
	/**
	 * @return boolean Whether the current module option specifies not to trim fields
	 * @author Phil Calder
	 */
	function getNoTrim() {
		return ( (isset($this->data_options['options']['no_trim']) && $this->data_options['options']['no_trim']) ? true : false);	
	}
	
	/**
	 * @param boolean Whether the current module option specifies not to trim fields
	 * @author Phil Calder
	 */
	function setNoTrimOption() {
		if (isset($this->data_options['options']) && $this->data_options['options']) {
			$this->data_options['options']['no_trim'] = 1;
		}
	}
	
	/**
	* returns an array of items with the data_definition formation information
	*
	* @param array $items array
	* @return array items with formating information, or prints an error and exits (if the item parameter is not an array)
	*/
	function getDataStructureWithData($item, $sort_priority = '') {
		if (is_array($item)) {
			$dd_items = array();
			$fields = $this->getDataDictionary();
			// print '<div>getDataStructureWithData: field keys:</div>'; print_r(array_keys($fields)); print '<hr>';
			
			//sort $fields here if necessary
			if ($sort_priority) {
				// print '<p>' . $sort_priority . ' -- ' . implode(' | ', array_keys($fields)) . ' --</p>';
				// Obtain a list of columns
				foreach ($fields as $k => $v) {
				   $sort[$k] = $v[$sort_priority];
				}
				
				// Sort the data with volume descending, edition ascending
				// Add $data as the last parameter, to sort by the common key
				if (is_array($fields) && is_array($sort)) {
					array_multisort($sort, SORT_ASC, $fields);
				}
			}
			
			foreach ($fields as $k => $v) {
				if ( ( isset($fields[$k]['foreign']) && ($fields[$k]['foreign'] == 1) ) && !$item[$v['field_key']] && (substr($v['field_key'], 0, 1) != '_') ) {
					$fields[$k]['value'] = $item['_' . str_replace('.', '_', $v['field_key'])];
					// $fields[$k]['hidden'] = $item[$v['hidden']]; // Phil
					$dd_items[$v['field_key']] = $fields[$k];
					// print '<div>**getDataStructureWithData: ' . $v['field_key'] . ', ' .  $item[$v['field_key']] . '</div>'; print_r($dd_items[$v['field_key']]); print '<div>---------------</div>'; print_r($fields[$k]);  print '<hr>';
				} else {
					$fields[$k]['value'] = $item[$v['field_key']];
					// $fields[$k]['hidden'] = $item[$v['hidden']]; // Phil
					$dd_items[$v['field_key']] = $fields[$k];
					
				}
				// print '<div>getDataStructureWithData: ' . $v['field_key'] . ', ' .  $item[$v['field_key']] . '</div>'; print_r($dd_items[$v['field_key']]); print '<div>---------------</div>'; print_r($fields[$k]);  print '<hr>';
			}
			// print '<div>getDataStructureWithData returning:</div>'; print_r($dd_items); print '<hr>';
			return $dd_items;
			
		} else {
			global $SERVICES, $L;
			$SERVICES->getService('Log');
			$title = '<strong>' . $L['errDataDictionary'] . ':</strong>';
			$msg = $L['errNoItem'];
			$message = $title . ': ' . $msg;
			$SERVICES->log->logAction($message);
			
			quickPage($message);
			exit();
		}
	}
	
	/**
	 * If the dropdown values are from a table it gets the dropdown_values from the DB table and updates $this->data_dictionary.
	 * Sets the data_dictionary field dropdown_values variable to an error message if it cannot get the values.
	 * 
	 */
	function getDropdownValues() {
		foreach ($this->data_dictionary as $k => $v) {
			if (isset($v['dropdown_values']) && strstr($v['dropdown_values'], '__table(')) {
				$temp =  explode('.', str_replace(array('__table(', ')__'), '', $v['dropdown_values']) );
				if (is_array($temp) &&  (count($temp) > 1) ) {
					if (!isset($temp[2]) || !$temp[2]) {
						$temp[2] = false;
					}
					if (!isset($v['custom_where'])) {
						$v['custom_where'] = false;
					}
					
					// Phil 20070314 Dropdown values stored / retrieved from DataDefinition Service cache
					global $SERVICES;
					$dd_service =& $SERVICES->getService('DataDefinition');
					if (!is_array($dropdown_values = $dd_service->getWebMATCachedDropdownValues($temp[0], $temp[1], $temp[2]))) {
						$dropdown = getWebMATObject(
							'WebMATDropdown', 
							$temp[0], // module
							array(
								'module' => $temp[0], 
								'table' => $temp[1], 
								'field_name' => $temp[2]
							)
						); // new WebMATDropDown($temp[0], $temp[1], $temp[2], $v['custom_where']);
						$dropdown_values = $dropdown->getValues();
						$dd_service->setWebMATCachedDropdownValues($dropdown_values, $temp[0], $temp[1], $temp[2]);
					}
					
				} else {
					$dropdown_values = 'Invlalid dropdown selected in Data Dictionary';
				}
				
				$this->data_dictionary[$k]['dropdown_values'] = $dropdown_values;
				// Phil 20060817 store the original table name as well (eg __table(Module.table_name)__)
				$this->data_dictionary[$k]['_dropdown_table_name'] = $v['dropdown_values']; 
				unset($dropdown);
			}
		}
	}
	
	function setDictionaryIncludeField($field, $force_merge = false) {
		// print '<div>' . get_class($this) . '::setDictionaryIncludeField<div>'; print_r($field); print '<br>---<br>'; print_r($this->data_dictionary); print '<br>-----<br>';
		if (is_array($field) && $field['field_key'] && is_array($this->data_dictionary)) {
			// Phil 20060601
			// Either add the field at the beginning or end of the list of fields, or retain its current position
			if ($force_merge) { 
				// beginning (eg contact view with company depth adds the contact field at the beginning)
				// print '<div>force_merge ' . $field['field_key'] . '</div>';
				$this->data_dictionary = $this->arrayMergeReplace(array($field['field_key'] => $field), $this->data_dictionary);
			} else if (array_key_exists($field['field_key'], $this->data_dictionary)) {
				// retain position (overwriting current field with $field)
				// print '<div>key exists (retain order) ' . $field['field_key'] . '</div>';
				$this->data_dictionary = $this->arrayMergeReplace($this->data_dictionary, array($field['field_key'] => $field));
			} else {
				// Add new field to the end of the list
				// print '<div>new field ' . $field['field_key'] . '</div>';
				$this->data_dictionary[$field['field_key']] = $field;
			}	
		}
		// print '<div>After include:<div>'; print '<br>'; print_r($this->data_dictionary); print '<hr>';
	}
	
	/**
	 * Removes all fields from this->data_dictionary that are not in the $fields array
	 *
	 * @param array $fields array
	 */
	function setDictionaryIncludeFields($fields) {
		//print '<div>' . get_class($this) . '::$data_dictionary:<div>'; print_r(array_keys($this->data_dictionary)); print '<hr>';
		//print '<div>' . get_class($this) . '::setDictionaryIncludeFields<div>'; print_r($fields); print '<hr>';
		
		foreach ($this->data_dictionary as $k => $v) {
			if (!in_array($v['field_key'] , $fields)) {
				// print '<div>remove field ' . $v['field_key'] . '</div>';
				$this->removed_fields[$k] = $this->data_dictionary[$k];
				bcrm_unset($this->data_dictionary, $k);
			}
		}
		//print '<div>remove:'; print_r(array_keys($this->removed_fields)	); print '<hr>';
		//print '<div>remaining (data_dictionary):'; print_r(array_keys($this->data_dictionary)); print '<hr>';
		 
	}
	
	/**
	* Updates this->data_dictionary by unsetting all values that are in the specified $fields array
	*
	* @param array $fields array
	*/
	function setDictionaryExcludeFields($fields) {
		// print '<div>' . get_class($this) . '::setDictionaryExcludeFields<div>'; print_r($fields); print '<hr>';
		foreach ($this->data_dictionary as $k => $v) {
			if (is_array($fields) && in_array($v['field_key'] , $fields)) {
				$this->removed_fields[$k] = $this->data_dictionary[$k];
				bcrm_unset($this->data_dictionary, $k);
			}
		}
	}
	
	/**
	 *
	 * @author Phil Calder
	 */
	function setDictionaryExcludeHideEmpty() {
		// print '<div>' . get_class($this) . '::setDictionaryExcludeHideEmpty<div>'; print '<hr>';
		foreach ($this->data_dictionary as $k => $v) {
			if (isset($v['hide_empty']) && $v['hide_empty']) { // Phil
				// print '<div>unsetting: ' . $k . '</div>';
				$this->removed_fields[$k] = $this->data_dictionary[$k];
				bcrm_unset($this->data_dictionary, $k);
			}
		}
	}
	
	/**
	* Removes all hidden and password values from this->data_dictionary
	*/
	function setDictionaryExcludeHidden(){
		// print '<div>' . get_class($this) . '::setDictionaryExcludeHidden<div>'; print '<hr>';
		foreach ($this->data_dictionary as $k => $v) {
			if ($v['type'] == 'hidden' || $v['type'] == 'password') {
				$this->removed_fields[$k] = $this->data_dictionary[$k];
				bcrm_unset($this->data_dictionary, $k);
			} else if (isset($v['hidden']) && $v['hidden']) { // Phil
				// print '<div>unsetting: ' . $k . '</div>';
				$this->removed_fields[$k] = $this->data_dictionary[$k];
				bcrm_unset($this->data_dictionary, $k);
			}
		}
	}
	
	/**
	* Removes all calculated values from this->data_dictionary
	* Also excludes foreign fields
	*/
	function setDictionaryExcludeCalculated() {
		// print '<div>setDictionaryExcludeCalculated</div>';
		foreach ($this->data_dictionary as $k => $v) {
			//if ( (isset($v['calculated']) && ($v['calculated'] == '1')) || (isset($v['foreign']) && ($v['foreign'] == '1')) ) {
			if ( (isset($v['calculated']) && ($v['calculated'] == '1')) ) {
				$this->removed_fields[$k] = $this->data_dictionary[$k];
				bcrm_unset($this->data_dictionary, $k);
			}
		}
	}
	
	/**
	 * remove all custom fields from data dictionary
	 * @author Phil Calder
	 */
	function setDictionaryExcludeCustom() {
		// print '<div>setDictionaryExcludeCustom</div>';
		foreach ($this->data_dictionary as $k => $v) {
			// Phil 20051031 can be 1, 2, or from a _custom table
			if ( ($v['field_is_custom']) || (strpos($v['field_key'], '_custom.') !== false)) { 
				$this->removed_fields[$k] = $this->data_dictionary[$k];
				bcrm_unset($this->data_dictionary, $k);
				// print '<div>exclude: ' . $v['field_key'] . ', flag: ' . $v['field_is_custom'] . '</div>';
			} // else {
				// print '<div>include: ' . $v['field_key'] . ', flag: ' . $v['field_is_custom'] . '</div>';	
			// }
		}
	}
	
	/**
	 * remove all custom fields from data dictionary
	 * @author Phil Calder
	 */
	function setDictionaryExcludeForeign() {
		// print '<div>setDictionaryExcludeForeign</div>';
		foreach ($this->data_dictionary as $k => $v) {
			if ($v['foreign'] == '1') {
				$this->removed_fields[$k] = $this->data_dictionary[$k];
				bcrm_unset($this->data_dictionary, $k);
			}
		}
	}
	
	/**
	 * gets default values
	 *
	 * @param array $default_overrides values that overwrite the data definition defaults.
	 */
	function getDefaultItem($default_overrides = false) {
		// print '<div>Data dictionary</div>'; print_r($this->data_dictionary); print '<hr>';
		$items = array();
		foreach ($this->data_dictionary as $k => $v) {
			if (!$v['default_value'] && strstr($v['field_key'], '_group_id')) {
				global $USER, $SERVICES;
				$SERVICES->getService('Preferences');
				switch ($v['field_key']) {
					case 'view_group_id':
						$pref = 'frontDefaultGroupView';
					break;
					case 'edit_group_id':
						$pref = 'frontDefaultGroupEdit';
					break;
					case 'owner_group_id':
						$pref = 'frontDefaultGroupOwner';
					break;
					case 'admin_group_id':
						$pref = 'frontDefaultGroupAdmin';
					break;
				}
				$v['default_value'] = $SERVICES->preferences->getUserPref($pref, 'private_group_id');
				if ($v['default_value'] == 'private_group_id') {
					$user_private_group = $USER['private_group_id'];
					$v['default_value'] = $user_private_group;
				}
			}
			
			if ($v['default_value']) {
				$items[$v['field_key']] = $v['default_value'];
			}
			
			if ( ($default_overrides) && (in_array($v['field_key'], array_keys($default_overrides))) ) {
				$items[$v['field_key']] = $default_overrides[$v['field_key']];
			}
		}
		return $items;
	}
	
	/**
	* disable list detail triming
	*/
	function setNoTrim() {
		foreach ($this->data_dictionary as $k => $v) {
			if (isset($v['list_detail_length']) && $v['list_detail_length']) {
				$this->removed_fields[$k] = $this->data_dictionary[$k];
				bcrm_unset($this->data_dictionary, $k, 'list_detail_length');
			}
		}
	}
	
	/**
	 * Sets the given field as read only (temporarily)
	 * @param string $field_key The data definition field key
	 *
	 */
	function setReadOnly($field_key) {
		// print '<div>' . get_class($this) . '::setReadOnly ' . $field_key . '<div>'; print '<hr>';
		foreach ($this->data_dictionary as $k => $v) {
			if ($v['field_key'] == $field_key) {
				$this->data_dictionary[$k]['read_only'] = 1;
				return;
			}
		}
	}
	
	/**
	 * Sets the given field as not read only (temporarily)
	 * @param string $field_key The data definition field key
	 *
	 */
	function setNotReadOnly($field_key) {
		foreach ($this->data_dictionary as $k => $v) {
			if ( ($field_key = '__all__') || ($v['field_key'] == $field_key)) {
				$this->data_dictionary[$k]['read_only'] = 0;
			}
		}
	}
	
	/**
	 * Usually for returning foreign/linked fields for a particular table
	 *
	 * Eg: keys beginning with 'ModuleName__table'
	 * @param string $prefix Look for field_keys beginning ith this prefix. Optional, default '' (ie all field_keys)
	 * @return array Subset of fields - those with keys beginning with $prefix
	 * @author Phil Calder
	 */
	function getFieldsBeginningWith($prefix = '') {
		$subset = array();
		foreach ($this->data_dictionary as $k => $v) {
			// print '<div>field</div>'; print_r($v); print '<hr>';
			if (substr($k, 0, strlen($prefix)) == $prefix ) { 
				// print '<div>field</div>'; print_r($v); print '<hr>';
				if (isset($v['field_key']) && $v['field_key']) {
					$subset[] = $v['field_key'];
				}
			}
		}
		return $subset;
	}
	
	/**
	 * Usually for returning foreign/linked fields for a particular table
	 *
	 * Eg: keys beginning with 'ModuleName__table'
	 * @param string $prefix Look for field_keys beginning ith this prefix. Optional, default '' (ie all field_keys)
	 * @return array Subset of fields - those with keys beginning with $prefix
	 * @author Phil Calder
	 */
	function getCustomFields() {
		$subset = array();
		foreach ($this->data_dictionary as $k => $v) {
			// print '<div>field</div>'; print_r($v); print '<hr>';
			if (isset($v['field_is_custom']) && $v['field_is_custom']) { 
				$subset[] = $v['field_key'];
			}
		}
		return $subset;
	}
	
	/**
	 * Not used
	 *
	 * @param string $text Look for field_keys containing this text. Optional, default '' (ie all field_keys)
	 * @return array Subset of fields - those with keys with $text in them
	 * @author Phil Calder
	 */
	function getFieldsContaining($text = '') {
		$subset = array();
		if ($text) {
			foreach ($this->data_dictionary as $k => $v) {
				// print_r($v);
				if (isset($v['field_key']) && $v['field_key'] && (strpos($v['field_key'], $text) !== false) ) { 
					$subset[] = $v['field_key'];
					// print '<div>YES ' . $v['field_key'] . '</div>';
				} // else {
					// print '<div>NOT ' . $v['field_key'] . '</div>';
				// }
			}
		}
		return $subset;
	}
	
	/**
	 * @author Phil Calder
	 */
	function getPermissionsFields() {
		$all_permissions_fields = array(
			'lock_permissions', 
			'view_group_id',  
			'edit_group_id',  
			'assignedto_group_id', 'assigned_to_group_id',   
			'owner_group_id',  
			'admin_group_id'
		);
		
		$subset = array();
		foreach ($this->data_dictionary as $k => $v) {
			if (in_array($k, $all_permissions_fields)) { 
				$subset[] = $v['field_key'];
			}
		}
		// print '<div>permissions fields</div>'; print_r($subset); print '<hr>';
		return $subset;
	}
	
	/**
	 * @author Phil Calder
	 */
	function getMetaFields() {
		// Primarily used in WebMATEdit and Contact View
		$all_meta_fields = array(
			'created', 
			'creator_id', 
			'last_update',  
			'last_action', 
			'last_reach'
		);
		$subset = array();
		foreach ($this->data_dictionary as $k => $v) {
			if (in_array($k, $all_meta_fields)) { 
				$subset[] = $v['field_key'];
			}
		}
		// print '<div>meta fields</div>'; print_r($subset); print '<hr>';
		return $subset;
	}
	
	function getUserFields() {
		$all_user_types = array(
			'__USERS__', 
			'__USERSALL__'
		);
		$subset = array();
		foreach ($this->data_dictionary as $k => $v) {
			// print_r($v); print '<hr>';
			if ($v['dropdown_values'] && in_array($v['dropdown_values'], $all_user_types)) { 
				$subset[] = $v['field_key'];
			}
		}
		// print '<div>user fields</div>'; print_r($subset); print '<hr>';
		return $subset;
	}
	
	/**
	 * @return array Foreign (and linked) field keys
	 * @author Phil Calder
	 */
	function getForeignFields() {
		$foreign_fields = array();
		foreach ($this->data_dictionary as $k => $v) {
			if ($v['foreign'] || ($v['type'] == 'foreign') || $v['linked']) { // Phil 20050926 adding linked fields
				$foreign_fields[] = $v['field_key'];
			}
		}
		return ($this->foreign_fields = $foreign_fields);
	}
	
	/**
	 * @param string $field The field to test
	 * @return boolean Whether the field is a foreign field
	 * @author Phil Calder
	 */
	function isForeignField($field_name) {
		// print '<div>In dd, checking ' . $field_name . '</div>';
		if (!is_array($this->foreign_fields)) {
			$this->foreign_fields = $this->getForeignFields();
		}
		if (is_array($this->foreign_fields)) {
			// print '<div>is array</div>'; print_r($this->foreign_fields); print '<hr>';
			if (in_array(trim($field_name, '_'), $this->foreign_fields)) {
				return true;
			}
		}
		return false;
	}
	
	/**
	 * Linked fields are a subset of foreign fields
	 *
	 * There is no primary record restriction for linked fields. As such, there is no default 
	 * mechanism for determining a particular linked item for a contact. 
	 * Usually used to add a new item to the given contact (eg new contact form, contact webforms)
	 * @return array of linked field keys
	 * @author Phil Calder
	 */
	function getLinkedFields() {
		$linked_fields = array();
		foreach ($this->data_dictionary as $k => $v) {
			if (isset($v['linked']) && $v['linked']) { 
				$linked_fields[] = $v['field_key'];
			}
		}
		return $linked_fields;
	}
	
	/*
	 * Custom fields (WIP)
	 *
	 * @return array of custom field keys
	 * @author Phil Calder
	 */
	// function getCustomFields() {
		// $linked_fields = array();
		// foreach ($this->data_dictionary as $k => $v) {
			// if ( isset($v['field_is_custom']) && $v['field_is_custom'] ) { 
				// $linked_fields[] = $v['field_key'];
			// }
		// }
		// return $linked_fields;
	// }
	
	/**
	 * disables the read only fields
	 * 
	 * @param boolean $incl_foreign By default, foreign fields are excluded as well
	 */
	function setNoReadOnly($incl_foreign = true) {
		// print '<div>setNoReadOnly</div>'; print '<hr>';
		foreach ($this->data_dictionary as $k => $v) {
			if ($v['foreign'] || ($v['type'] == 'foreign')) {
				// print '<div>setNoReadOnly</div>'; print_r($v); print '<hr>';
				// $this->changeAttributes(
					// array(
						// 'field_key' => $v['field_key'], 
						// 'type' => 'free'
					// )
				// );
				if (!$incl_foreign) { // Phil 20050631 If including foreign fields, make sure the key is prefixed with '_'
					// print '<div>k=' . $k . '</div>'; print_r($this->data_dictionary[$k]); print '<hr>';
					if (substr($this->data_dictionary[$k]['field_key'], 0, 1) != '_') {
						$this->data_dictionary[$k]['field_key'] = '_' . $this->data_dictionary[$k]['field_key'];
					}
				} else {
					$this->removed_fields[$k] = $this->data_dictionary[$k];
					bcrm_unset($this->data_dictionary, $k);
				}
			} else if ($v['read_only']) {
				$this->removed_fields[$k] = $this->data_dictionary[$k];
				bcrm_unset($this->data_dictionary, $k, 'read_only');
			}
		}
	}
	
	/**
	* Changes the attributes of a field or several fields in the data definition.
	*
	* @param array $overrides
	*/
	function changeAttributes($overrides) {
		if (is_array($overrides)) {
			switch ($this->rankOfArray($overrides)) {
				case 1:
					$this->updateField($overrides);
				break;
				
				case 2:
					foreach($overrides as $k => $v) {
						$this->updateField($v);
					}
				break;
				
				default:
					//do nothing
				break;
			}
		}
	}
	
	/**
	* returns the depth of an array.
	*
	* @param array $arr the array you want the depth of
	* @param integer $level the current level of the array is used in for each
	* @return integer the depth of the array
	*/ 
	function rankOfArray($arr, $level=0) {
		static $maxLevel = 0;
		if (is_array($arr)) {
			if ($maxLevel < ++$level) {
				$maxLevel = $level;
			}
			foreach($arr as $k => $v) {
				//NLH - 20041203 - Bug Fix, was counting dropdown_values as an extra depth in the array.
				//NLH - 20041206 - Added rank_array_exclusions as a class array.
				//NLH - 20050119 - Bug Fix, multiple depth arrays without array_exclusions existant fail, loop syntax corrected.
				foreach ($this->rank_array_exclusions as $kk => $vv){
					if (array_key_exists($vv, $arr)){
						return($maxLevel);
					}
				}
				$this->rankOfArray($v, $level);
			}
			$level--;
		}
		if ($level == 0) {
			return($maxLevel);
		}
	}
	
	/**
	* replace attributes for a field with the ones from the array.
	*
	* @param array $field_update the array of new values
	*/ 
	function updateField($field_update) {
		foreach($this->data_dictionary as $k => $v) {
			if ($v['field_key'] == $field_update['field_key']) {
				foreach($field_update as $kk => $vv) {
					$this->data_dictionary[$k][$kk] = $vv;
				}
			}
		}
	}
	
	/**
	 * Get a feild definition
	 *
	 * Will return fields in the current data dictionary OR those removed from the current data dictionary
	 * use {@see fieldExists} to see if a field is in the current data dictionary
	 * 
	 * @param sting $field the field key you are searching for.
	 * @return mixed boolean false or the field array
	 * @author Phil Calder
	 */
	function getField($field) {
		// print '<div>[field=' . $field . '], getFields - included fields</div>';
		foreach ($this->data_dictionary as $k => $v) {
			// print '<div>k=' . $k . ', field_key= ' . $v['field_key'] . '</div>';
			if ( ($field == $v['field_key']) || ( (substr($field, 0, 1) == '_') && (substr($field,1) == $v['field_key']) ) ) {
				return $v;
			}
		}
		// print '<div>getFields - removed fields</div>';
		foreach ($this->removed_fields as $k => $v) {
			// print '<div>k=' . $k . ', field_key= ' . $v['field_key'] . '</div>';
			if ( ($field == $v['field_key']) || ( (substr($field, 0, 1) == '_') && (substr($field,1) == $v['field_key']) ) ) {
				return $v;
			}
		}
		return false;
	}
	
	/**
	 * Check if a field is in the data definition
	 * 
	 * @param sting $field the field you are searching for.
	 * @return boolean TRUE if the field is in that data definition
	 */
	 function fieldExists($field) {
		 if (!$this->fields) {
			 $this->fields = array();
			 foreach ($this->data_dictionary as $k => $v) {
				 $this->fields[] = $v['field_key'];
			 }
		 }
		return in_array($field, $this->fields);
	 }
}
?>