<?php
/**
 * Handles the dropdown functionality
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 */

includeWebMAT('webmat.inc');

/**
 * this class handles the dropdown list functionality
 *
 * @author Matthew Corich, Phil Calder
 * @abstract
 */
class WebMATStandardDropdown extends WebMAT{
	/**
	 * @var string the table used to store the dropdown data
	 */
	var $table_name;
	/**
	 * @var object the module class the dropdown in in
	 */
	var $mod;
	/**
	 * @var string the field name that holds the dropdown detail
	 */
	var $field_name;
	/**
	 * @var array the dropdown values
	 */
	var $dropdown_values = array();
	/**
	 * @var array containing the database colums
	 */
	var $columns = array();
	/**
	 * @var array the where array 
	 * @see WebMATDatabaseTable->setDesignTimeFilters for array paramaters
	 */
	var $where = array();
	
	/**
	 * Constructor function sets all the class variables
	 *
	 * Prefixes the given table_name if necessary (incase the name 'item' was supplied for the 'Module__item' table)
	 * @param  string  $table_name the table used to store the dropdown data
	 * @param  string  $module_name the name of the module the dropdown is in
	 */
	function WebMATStandardDropdown($module_name, $table_name, $field_name = false, $where = false) {
		global $SERVICES;
		$this->mod =& $SERVICES->modules->getModule($module_name);
		// $this->db = $SERVICES->getDatabase($module_name);
		$this->db =& $SERVICES->getService('Database');
		// $this->table_name = $table_name;
		// TODO: Check that table_name is not already prefixed - done
		$this->table_name = $module_name . '__' . str_replace($module_name . '__', '', $table_name); // $module_name . '__' . $table_name;
		if ($field_name) {
			$this->field_name = $field_name;
		} else {
			$this->field_name = 'name';
		}
		$this->module_name = $module_name; // Phil 20051009 for getFilesetConfigPath
		$this->columns =  $this->db->getColumns($this->table_name);
		if ($where) {
			$this->setWhereStatement($where);
		}
	}
	
	/**
	 * sets the where statement array
	 * 
	 * @param array $where the where statement array 
	 * @see WebMATDatabaseTable::setDesignTimeFilters for array parameters
	 */
	function setWhereStatement($where = array()) {
		if (in_array('active', $this->columns)) {
			// $where[] = 'active = 1';// Phil, 20041123, old code replaced
			$where['active'] = 1;
		}
		$display_by_group_where = $this->getDisplayByGroupWhere();
		
		if (is_array($where) && is_array($display_by_group_where)) {
			$where = $this->arrayMergeReplace($where, $display_by_group_where);
		}
		// print '<div>merged: [' . $this->module_name . ' -  ' . $this->getTableName() . ']</div>'; print_r($where); print '<hr>';
		$this->where = $where;
	}
	
	function getDisplayByGroupWhere() {
		$where = $permissions = array();
		// Phil 20051003 WIP
		if ($this->getDisplayByGroupFlag() > 0) {
			global $USER;
			if (in_array('view_group_id', $this->columns) ) {
				$permissions['view_group_id'] = array( 'where' => '(view_group_id IS NULL OR view_group_id IN (' . $USER['group_string'] . '))');
			}
			if (in_array('edit_group_id', $this->columns) ) {
				$permissions['edit_group_id'] = array( 'where' => '(edit_group_id IS NULL OR edit_group_id IN (' . $USER['group_string'] . '))');
			}
			if (in_array('assigned_to_group_id', $this->columns) ) {
				$permissions['assigned_to_group_id'] = array( 'where' => '(assigned_to_group_id IS NULL OR assigned_to_group_id IN (' . $USER['group_string'] . '))');
			}
			if (in_array('owner_group_id', $this->columns)) {
				$permissions['owner_group_id'] = array( 'where' => '(owner_group_id IS NULL OR owner_group_id IN (' . $USER['group_string'] . '))');
			}
			if (in_array('admin_group_id', $this->columns)) {
				$permissions['admin_group_id'] = array( 'where' => '(admin_group_id IS NULL OR admin_group_id IN (' . $USER['group_string'] . '))');
			}
			$group_permissions = '';
			if (is_array($permissions) && sizeof($permissions)) {
				foreach ($permissions as $k => $v) {
					if (isset($v['where']) && $v['where']) {
						$group_permissions .= ' ' . $v['where'] . ' OR';
					}
				}
				$group_permissions = trim($group_permissions, " OR ");
				if (strlen($group_permissions)) {
					// print '<div>strlen true</div>';
					$group_permissions = '(' . $group_permissions . ')';
				}
				$where['view_group_id'] = array('where' => $group_permissions);
			}
		}
		return $where;
	}
	
	/**
	 * gets the where statement
	 * 
	 * @return string the where statement
	 */
	function getWhereStatement() {
		$where_array = array();
		if (!$this->where) {
			$this->setWhereStatement();
		}
		foreach ($this->where as $k => $v) {
			if ( in_array($k, $this->columns) ) {
				if ( is_array($v) ) {
					$where_array[] = $v['where'];
				} else {
					$where_array[] = $k . "='" . $v . "'";
				}
			}
		}
		return implode(' AND ', $where_array);
	}
	
	/**
	 * @author Phil Calder
	 * 20050519
	 */
	function processLanguageText($label) {
		$label = stripslashes($label);			
		if (substr($label, 0, 2) == '__') {
			global $L, $SERVICES;
			$SERVICES->getService('Language');
			$label = $SERVICES->language->getLanguageText(substr($label, 2));
		}
		return $label;
	}
	
	/**
	 * Updates the values from the database
	 * 
	 * @param boolean $multi_level true if multilevel else false
	 * @param integer $current_id the current id used to stop endless loops
	 * @return array of dropdown values (key is the id, value is the name)
	 * 
	 */
	function updateValues($multi_level = false, $current_id = false) {		
		$order_by = '';
		$values = array();
		if (in_array('priority', $this->columns)) {
			$order_by .= 'priority, ';
		}
		$where = $this->getWhereStatement();
		$order_by .= $this->field_name;
		$result = $this->db->select('*', $this->table_name, $where, $order_by);
		
		if ($this->db->hasError()) {
			$this->db->printError();
		}
		if (in_array('parent_id', $this->columns) ){
			while ($row = $this->db->getRow($result)) {
				$values[$row['id']] = $row;
			}
			
			if ($multi_level) {
				$multi_level_values = $this->getMultiLevelValues($values, $current_id);
				$values = array(); // reset the values array
				foreach ($multi_level_values as $k => $v) {
					// $name = str_repeat($multi_level, $v['indent']) . $v[$this->field_name];
					$name = str_repeat($multi_level, $v['indent']) . $this->processLanguageText($v[$this->field_name]);
					$values[$k] = $name;
				}
			} else {
				if (is_array($values)) {
					$multi_level_values = $this->getMultiLevelValues($values, $current_id, true);
					$values = array(); // reset the values array
					foreach ($multi_level_values as $k => $v) {
						if ($v['indent']) {
							// $name = ltrim($v['indent'] . ' > ' . $v[$this->field_name], ' > ');
							$name = ltrim($v['indent'] . ' > ' . $this->processLanguageText($v[$this->field_name]), ' > ');
						} else {
							$name = $this->processLanguageText($v[$this->field_name]); // $v[$this->field_name];
						}
						$values[$k] = $name;
					}
				}
			}
		} else {
			while ($row = $this->db->getRow($result)) {
				$values[$row['id']] = $this->processLanguageText($row[$this->field_name]); // $row[$this->field_name];
			}
		}
		$this->dropdown_values = $values;
	}
	
	/**
	 * Gets the items for a multilevel dropdown
	 * 
	 * @param array $values the current values for the array
	 * @param integer $current_id the current item id.
	 * @return array the items in an indented array
	 */
	function getMultiLevelValues($values, $current_id = false, $add_name = false){
		$this->parent_id = 0; //used for the call back function
		$this->temp_items = $values;
		if (!$values) {
			$values = array();
		}
		$top_level =  array_filter($values , array($this, 'getForParentId'));
		$indented_items = array();
		foreach ($top_level as $k => $v) {
			if ($current_id != $v['id']) {
				if ($add_name) {
					$indent = '';
				} else {
					$indent = 0;
				}
				$indented_items[$k] = array(
					$this->field_name => $v[$this->field_name], 
					'indent' => $indent
				);
				$indented_items = $this->getChildren($indented_items, $indent, $v['id'], $current_id, $add_name);
			}
		}
		return $indented_items;
	}
	
	/**
	 * Gets all the child items for an item
	 * 
	 * @param array $indented_items the current items array
	 * @param string $indent 
	 * @param integer $parent_id
	 * @param integer $current_id
	 * @return array $indented_items
	 */
	function getChildren($indented_items, $indent , $parent_id, $current_id = false, $add_name = false) {
		$this->parent_id = $parent_id;
		if ($add_name) {
			$indent = $indent . ' > ' . $this->temp_items[$this->parent_id][$this->field_name];
		} else {
			$indent ++;
		}
		$items = array_filter($this->temp_items , array($this, 'getForParentId') );
		
		if (count($items)) {
			foreach ($items as $k => $v) {
				if ( (!isset($indented_items[$k])) && ($current_id != $v['id']) ) {
					
					$indented_items[$k] = array(
						$this->field_name => $v[$this->field_name], 
						'indent' => $indent
					);
					
					$indented_items = $this->getChildren($indented_items, $indent, $v['id'], $current_id, $add_name);
				
				} else {
					return $indented_items;
				}
			}
		}
		return $indented_items;
	}
	
	/**
	 * returns the parent names for an item as a string seprated by " > "
	 * 
	 * @param array $values
	 * @param integer $id
	 * @return string
	 */
	function getParentNames($values, $id) {
		$current_id = $id;
		$name = $values[$id]['name'];
		
		while ($current_id != 0) {
			$current_id = $values[$current_id]['parent_id'];
			$name = $values[$current_id]['name'] . ' > ' . $name;
		}
		
		return ltrim($name, ' > ');
	}
	
	function getForParentId($val) {
		if ($val['parent_id'] == $this->parent_id) {
			return true;
		}
	}
	
	/**
	 * Returns items from the database. Call setWhereStatement before this function to filter.
	 * @return array of dropdown values (key is the id, value is an array of fields)
	 * 
	 */
	function getItems($extra_where = '') {
		$order_by = '';
		$values = array();
		
		if (in_array('priority', $this->columns)) {
			$order_by .= 'priority, ';
		}
		
		$where = $this->getWhereStatement();
		
		$where .= ( (strlen($where) && strlen($extra_where)) ? ' AND ' : '') . $extra_where; 
		$order_by .= $this->field_name;
		
		$result = $this->db->select('*', $this->table_name, $where, $order_by);
		
		if ($this->db->hasError()) {
			$this->db->printError();
		}
		
		while ($row = $this->db->getRow($result)) {
			$values[$row['id']] = $row;
		}	
		return $values;
	}
	
	/**
	 * Gets a list of dropdown values
	 *
	 * @return array the list of dropdown values
	 */
	function getValues($multi_level = false, $current_id = false) {
		if ( (count($this->dropdown_values) == 0) ) {
			$this->updateValues($multi_level, $current_id);
		}
		return $this->dropdown_values;
	}
	
	/**
	 * Gets a list of dropdown values
	 *
	 * @return array the list of dropdown values
	 * @author Phil Calder
	 * @access private
	 */
	function getValuesArray($order_by_str = 'priority ASC') {
		$where_str = $this->getWhereStatement();
		// print '<div>where: ' . $where_str . '</div>'; print '<hr>';
		$dropdown_array = $this->db->quickGetAllRows('*', $this->table_name, $where_str, $order_by_str);
		return $dropdown_array;
	}
	
	/**
	 * Gets a single dropdown value
	 *
	 * @param integer $id the dropdown id
	 * @return array the dropdown value, or false if not found.
	 */
	function getValue($id) {
		$dropdown = $this->db->quickGet('*', $this->table_name, "id='$id'");
		if ($this->db->hasError())	$this->db->printError();
		return $dropdown;
	}
	
	/**
	 * Updates or creates a dropdown value in the database
	 *
	 * @param array $item the dropdown item
	 * @return integer the id of the dropdown value updated/created, or false if there was an error
	 */
	function setValue($item) {
		// The item exists. Lets get it, and update the values.
		if (isset($item['id']) && $item['id']) {
			$id = $item['id'];
			$temp = $this->db->quickGet('*', $this->table_name, "id='$id'");
		}
		
		if (isset($temp) && is_array($temp)) { // update
			$this->db->update($this->table_name, $item, 'id=' . $id);
			if ($this->db->hasError()) {
				$this->db->printError();
				return false;
			} else {
				$value = (isset($item['name']) ? $item['name'] : $value);
				$this->updateLogs($id, $value, 'updated');
				$this->updateValues();
				return ($id);
			}
			
		} else { // insert
			$this->db->insert($this->table_name, $item); 
			if ($this->db->hasError()) {
				$this->db->printError();
				return false;
			} else {
				
				$insert_id = $this->db->getInsertId();
				$value = (isset($item['name']) ? $item['name'] : $value);
				$this->updateLogs($insert_id, $value, 'created');
				$this->updateValues();
				return $insert_id;
			}
		}
	}
	
	/**
	 * Updates the log files by building a log message and calling the Log service to log the action.
	 *
	 * @param integer $id the dropdown id
	 * @param string $value the dropdown value
	 * @param string $update_type either created or deleted
	 */
	 function updateLogs($id, $value, $update_type,  $include_table_info = true) {
		$msg = 'Dropdown item ' . $update_type . ' id=' . $id . ' value=' . $value;
		
		$module_name = '';
		if (is_object($this->mod)) {
			$module_name = $this->mod->getModuleName();
		}
		
		if ($include_table_info) {
			$msg .= ' [' . str_replace($module_name . '__', '', $this->table_name) . ']';
		}
		 
		global $SERVICES;
		$SERVICES->getService('Log');
		$SERVICES->log->logAction($msg, $module_name);
	 }
	
	/**
	 * Deletes a dropdown value from the database. 
	 *
	 * This is done either by making the item not 'active' (where appropriate), or by 'hard deleting' the item.
	 * Display a database error message, or updates the dropdown values.
	 * @param integer $id the dropdown id
	 */
	function deleteValue($id) {
		if (in_array('active', $this->columns)) {
			$this->db->update($this->table_name, array("active" => "0"), "id='$id'"); 
		} else {
			$this->db->delete($this->table_name, "id='$id'");
		}
		
		if ($this->db->hasError()) {
			$this->db->printError();
		} else {
			$this->updateLogs($id, $value, 'deleted');
			$this->updateValues();
		}
	}
	
	/**
	 * returns the display by group flag for the current file set
	 * @return integer Display by group flag (0,1,2)
	 * @author Phil Calder 20051009
	 * @see WebMATDatabaseTable::getDisplayByGroupFlag
	 */
	function getDisplayByGroupFlag() {
		// Note: could return 0 here to always disable group permission filtering
		global $SERVICES;
		if (!isset($this->display_by_group_flag)) {
			$this->display_by_group_flag = $SERVICES->config->lookup($this->getFilesetConfigPath() . 'display_by_group');
		}	
		// print '<div>flag: ' . $this->display_by_group_flag . '</div>';
		return $this->display_by_group_flag;
	}
	
	/**
	 * returns the fileset config name
	 * 
	 * @return string the Path for the current fileset
	 * @author Phil Calder 20051009
	 * @see WebMATDatabaseTable::getFilesetConfigPath
	 */
	function getFilesetConfigPath() {
		if (!isset($this->fileset_config_path) || !$this->fileset_config_path) {
			$this->fileset_config_path = 'modules.' .  $this->module_name . '.file_set.' . 
				str_replace($this->module_name . '__', '', $this->getTableName()) . '.';
		}
		// print '<div>config path: ' . $this->fileset_config_path . '</div>';
		return $this->fileset_config_path;
	}
}
?>