<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 */

includeWebMAT('display.inc');
includeWebMAT('data_dictionary.inc');

/**
 * This abstract class serves as a base for edit and view classes 
 * to handle the display of an item.
 *
 * @package WebMAT
 * @author Phil Calder
 * @abstract
 */
class WebMATStandardItem extends WebMATDisplay {
	/**
	 * @var array $item the item array
	 * @access private
	 */
	var $item = array();
	
	/**
	 * @access private
	 */
	var $restrict_level = 'read';
	
	/**
	 * @var boolean $container_not_table switch to set style of output from getTable()
	 * @access public 
	 */
	var $container_not_table = false;

	/**
	 * @var string the switch case for function getBeginMainContent()
	 * @access public
	 */
	var $begin_main_content_case = 'item';


	/**
	 * @var 
	 * @access public 
	 */
	var $xx_container_not_table = false;
	
	/**
	 * init function sets all the class variables
	 *
	 * @param  string $module  the current module name
	 * @param  string $data_option  the current data option used for DataDictionary
	 * @param  array $post the post array
	 * @param  array $get the get array
	 * @param  string $table_name the name of the table the page is using
	 * @param  array $language_overwrite has the keys title and description
	 */
	function init($module, $data_option, $post = false, $get = false, $table_name = false, $language_overwrite = false) {
		parent::init($module, $data_option, $post, $get, $table_name, $language_overwrite);
		$this->setItemVariable();
		$this->substituteTagContents();
		$this->setPriorityEdit();
		$this->afterDataDictionary(); // Phil 20060531 Used to be only in WebMATList::main
	}
	
	/** 
	 * Method placeholder
	 * 
	 * Can be implemented by subclasses with individual Output method calls.
	 * @see Output::substituteTagContent
	 *
	 * @author Phil Calder
	 */
	function substituteTagContents() {
		// $this->output->substituteTagContent('outerTable', 'outerTableEdit');
	}

	/**
	 * sets the item class variable
	 */
	function setItemVariable() {
		if ($this->form_values['item']) {
			$this->item = $this->form_values['item'];
		} else {
			$this->delete = false;
		}
	}

	/**
	 * returns the item class variable
	 *
	 * @return array a normal item array
	 */
	function getItemVariable() {
		return $this->item;
	}

	/**
	 * sets the return page
	 */
	function run() {
		$this->setRefererAsReturnPage();
	}

	/**
	 * Sets the log action
	 *
	 * @author Phil Calder
	 */
	function setLogAction() {
		$this->db_functions->setLogAction('view');
	}

	/**
	 * Builds the item page
	 *
	 * @param  integer  $id  an item id
	 * @param  array  $item  the item array from a form that failed validation
	 * @param  array  $error_msg  an array of error messages.
	 * @author Phil Calder
	 */
	function main($id = '', $item = '', $error_msg = '') {
		// print '<div>WebMATItem main id: ' . $id . ' form_values[id]: ' . $this->form_values['id'] . '</div>';
		$this->checkForBrowse(); // Phil 20060926
		// Phil 20060809 Re-implemented template option
		// TODO: handle second menu, action links
		if ($this->output->getHasTemplate()) {
			if ($this->getWebMATType() == 'view') {
				$title_key = 'titleRow';
				$desc_key = 'description';
				$menu_key = 'newMenu';
				$form_key = 'table';
			} else {
				$title_key = 'titleRow';
				$desc_key = 'description';
				$menu_key = 'menu';
				$form_key = 'form';
			}
			
			$content = array();
			$content = array_merge(
				array($title_key => $this->getTitleContent()),
				array($desc_key => $this->getDescriptionContent()),
				array($menu_key => $this->getActionLinksContent()), 
				array($form_key => $this->getDataContent($item, $error_msg))
			);
			
		} else {
			
			$this->item = $this->getItem($this->form_values['id']);
			
			// Build a content array
			$content = array();
			// LastViewed
//MCR 20070314
//idea is to move to a side panel working with AJAX 
//			$content['lastViewed']          = $this->getPastLinksTable();
			// ---------------
			
			
			$content['beginMain']           = $this->getBeginMainContent(true);
			// Title (and links)
			$content['title']               = $this->getTitleContent();
			// Description (optional)
			$content['description']         = $this->getDescriptionContent();
			// Browse / paging
			$content['browsePaging_top']    = $this->getBrowsePagingContent();
			// ActionLinks
			$content['actionLinks']         = $this->getActionLinksContent();
			// ----
			// Main Content
			// $content['tmp2']				= '<table width="100%" border="0" cellpadding="0" cellspacing="0" id="viewTbl">';
			$content['endMain']             = $this->getEndMainContent();

			//$content['beginDataCell']				= $this->getBeginDataCell();
			
			
			$content['beginContainer']      = $this->getBeginContainer();
			$content['data']                = $this->getDataContent($item, $error_msg);
			$content['endContainer']      = $this->getEndContainer();
			
			
			// $content['tmp3']				= '</table>';
			//			$content['endDataCell']					= $this->getEndDataCell();
						
						// ----
						// Browse / paging
			//MCR 20070427
			//try with just paging in toolbar 
			//			$content['browsePaging_bottom'] = $content['browsePaging_top']; // $this->getBrowsePagingContent();
			// ---------------

			
		}
		return $this->display($content);
		
	}
	
	/**
	 * An edit page returns an edit form, view page return a view table.
	 *
	 * @param array $item An item array (optional for view pages)
	 * @param array $error_msg error message (optional for view pages)
	 * @return string Content
	 * @author Phil Calder
	 */
	function getDataContent($item, $error_msg) {
		// Phil 20060105 An edit page would return getForm
		return $this->getTable(false);
	}
	
	/**
	 * builds a nested table displaying all the content for the page
	 *
	 * @return string returns the main tabular content of the page
	 */
	function getTable($item = false) {
		// print '<div>' . get_class($this) . '::getTable form_values[id]: ' . $this->form_values['id'] . '</div>'; 
		global $SERVICES, $L;

		if (!$item) {
			$item = $this->getItem($this->form_values['id']);
		}
		
		$table_labels = array();
		
		$table_labels = $this->getDataDictionary(); // Phil 20051027 using wrapper function
		foreach ($table_labels as $k => $v) {
			if ($v['calculated'] == '1') {
				$table_labels[$k]['field_key'] = '';
			}
		}

		if ($this->output->getHasTemplate()) {
			$table_content = array();
		} else {
			$table_content = '';
		}
		
		if (is_array($item) && sizeof($item)) {
			//get dropdown list sets
			$ua =& $SERVICES->getService('Authentication');
			$users = $ua->users->getUsersList();
			$groups = $ua->groups->getGroupList();

			$this->markAsSeen($item); //Marks the item as seen when you view it
			//$style_flag = '';
			$this->dd->setDictionaryExcludeHidden(); //Gets rid of the hidden variables
			
			// Phil 20070815 Exclude any fields with hide_empty set
			// Note: this would exclude these fields whether they are empty or not. See getFieldRows.
			// $this->dd->setDictionaryExcludeHideEmpty();
			
			$this->dd->setNoTrim(); //disables trim
			
			// $table_content .= $this->getBeginDataCell();
			if(!$multiCol) {
				$tblType = 'viewTbl';
			} else if($multiCol === true) {
				$tblType = 'contentTbl';
			} else {   
				$tblType = array('tag_name' => 'contentTbl', 'tag_extras' => ' width="' . ceil(100/$multiCol) . '%"'); 
			}
			
			//new 20070531
			//todo move box start 7 end to Output functions
			// maybe do this for all viewTbl ??
			/*if ($this->container_box_not_viewTbl) {
				$table_content .= '
				<table border="0" ><tr><td>
				<div class="x-box-tl"><div class="x-box-tr"><div class="x-box-tc">
				</div></div></div>
				<div class="x-box-ml"><div class="x-box-mr"><div class="x-box-mc">
				';
				$table_content .= $this->output->getBeginTable('zeroTbl');
				$table_content .= $this->getFieldRows($item, $tabs);
				$table_content .= $this->output->getEndTable();
				$table_content .= '
				</div></div></div>
				<div class="x-box-bl"><div class="x-box-br"><div class="x-box-bc">
				</div></div></div>
				</td></tr></table>
				';
				
				
			} else
			*/
			if ($this->output->getHasTemplate()) {
				//TODO
				$table_content = $this->getFieldRows($item, $tabs); // returns an array
			} else {
				// $table_content .= '
				// <div class="x-box-tl"><div class="x-box-tr"><div class="x-box-tc">
				// </div></div></div>
				// <div class="x-box-ml"><div class="x-box-mr"><div class="x-box-mc">
				// ';
				
				if ($this->container_not_table) {
					$table_content .= $this->getFieldRows($item, $tabs);
				} else {
					$table_content .= $this->output->getBeginTable($tblType);
					$table_content .= $this->getFieldRows($item, $tabs);
					$table_content .= $this->output->getEndTable();
				}
				// $table_content .= $this->getEndDataCell();
				
				// $table_content .= '
				// </div></div></div>
				// <div class="x-box-bl"><div class="x-box-br"><div class="x-box-bc">
				// </div></div></div>
				// ';
				
			}

		} else {
			//no items in the table
			$ua =& $SERVICES->getService('Authentication');
			if ($this->output->getHasTemplate()) {
				$table_content['noItems'] = $L[$this->module_name_lower . 'NoItems'];
			} else {
				// $table_content .= $this->getBeginDataCell();
				$table_content .= $this->output->getTextRow($L[$this->module_name_lower . 'NoItems'] , sizeof($table_labels));
				// $table_content .= $this->getEndDataCell();
			}
		}
		
		// Phil - moved multiCol (tabset) code to contact_view.php page
		
		return $table_content;
	}
	
	/**
	 * @author Phil Calder
	 */
	function getBrowsePagingContent() {
		if (is_array($links = $this->getBrowseLinks()) && count($links)) {
			return $this->output->getTextRow(implode(' ', $links), 1, 'itemPagingCell', true);
		}
		return false;
	}
	
	/**
	 * @author Phil Calder
	 */
	function getActionLinksContent() {
		$this->toolbar_link_mode = true;
		
		$content1 = (array)$this->getNewLinks();
		$content2 = (array)$this->getTableLinks($this->item);
		// print '<div>getActionLinksContent --</div>'; print_r($content1); print_r($content2); print '<hr>';
		if (count($content2)) {
			if (isset($content2['list']) && $content2['list']) {
				$content1['list'] = $content2['list'];
				bcrm_unset($content2, 'list');
			}
		}
		
		//20070427
		$this->output->setActionLinksMenu($content1, $content2);
		
		$this->toolbar_link_mode = false;
	}
	
	/**
	 * Adds the extra links from the module.config
	 *
	 * @param boolean $as_array Whether to return each link as a processed href (html a tag) or an array (eg for inclusion in a toolbar). Default is true for WebMATitem.
	 * 
	 * @return array An array of extra config links
	 * @author Phil Calder
	 */
	function getExtraConfigLinks($as_array = true) {
		return parent::getExtraConfigLinks($as_array);
	}
	
	/**
	 * Determines whether the item is viewable by the current user
	 *
	 * @param integer $id the item id
	 * @param mixed item - item array or false
	 * @return boolean true if editable, false if not (note: new items are editable)
	 * @author Phil Calder
	 */
	function isViewable($id, $item = false) {
		if ($id) { // id passed in is for an existing item
			return $this->db_functions->isViewable($id, $item);
		}
		return true; // 20050902
	}
	
	/**
	 * Marks an item as seen
	 *
	 * @param  array  $item  the item array
	 */
	function markAsSeen($item, $extra_values = false) {
		// Phil 20070619 mainly for automated scripts that we dont want to log for
		global $USER;
		if (!isset($USER['no_log']) || !$USER['no_log']) {
			$this->db_functions->setItemSeenStatus($item['id'], 1, true, $extra_values);
		}
	}
	
	/**
	 * Returns the title row
	 * 
	 * @return string the title row
	 */
	function getTitleContent() {
		return $this->output->getTitleRow($this->getTitleText(), $this->getTitleRowLinks());
	}
	
	/**
	 * Gets the title text for the page
	 *
	 * Appends the item display name if applicable
	 *
	 * @param string $prefix what to prefix the file name with (optional)
	 * @return string with the title
	 */
	function getTitleText($prefix = 'Title_') {
		if (($prefix == 'Title_') && !$this->form_values['id']) {
			$prefix = 'TitleNew_';
		}
		$title_text = parent::getTitleText($prefix);
		
		if (is_object($this->db_functions) && method_exists($this->db_functions, 'getDisplayNameField')) {
			$display_name = $this->db_functions->getDisplayNameField();
			if (isset($this->item) && isset($this->item[$display_name]) && $this->item[$display_name]) {
				$title_text = ($title_text ? $title_text . ': ' : '') . $this->item[$display_name];
			}
		}
		return $title_text;
	}
	
	
	/**
	 * Suffix for the description row language text key
	 *
	 * 
	 * @return string the description row content language key suffix
	 */
	function getDescriptionContentSuffix() {
		return 'DisplayDescription';
	}

	/**
	 * Builds content for the page description row
	 *
	 * Description can be 'description' key in the langauge overwrite array,
	 * or 'modulenameDisplayDescription' language key
	 * @return string the description row content
	 */
	function getDescriptionContent() {
		global $L, $SERVICES;
		$content = '';
		if ($this->language_overwrite['description']) {
			$description = $SERVICES->language->getLanguageText($this->language_overwrite['description'], $this->module_name);
		} else {
			$description = $L[$this->module_name_lower . $this->getDescriptionContentSuffix()];
		}
		if ($description) {
			// $content = $this->output->getTextRow($description);
			// $content .= $this->output->getSpacerRow();
			// Phil 20060223
			$padding = true;
			$content = $this->output->getBeginRow('', true) . '<td id="bodyCell"' . ($padding ? 'style="padding: 10px"' : '') . '>';
			$content .= $description;
			$content .= '</td>' . $this->output->getEndRow(true);
		}
		return $content;
	}

	/**
	 * Menu row wrapper function
	 *
	 * Calls output method getMenuRow
	 *
	 * @param string $menu The menu
	 * @param integer $colspan he colspan (Optional, see Output)
	 * @param string $tag (Optional, see Output)
	 * @return string the menu row
	 * @author Phil Calder
	 */
	function getMenuRow($menu, $colspan = 2) {
		// Phil 20060112 Removed default param , $tag = 'lMainMenuCell'
		if ( is_object($this->output) && method_exists($this->output, 'getMenuRow') ) {
			return $this->output->getMenuRow($menu, $colspan);
		}
		return '';
	}
	
	/**
	 * returns the second menu row
	 *
	 * Based on second menu links
	 * @see addSecondMenuLinks()
	 *
	 * @return string the second menu row
	 * @author Phil Calder
	 */
	function getSecondMenuRow() {
		if (count($this->second_menu_links)) {
			return $this->output->getMenuRow( implode($this->getMenuSeparator() , $this->second_menu_links) );
		}
	}

	/**
	 * adds links below the menu row
	 *
	 * @param array an array with the link info 
	 * (with the keys 'href', 'text' , 'title' and 'tag_extras')
	 */
	function addSecondMenuLinks($links) {
		if (!$this->second_menu_links) {
			$this->second_menu_links = array();
		}
		
		if (isset($links['href'])) {
			//$this->second_menu_links[] = $this->output->getLinkHref($links);
			$this->second_menu_links[] = $links;
		} else {
			if (is_array($links)) {
				foreach ($links as $k => $v) {
					//$this->second_menu_links[] = $this->output->getLinkHref($v);
					$this->second_menu_links[] = $v;
				}
			}
		}
	}
	
	/**
	 * Checks if We can add Browse Navigation Links
	 * @author Phil Calder
	 * @return boolean
	 * @see WebMATDatabaseTable::isUsingBrowseQuery
	 */
	function isBrowseable() {
		global $SERVICES, $USER;
		if (is_object($this->db_functions) && !$this->db_functions->isUsingBrowseQuery()) { // Phil 20051104
			// Checked webmat config setting
			return false;
		}
		if ($SERVICES->config->lookup($this->db_functions->getFileSetConfigPath().'not_browseable') == 1) {
			// Checked modules config setting
			return false;
		}
		if (empty($USER['browse_query']) || !is_array($USER['browse_query'])) {
			// print 'no browse - USER[browse_query] empty<hr>';
			return false;
		}
		return true;// Phil 20060927 getBrowseQuery will determine whether we can browse
	}
	
	/**
	 * @author Phil Calder
	 */
	function checkForBrowse() {
		if ($this->form_values['id']) {
			if ($this->form_values['browse']) {
				$query = $this->getBrowseQuery();
				// print_r($query); print '<hr>';
				// print '<div>browse query data option: ' . $query['data_option'] . '</div>';
				$new_id = $new_pos = $current_pos = false;
				switch ($this->form_values['browse']) {
				case 'back1':
					$offset = -1;
					break;
				case 'back10':
					$offset = -10;
					break;
				case 'fwd1':
					$offset = 1;
					break;
				case 'fwd10':
					$offset = 10;
					break;
				case 'first':
					$offset = 0;
					$new_pos = 0;
					// $new_id = $this->form_values['id'];
					break;
				case 'last':
					$offset = 0;
					// print '<div>count: ' . $query['count'] . '</div>'; exit();
					$new_pos = (($query['count'] > 0) ? ($query['count'] - 1) : 0);
					// $new_id = $this->form_values['id'];
					break;
				default:
					$offset = 0; // No change
					break;
				}
				if (is_array($query['cache'])) {
					foreach ($query['cache'] as $k => $v) {
						// print '<div>k=' . $k . ', v=' . $v . '</div>';
						if ($v == $this->form_values['id']) {
							$current_pos = $k;
							$new_pos = $current_pos + ($offset);
							$new_pos = ( ($new_pos >= 0) ? $new_pos : 0);
							
							// $new_pos = $k + intVal($this->form_values['browse_next']) - intVal($this->form_values['browse_previous']);
							if (isset($query['cache'][$new_pos]) && $query['cache'][$new_pos]) {
								$new_id = $query['cache'][$new_pos];
								// print '<div>*** new_id: ' . $new_id . '</div>';
							}
							break;
						}
					}
				}
				//--
				if (!$new_id) { // We know the position but need to get the id
					// print '<div>new pos: ' . $new_pos . ', offset: ' . $offset . ' [new id not in cache]</div>';
					$query['limit'] = 'LIMIT ' . $new_pos . ', 20'; // TODO
					// print '<div>calling getItems with new query</div>'; print_r($query); print '<hr>';
					$this->db_functions->setDataOption($query['data_option']);
					// print '<div>calling getItems with new query limit: ' . $query['limit'] . '</div>';
					$new_items = $this->db_functions->getItems($query['order_by'], $query['limit'], $query['where']);
					// print '<div>new items</div>'; print_r($new_items); print '<hr>';
					$query = $this->getBrowseQuery();
					// print '<div>query</div>'; print_r($query); print '<hr>';
					if (is_array($new_items) && ($new_item = current($new_items))) {
						// print_r(array_keys($new_items)); print '<hr>';
						if ($new_item['id']) {
							$new_id = $new_item['id'];
						}
					}
					// exit();
				}
				//--
				// print '<div>checkForBrowse<br>id: ' . $this->form_values['id'] . '<br>current pos: ' . $k . '<br>browse: ' . $this->form_values['browse'] . '<br>new_pos: ' . $new_pos . '<br>new id: ' . $new_id . '</div>';
				// exit();
				// if (!$new_id) {
					// $new_id = $this->form_values['id']; // last viewed id
				// }
				
				if (!$new_id) {
					// print '<div>no new id new_pos: ' . $new_pos . ', current_pos: ' . $current_pos . '</div>'; exit();
					if ($new_pos <= $current_pos) {
						if (isset($query['first']) && $query['first']) {
							$new_id = $query['first'];
						}
					} else if (isset($query['last']) && $query['last']) {
						$new_id = $query['last'];
					}
				}
				
				if ($new_id) {
					if ($new_id != $this->form_values['id']) {
						global $SERVICES;
						$href = $SERVICES->getGlobal('php_self') . '?id=' . $new_id;
						// print '<div><a href="' . $href . '">' . $href . '</a></div>'; exit();
						header('Location: ' . $href);
						exit();
					} else {
						return;
					}
				} 
				
				$this->redirectToList();
			}
		}
	}
	
	/**
	 * @author Phil Calder
	 */
	function getBrowseQuery() {
		$query = $this->db_functions->getBrowseQuery(
			$this->getModuleNameLower(), 
			$this->getTableName(), 
			$this->getBrowseQueryDataOption()
		);
		// print_r($query); print '<hr>';
		return $query;
	}
	
	/**
	 * @author Phil Calder
	 */
	function updateBrowseQuery($values) {
		if (is_array($values)) {
			$this->db_functions->updateBrowseQuery(
				$this->getModuleNameLower(), 
				$this->getTableName(), 
				$this->getBrowseQueryDataOption(), 
				$values
			);
		}
	}
	
	/**
	 * @author Phil Calder
	 */
	function getBrowseLinks() {
		global $L, $SERVICES;
		// print '<div>getBrowseLinks</div>';
		if ($this->isBrowseable() && $this->form_values['id']) {
			$links = array();
			$count = $current_pos = false;
			$query = $this->getBrowseQuery();
			// print '<div>query:</div>'; print_r($query); print '<hr>';
			if (is_array($query) && !empty($query)) {
				// 20060926
				$links = array();
				
				if (
					isset($query['first']) && $query['first'] && 
					isset($query['last']) && $query['last'] && 
					isset($query['count']) && $query['count'] && 
					isset($query['cache']) && is_array($query['cache']) && 
					in_array($this->form_values['id'], $query['cache'])
				) {
					$first_id = $query['first'];
					$last_id = $query['last'];
					$count = $query['count'];
					if ($count < 2) {
						// print '<div>1) count = ' . $count . '</div>'; exit();
						return false;
					}
				} else {
					$db =& $SERVICES->getService('Database');
					$result = $db->select($query['fields'], $query['tables'], $query['where'], $query['sort']);
					if (!$db->isGood($result)) {
						return false;
					}
					// print '<div>WebMATList::getBrowseLinks finished calling select</div>'; print_r($query); print '<hr>';
					// Count the items
					$count = $db->getNumRows($result);
					// print '<div>WebMATList::getBrowseLinks item count: ' . $count . '</div>';
					 
					// get the first and last rows into variables
					$first_item = @$db->getRow($result, 0);
					$limit_last_value = ($count - 1) < 0 ? 0 : $count - 1;
					$last_item = @$db->getRow($result, $limit_last_value); // Phil - added
					$first_id = (is_array($first_item) && $first_item['id'] ? $first_item['id'] : false);
					$last_id = (is_array($last_item) && $last_item['id'] ? $last_item['id'] : false);
					$this->updateBrowseQuery(
						array(
							'first' => $first_id, 
							'last' => $last_id, 
							'count' => $count
						)
					);
					if ($count < 2) {
						// print '<div>2) count = ' . $count . '</div>'; exit();
						return false;
					}
					//--
				}
				
				if (isset($query['cache']) && is_array($query['cache'])) {
					foreach ($query['cache'] as $k => $v) {
						// print '<div>k='.$k.', v='.$v.'</div>';
						if ($v == $this->form_values['id']) {
							$current_pos = $k;
							break;
						}
					}
				}

				// eg Item 1 of 20
				$item_pos = 1 + intVal($current_pos); // We index from 0 and the first item in the list should be 1
				// print '<div>count: ' .$count . ', current_pos: ' . $current_pos . ', item_pos: ' . $item_pos . '</div>';
				if ($count && ($current_pos !== false) && $item_pos) {
					$links[] = $L['browseItemLabel'] . ' ' . $item_pos  . ' ' . $L['browseItemOfLabel'] . ' ' . $count . ' &nbsp; &nbsp; ';
				}
				
				// First
				if ($first_id && ($first_id != $this->form_values['id'])) {
					$links[] = $this->output->getLinkHref(
						array(
							'href' => $SERVICES->getGlobal('php_self') . '?id=' . $first_id . '&browse=first', 
							'text' => $L['browseItemFirstText'], 
							'title' => $L['browseItemFirstTitle'], 
							'tag_extras' => ' class="pg"'
						)
					);
				} else {
					$links[] = '<span class="npg" title="' . $L['browseItemFirstTitle'] . '">' . $L['browseItemFirstText'] . '</span>';
				}
				// Back 10
				if ($count && (($item_pos - 10) > 0)) {
					$links[] = $this->output->getLinkHref(
						array(
							'href' => $SERVICES->getGlobal('php_self') . '?id=' . $this->form_values['id'] . '&browse=back10', 
							'text' => $L['browseItemPreviousTenText'], 
							'title' => $L['browseItemPreviousTenTitle'], 
							'tag_extras' => ' class="pg"'
						)
					);
				} else {
					$links[] = '<span class="npg" title="' . $L['browseItemPreviousTenTitle'] . '">' . $L['browseItemPreviousTenText'] . '</span>';
				}
				// Back 1
				if ($count && (($item_pos - 1) > 0)) {
					$links[] = $this->output->getLinkHref(
						array(
							'href' => $SERVICES->getGlobal('php_self') . '?id=' . $this->form_values['id'] . '&browse=back1', 
							'text' => $L['browseItemPreviousText'], 
							'title' => $L['browseItemPreviousTitle'], 
							'tag_extras' => ' class="pg"'
						)
					);
				} else {
					$links[] = '<span class="npg" title="' . $L['browseItemPreviousTitle'] . '">' . $L['browseItemPreviousText'] . '</span>';
				}
				// Next 1
				if ($count && (($item_pos + 1) <= $count)) {
					$links[] = $this->output->getLinkHref(
						array(
							'href' => $SERVICES->getGlobal('php_self') . '?id=' . $this->form_values['id'] . '&browse=fwd1', 
							'text' => $L['browseItemNextText'], 
							'title' => $L['browseItemNextTitle'], 
							'tag_extras' => ' class="pg"'
						)
					);
				} else {
					$links[] = '<span class="npg" title="' . $L['browseItemNextTitle'] . '">' . $L['browseItemNextText'] . '</span>';
				}
				// Next 10
				if ($count && (($item_pos + 10) <= $count)) {
					$links[] = $this->output->getLinkHref(
						array(
							'href' => $SERVICES->getGlobal('php_self') . '?id=' . $this->form_values['id'] . '&browse=fwd10', 
							'text' => $L['browseItemNextTenText'], 
							'title' => $L['browseItemNextTenTitle'], 
							'tag_extras' => ' class="pg"'
						)
					);
				} else {
					$links[] = '<span class="npg" title="' . $L['browseItemNextTenTitle'] . '">' . $L['browseItemNextTenText'] . '</span>';
				}
				// Last
				if ($last_id && ($last_id != $this->form_values['id'])) {
					$links[] = $this->output->getLinkHref(
						array(
							'href' => $SERVICES->getGlobal('php_self') . '?id=' . $last_id . '&browse=last', 
							'text' => $L['browseItemLastText'], 
							'title' => $L['browseItemLastTitle'], 
							'tag_extras' => ' class="pg"'
						)
					);
				} else {
					$links[] = '<span class="npg" title="' . $L['browseItemLastTitle'] . '">' . $L['browseItemLastText'] . '</span>';
				}
				
				return $links;
			}
		} 
		return false;	
	}
	
	/**
	 * Returns '' so table dosen't sort
	 *
	 * @param string $sort (not used for view or edit)
	 * @return string the string is = ''
	 */
	function sortTable($sort = '') {
		return '';
	}
	
	/**
	 * Sets the priority edit name
	 * 
	 * View and Edit pages can use a different value for ordering fields.
	 * This only needs to be changes if you do not want to use the priority_edit field for ordering.
	 * 
	 * @param string $priority_edit the priority edit name
	 * @see getPriorityEdit()
	 * @author Phil Calder
	 */
	function setPriorityEdit($priority_edit = 'priority_edit') {
		$this->priority_edit = $priority_edit;	
	}
	
	/**
	 * Gets the priority edit name
	 *
	 * View and Edit pages can use a different value for ordering fields.
	 * 
	 * @return string the priority edit name
	 * @see setPriorityEdit()
	 * @author Phil Calder
	 */
	function getPriorityEdit() {
		return $this->priority_edit;
	}
	
	
	/**
	 * returns the edit_link variable
	 * 
	 * @param string the edit_link variable
	 */
	function getEditLinkVar($name = false) {
		return $this->edit_link;
	}
	
	/**
	 * redirects back to the list page
	 * @see getListPageUrl()
	 * @author Phil Calder
	 */
	function redirectToList() {
		$list_page = $this->getListPageUrl();
		header('Location: ' . $list_page);
		exit();
	}
}
?>