<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 */

includeWebMAT('display.inc');
includeWebMAT('filters.inc');

/**
 * this class handles the tabluar display of content
 *
 * @author Phil Calder, Matthew Ross, Matthew Corich, Brenton Bartel, Simon Cook, Syed, Bosco
 * @abstract
 * @package WebMAT
 */
class WebMATStandardList extends WebMATDisplay {
	
	/**
	 * @var used to disable filters
	 * @access private
	 */
	var $no_filters = false;
	
	/**
	 * @var used to disable filters
	 * @access private
	 */
	var $alert_ids = array();
	

	/**
	 * @var integer the item id for the item highlighted
	 * @access private
	 */
	var $highlight_id = false;
	
	/**
	 * @var string The current bookmark name
	 * @access private
	 */
	var $bookmark_name = '';

	/**
	 * @var string The current data row in the list table
	 * @access private
	 */
	var $data_row_counter = 0;
	
	/**
	 * @access private
	 */
	var $restrict_level = 'read';
	
	/**
	 * init function sets all the class variables
	 *
	 * @param  string $module  the current module name
	 * @param  string $data_option  the current data option used for DataDictionary
	 * @param  array $post the post array
	 * @param  array $get the get array
	 * @param  string $table_name the name of the table the page is using
	 * @param  array $language_overwrite has the keys title and description
	 */
	function init($module, $data_option, $post = false, $get = false, $table_name = false, $language_overwrite = false) {
		// print '<div>call processAltDataOption(' . $data_option . ', ' . $module . ', ' . $table_name . ', [get:])</div>'; print_r($get); print '<hr>';
		$this->processAltDataOption($data_option, $module, $table_name, $get);
		parent::init($module, $data_option, $post, $get, $table_name, $language_overwrite);
		
		$this->setMasterIds();
		$this->loadFromBookmark(); // Phil 20060530 Extended to load from an advanced search
		$this->setHighlightId();
		$this->setSortTableName(); // blank means dont use sort table name
		$this->setLinkCellFormatting();
		$this->setBrowseQueryDataOption();
		$this->output->req_list_js = true;
		$this->output->req_popup_menu_js = true;
		// $this->output->req_config_js = true; // Phil 20060413 See getConfigIcon
		
		$this->initXajax();	
	}
	
	/**
	 * main function either prints the content or returns it as a string
	 *
	 * @param  bool  $return_string  used to see if you want the content returned as a string or printed to the string
	 * @return mixed if $return_string = true it returns a string with the content for the page else returns true.
	 * @see display
	 * @author Phil Calder
	 */
	function main($return_string = false) {
		$this->setReturnString($return_string); // Phil 20060113
		
		// -- specific to list pages --
		$this->clearBrowseQuery(); // Phil 20050823
		$this->setFilters();
		// print '<div>filters have been set</div>';
		$contact_id = $this->form_values['contact_id'];
		$items = $this->form_values['items'];
		if ($contact_id || $this->paging_overwrite) {
			if (!$this->form_values['filter']) {
				$this->form_values['filter'] = 'fnone';
			}
		}
		$this->setLogAction();
		$this->db_functions->setLog();
		$this->afterDataDictionary();
		// --
		
		// Build a content array
		$content = array();
		// Last Viewed and Bookmarks Table
//MCR 20070314
//idea is to move to a side panel working with AJAX 
//		$content['pastLinks']			= $this->getPastLinksTable();
		// ---------------
		$content['beginMain']           = $this->getBeginMainContent();
		// Title (and links)
		$content['title']               = false; // placeholder (see after getFiltersContent) // = $this->getTitleContent();
		// Description (optional)
		$content['description']         = $this->getDescriptionContent();
		
		// Summary (optional)
		$content['summary']             = false; // placeholder$this->getSummaryContent();
		
		
		// Browse / paging

//MCR 20070314
//trying idea of just bottom paging
//to keep toolbar clean
//		$content['browsePaging_top']    = false; // placeholder (for ordering) - set AFTER getDataContent
		
		
		$content['beginDataCell']       = $this->getBeginDataCell(false);
		
		// Filters
		$content['filters']             = $this->getFiltersContent($title_text);
		// populate the title content (placeholder above)
		$content['title']               = $this->getTitleContent($title_text);
		// ActionLinks
		$content['actionLinks']         = $this->getActionLinksContent();
		
		$content['bodyTable']			= $this->output->getBeginTable('bodyTbl');
		// ----
		
		// Main Content
		$content['data']                = $this->getDataContent($item, $error_msg);
		// ----
		
		$content['endBodyTable']        = $this->output->getEndTable();;
		
		$content['endDataCell']         = $this->getEndDataCell();
		
		// Checkbox menu content
		$content['checkboxMenu']        = $this->getCheckboxMenuContent();
		
		// Browse / paging - both paging rows are set here - has to be done AFTER getdataContent
//		$content['browsePaging_bottom'] = $content['browsePaging_top'] = $this->getBrowsePagingContent();
		$content['browsePaging_bottom'] = $this->getBrowsePagingContent();
		
		$content['summary']             = $this->getSummaryContent();
		// ---------------
		$content['endMain']             = $this->getEndMainContent();
		
		return $this->display($content);
	}

	/**
	 * returns the WebMAT type
	 */
	function getWebmatType() {
		return 'list';
	}
	
	/**
	 * @author Phil Calder
	 */
	function showXajaxPopupIcon() {
		return false; // Phil: todo when implementation complete
	}
	
	
	/**
	 * register xajax, the xajaxView function and process any incoming request
	 *
	 * @author Matthew Ross
	 */
	function initXajax() {
		// Phil - a switch to disable
		// Will need to allow for other xajax functions (not just the popup view) in future
		if ($this->showXajaxPopupIcon()) { 
			// print '<div>initXajax</div>';
			$this->output->initXajax();
			$this->output->xajax->registerFunction(array('xajaxView', &$this,'xajaxView'));
			$this->output->xajax->processRequest();
		}
	}
	
	/**
	 * handle xajax request
	 *
	 * @param integer $id   the item id 
	 * @param string $module the module name
	 * I had expected to be able to use $this->module_name instead of the 2nd paratmer 
	 * but it doesn't seem to work with our tabs???
	 *
	 * @return string  the html content piece
	 *
	 * @author Matthew Ross
	 */
	function xajaxView($id, $module) {
		// print '<div>xajaxView(' . $id . ', ' . $module . ')</div>';
		$this->response = new xajaxResponse();
		
		$view = getWebMATObject(
			'WebMATView', 
			//$this->module_name,
			$module,
			array(
				'data_option'=>'view'
			)
		);
		
		//this is required for getActionLinksContent() to work
		$view->item = array('id' => $id);
		
		//this is required for getDataContent() to work
		$view->form_values['id'] = $id;
		
		
		//$this->response->assign('xajax_view_title','innerHTML',$view->getTitleText() );
		
		$this->response->assign('xajax_view_content','innerHTML',
			//'DEBUG: module: ' . $module . ' id: ' . $id . '<br>' .
			'<table width="100%"><tr><td>'
				. '<table width="100%"><tr><td>'
				. $view->getActionLinksContent()
				. '</td></tr></table>'
			. '</td></tr>'
			
			. '<tr>'
			. '<td id="bodyCell" style="padding: 10px;">'
			//. $view->getDataContent(array('item_id' => $id), $error ='')
			. $view->getTable()
			. '</td>'
			. '</tr></table>'
		);
			
		//@todo set position relative to click
		
		//make space visible
		$this->response->assign('xajax_space_div','style.visibility', 'visible');
		
		//set
		//$this->response->assign('xajax_space','innerHTML', is_object($view) ? 'yes': 'no' );
		return $this->response;
	}
	
	/**
	 * Changes the data option
	 *
	 * @param array $get The get variable, specifically the 'alt_data_option'
	 * @return string $data_option The data option to use, either the original or alternative data option
	 */
	function processAltDataOption(&$data_option, $module, $table_name, $get) {
		global $SERVICES;
		$this->has_alt_data_option = false;
		$this->use_alt_data_option = 0;
		$table_name = ($table_name ? $table_name : 'item');
		// print '<div>module_name: ' . $module . ' table_name: ' . $table_name . ' get:</div>'; print_r($get); print '<hr>';
		if (is_object($config =& $SERVICES->getService('Config'))) {
			// Phil 20061110 TODO: implement 3rd option, short_data_option: '__shortlist__';
			$alt_data_option = $config->lookup('modules.' . $module . '.file_set. ' . $table_name . '.alt_data_option.' . $data_option);
			// print '<div>standard data_option: ' .$data_option . ' alt_data_option: ' . $alt_data_option . ' for module ' . $module . '</div>';
			if ($alt_data_option && is_string($alt_data_option)) {
				$this->has_alt_data_option = true;
				$module_lower = strtolower( substr($module, 0, 1) ) . substr($module, 1);
				$pref_name = $module_lower . 'AltDataOption' . $data_option;
				if (is_array($get) && isset($get['alt_data_option'])) {
					// print '<div>[get] alt data option: ' . $get['alt_data_option'] . '</div>';
					// $data_option = $get['alt_data_option'];
					if (is_object($prefs =& $SERVICES->getService('Preferences'))) {
						$this->use_alt_data_option = $new_value = ($get['alt_data_option'] ? '1' : '0');
						// print '<div>[set] pref_name ' . $pref_name . ' new value: ' . $new_value . '</div>';
						$prefs->setUserPref($pref_name, $new_value);
						// $data_option = $alt_data_option;
					}
				} else if (is_object($prefs =& $SERVICES->getService('Preferences'))) {
					
					$this->use_alt_data_option = $prefs->getUserPref($pref_name);
					// print '<div>[pref] pref_name ' . $pref_name . ' ' . ($this->use_alt_data_option ? 'yes' : 'no') . '</div>';
				}
				if ($this->use_alt_data_option) {
					$data_option = $alt_data_option;
					// print '<div>using data option: ' . $data_option . '</div>';
				}
			}
		}
	}
	
	// TODO: processAltDataOption could call this
	// function getAltDataOption($data_option, $module, $table_name) {
		// global $SERVICES;
		// if (is_object($config =& $SERVICES->getService('Config'))) {
			// // Phil 20061110 TODO: implement 3rd option, short_data_option: '__shortlist__';
			// $alt_data_option = $config->lookup('modules.' . $module . '.file_set. ' . $table_name . '.alt_data_option.' . $data_option);
		// }
		// return ( ($alt_data_option && is_string($alt_data_option)) ? $alt_data_option : false);
	// }

	/**
	 * Loads a Bookmark and/or advanced filter
	 * 
	 * loads the bookmark values and updates form values
	 * Will first check for Advanced Filter and load form values based on that filter
	 */
	function loadFromBookmark() {
		// print '<div>loadFromBookmark</div>';
		global $SERVICES;
		// Phil 20060531 Added Advanced Filter code
		if (isset($this->form_values['adv_filter']) && $this->form_values['adv_filter']) {
			global $USER; // Advanced Filter values are stored in the USER array
			// Phil 20060824 - Retain advanced filter values between page views
			if (isset($this->form_values['submit']) && $this->form_values['submit']) {
				$USER['adv_filter'][$this->getModuleName()][$this->getTableName()] = $this->form_values;
				// print '<div>(loadFromBookmark) set user adv_filter</div>'; print_r($USER['adv_filter']); print '<hr>';
			}
			
			if (isset($USER['adv_filter']) && isset($USER['adv_filter'][$this->getModuleName()]) && isset($USER['adv_filter'][$this->getModuleName()][$this->getTableName()])) {
				if (is_array($form_values = $USER['adv_filter'][$this->getModuleName()][$this->getTableName()])) {
					// We (possibly) need to retain certain form values
					// print '<div>current form_values</div>'; print_r($this->form_values); print '<hr>';
					// print '<div>new form_values</div>'; print_r($form_values); print '<hr>';
					
					$current_form_values = array();
					if (isset($this->form_values['sort'])) {
						$current_form_values['sort'] = $this->form_values['sort'];
					}
					if (isset($this->form_values['show_secondary'])) {
						$current_form_values['show_secondary'] = $this->form_values['show_secondary'];
					}
					// This loads the saved Advanced Filter form values into this class
					$this->form_values = $form_values;
					// Restore certain form values
					if (isset($current_form_values['sort'])) {
						$this->form_values['sort'] = $current_form_values['sort'];
					}
					if (isset($current_form_values['show_secondary'])) {
						$this->form_values['show_secondary'] = $current_form_values['show_secondary'];
					}
					$this->processAdvancedFilters($form_values['item']);
				}
			}
		}
		
		// Bookmarks code
		if ($SERVICES->modules->exists('SystemBookmarks')) {
			if (isset($this->form_values['bookmark']) && $this->form_values['bookmark']) { // Phil 20051111 check isset
				global $SERVICES;
				$mod =& $SERVICES->modules->getModule('SystemBookmarks');
				if ($bookmark = $mod->getBookmark($this->form_values['bookmark'])) { // Phil 20051111 check if
					$this->bookmark_name = $bookmark['name']; // Phil 20060123 - store for later use
					$list_values = $bookmark['extra_values'];
					// Phil 20060602 Retain bookmark in query string
					// $new_query = array('bookmark' => '');
					$new_query = array(
						'bookmark' => $this->form_values['bookmark'], 
						'custom_page' => '1'
					);
					
					// Phil 20060602
					if (!isset($this->form_values['custom_page']) || !$this->form_values['custom_page']) {
						//bcrm_unset($this->form_values, 'bookmark');
						if ($list_values['view_all']) {
							$this->form_values['view_all'] = $list_values['view_all'];
							$new_query['view_all'] = $this->form_values['view_all'];
						}
		
						if ($list_values['page_num']) {
							$this->form_values['page_num'] = $list_values['page_num'];
							$new_query['page_num'] = $this->form_values['page_num'];
						}
		
						if ($list_values['sort']) {
							$this->form_values['sort'] = $list_values['sort'];
						}
		
						if ($list_values['filters']['contact']) {
							$this->form_values['contact_id'] = $list_values['filters']['contact'];
							$new_query['contact_id'] = $this->form_values['contact_id'];
						}
					}
	
					if ($list_values['filters']['ids']) {
						$this->form_values['ids_session']  = $list_values['filters']['ids'];
					}
	
					if ($list_values['filters']['search_filter']) {
						$this->form_values['search_filter'] = 'on';
						$this->form_values['item'] = $list_values['filters']['search_filter'];
						
						// Need to retain the current bookmark id, page_num, and view_all?
						
						// Phil 20060601 Need to process fields here too (eg like adv filter)
						$this->processAdvancedFilters($this->form_values['item']);
						
					} else {
						$this->form_values['search_filter'] = 'off';
					}
	
					if (isset($list_values['filters']['filter'])) { //Syed 20050801 | includes the config filter 'f0'
						$this->form_values['filter'] = 'f' . $list_values['filters']['filter'];
					}
					
					if (isset($list_values['filters']['qfilter'])) { // Phil 20050824 - Looks for stored qfilter value in bookmark
						$this->form_values['qfilter'] = $list_values['filters']['qfilter'];
					}
	
					$_SERVER['QUERY_STRING'] = $this->output->updateQueryString($new_query, false);
					//Syed 20050727 | show/hide the search filter form
					if (!$bookmark['show_filter']) {
						$this->output->show_filter_form= false;
					}
				}
			}
		}
	}
	
	/**
	 * Internal function to process advanced filters
	 *
	 * @link loadFromBookmark
	 *
	 * @param array $item The form values item array to process
	 * @access private
	 * @author Phil Calder
	 */
	function processAdvancedFilters($item) {
		//print '<div>processAdvancedFilters item:</div>'; print_r($item); print '<hr>';
		// Make sure this is processed now
		$this->db_functions->getDataDictionaryfields();
		// Get all item keys that dont start with _changed_ or _options_ and add them to the data dictionary
		if (isset($item) && is_array($item)) {
			$ignore = array('_changed_', '_options_');
			foreach ($item as $k => $v) {
				$prefix = substr($k, 0, 9);
				if (strlen($prefix) && !in_array($prefix, $ignore)) {
					if ($field = $this->dd->getField($k)) {
						$this->dd->setDictionaryIncludeField($field); //, true); // Phil 20060929 dont force merge
					}
				}
			}
		}
		// Phil, should (probably) only do this if we have called setDictionaryIncludeField
		$this->dd->getDropdownValues(); 
	}
	
	/**
	 * sets the highlight class variable
	 *
	 * @param integer $highlight_id the id of the item you want to highlight
	 */
	function setHighlightId($highlight_id = false) {
		if ($highlight_id) {
			$this->highlight_id = $highlight_id;
		} else if (isset($this->form_values['highlight_id']) && $this->form_values['highlight_id']) {
			$this->highlight_id = $this->form_values['highlight_id'];
		} else {
			$this->highlight_id = false;
		}
	}

	/**
	 * gets the highlight class variable
	 *
	 * @return mixed $highlight_id the id of the item you want to highlight or
	 * false if nothing is highlighted
	 */
	function getHighlightId() {
		return $this->highlight_id;
	}
	
	/**
	 * Defaults to true
	 *
	 * @author Phil Calder
	 */
	function showAlertStatus() {
		return true;
	}
	
	/**
	 * Defaults to true
	 *
	 * @author Phil Calder
	 */
	function showSeenStatus() {
		return true;
	}

	/**
	 * sets the filters object
	 */
	function setFilters() {
		if (!$this->no_filters) {
			
			$this->filters = new WebMATFilters($this->module_name, $this->form_values, $this->table_name, $this->dd, $this->db_functions);
			//print_r($this->filters);
			$this->setDefaultConstraint(); // Phil 20061129
			
			// Phil 20050824 - Sets query type (qfilter) from query string or bookmark (see loadFromBookmark)
			if (isset($this->form_values['qfilter']) && is_object($this->db_functions)) {
				$this->db_functions->setQueryType($this->form_values['qfilter']);
			}
			if (is_array($fields = $this->filters->getFilteredFields())) { // Phil 20060927
				// print '<div>filtered fields:</div>'; print_r($fields); print '<hr>';
				if (is_array($fields) && count($fields)) {
					$filtered = array();
					foreach ($fields as $k => $v) {
						$filtered[$v] = 1;
					}
					$this->processAdvancedFilters($filtered);
				}
			}
		}
	}
	
	/**
	 * @author Phil Calder
	 */
	function setDefaultConstraint() {
		// TODO: cache getQFilters, since it is now called 2 times
		// TODO: need to be able to override this depending on main filter 
		// eg no point setting default constraint to 'alert' when 
		// viewing Mail sent items, or archived Memos, or closed Tasks
		
		// TODO: config setting?
		$mods_with_constraints = array('Contact', 'Person', 'Company', 'Tasks', 'Mail', 'Opportunities', 'Jobs');
		if (!$this->form_values['bookmark'] && in_array($this->getModuleName(), $mods_with_constraints)) {
			if (!isset($this->form_values['qfilter'])) {
				$qfilters = $this->filters->getQFilters();
				//print '<div>qfilters: </div>'; print_r($qfilters); print '<hr>';
				//echo '#####'; print_r($qfilters); echo 'XXXX'; 
				// $default_qfilter = 'alert';
				$default_qfilter = $this->getDefaultConstraint();
				if ($default_qfilter && array_key_exists($default_qfilter, $qfilters)) {
					$this->form_values['qfilter'] = $default_qfilter;
					//echo 'here';
				}
				$this->form_values['qfilter'] = $default_qfilter;
				//echo $this->form_values['qfilter'] . '***********';
			} // else {
				// print '<div>qfilters already set: ' . $this->form_values['qfilter'] . '</div>';
			// }
		}
	}
	
	/**
	 * @author Phil Calder
	 */
	function getDefaultConstraint() {
		global $SERVICES;
		$prefs =& $SERVICES->getService('Preferences');
		$pref_name = $this->getModuleNameLower() . 'DefaultConstraint';
		return $prefs->getUserPref($pref_name);
	}

	/**
	 * Checks whether we are using filters and the filters object exists 
	 *
	 * @return boolean $use_filters
	 *
	 * @author Phil Calder
	 */
	function useFilters() { // Phil 20050503
		return (!$this->no_filters && is_object($this->filters));
	}
	
	/**
	 * @return boolean Whether to show the qfilters row
	 * @author Syed, Phil Calder
	 */
	function showQfilters() {
		if (is_object($this->filters) && $this->filters->show_qfilters) {
			return true;
		} 
		return false;
	}
	
	/**
	 * @return boolean Whether to show the qfilters row
	 * @author Phil Calder
	 */
	function showIdsfilters() {
		if (is_object($this->filters)) { // && $this->filters->show_idsfilters) {
			return true;
		} 
		return false;
	}
	
	/**
	 * @return boolean Whether to show the config filters row
	 * @author Phil Calder
	 */
	function showConfigFilters() { 
		return true;
	}

	/**
	 * @return boolean When to display the 'Filtered on' row
	 * @author Phil Calder
	 */
	function showFilteredOnRow() {
		return true;
	}

	/**
	 * @return boolean Whether to show the checkbox (process multi) row
	 * @author Phil Calder
	 */
	function showCheckboxMenuRow() {
		if (isset($this->item_count) && !$this->item_count) {
			return false;
		}
		return true;
	}
	
	/**
	 * @return boolean Whether to show the search filter form
	 * @author Phil Calder
	 */
	function showSearchFilterForm() {
		return true;
	}
	
	/**
	 * Sets the Log Action | Bosco 20051209
	 * @author Bosco
	 */
	function setLogAction() {
		$this->db_functions->setLogAction('list');
	}

	/**
	 * Gets the table links
	 *
	 * Custom get table links method to remove the list and duplicate links
	 *
	 * @param array $item the item array
	 * @return array an array of links
	 */
	function getTableLinks($item) {
		$links = parent::getTableLinks($item);
		bcrm_unset($links, 'list');
		bcrm_unset($links, 'duplicate');
		return (array)$links;
	}
	
	/**
	 * @author Phil Calder
	 */
	function getListTableLinks($item, $identifier, $default = 'edit') {
		$table_links = (array)$this->getTableLinks($item);
		return $this->output->getListTableLinks($table_links, $identifier, $default);
	}
	
	/**
	 * @return mixed false or a string containing the checkbox menu content
	 */
	function getCheckboxMenuContent() {
		$content = array();
		if ($this->showCheckboxMenuRow()) {
			if ($this->getItemCheckbox()) { // Phil 20060117
				global $L;
				$dropdown_link = $this->getItemCheckbox(false) . ' ';
				$items = bcrm_array_merge(
					array($L['labelWithSelected']), 
					$this->getCheckboxMenuItems(), 
					$this->getCheckboxMenuExtraItems()
				);
				$dropdown = array(
					'label'=>'', 
					'field_key'=>'_process_multi', 
					'type'=>'dropdown',
					'dropdown_values'=>$items, 
					'notnull'=>1, 
					'width' => 110
				);
				$dropdown_link .= $this->output->getLinkHref(
					array(
						'href' => 'javascript:hrefChg(\'chk\')', 
						'text' => $L['labelCheckAll']
					)
				) . ' '; //mcr 20060130  | ';
				$dropdown_link .= $this->output->getLinkHref(
					array(
						'href' => 'javascript:hrefChg(\'uchk\')', 
						'text' => $L['labelUncheckAll'])
					) . ' ';//mcr 20060130  | ';
				$dropdown_link .= $this->output->getLinkHref(
					array(
						'href' => 'javascript:hrefChg(\'ichk\')', 
						'text' => $L['labelInverseCheck'])
					) . ' &nbsp; ';
				$dropdown_link .= $this->output->getHtmlField($dropdown);
				if (is_object($this->filters) && $this->filters->getContactFilter()) {
					$contact_id = array(
						'label'=>'', 
						'field_key'=>'contact_id', 
						'type'=>'hidden',
						'value'=>$this->filters->getContactFilter()
					);
					$dropdown_link .= $this->output->getHtmlField($contact_id);
				}
				if ($this->select_contacts) {
					$contact_fields = array(
						'label'=>'', 
						'field_key'=>'contact_fields', 
						'type'=>'hidden',
						'value'=>$this->select_contacts
					);
					$dropdown_link .= $this->output->getHtmlField($contact_fields);
				}
				$dropdown_link .= $this->output->getHtmlSubmit($L['labelGo'], 'submit'); 
				return $this->output->getTextRow($dropdown_link, 1, 'lCheckAllCell', true) . $this->output->getEndForm();
			}
		}
		return false;
	}
	
	/**
	 * @return string The paging row
	 * @author Phil Calder
	 * @see getPagingRow
	 */
	function getBrowsePagingContent() {
		return $this->getPagingRow();
	}
	
	/**
	 * @author Phil Calder
	 */
	function getActionLinksContent() {
		//return $this->getSecondMenuRow();
		$this->toolbar_link_mode = true;
		$this->output->setActionLinksMenu($this->getSecondMenuRow());
		$this->toolbar_link_mode = false;
	}
	
	/** 
	 * Whether to allow the Advanced search filter link
	 *
	 * Does not imply that this link is available, but if it is then request it
	 * Override to toggle this value
	 * @return boolean true
	 * @author Phil Calder
	 */
	function allowAdvancedFilterLink() {
		return true;
	}
	
	/**
	 * Build an array of content and pass to an output handler
	 *
	 * @param string $title_text Populated and returned by reference
	 * @return array A content array with possible keys: 
	 * showFilters, contact, qFilters, idsFilters, configFilters
	 * @see Output::getFiltersContent
	 * @author Phil Calder
	 */
	function getFiltersContent(&$title_text) {
		$content = array();
		$title_text = false;
		if ($this->useFilters()) {
			// $content['collapse'] = $this->getBeginSearchFilter();
			// $content['activate'] = $this->getSearchFilterActivate();
			$content['showFilters'] = $this->filters->getSearchFilterLink($this->allowAdvancedFilterLink());
			if ($this->showFilteredOnRow()) {
				if (isset($this->form_values['bookmark']) && $this->form_values['bookmark']) {
					$content['bookmark'] = array(
						'id' => $this->form_values['bookmark'], 
						'label' => $this->bookmark_name
					);
				}
				$content['contact'] = $this->filters->getContactFilterContent();
				if ($this->showQfilters()) {
					$content['qFilters'] = $this->filters->getQFilterContent();
				}
				if ($this->showIdsfilters()) {
					// print '<div>list, getting ids filter content</div>';
					$content['idsFilters'] = $this->filters->getIdsFilterContent();
				}
			}
			if ($this->showConfigFilters()) {
				$content['configFilters'] = $this->filters->getConfigFiltersContent($this->form_values['filter']); // Lists config filters
				//$content['configFilters']['options'][2]['where'] = " (Contact__item.acc_type_id = 2)";
				//print_r($content['configFilters']);
			}
		} else {
			return ''; // Phil 20060306	
		}
		return $this->output->getFiltersContent($content, $title_text);
	}
	
	/**
	 * Override to implement
	 */
	function getSummaryContent() {
		return '';
	}
	
	/**
	 * A list page returns a list table.
	 *
	 * @param array $item An item array (optional for view pages)
	 * @param array $error_msg error message (optional for view pages)
	 * @return string Content
	 * @author Phil Calder
	 */
	function getDataContent() {
		$cells = array();
		if ($this->showCheckBoxMenuRow()) {
			$cells['__cb'] = ''; // item checkbox
		}
		if ($this->showListLinks()) {
			$cells['__links'] = ''; // view, edit, etc, links
		}
		if ($this->showListFlags()) {
			$cells['__flags'] = ''; // alert, unseen/edited, etc
		}
		
		$cells = array_merge($cells, $this->getDataDictionary());
		$cells = $this->excludeCalculated($cells); // flags calculated fields
		// $cells = $this->headerRowSpacer($cells);
		// print '<div>' . get_class($this) . ' ' . $this->getModuleName() . '</div>'; print_r($cells); print '<hr>';
		
		if ($this->useFilters()) {
			$cells = $this->filters->getSearchFilterTableLabels($cells, $this->showSearchFilterForm());
			// print '<div>cells</div>';print_r($cells); print "\n<hr>\n";
		}

		$sort = $this->sortTable($this->form_values['sort']);
		
 		$contact_id = $this->form_values['contact_id'];
		
		if (!isset($this->item_count) || !$this->item_count) { // Phil 20070212 can sometimes already have been called (eg contact view, desktop)
			// print '<div>List, calling db_functions->countItems [item_count: ' . $this->item_count . ']</div>'; 
			$this->item_count = $this->db_functions->countItems($contact_id); // only needed for setPagingLimits, and showCheckboxMenu
		} // else {
			// print '<div>[cached] item_count: ' . $this->item_count . '</div>'; 
		// }
		$table_content = $this->getSubTableHeader($cells, $sort); //Syed 20050704

		if ($this->getItemCheckbox()) { // Phil 20060117
			$table_content .= $this->beginSelectItemForm();
		}

		if ($this->paging_overwrite['limit']) {
			$limit = 'LIMIT 0, ' . $this->paging_overwrite['limit'];
		} else {
			$limit = $this->output->setPagingLimits($this->pref_name . 'ListSize', $this->item_count);
		}
		if ($this->paging_overwrite['sort']) {
			$sort = $this->paging_overwrite['sort'];
		}
		
		$table_content .= $this->startDataBlock();
		
		$this->beforeGetItems($sort, $limit);
		
		if (!$this->useGetItemsIterator()) {
			// print '<div>mod: ' . $this->getModuleName() . ' NOT using iterator</div>';
			// Phil 20051201 This is the standard way of handling getItems
			// Database tables class returns an entire (processed) items array
			$items = $this->getItems($sort, $limit);
			// print '<div>items:</div><pre>' . print_r($items, 1) . '</pre><hr>';
			$table_content .= $this->afterGetItems($items);
	
			if (is_array($items) && sizeof($items)) {
				$this->alert_ids = false;
				if ($this->showAlertStatus()) { // Phil 20051114 Will not calculate or show alert flag if this is false
					// print '<div>list: no iterator</div>';
					$this->alert_ids = $this->db_functions->getAlertIds($limit, array_keys($items)); //passing item ids
				}
				// Phil 20051031 Tell Output if we dont want to trim fields
				$no_trim = $this->dd->getNoTrim();
				$current_no_trim = $this->setNoTrim($no_trim);
				
				foreach ($items as $k => $item) {
					$has_alerts = count($this->alert_ids);
					$table_content .= $this->buildListTable($item, $has_alerts);
				}
				$this->setNoTrim($current_no_trim); // Phil 20051031 Reset the no trim value
			} else {
				global $L;
				$table_content .= $this->output->getTextRow($L['msgNoItemsForView'] , sizeof($cells)+1);
			}
		} else {
			// Phil 20051201 This is the more efficient way of handling getItems
			// Database tables class returns a resource/handle which we 
			// iterate through.
			// print '<div>mod: ' . $this->getModuleName() . ' using iterator</div>';
			if ( $this->item_count && (($items_resource = $this->getItemsResource($sort, $limit)) !== false) ) { // add other checks (ie num rows)
				// rewrite
				$this->alert_ids = false;
				if ($this->showAlertStatus()) { // Phil 20051114 Will not calculate or show alert flag if this is false
					// $this->alert_ids = $this->db_functions->getAlertIds($limit);
					// print '<div>list - using iterator</div>';
					$this->alert_ids = $this->db_functions->getHasAlerts(); // We just need to know if there are any alerts
					// print '<div>[real page limit: ' . $limit . '] alert_ids:</div>'; print_r($this->alert_ids); print '<hr>';
				}
				
				// Phil 20051031 Tell Output if we dont want to trim fields
				$no_trim = $this->dd->getNoTrim();
				$current_no_trim = $this->setNoTrim($no_trim);
				
				while ($row = $this->db_functions->db->getRow($items_resource)) {
					$item = $this->db_functions->processRow($row['id'], $row);
					$has_alerts = count($this->alert_ids);
					$table_content .= $this->buildListTable($item, $has_alerts);
				}
				$this->setNoTrim($current_no_trim); // Phil 20051031 Reset the no trim value
			} else {
				global $L;
				$table_content .= $this->output->getTextRow($L['msgNoItemsForView'] , sizeof($cells)+1);
			}
		}

		$table_content .= $this->endDataBlock();
		return $table_content;
	}
	
	/**
	 * Specify whether to use an iterator for getItems, or process and return the entire items array
	 *
	 * Added 200512xx. Extend this method for modules which rely on bulk processing 
	 * of items in the {@link afterGetItems} method
	 *
	 * @return boolean true or false (default is true)
	 * @author Phil Calder
	 */
	function useGetItemsIterator() {
		return true;
	}
	
	/**
	 * Wrapper function for database tables setDataOption
	 *
	 * @author Phil Calder
	 * @see getBrowseQueryDataOption
	 * @see WebMATDatabaseTable::setDataOption
	 */
	function setBrowseQueryDataOption() {
		if (is_object($this->db_functions)) {
			// print '<div>setBrowseQueryDataOption '. $this->getBrowseQueryDataOption() . ' class: ' . get_class($this) . ' db_functions: ' . get_class($this->db_functions) . '</div>';
			$this->db_functions->setDataOption($this->getBrowseQueryDataOption());
		}
	}
	
	/**
	 * Wrapper function for database tables clearBrowseQuery
	 * @author Phil Calder
	 */
	function clearBrowseQuery() {
		// print '<div>' . get_class($this) . '::clearBrowseQuery</div>';
		if (($this->getBrowseQueryDataOption() == $this->getWebMatType()) && is_object($this->db_functions)) {
			// print '<div>clearBrowseQuery '. $this->getBrowseQueryDataOption() . ' class: ' . get_class($this) . ' db_functions: ' . get_class($this->db_functions) . '</div>';
			$this->db_functions->clearBrowseQuery(
				$this->getModuleNameLower(), 
				$this->getTableName(), 
				$this->getBrowseQueryDataOption()
			);
		}
	}

	/**
	 * get the items from the db_function object
	 *
	 * @return array the array of items
	 * @author Phil Calder
	 */
	function getItems($sort, $limit) {
		return $this->db_functions->getItems($sort, $limit);
	}
	
	/**
	 * get the items from the db_function object
	 *
	 * @return array the array of items
	 * @author Phil Calder
	 */
	function getItemsResource($sort = false, $limit = false, $where = false) {
		return $this->db_functions->getItemsResource($sort, $limit, $where);
	}
	
	/**
	 * Determines whether to allow foreign field sorting in a list page. 
	 * 
	 * Config based
	 *
	 * @return boolean whether the class excludes foreign field sorting
	 * @author Phil Calder
	 */
	function isExcludingForeignFieldSorting() {
		if (!isset($this->exclude_foreign_sorting)) {
			global $SERVICES;
			$SERVICES->getService('Config');
			if ($SERVICES->config->lookup('modules.Front.webmat.list.exclude_foreign_sorting')) {
				$this->exclude_foreign_sorting = true;
			} else {
				$this->exclude_foreign_sorting = false;
			}
		}
		return $this->exclude_foreign_sorting;
	}
	
	/**
	 * Determines whether to allow custom field sorting in a list page. 
	 * 
	 * Config based
	 *
	 * @return boolean whether the class excludes custom field sorting
	 * @author Phil Calder
	 */
	function isExcludingCustomFieldSorting() {
		if (!isset($this->exclude_custom_sorting)) {
			global $SERVICES;
			$SERVICES->getService('Config');
			if ($SERVICES->config->lookup('modules.Front.webmat.list.exclude_custom_sorting')) {
				$this->exclude_custom_sorting = true;
			} else {
				$this->exclude_custom_sorting = false;
			}
		}
		return $this->exclude_foreign_sorting;
	}

	/**
	 * Excludes the calculated fields from the specified data definition
	 *
	 * Also sets foreign field key appropriately, and changes email type to text (for proper js validation)
	 *
	 * @param array $table_labels a data_definition
	 * @return array the transformed data_definition
	 * @author Phil Calder, Syed
	 */
	function excludeCalculated($table_labels) {

		foreach ($table_labels as $k => $v) {
			if (isset($v['calculated']) && ($v['calculated'] == '1')) {
				$table_labels[$k]['no_sort'] = 1;
			}
			if (isset($v['type']) && ($v['type'] == 'contact')  && ($v['field_key'] == 'contact_id')) {
				$table_labels[$k]['field_key'] =  '_' . $v['field_key'] . '.display_name';
				//$table_labels[$k]['field_key'] =  '_Contact__item.display_name';
				$table_labels[$k]['type'] =  'text';
			} else if (isset($v['foreign']) && $v['foreign']) { // Syed 20050625 to enable foreign field filtering
				// Phil 20051107 Need to check this field is not already prefixed
				$table_labels[$k]['field_key'] =  ( ( isset($v['field_key']) && (substr($v['field_key'], 0, 1))  != '_') ? '_' . $v['field_key'] : $v['field_key']);
			}
			if (isset($v['type']) && ($v['type'] == 'email')) { // Syed 20050628 To avoid Js validation
				$table_labels[$k]['type'] =  'text';
			}
		}
		//echo '<br>'; print_r(array_keys($table_labels));
		return $table_labels;
	}
	
	/**
	 * Excludes foreign fields from the given fields array based on config setting
	 *
	 * Note: In the Contact module this will NOT exclude the contact company field
	 *
	 * @see isExcludingForeignFieldSorting
	 * @param array $fields a data definition
	 * @return array the transformed data definition
	 * @author Phil Calder
	 */
	function excludeForeign($fields) {
		if ($this->isExcludingForeignFieldSorting()) {
			foreach ($fields as $k => $v) {
				if (isset($v['foreign']) && $v['foreign']) {  
					$fields[$k]['no_sort'] = 1;
				}
			}
		}
		return $fields;
	}
	
	/**
	 * Excludes custom fields from the given fields array
	 *
	 * @see isExcludingCustomFieldSorting
	 * @param array $fields a data_definition
	 * @return array the transformed data_definition
	 * @author Phil Calder
	 */
	function excludeCustom($fields) {
		if ($this->isExcludingCustomFieldSorting()) {
			foreach ($fields as $k => $v) {
				if ($v['field_is_custom']) {  
					// $fields[$k]['field_key'] =  '-1';
					$fields[$k]['no_sort'] = 1;
				}
			}
		}
		return $fields;
	}
	
	/**
	 * Returns the title row content and adds new and extra config links to the toolbar
	 * 
	 * @param string $title_text, optional
	 * @return string the title row
	 * @author Phil Calder
	 */
	function getTitleContent($title_text = '') {
		$standard_title_text = $this->getTitleText();
		$qfilter_text = $constraint_text = '';
		if (is_object($this->filters)) {
			$qfilter_text = $this->filters->getQfilterLabel();
		}
		if (strlen($title_text) && strlen($standard_title_text) && (substr($title_text, 0, strlen($standard_title_text)) == $standard_title_text)) {
			$standard_title_text = '';
		}
		$title_text = $standard_title_text . ( (strlen($standard_title_text) && strlen($title_text)) ? ' - ' : '') . $title_text;
		if (strlen($qfilter_text) && ((substr($title_text, (strlen($title_text) - (strlen($qfilter_text)+2)))) !== '(' . $qfilter_text . ')')) {
			$title_text .= ' (' . $qfilter_text . ')';
		}
		
		//MCR 20070318 break this into separate rows like item, for a toolbar
		$new_links = (array)$this->getNewLinks();
		$extra_links = (array)$this->getExtraTitleContentLinks();
		// print_r($new_links); print_r($extra_links); print '<hr>';
		// Phil 20070508 Add links to toolbar
		$this->output->setActionLinksMenu($new_links, $extra_links);
		
		$content = $this->output->getTitleRow(
			$title_text, 
			$this->getTitleRowLinks()
		);
		return $content;
	}

	/**
	 * builds the description row for the page
	 *
	 * @return string with the description
	 */
	function getDescriptionContent() {
		// $this->printDebug('getDescriptionContent');
		global $L, $SERVICES;
		$content = '';
		if ($this->language_overwrite['description']) {
			$description = $SERVICES->language->getLanguageText(
				$this->language_overwrite['description'], 
				$this->module_name
			);
		} else {
			$description = $L[$this->mod->getModuleNameLower() . 'DisplayDescription'];
		}
		if ($description) {
			// $content = $this->output->getTextRow($description);
			// Phil 20060223
			$padding = true;
			$content = $this->output->getBeginRow('', true) . '<td id="bodyCell"' . ($padding ? 'style="padding: 10px"' : '') . '>';
			$content .= $description;
			$content .= '</td>' . $this->output->getEndRow(true);
		}
		$content .= $this->addToDescription();
		return $content;
	}

	/**
	 * returns the paging row
	 *
	 * @return string the paging row
	 * @see getBrowsePagingContent
	 */
	function getPagingRow() {
		return $this->output->getPagingRow();
	}
	
	function getExtraTitleContentLinks() {
		$links = false;
		if ($this->has_alt_data_option) {
			if (is_object($this->output)) {
				global $L;
				$tag_extras = ' style="padding: 0px; margin: 0px;" ';
				// print '<div>has alt data option</div>';
				if ($this->use_alt_data_option) {
					$link_href = $php_self . $this->output->updateQueryString(array('alt_data_option' => '0'));
					$text = $L['LinkStandardView'];
					
					$links[] = array(
						'href' => $link_href,
						'text' => $text, 
						'title' => $text, 
						'icon' => 'list_standard.png'
					);
				} else {
					$link_href = $php_self . $this->output->updateQueryString(array('alt_data_option' => '1'));
					$text = $L['LinkAlternativeView'];
					
					$links[] = array(
						'href' => $link_href,
						'text' => $text, 
						'title' => $text, 
						'icon' => 'list_detail.png'
					);
				}
				
			}
		}
		return $links;
	}

	/**
	 * Adds a value to the description row
	 *
	 * Override this method to return the extra content for the description row
	 *
	 * @return the value you want to add to the description row
	 */
	function addToDescription() {}

	/**
	 * @return mixed A contact id from filters or form values
	 * @author Phil Calder
	 */
	function getContactId() {
		if ($this->useFilters()) {
			$contact_id = $this->filters->getContactFilter();
		} else {
			$contact_id = $this->form_values['contact_id'];
		}
		return $contact_id;
	}

	/**
	 * Checks if the value passed is a data option
	 *
	 * @param string $data_option the data option you are checking for
	 * @param boolean true if exists
	 * @author Phil Calder
	 */
	function isDataOption($data_option) {
		$data_definition = $this->db_functions->getDataDefinition();
		return isset($data_definition['data_options'][$data_option]);
	}
	
	/**
	 * @param mixed $formatting - Will set the link cell formatting to this if it is an array. Otherwise, default values are set. 
	 * @author Phil Calder
	 * @see buildListTable
	 */
	function setLinkCellFormatting($formatting = false) {
		// Phil 20050810 - Output will use these style setting for cells with these keys
		// eg view, edit, delete links. See buildListTable
		if (!is_array($formatting)) {
			$this->cell_formatting = array(
				'view' => ' style="width:50" ',
				'edit' => ' style="width:50" ',
				'delete' => ' style="width:50" '
			);
		} else {
			$this->cell_formatting = $formatting;
		}
	}
	
	/**
	 * @param int $id An item id
	 * @return boolean Whether the given id is an alerting item
	 * @author Phil Calder
	 */
	function getItemIsAlert($id) {
		if ($id && $this->showAlertStatus()) {
			if (!$this->useGetItemsIterator()) {
				if ( is_array($this->alert_ids) && in_array($id, $this->alert_ids) ) {
					// print '<div>no iterator, alert - true</div>';
					return true;
				} else {
					// print '<div>no iterator, alert - false</div>';
					return false;
				}
			} else {
				// print '<div>iterator, alert - todo</div>';
				$is_alert = $this->db_functions->getAlertIds(false, (array)$id);
				// print_r($is_alert); print '<hr>';
				// todo
				return (boolean)$is_alert;
			}
			
		}
		return false;
	}

	/**
	 * builds a table row
	 *
	 * @param array $item the items array
	 * @param boolean $has_alerts Whether the current data set has alerts
	 * @return string returns a table row with data.
	 */
	function buildListTable($item, $has_alerts = false, $style = false) {
		// print '<div>buildListTable item:</div>'; print_r($item); print '<hr>';
		// check to make sure we have an array!
		if (!is_array($item)) {
			return '';
		}

		$this->data_row_counter++;
		//table row items as an array
		$style_flag = array();
		if ($has_alerts) {
			if ($this->getItemIsAlert($item['id'])) {
				$style_flag[] = 'rAlert';
			}
		}
		
		if ($this->showSeenStatus()) { // Phil 20051114 Will not display seen status (flag) if false
			if (array_key_exists('seen', $item)) {
				switch ($item['seen']) {
					case 0:
						$style_flag[] = 'rUnseen';
					break;
					case 2:
						$style_flag[] = 'rEdited';
					break;
					// default:
					//	do nothing
					// break;
				}
			}
		}

		if ( ($highlight_id = $this->getHighlightId()) && ($item['id'] == $highlight_id) ) {
			$style_flag[] = 'rHighLight';
		}

		if ($style) {
			$style_flag[] = $style;
		}
		
		$table_row = $this->buildStringFieldRow($item);
		
		// --
		// [Phil 20060130] todo, pass id and links to a new 'output' function
		// eg $table['__links'] = $this->output->getTableLinks($this->getTableLinks($item), $item['id']);
		// $table_links = $this->getTableLinks($item);
		
		if ($this->showListLinks()) {
			list($__link, $__menu) = $this->getListTableLinks($item, $this->getHtmlIdPrefix() . $this->data_row_counter);
			$table_row['__links'] = $__link . '</th><th>' . $__menu;
		}
		//write row; style can be 'new' or 'alert'
		// Phil 20050810 - Uses class variable cell_formatting to format item links (eg view, edit, delete)
		return $this->output->getTableRow($table_row, $style_flag, $this->cell_formatting, $this->getHtmlIdPrefix() . $this->data_row_counter);
	}

	/**
	 * Gets a unique piece identifier to prefix to the html id
	 * is overriden in webmat_pieces_view
	 *
	 * @return string unique piece identifier
	 * @author Matthew Ross
	 *
	 */
	function getHtmlIdPrefix() {
	}
	
	/**
	 * Gets the table links and adds them to the item row
	 *
	 * @param array $table_row
	 * @param array $cell_formatting Optional array to add per-cell formatting see {@link Output::getTableRow}
	 * @return array the table row array with the extra values.
	 */
	function getItemLinks($table_row, $item) {
		global $SERVICES;
		if (!is_array($table_row)) {
			$table_row = array();
		}
		foreach ($this->getTableLinks($item) as $k => $v) {
			//you can remove links here
			$table_row[$k] = $v;
		}
		return $table_row;
	}
	
	/**
	 * Whether to show list page 'links' (edit link and menus for each item)
	 *
	 * @return boolean Default is true. Override to disable.
	 * @author Phil Calder
	 */
	function showListLinks() {
		return true;
	}
	
	/**
	 * Whether to show list page 'flags' (unseen, edited, alerts)
	 *
	 * @return boolean Default is true. Override to disable.
	 * @author Phil Calder
	 */
	function showListFlags() {
		return true;
	}
	
	/**
	 * gets the content for a table cell
	 *
	 * @param array $item the items array
	 * @return string the table cell content.
	 */
	function buildStringFieldRow($item) {
		
		$cells = array();
		if ($this->showCheckBoxMenuRow()) {
			$cells['__cb'] = $this->getItemCheckbox($item['id']);
		}
		
		if ($this->showListLinks()) {
			$cells['__links'] = ''; // '[edit]'; // view, edit, etc, links
		}
		if ($this->showListFlags()) {
			$cells['__flags'] = ''; // '[flags]'; // alert, unseen/edited, etc
		}
		
		$dd_item = $this->dd->getDataStructureWithData($item);
		// print '<div>' . get_class($this) . '::buildStringFieldRow ' . (method_exists($this, 'getPieceId') ? 'piece: ' . $this->getPieceId() : '') . ' dd structure keys:</div>'; print_r(array_keys($dd_item)); print '<hr>';
		$this->calculateValues($dd_item, $item);
		bcrm_unset($dd_item, 'current_item_id');
		
		foreach ($dd_item as $k => $field) {
			if ($field['type'] == 'contact') {
				if (isset($field['value'])
					&& !is_array($field['value'])
					&& isset($item['_' . $field['field_key'] . '_display_name'])
				) {
					$field['value'] = array(
						'id' => $field['value'],
						'display_name' => $item['_' . $field['field_key'] . '_display_name']
					);
				}
				$cells[$field['field_key']] = $this->getStringField($field);
			} else if ($field['display_name'] && !$field['no_link'] && !in_array($field['type'], array('email', 'url', 'contact'))) {
				// add a view link to the display name
				if ($view_href = $this->getFileSetHref('view')) {
					// print '<p>WebMATList::buildStringFieldRow:</p>'; print_r($field); print_r($item); print '<hr>';
					
					global $L;
					$text = (strlen(trim($field['value'])) ? trim($field['value']) : $L['noName']);
					
					$xajax_icon = false;
					//MCR icon (link) for xajax view
					if (isset($item['id']) && $item['id'] && $this->showXajaxPopupIcon()) {
						$xajax_icon = ' <span onclick="xajax_xajaxView(' . $item['id'] . ',\'' . $this->module_name . '\'); return false;">' . $this->output->getIconXajaxPopup() .'</span> ';
					}
						
					// Phil - display icon to left of display link
					
					if ( ($this->getModuleName() == 'Contact') && isset($item['id']) && $item['id']) { // Phil 20060504
						// $cells[$field['field_key']] = $this->output->getLinkContact(
							// array(
								// 'id'=>$item['id'], 
								// 'display_name'=> $text
							// )
						// ) . $xajax_icon;
						$cells[$field['field_key']] = $xajax_icon . $this->output->getLinkContact(
							array(
								'id'=>$item['id'], 
								'display_name'=> $text
							)
						);
					} else {
						$view_href = $this->getModulePath() . $view_href;
						
						$view_link = $this->output->getLinkHref(
							array(
								// 'text' => $field['value'], 
								'text' => $text, 
								'href' => $view_href
							)
						);
						
						$view_link = $this->replaceLinkVariables($item['id'], $item, array($view_link));
						
						// $cells[$field['field_key']] = current($view_link) . $xajax_icon;
						$cells[$field['field_key']] = $xajax_icon . current($view_link);
						// print $cells[$field['field_key']] . '<hr>';
					}
				} else {
					$cells[$field['field_key']] = $this->getStringField($field);
				}
			} else {
				$cells[$field['field_key']] = $this->getStringField($field);
			}
			
			
		}
		return $cells;
	}
	
	/**
	 * Wrapper function for Data Dictionary 
	 *
	 * @return boolean The current no trim value
	 * @author Phil Calder
	 */
	function getNoTrim() {
		if (!is_object($this->dd)) {
			global $SERVICES;
			$this->dd = $this->getDataDictionary();
		}
		return $this->dd->getNoTrim();
	}
	
	/**
	 * Calls Output 
	 *
	 * @param boolean $value The no trim value to set (Optional, default true)
	 * @return boolean The current no trim value
	 * @author Phil Calder
	 */
	function setNoTrim($value = true) {
		if (!is_object($this->output)) {
			global $SERVICES;
			$this->output =& $SERVICES->getService('Output');
		}
		return $this->output->setNoTrim($value);
	}
	
	/**
	 * returns a checkbox used for selecting rows
	 *
	 * @param string $id the item id
	 * @return mixed false or a string containing the item checkbox
	 */
	function getItemCheckbox($id =true) { // Phil - 20050503 - only if list has items
		if ( 
		 	($this->getFileSetHref('multi_edit') || $this->getFileSetHref('mass_edit') || $this->getFileSetHref('mass_delete')) 
		) {
			if ($id) {
				$checkbox = $this->output->getHtmlField(
					array(
						'label'=>'', 
						'field_key'=>'_select_checkbox][', 
						'value'=>$id, 
						'html_id'=> 'cb' . $this->getHtmlIdPrefix() . $this->data_row_counter, 
						'type'=>'checkbox', 
						'checked'=>0, 
						'tag_extras'=>'onclick="chgCB(\'' . $this->getHtmlIdPrefix() . $this->data_row_counter . '\');"'
						/*'tag_extras'=>'onchange="toggleMain();"'*/
					)
				);
			} else if ($this->showCheckboxMenuRow()) {
				$checkbox = $this->output->getHtmlMasterCheckbox(
					'_select_checkbox', 
					false, 
					'select_item_form'
				);
				
			}
			return $checkbox;
		}
		return false;
	}

	/**
	 * get the first form tag for the Select ItemForm
	 *
	 * @return string the begin form tag
	 */
	function beginSelectItemForm() {
		return $this->output->getBeginForm($this->getFileSetHref('process_multi'), false, false, false, 'select_item_form');
	}

	function getCheckboxMenuItems() {
		$val = '';
		$items = array();
		if ($val = $this->getMultiEditItem()) {
			$items = bcrm_array_merge($items, $val);
		}
		if ($val = $this->getMassEditItem()) {
			$items = bcrm_array_merge($items, $val);
		}
		if ($val = $this->getMassDeleteItem()) {
			$items = bcrm_array_merge($items, $val);
		}
		if ($val = $this->getMassCreateItem()) { // Phil 20060831
			$this->masscreate = $val['contact_fields'];
			bcrm_unset($val, 'contact_fields');
			$items = bcrm_array_merge($items, $val);
		}
		if ($val = $this->getSelectItems()) {
			$items = bcrm_array_merge($items, $val);
		}
		if ($val = $this->getSelectContacts()) {
			$this->select_contacts = $val['contact_fields'];
			bcrm_unset($val, 'contact_fields');
			$items = bcrm_array_merge($items, $val);
		}
		if ($val = $this->getMailmergeContacts()) { // Phil 20050506
			$this->mailmerge = $val['contact_fields'];
			bcrm_unset($val, 'contact_fields');
			$items = bcrm_array_merge($items, $val);
		}
		if ($val = $this->getQuickExportItems()) {
			$items = bcrm_array_merge($items, $val);
		}
		if ($val = $this->getExportItems()) {
			$items = bcrm_array_merge($items, $val);

		}
		//('---To Do---', 'View', 'Print View', 'Export (CSV)', 'Mail Merge', 'Add Note', 'Add Task', 'Add Appointment');
		$example = array();
		$items = bcrm_array_merge($items, $example);
		return $items;
	}

	/**
	 * returns the extra items to add to the checkbox menu
	 *
	 * @return string the second menu row
	 */
	function getCheckboxMenuExtraItems() {
		if (is_array($this->checkbox_menu_extra_items)) {
			return $this->checkbox_menu_extra_items;
		}
		return array();
	}
	
	/**
	 * adds links below the menu
	 *
	 * @param array a array with the link info (with the keys 'href', 'text' , 'title'
	 * and 'tag_extras')
	 */
	function addCheckboxMenuExtraItems($items) {
		if (is_array($items)) {
			if (!$this->checkbox_menu_extra_items) {
				$this->checkbox_menu_extra_items = array();
			}
			$this->checkbox_menu_extra_items = bcrm_array_merge($this->checkbox_menu_extra_items, $items);
		}
	}

	/**
	 * returns the mass edit link
	 *
	 * @return string the mass edit link
	 */
	function getSelectItems() {
		if ($this->getFileSetHref('process_multi')) {
			global $L;
			return array('select_ids' => $L['labelSelectIds']);
		}
		return false;
	}

	/**
	 * returns the mass edit link
	 *
	 * @return string the mass edit link
	 */
	function getSelectContacts() {
		$this->output->setWhitespaceCompressionSwitch(false);
		if ($this->getFileSetHref('process_multi')) {
			
			$dd_items = $this->getDataDictionary(); // Phil 20051027 using wrapper function
			$contact_fields = array();
			foreach ($dd_items as $k => $v) {
				if ($v['type'] == 'contact') {
					$contact_fields[] = $v['field_key'];
				}
			}

			if (count($contact_fields)) {
				global $L;
				return array('select_contacts' => $L['labelSelectContacts'], 'contact_fields' => implode(',', $contact_fields));
			}
		}
		return false;
	}

	/**
	 * returns the mail merge link
	 *
	 * @return string the mail merge link
	 * @author Phil Calder
	 * @todo Remove duplication from this function and WebMATList::getSelectContacts() (ie write function getContactFields())
	 */
	function getMailmergeContacts() {
		$this->output->setWhitespaceCompressionSwitch(false);
		if ($this->getFileSetHref('process_multi')) {
			$dd_items = $this->getDataDictionary(); // Phil 20051027 using wrapper function
			$contact_fields = array();
			foreach ($dd_items as $k => $v) {
				if ($v['type'] == 'contact') {
					$contact_fields[] = $v['field_key'];
				}
			}
			if (count($contact_fields)) {
				global $L;
				return array('mailmerge' => $L['labelEmailmergeContacts'], 'contact_fields' => implode(',', $contact_fields));
			}
		}
		return false;
	}

	/**
	 * returns the mass edit link
	 *
	 * @return string the mass edit link
	 */
	function getMassEditItem() {
		if ($link_href = $this->getFileSetHref('mass_edit')) {
			global $L;
			return array('mass_edit' => $L['labelMassEdit']);
		}
		return false;
	}
	
	/**
	 * returns the mass delete link
	 *
	 * @return string the mass delete link
	 */
	function getMassDeleteItem() {
		if ($link_href = $this->getFileSetHref('mass_delete')) {
			global $L;
			return array('mass_delete' => $L['labelMassDelete']);
		}
		return false;
	}
	/**
	 * returns the mass create link
	 *
	 * @return string the mass create link
	 * @author Phil Calder
	 */
	function getMassCreateItem() {
		global $L;
		// return array('mass_create' => $L['labelMassCreate']);
		
		$this->output->setWhitespaceCompressionSwitch(false);
		if ($this->getFileSetHref('process_multi')) {
			$dd_items = $this->getDataDictionary(); // Phil 20051027 using wrapper function
			$contact_fields = array();
			// $has_contact_id = false;
			foreach ($dd_items as $k => $v) {
				if ($v['type'] == 'contact') {
					$contact_fields[] = $v['field_key'];
				}
				// if ( ($v['type'] == 'contact') && ($v['field_key'] == 'contact_id') ) {
					// $has_contact_id = true;
				// }
			}
			if (count($contact_fields)) {
			// if ($has_contact_id) {
				global $L;
				// return array('mass_create' => $L['labelMassCreate']);
				return array('mass_create' => $L['labelMassCreate'], 'contact_fields' => implode(',', $contact_fields));
			}
		}
		return false;
	}

	/**
	 * returns the multi edit link
	 *
	 * @return string the multi edit link
	 */
	function getMultiEditItem() {
		if ($link_href = $this->getFileSetHref('multi_edit')) {
			global $L;
			return array('multi_edit' => $L['labelMultiEdit']);
		}
	}

	/**
	 * returns the export link
	 *
	 * @return string the export link
	 */
	function getQuickExportItems() {
		if ($link_href = $this->getFileSetHref('quickexport')) {
			global $L;
			return array('quickexport' => $L['labelQuickExport']);
		}
	}

	function getExportItems() {
		if ($link_href = $this->getFileSetHref('export')) {
			global $L;
			return array('export' => $L['labelExport']);
		}
	}

	/**
	 * Gets the header row from the output class
	 *
	 * @param array $table_lables
	 * @param array $sort
	 * @return string the header row
	 */
	function getSubTableHeader($table_labels, $sort) {
		$table_labels = $this->excludeCalculated($table_labels);
		$table_labels = $this->excludeForeign($table_labels);
		$table_labels = $this->excludeCustom($table_labels);
		
		if ($item_checkbox = $this->getItemCheckbox(false)) {
			if (array_key_exists('__cb', $table_labels)) {
				$table_labels['__cb'] = $item_checkbox;
				$labels = $table_labels;
			} else {
				$labels = bcrm_array_merge(array('__cb'=>$item_checkbox), $table_labels);
			}
		} else {
			$labels = $table_labels;
		}
		
		if ($config_icon = $this->getConfigIcon()) {
			$labels['__links'] = $config_icon;
		}
		
		if ($this->useFilters() && $this->showSearchFilterForm() ) { // Phil 20050503
			$search_filter = true;
		} else {
			$search_filter = false;
		}
		
		// Phil 20050809 We need to be able to specify that search filtering is off 
		// (ie contact view, desktop, and maybe dashboard)
		$extra_action_query = $this->getExtraActionQuery();
		return $this->output->getHeaderRow($labels, $sort,$search_filter, $extra_action_query);
	}
	
	/**
	 * @return string $content - Extra query string for use in search form
	 *
	 * @author Phil Calder
	 * @see getSubTableHeader
	 */
	function getExtraActionQuery() {
		$content = '';
		if (isset($this->form_values['adv_filter']) && $this->form_values['adv_filter']) {
			$content = 'adv_filter=1';
		} else if (isset($this->form_values['bookmark']) && $this->form_values['bookmark']) {
			$content = 'adv_filter=1'; // Yes, same as above
		}
		return $content;
	}
	
	/**
	 * Disabled
	 *
	 * @author Phil Calder
	 * @access private
	 */
	function getConfigIcon() {
		// Phil 20060413 Temporarily disabled this pending further development
		// Also commented "$this->output->req_config_js = true;" from init()
		return false;
		
		// ADD CONFIG ICON TO HEADER ROW
		if ($this->output->req_config_js) { // Phil 20060209 Added if
			$labels['__links'] = '
				<a href="javascript:showConfig(\'config\');" id="config">
					<img src="' . $this->output->getSkinWebPath() . 'img/config.gif" width="11" height="13" title="Configure the view of the data">
				</a>
					<img src="' . $this->output->getSkinWebPath() . 'img/configsort.gif" width="11" height="13" border="0" title="Other Data Sorting Options">
			';
		}		
	}

	/**
	 * returns the 'new' menu row
	 *
	 * @return string the 'new' menu row
	 */
	function getNewMenuContent() {
		foreach ($this->new_menu_links as $k => $v) {
			if (!$v) {
				bcrm_unset($this->new_menu_links, $k);
			}
		}
		if (count($this->new_menu_links)) {
			return $this->output->getMenuRow( implode($this->getMenuSeparator(), $this->new_menu_links), 2, 'lNewMenuCell' );
		}
	}

	/**
	 * add a link to the 'new' menu row
	 *
	 */
	function addNewMenuLink($link) {
		$this->new_menu_links[] = $link;
	}


	/**
	 * returns the second menu row
	 *
	 * @return string the second menu row
	 */
	function getSecondMenuRow() {
		if (count($this->second_menu_links)) {
			//return $this->output->getMenuRow( implode($this->getMenuSeparator(), $this->second_menu_links) );
			return $this->second_menu_links;
		}
	}

	/**
	 * adds links below the menu row
	 *
	 * @param array a array with the link info (with the keys 'href', 'text' , 'title'
	 * and 'tag_extras')
	 */
	function addSecondMenuLinks($links) {
		if (!$this->second_menu_links) {
			$this->second_menu_links = array();
		}
		if (isset($links['href'])) {
			$this->second_menu_links[] = $this->output->getLinkHref($links);
		} else {
			foreach ($links as $k => $v) {
				//$this->second_menu_links[] = $this->output->getLinkHref($v);
				$this->second_menu_links[] = $v;
			}
		}
	}
	
	/**
	 * gets the filter info
	 *
	 * @return array an array of filter information
	 */
	function getConfigFilterInfo() {
		if ($this->useFilters()) {
			return $this->filters->getConfigFilterInfo();
		} else {
			return false;
		}
	}

	function setSortTableName($sort_table = '') {
		if ($sort_table) {
			$this->sort_table = $sort_table;
			$this->output->sort_table = $this->sort_table;
		}
	}

	/**
	 * if sort = '' it returns the user pref for sort, else sets the user pref for sort and returns the sort values.
	 *
	 * @param string $contact_id the contact id if required
	 * @return array sort
	 */
	function sortTable($sort = false) {
		global $SERVICES;
		$update_pref = true; // Phil 20051104
		$SERVICES->getService('Preferences');
		if (!is_array($sort)) {
			if ($SERVICES->preferences->getUserPref($this->pref_name . 'SortField', 0)) {
				$sort['field'] = $SERVICES->preferences->getUserPref($this->pref_name . 'SortField', 0);
				$sort['order'] = $SERVICES->preferences->getUserPref($this->pref_name . 'SortOrder', 0);
			} else {
				$sort = $this->getDefaultSort();
			}
			$update_pref = false;
		}
		
		//print_r($sort);
//		if ($sort['field'] == '_contact_id.display_name') {
//			//echo 'here';
//			//
//			$sort['field'] = 'Contact__item.display_name';
//		}
		
		if ($this->sort_table && $this->form_values[$this->sort_table]['sort']) {
			$sort = $this->form_values[$this->sort_table]['sort'];
			$update_pref = true;
		}
		
		if (is_array($sort) && !isset($sort['sort_table']) && $this->sort_table) { // Phil 20050503
			$sort['sort_table'] = $this->sort_table;
		}
		
		if ($this->isExcludingForeignFieldSorting() && $this->isForeignField($sort['field'])) {
			$sort = array('field'=>'', 'order'=>'');
			$update_pref = true;
		} 
		
		// Phil 20051109 Prevent trying to sort on a field that is not in the data dictionary
		if (isset($sort['field']) && $sort['field']) {
			$data_dictionary = $this->getDataDictionary();
			//print_r($data_dictionary);print_r($this->db_functions->columns);

			// Very ugly way to do this
			// Phil 20060824 - added tmp_keys variable prevent PHP 5.05 fatal error
			if (is_array($data_dictionary) && !is_numeric(current($tmp_keys = array_keys($data_dictionary))) ) {
				if (
					!array_key_exists(trim($sort['field'], '_'), $data_dictionary) 
					&& (is_array($this->db_functions->columns) && !in_array($sort['field'], $this->db_functions->columns))
				) {
					//print '<div>----excluding sort field not in data dictionary or columns----------</div>';
					//$sort = array('field'=>'', 'order'=>'');	
				}
			}
		}
		
		if ($update_pref && is_array($sort)) {
			$SERVICES->preferences->setUserPref($this->pref_name . 'SortField', $sort['field']);
			$SERVICES->preferences->setUserPref($this->pref_name . 'SortOrder', $sort['order']);
		}
		
		return $sort;
	}
	
	function isForeignField($field_name = false) {
		if ($field_name) {
			if (!$this->dd) {
				$this->dd = $this->getDataDictionary();
			}
			return $this->dd->isForeignField($field_name);
		}
		return false;
	}
	
	/**
	 * @author Phil Calder
	 */
	function getDefaultSort() {
		return 'none'; //Can set default values here
	}
	
	function startDataBlock() {
		return $this->output->startDataBlock();
	}
	
	function endDataBlock() {
		return $this->output->endDataBlock();
	}
	
	/**
	 * runs before the items are got from the databse
	 */
	function beforeGetItems(&$sort, &$limit) {
		// Jack 20060120
	}

	/**
	 * Runs after the items are got from the database
	 *
	 * Note: if {@link useGetItemsIterator} returns true, this function will/should not be called. 
	 * For module that require afterGetItems functionality, override useGetItemsIterator to return false
	 * Phil Calder 20051201
	 */
	function afterGetItems(&$items) {
	}

	/**
	 * Override this to set the master ids when using WebMATDatabaseDetailTable
	 */
	function setMasterIds() {
		//$this->db_functions->setMasterIds($this->form_values['id']);
	}
	
	// ----- deprecated -----
	
	/**
	 * Builds a nested table displaying the main content for the list page
	 *
	 * @return string returns the nested table with data
	 * @see getDataContent
	 * @deprecated version 4.5
	 */
	function getTable() {
		return $this->getDataContent(); 
	}
	
	/**
	 * returns the checkbox menu row
	 *
	 * @return string the checkbox menu row
	 * @deprecated 20060117
	 * @see getCheckboxMenuContent
	 */
	function getCheckboxMenuRow() {
		return $this->getCheckboxMenuContent();
	}
	
	/**
	 * @deprecated 20050117
	 * @see addCheckboxMenuExtraItems
	 */
	function addCheckboxMenuRowExtraItems($items) {
		$this->addCheckboxMenuExtraItems($items);
	}
	
	/**
	 * @deprecated 20060119
	 * @see getConfigFilterInfo
	 */
	function getFilterInfo() {
		return $this->getConfigFilterInfo();
	}
	
	/**
	 * Returns the preferences button
	 *
	 * see commented code for an example on how to get the preferences button
	 * @deprecated version 4
	 */
	function getPrefLink() {
		// Usually as below.
		//return $this->output->getLinkPref( array('pref1', 'pref2') )
	}
}
?>