<?php
/**
 * The menu class
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 */

includeWebMAT('list.inc');

class WebMATStandardMenu extends WebMATList{
	
	/**
	 * Constructor function sets all the class variables
	 *
	 * @param  string  $module  the current module name
	 * @param  array  $post  the post array
	 * @param  array  $get  the get array
	 * @param  array $language_overwrite has the keys title and description
	 */
	function WebMATStandardMenu($module, $post, $get, $menu_links, $language_overwrite = false) {
		global $SERVICES;
		// print '<div>language_overwrite: ' . $language_overwrite . '</div>'; print_r($language_overwrite); print '<hr>';
		$this->module_name = $module;
		$SERVICES->getService('Modules');
		$this->mod =& $SERVICES->modules->getModule($this->module_name);
		$this->module_name_lower = $this->mod->getModuleNameLower();
		// $this->form_values = array_merge_recursive($post, $get);
		$this->form_values = $this->arrayMergeReplace($post, $get);
		$this->menu_links = $menu_links;
		$this->language_overwrite = $language_overwrite;
		$this->output =& $SERVICES->getService('Output');
	}
	
	/**
	 * Overrides the WebMATList constructor function to disable it
	 */
	function WebMATList() {}
	
	/**
	 * Optional title (default is blank)
	 *
	 * @return string title
	 */
	function getMenuLinksTitle() {
		return '';
	}
	
	/**
	 * Return the rows of menu links
	 * 
	 * @return string the table rows of menu links
	 */
	function getMenuLinks($menu = false) {
		$menu_links = '';
		
		if (is_array($this->menu_links)) {
			foreach ($this->menu_links as $k => $v) {
				// print '<div>k='. $k . '</div>'; print_r($v); print '<hr>';
				$v = str_replace(array('%id%', '%contact_id%') , array($this->form_values['id'], $this->form_values['id']), $v);
				
				if (isset($v['href'])) { // contains 1 link (array)
				
					if ($menu) {
						// print '<br>1<hr>';
						$menu_links .= $this->output->getLinkHref($v) . "\n";
					} else {
						// print '<br>2<hr>';
						$menu_links .= $this->output->getTextRow($this->output->getLinkHref($v)) . "\n";
					}
				} else { // may contain multiple link arrays
					if (is_array($v)) {
						$tmp_links = array();
						foreach ($v as $kk => $vv) {
							if (isset($vv['href'])) { 
				
								// if ($menu) {
									// print '<br>1<hr>';
								$tmp_links[] = $this->output->getLinkHref($vv);
								// } else {
									// print '<br>2<hr>';
									// $tmp_links[] = $this->output->getTextRow($this->output->getLinkHref($vv));
								// }
							}
						}
						if (count($tmp_links)) {
							if ($menu) {
								// $menu_links .= '<div>' . implode('&nbsp;|&nbsp;', $tmp_links) . "</div>\n";
								$menu_links .= '<div>' . implode(' ', $tmp_links) . "</div>\n";
							} else {
								// $menu_links .= $this->output->getTextRow(implode('&nbsp;|&nbsp;', $tmp_links)) . "\n";
								$menu_links .= $this->output->getTextRow(implode(' ', $tmp_links)) . "\n";
							}
						}
					}
				}
			}
		}
		return $menu_links;
	}
	
	function getLinks() {
		// print '<div>getLinks</div>';
		// print '<div>getConfigMenuLinks</div>'; print_r($this->getConfigMenuLinks()); print '<hr>';
		$this->menu_links = $this->getConfigMenuLinks();
		return $this->menu_links;
	}
	
	/**
	 * Adds the extra links from the module.config
	 * 
	 * @return array An array of extra config links
	 */
	function getConfigMenuLinks($suffix = 'menu_links') {
		if (!isset($this->config_menu_links)) { // Phil - need to check not set - may be set but not have any extra links
			global $SERVICES;
			$config =& $SERVICES->getService('Config');
			$language =& $SERVICES->getService('Language');
			$table_name = $this->getTableName();
			$extra_links = $config->lookup('modules.' .  $this->module_name . '.file_set.' . $table_name . '.' . $suffix);
			// print '<div>extra links for [modules.' .  $this->module_name . '.file_set.' . $table_name . '.' . $suffix . ']</div>'; print_r($extra_links); print '<hr>';
			$links = array();
			
			if (is_array($extra_links)) {
				$module_path = $this->getModulePath();
				foreach ($extra_links as $k => $v) {
					if (! ($href = $this->getFileSetHref($v)) ) {
						$href = $v;
					}
					// Phil 20050906 - Objects  can be part of combined view pages such as desktop and contact view
					// Need to link to correct module
					$href = $module_path . $href; 
					$text = $language->getLanguageText($k, $this->module_name);
					$links[$k] = array(
						'href' => $href, 
						'text' => $text
					);
				}
			}
			$this->config_menu_links = $links;
		}
		$this->appendContactLinks(); // Phil 20060222
		return $this->config_menu_links;
	}
	
	function getTableName() {
		$table_name = parent::getTableName();
		if (!$table_name) {
			$table_name = 'item';
		}
		return $table_name;
	}
	
	/**
	 * Appends contact related links to config menu links
	 * @author Phil Calder
	 */
	function appendContactLinks() {
		if (!isset($this->form_values['id']) || !$this->form_values['id']) {
			return;
		}
		
		if (!isset($this->config_menu_links) || !is_array($this->config_menu_links)) {
			$this->config_menu_links = array();
		}
		
		global $SERVICES, $L;
		if (is_object($config =& $SERVICES->getService('Config'))) {
			
			$module_list = $config->lookup('modules.Contact.menu');
			if (is_array($module_list)) {
				// print '<div>module list:</div>'; print_r($module_list); print '<hr>';
				// Phil 20060223 Dont need this in the menu, but need the config setting for the module
				bcrm_unset($module_list, 'TemplateNewModule'); 
				foreach ($module_list as $k => $v) {
					if (isset($v['module']) && $v['module']) {
						// print '<div>module: ' . $v['module'] . '</div>';
						// print '<div>module: ' . $v['module'] . '<br>tables:</div>'; print_r($v['tables']); print '<hr>';
						if (isset($v['tables']) && is_array($v['tables'])) {
							foreach ($v['tables'] as $tk => $tv) {
								// print '<div>table: ' . $tv . '</div>';
								$new_link = $list_link = '';
								
								/*
								// WIP - formatting issues in iframe
								if ($list_href = $config->lookup('modules.' . $v['module'] . '.file_set.' . $tv . '.functions.list')) {
									if ($SERVICES->modules->isCustom($v['module'])) {
										$path = '../../modules_custom/' . $v['module'];
									} else {
										$path = '../../modules/' . $v['module'] ;
									}
									$list_href = $path . '/' . str_replace(array('%id%', '%contact_id%') , array('', $this->form_values['id']), $list_href);
									// print '<div>href: ' . $href . '</div>';
									
									// (see below) could use strtolower( substr($v['module'], 0, 1) ) . substr($v['module'], 1) . Title_itemlist
									$list_label = $L['ListView'];
									
									$list_link = array('href'=>$list_href, 'text'=> $list_label);
								}
								*/
								
								if ($new_href = $config->lookup('modules.' . $v['module'] . '.file_set.' . $tv . '.functions.edit')) {
									// if ($SERVICES->modules->isCustom($v['module'])) {
										// $path = '../../modules_custom/' . $v['module'];
									// } else {
										// $path = '../../modules/' . $v['module'] ;
									// }
									$path = $SERVICES->modules->getModulePath($v['module'], true);
									
									$new_href = $path . '/' . str_replace(array('%id%', '%contact_id%') , array('', $this->form_values['id']), $new_href);
									// print '<div>href: ' . $href . '</div>';
									$new_label_key = strtolower( substr($v['module'], 0, 1) ) . substr($v['module'], 1) . 'TitleNew_itemedit';
									$new_label = $SERVICES->language->getLanguageText($new_label_key, $v['module']);
									// $this->config_menu_links[] = array('href'=>$new_href, 'text'=> $new_label);
									$new_link = array('href'=>$new_href, 'text'=> $new_label);
								}
								
								if (is_array($list_link) && is_array($new_link)) {
									$this->config_menu_links[] = array(
										$list_link, 
										$new_link
									);
								} else {
									if (is_array($list_link)) {
										$this->config_menu_links[] = $list_link;
									} else if (is_array($new_link)) {
										$this->config_menu_links[] = $new_link;
									}
								}
								
							}
						}
					}
				}
			}
		}
	}
	
	/** 
	 * @author Phil Calder
	 */
	function getTitleText($prefix = 'Title_') {
		global $L;
		$content = $L['titleWebMATMenu'];
		$suffix = WebMATDisplay::getTitleText($prefix);
		if (strlen($suffix)) {
			$content .= ': ' . $suffix;
		}
		return $content;
	}	
	
	/**
	 * Bulids the menu page
	 * 
	 * @param boolean $get_links Optional, default false. When true, will re-get links from {@link getLinks}
	 */
	function main($get_links = false, $menu = false) { // Phil 20060203 get_links
		if ($get_links) {
			$this->getLinks();	
		}
		if (!$menu) {
			global $L;
			$menu_links = $this->getMenuLinks($menu);
			if (!strlen($menu_links)) {
				$menu_links = $L['msgNoItemsForView'];
			} else {
				$menu_links = $this->output->getBeginRow() . $this->output->getBeginCell() . $this->getMenuLinksTitle() . $this->output->getEndCell() . $this->output->getEndRow() . $menu_links;	
			}
			// print '<div>menu_links: ' . $menu_links . '</div>';
			// print '<div>title text: ' . $this->getTitleText() . '</div>';
			$content = '';
			$content .= $this->getBeginMainContent();
			$content .= $this->output->getTitleRow($this->getTitleText());
			$content .= $this->getBeginDataCell();
			$content .= $this->output->getBeginTable('adminTbl');
			$content .= $this->getDescriptionContent();
			$content .= $menu_links;
			$content .= $this->output->getEndTable();
			$content .= $this->getEndDataCell();
			$content .= $this->getEndMainContent();
			$this->output->addToBody($content);
			$this->output->display();
		} else {
			$content = '';
			// $content = '<div style="font-size: 8px; padding: 0px; margin: 0px">';
			$content .= $this->getMenuLinks($menu);
			// $content .= '</div>';
			echo $content;
		}
	}
}
?>