<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 */

includeWebMAT('edit.inc');

/**
 * This class handles the Multi-edit view for the content
 *
 * An extended version of the edit page, this class enables you to edit more then one
 * record at once.
 *
 * @author Matthew Corich
 * @package WebMAT
 */
class WebMATStandardMultiEdit extends WebMATEdit{
	
	/**
	 * @var The number of blank rows to display
	 * @access private
	 */
	var $extra_rows = 0;
	
	/**
	 * @var The current row number
	 * @access private
	 */
	var $row_number = 0;
	
	var $extra_rows_selector_switch = false;
	
	/**
	 * returns the WebMat type
	 */
	function getWebmatType() {
		return 'multi_edit';
	}
	
	/**
	 * Checks if for a delete or update runs the submit function
	 * 
	 * Changed to add extra rows and paging info
	 */
	function run() {
		if (isset($this->form_values['submit'])) {
			$this->db_functions->setLogAction('multi_update');
			$this->doSubmit();
		}
		
		if (isset($this->form_values['add_rows'])) {
			$this->addExtraRows($this->form_values['add_rows']);
		}
		
		if ($this->form_values['method']) {
			$this->method = $this->form_values['method'];
			$this->values = $this->form_values['values'];
		} else {
			$this->method = 'paging';
			$this->values = array('pref' => 'templateindex', 'page_num' => $this->form_values['page_num'], 'view_all' => $this->form_values['view_all']);
		}
	}
	
	function doSubmitValidate() {
		if (!$this->items) {
			$this->items = $this->form_values['items'];
		}
		$error_msgs = array();
		$extra_count = 0;
		$items = $this->reassignExtraFields($this->items);
		foreach ($this->items as $k => $item) {
			$id = $item['id'];
			if ($id < 0) {
				if ($item['_ignore_row']) {
					bcrm_unset($this->items, $k);
				} else {
					bcrm_unset($this->items, $k, 'id');
					$error = $this->validateFields($item);
					if ($error) {
						$error_msgs[$id] = $error['error_msg'];
					}
					$items[$k] = $item;
				}
			} elseif ( ($this->isEditable($id)) ) {
				$error = $this->validateFields($item);
				if ($error) {
					$error_msgs[$id] = $error['error_msg'];
				}
				$this->items[$k] = $item;
			} else {
				bcrm_unset($this->items, $k);
			}
		}
		if (count($error_msgs)) {
			bcrm_unset($this->form_values, 'override_method');
			$current_extras = count(array_filter($items, array($this, 'filterExtraFields')));
			$this->setExtraRowCount($new_extras + $current_extras);
			$this->main('items', $items, $error_msgs);
			exit();
		}
	}
	
	
	function doSubmitProcess() {
		foreach ($this->items as $k => $item) {
			
			if ($k > 0) {
				$item['id'] = $k;
			}
			
			$this->setErrorMessage(false);
			
			$ids[] = $this->setItem($item);
			
			if ($error = $this->getErrorMessage()) {
				$this->dbwrite_error[$id] = $error;
			}
		}
		return $ids;
	}
	
	function doSubmitPrintErrors($id) {
		if (count($this->dbwrite_error) != 0) {
			global $L;
			$quick_page = $L['errDatabaseWrite'] . '<br>';
			$tmp = 0;
			$query_string = '';
			foreach ($this->dbwrite_error as $k => $v) {
				$quick_page .= '&nbsp;&nbsp;&nbsp;id ' . $k . ': ' . $L[$v] . '<br>';
				if ($v = 'errUpdated') {
					$query_string .= 'values[' . $tmp . ']' . '=' . $k . '&';
					$tmp++;
				}
			}
			if ($query_string) {
				global $SERVICES;
				$php_self = $SERVICES->getGlobal('php_self');
				$href = $php_self . '?override_method=ids&' . rtrim($query_string, '&');
				$quick_page .= '<br> ' . $this->output->getLinkHref( array('href' => $href, 'text' => $L['errReEditItems']) );
			} else {
				$quick_page .= '<br> ' . $this->output->getLinkHref( array('href' => $_SESSION['return_page'], 'text' => $L['Back']) );
			}
			quickPage($quick_page);
			exit();
		}
	}
	
	
	/**
	 * Changes the number of extra rows on the page
	 */
	function addExtraRows($method = false, $values = false) {
		
		if ($items = $this->form_values['items']) {
			$items = $this->reassignExtraFields($items);
			//need to validate all the fields for percentages etc.
			foreach ($items as $k => $item) {
				if (!$item['_ignore_row']) {
					$this->validateFields($item);
					$items[$k] = $item;
				} else {
					bcrm_unset($items, $k);
				}
			}
		
			$new_extras = $this->form_values['item']['_extra_rows'];
			$current_extras = count(array_filter($items, array($this, 'filterExtraFields')));
			$this->setExtraRowCount($new_extras + $current_extras);
			$this->main('items', $items);
			exit();
		} else {
			$this->main($method, $values);
			exit();
		}
	}
	
	
	/**
	 * Callback function to filter the items array and only return extra fields
	 * 
	 * @param array $item an item array
	 * @return boolean true if extra field else false
	 */
	function filterExtraFields($item) {
		if (($item['id'] < 1) && (!$item['_ignore_row']) ) {
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * Builds the multi-edit form
	 *
	 * @param array $items an array of items
	 * @param array $error_msgs an array of error messages
	 * updated Phil, 20060223
	 */
	function main($method = false, $values = false, $error_msgs = false) {
		global $SERVICES;
		if (!$method && !$values) {
			 $method = $this->method;
			 $values = $this->values;
		}
		
		if ($this->form_values['override_method']) { //Added so that the re-edit function works
			$method = $this->form_values['override_method'];
			$values = $this->form_values['values'];
		}
		
		if ( isset($this->form_values['contact_id']) ) {
			$this->db_functions->setDesignTimeFilters( array('contact_id' => $this->form_values['contact_id']) );
		}
		
		switch ($method) {
			case 'items': //The items array is passed.
				$items = $this->getItemsItems($values);
			break;
			
			case 'session': //The items array is passed.
				$ids = implode(',', $_SESSION[$values]);
				$ids_where = array('id' => array('where' => 'id IN (' . $ids . ')'));
				
				$items = $this->getWhereItems($ids_where);
			break;
			
			case 'ids': //for a list of ids
			if (is_array($values)) {
						$ids = implode(',', $values);
				} else {
					$ids = $values;
				}
				$ids_where = array('id' => array('where' => 'id IN (' . $ids . ')'));
				$items = $this->getWhereItems($ids_where);
			break;
			
			case 'where': //pass in a where statement
				$items = $this->getWhereItems($values);
			break;
			
			case 'paging': //using paging information
				if (!$values) {}
				$items = $this->getPagingItems($values);
			break;
			
			case 'none': // no items only new rows
				$items = array();
			break;
			
			default: //If nothing is passed
				$items = $this->getDefaultItems($values);
			break;
		}
		
		
		$contact_id = $this->form_values['contact_id'];
		
		/*
		$content = '';
		$content .= $this->output->getBeginTable();
		
		$content .= $this->getTitleContent();
		
		$content .= $this->getNestedTable($items, $error_msgs);
		
		
		$content .= $this->output->getSubmitRowMultiEdit($this->extra_rows_selector_switch);
		$content .= $this->output->getEndTable();
		$content .= $this->output->getEndForm();
		
		
		if (!$this->row_number && !$this->extra_rows) {
			$this->redirectBack();
		}
		
		$this->output->addToBody($content);
		$this->output->display();
		*/
		
		// Build a content array
		$content = array();
		// Last Viewed and Bookmarks Table
		// $content['pastLinks']						= $this->getPastLinksTable();
		// ---------------
		$content['beginMain']           = $this->getBeginMainContent();
		// Title (and links)
		$content['title']               = $this->getTitleContent();
		// Description (optional)
		// $content['description']         = $this->getDescriptionContent();
		// Browse / paging
		// $content['browsePaging_top']    = false; // placeholder (for ordering) - set AFTER getDataContent
		// ActionLinks
		// $content['actionLinks']         = $this->getActionLinksContent();
		
		$content['beginDataCell']        = $this->getBeginDataCell(false);
		
		// Filters
		// $content['filters']             = $this->getFiltersContent();
		
		$content['bodyTable']			= $this->output->getBeginTable('bodyTbl');
		// ----
		// Main Content
		$content['data']                = $this->getDataContent($items, $error_msg);
		// ----
		
		$content['endBodyTable']        = $this->output->getEndTable();
		
		$content['endDataCell']         = $this->getEndDataCell();
		
		// Checkbox menu content
		// $content['checkboxMenu']        = $this->getCheckboxMenuContent();
		
		// Browse / paging - both paging rows are set here - has to be done AFTER getdataContent
		// $content['browsePaging_bottom'] = $content['browsePaging_top'] = $this->getBrowsePagingContent();

		// ---------------
		$content['endMain']             = $this->getEndMainContent();
		
		
		if (!$this->row_number && !$this->extra_rows) {
			$this->redirectBack();
		}
		
		return $this->display($content);
	}
	
	/**
	 * @author Phil Calder
	 */
	function getDataContent($items, $error_msgs) {
		$content = '';
		$content .= $this->output->getBeginForm($this->getAction(), 1, $this->encodingType());
		$content .= $this->getNestedTable($items, $error_msgs);
		$content .= $this->output->getSubmitRowMultiEdit($this->extra_rows_selector_switch);
		$content .= $this->output->getEndForm();
		
		return $content;
	}
	
	
	function getExtraRowsSelectorSwitch() {
		return $this->extra_rows_selector_switch;
	}
	
	function setExtraRowsSelectorSwitch($switch) {
		$this->extra_rows_selector_switch = $switch;
	}
	
	
	/**
	 * Gets the subtables for editing
	 * 
	 * @param array $items the items array
	 * @param array $error_msgs an array of error messages
	 * @return string the table rows.
	 */
	function getNestedTable($items, $error_msgs) {
		// print '<div>getNestedTable table labels:</div>'; print_r($this->getTableLabels()); print '<hr>';
		$content = '';
		$content .= $this->output->getBeginNestedTable();
		
		if (count($items)) {
			$content .= $this->output->getHeaderRow($this->getTableLabels());
			foreach ($items as $k => $v) {
				$id = $v['id'];
				if ( $this->isEditable($id) ) {
					$fields = $this->dd->getDataStructureWithData($v);
					// print '<div>getNestedTable fields:</div>'; print_r($fields); print '<hr>';
					if ($this->extra_rows > 0) {
						$content .= $this->output->getMultiHtmlFieldRow($id, $fields, $error_msgs[$id], $this->setRowNumber(1), '&nbsp;');
					} else {
						$content .= $this->output->getMultiHtmlFieldRow($id, $fields, $error_msgs[$id], $this->setRowNumber(1));
					}
				}
			}
		}
		
		if ( is_array($extra_rows = $this->getExtraRows($items)) ) {
			$checkbox_checked = array('_ignore_row' => array('label'=>'Checkbox', 'field_key'=>'_ignore_row', 'type'=>'checkbox', 'value' => true));
			$content .= $this->output->getHeaderRow($this->getTableLabels(true));
			foreach ($extra_rows as $k => $v) {
				$id = $v['id'];
				$checkbox = $checkbox_checked;
				$checkbox['_ignore_row']['value'] = $v['_ignore_row'];
				$fields = $this->dd->getDataStructureWithData($v);
				$content .= $this->output->getMultiHtmlFieldRow($id, $fields, $error_msgs[$id], $this->setRowNumber(1), $checkbox); //if extra feilds
			}
		}
		$content .= $this->output->getEndNestedTable();
		return $content;
	}
	

	/**
	 * Gets an array of table labels
	 *
	 * @param boolean $showcheckbox
	 * @return array an array of table labels
	 */
	function getTableLabels($show_checkbox = false) {
		global $L;
		$table_labels = array();
		$table_labels['row']['label'] = $L['labelRow']; // Phil 20060223 added 'row' key (was blank)
		if ($this->extra_rows > 0) {
			if ($show_checkbox) {
				$table_labels[]['label'] = $L['labelIgnore'];
			} else {
				$table_labels[]['label'] = '&nbsp;';
			}
		}
		$table_labels = $table_labels + $this->dd->getDataDictionary();
		foreach ($table_labels as $k => $v) {
			if (($v['type'] == 'hidden') || (isset($v['hidden']) && $v['hidden'])) {
				bcrm_unset($table_labels, $k);
			}
			if ($v['notnull'] == 1) {
				$table_labels[$k]['label'] = $v['label'] . '&nbsp;&nbsp;*';
				if ($v['list_label']) {
					$table_labels[$k]['list_label'] = $v['list_label'] . '&nbsp;&nbsp;*';
				}
			}
		}
		return $table_labels;
	}
	
	
	/**
	 * Gets the the items for a certain page using the paging functions
	 *
	 * @param array $values 
	 * @return array an array of items
	 */
	function getPagingItems($values){
		global $SERVICES;
		$item_count = $this->db_functions->countItems();
		$this->db_functions->setPrefName($values['pref']);
		$_GET['page_num'] = $values['page_num'];
		$_GET['view_all'] = $values['view_all'];
		$limit = $this->output->setPagingLimits($values['pref'] . 'ListSize', $item_count);
		if ($SERVICES->preferences->getUserPref($values['pref'] . 'SortField', 0)) {
			$sort['field'] = $SERVICES->preferences->getUserPref($values['pref'] . 'SortField', 0);
			$sort['order'] = $SERVICES->preferences->getUserPref($values['pref'] . 'SortOrder', 0);
		} else {
			$sort = ''; //Can set default values here
		}
		return $this->db_functions->getItems($sort, $limit);
	}
	
	
	/**
	 * Gets the items using a where statment
	 *
	 * @see WebMATDatabaseTable::setDesignTimeFilters()
	 *
	 * @param array $values 
	 * @return array an array of items
	 */
	function getWhereItems($values){
		$this->db_functions->setDesignTimeFilters($values);
		return $this->db_functions->getItems();
	}
	
	/**
	 * Gets the tiems from $values
	 *
	 * @param array $values 
	 * @return array an array of items
	 */
	function getItemsItems($values){
		return $values;
	}
	
	/**
	 * Gets all the items from the database
	 *
	 * @param array $values 
	 * @return array an array of items
	 */
	function getDefaultItems($values){
		return $this->db_functions->getItems();
	}
	
	/**
	 * gets default values
	 *
	 * Wrapper function
	 * @author Phil Calder
	 * @param array $default_overrides values that overwrite the data definition defaults.
	 */
	function getDefaultItem($default_overrides = false) {
		return $this->dd->getDefaultItem($default_overrides);
	}
	
	
	/**
	 * Sets the number of extra blank rows to display
	 * 
	 * @param integer $extra_row_count the number of extra rows to insert
	 */
	function setExtraRowCount($extra_row_count) {
		if (is_int($extra_row_count)) {
			$this->extra_rows = $extra_row_count;
		}
	}
	
	
	/**
	 * Adds extra blank rows the items array with a negative id
	 *
	 * @param array $all_items the items array
	 * @return array an items array of extra items or false if none
	 */
	function getExtraRows($all_items = array()) {
		if ($this->extra_rows) {
			$items = array();
			// Phil 20060530 Using webMATEdit wrapper function for data dictionary
			if (is_array($this->overrides)) {
				// $defaults = $this->dd->getDefaultItem($this->overrides);
				$defaults = $this->getDefaultItem($this->overrides);
			} else {
				// $defaults = $this->dd->getDefaultItem();
				$defaults = $this->getDefaultItem();
			}
			
			$i = 1;
			while ($i <= $this->extra_rows) {
				$key = ($i*-1); //Workaround for php 4.2.2 and comaring strings and integers
				if  (isset($all_items[$key])) {
					$items[$i*-1] = $all_items[$key];
					$items[$i*-1]['_ignore_row'] = false; //used to set the ingore checkbox to unchecked
				} else {
					$items[$i*-1] = $defaults;
					$items[$i*-1]['id'] = $i*-1;
					if (in_array('contact_id', $this->db_functions->columns)) {
						$items[$i*-1]['contact_id'] = $this->form_values['contact_id'];
					}
					$items[$i*-1]['_ignore_row'] = true; //used to set the ingore checkbox to checked
				}
				$i++;
			}
			return $items;
		} else {
			return false;
		}
	}
	
	
	/**
	 * gets the action for submit to pass contact id if required
	 * 
	 * @return string the url for the form submit
	 */
	function getAction() {
		global $SERVICES;
		$php_self = $SERVICES->getGlobal('php_self');
		if ($this->form_values['contact_id']) {
			return $php_self . '?contact_id=' . $this->form_values['contact_id'];
		}
		return $php_self;
	}
	
	
	/**
	 * Sets $row_number class variable
	 * 
	 * @param integer $var the number of extra rows to insert
	 * @param boolean $add weather to add to the current or overwrite
	 * @return integer the old value of rowcount
	 */
	function setRowNumber($var, $add = true) {
		if (is_int($var)) {
			if (!$this->row_number) {
				$this->row_number = 1;
			}
			$old = $this->row_number;
			if ($add) {
				$this->row_number += $var;
			} else {
				$this->row_number = $var;
			}
			return $old;
		}
	}
	
	/**
	 * reassign extra fields
	 * 
	 * removes the gaps between extra feilds.
	 */
	function reassignExtraFields($items = array()) {
		$temp_items = array();
		$extra_item_count = 1;
		foreach ($items as $k => $item) {
			if ($item['id'] < 0) {
				if (!$item['_ignore_row']) {
					$new_id = $extra_item_count * -1;
					$item['id'] = $new_id;
					$temp_items[$new_id] = $item;
					$extra_item_count++;
				}
			} else {
				$temp_items[$item['id']] = $item;
			}
		}
		return $temp_items;
	}
	
	/**
	 * Uses this modules list page as the return page
	 *
	 * @param string $page The relative page (Unused in this class)
	 * @author Phil Calder
	 */
	function setReturnPage($page = false) {
		global $SERVICES;
		$SERVICES->getService('Config');
		$_SESSION['return_page'] = $SERVICES->config->lookup('modules.Front.modulelink.' . $this->module_name . '.link');
	}
	
	/**
	 * builds the title row for the page
	 * 
	 * @param string $prefix what to prefix the file name with (optional)
	 * @return string with the title
	 */
	function getTitleText($prefix = 'Title_') {
		return WebMATDisplay::getTitleText($prefix);
	}
	
	/**
	 * Disabled in this class
	 * @author Phil Calder, 20050712
	 */
	function addMakePermissionsPrivate(&$id, &$item, &$error_msg) {
	}
	
	
	/**
	 * any processing required before adding to the db
	 * 
	 * @param array $item an item array passed by refrence.
	 */
	function beforeDBWrite(&$item) {}
	
	
	/**
	 * any processing required doing the database insert/update loop.
	 * 
	 * @param arrays $items the items array passed by refrence
	 * @param array $error_msgs the error messages array
	 */
	function beforeMultiDBWrite(&$items, &$error_msgs) {}
}
?>