<?php
/**
 * Class for opening a file
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 */

includeWebMAT('webmat.inc');
 
class WebMATStandardOpenFile extends WebMAT {
	
	/**
	 * @var array $form_values the combined post and get arrays
	 */
	var $form_values = array();
	/**
	 * @var object $mod The current module object
	 */
	var $mod = false;
	/**
	 * @var object $db_functions The current database object
	 */
	var $db_functions = false;
	/**
	 * @var string $module_name the current module name
	 */
	var $module_name = false;	
	/**
	 * @var boolean $use_database whether to use the database to get the file info
	 */
	var $use_database = false;
	/**
	 * @var boolean $use_database whether to use the database to get the file info
	 */
	var $master_dir = false;
	/**
	 * @var boolean $use_database whether to use the database to get the file info
	 */
	var $sub_dir = false;
	
	/**
	 * the constructor calls init
	 */
	function WebMATStandardOpenFile($module, $table_name, $post, $get, $use_database = true, $master_dir = false, $sub_dir = false) {
		$this->init($module, $table_name, $post, $get, $use_database, $master_dir, $sub_dir);
	}
	
	/**
	 * init sets the class variables
	 */
	function init($module, $table_name, $post, $get, $use_database = true, $master_dir = false, $sub_dir = false) {
		global $SERVICES;		
		$SERVICES->getService('Config');
		$SERVICES->getService('Modules');
		$SERVICES->getService('FileSystem');
		$this->use_database = $use_database;
		$this->master_dir = $master_dir;
		$this->sub_dir = $sub_dir;
		$this->mod =& $SERVICES->modules->getModule($module);
		$this->db_functions =& $this->mod->getDatabaseTable($table_name);
		//$this->form_values = array_merge_recursive($get, $post);
		$this->form_values = $this->arrayMergeReplace($get, $post);
		$this->module_name = $module;
		$dropdown_config = $SERVICES->config->lookup('modules.Admin.modulelink.documents_mime_type');
		$this->dropdown =  getWebMATObject(
				'WebMATDropdown', 
				$dropdown_config['module'], 
				array(
					'module' => $dropdown_config['module'], 
					'table' => $dropdown_config['table'], 
					'field_name' => $dropdown_config['field']
				)
			); // new WebMATDropDown($dropdown_config['module'], $dropdown_config['table'], $dropdown_config['field']);
	}
	
	
	/**
	 * returns the file info in an array
	 * 
	 * @return array an array of file info (with the keys 'mime_type', 
	 * 'file_name', 'location' and 'size')
	 */
	function getFileInfo() {
		global $SERVICES;
		$item = $this->db_functions->getItem($this->form_values['id']);
		//$this->db_functions->setMasterIds($item['item_id']);
		//$parent_item = $this->db_functions->getParentItems();
		$mime_type_info = $this->dropdown->getValue($item['mime_type_id']);
		$file_info['mime_type'] = $mime_type_info['name'];
		$file_info['file_name'] = $item['file_name'];
		// Phil, just testing, never used // $file_info['filename'] = trim($item['file_name'] . '.' . $item['file_extension'], '. ');
		// print '<div>DEBUG: ' . get_class($this) . '::getFileInfo() for ' . $this->form_values['id'] . '</div>';
		// print_r($item); print '<hr>';
		// print_r($file_info); print '<hr>';
		return $file_info;
	}
	
	
	/**
	 * makes the file appear as download window
	 * 
	 * @param boolean TRUE to open the file as a download or FALSE to view in 
	 * the browser
	 */
	function getFileForBrowser($download = false) {
		global $SERVICES;
		if ($this->use_database) {
			$file_info = $this->getFileInfo();
		} else {
			$file_info = false;
		}
		
		$SERVICES->filesystem->getFileForBrowser($this->form_values['id'], $this->module_name, $this->master_dir, $this->sub_dir, $file_info, $download);
		if ($SERVICES->filesystem->hasError()) {
			quickpage($SERVICES->filesystem->printError());
		}
	}
	
	/**
	 * the main function
	 */
	function main() {
		$this->getFileForBrowser($this->form_values['download']);
	}
}
?>