<?php
/**
 * 
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 */

includeWebMAT('list.inc');

/**
 * This class displays a simplified list view
 *
 * @author Phil Calder
 * @abstract
 * @package WebMAT
 */
class WebMATStandardShortList extends WebMATList {
	/**
	 * @access private
	 */
	var $list_link = '';
	
	/**
	 * returns the paging row
	 *
	 * @return string with the paging row
	 */
	function getPagingRow() {
		global $SERVICES;
		$SERVICES->getService('Language');
		$SERVICES->language->getLanguage('Contact');
		$last = $this->output->getPagingItemStartNumber($this->output->paging_list_size) + $this->output->paging_list_size;
		$total = $this->output->paging_items_count;
		$labels = array(
			'more' => $L['contactMoreLabel'], 
			'less' => $L['contactLessLabel']
		);
		
		return $this->output->getTableRow($this->output->getPagingLinkViewAll($last,$total, $labels));
	}
	
	function getMenuItems($contact_id) {} //hide MenuItems
	
	function getDescriptionContent() {} //hide DescriptionRow
	
	function sortTable() {} //Disable sort
	
	function getHelpLink() {
		return -1;
	}
	
	function main($return_string = false) {
		global $SERVICES, $USER, $L;
		$this->setReturnString($return_string);
		
		$this->pref_name = 'contactContactView';
		$this->list_link = 'list';
		// $this->menu_separator = ' | ';
		$this->setMenuSeparator(' | '); // Phil 20050511
		
		$contact_id = $this->form_values['contact_id'];
		$items = $this->form_values['items'];
		
		$content = '';
		$content .= '<div id="dashPieces">';
		$content .= '<div class="firstDiv">';
		$content .= $this->output->getBeginTable('zeroTable');
		$content .= $this->output->getTitleRow(
			$this->getTitleText(), 
			$this->getMenuItems($contact_id), 
			$this->getHelpLink()
		);
		$content .= $this->getTable();
		$content .= $this->getPagingRow();
		$content .= $this->output->getEndTable();
		$content .= '</div>';
		$content .= '</div>';
				
		return $content;
	}
	
	/**
	 * Links to the module, filtered by the contact_id
	 */
	function getAllLink($contact_id = '') {
		global $SERVICES, $L;
		
		if ( (($SERVICES->config->lookup('modules.' .  $this->module_name . '.file_set.' . str_replace($this->module_name . '__', '', $this->table_name) . '.contact_required') == 0) || ($contact_id != ''))  
		) {
			if ($link = $this->getFileSetHref($this->list_link)) {
				$link_href = $this->getModulePath() . str_replace('%id%', '', (str_replace('%contact_id%', $contact_id ,$link)) );
				return $this->output->getLinkHref( array('href' => $link_href, 'text' => $L['ListAll']) );
			}
		}
		return '';
	}
	
	function getNewLink($contact_id = '') {
		global $SERVICES, $L;
		
		if ( (($SERVICES->config->lookup('modules.' .  $this->module_name . '.file_set.' . str_replace($this->module_name . '__', '', $this->table_name) . '.contact_required') == 0) || ($contact_id != ''))  
		) {
			if ($link = $this->getFileSetHref($this->edit_link)) {
				$link_href = $this->getModulePath() . str_replace('%id%', '', (str_replace('%contact_id%', $contact_id ,$link)) );
				return $this->output->getLinkHref( array('href' => $link_href, 'text' => $L['LinkNew']) );
			}
		}
		return '';
	}
	
	function getTable() {
		global $SERVICES, $L;
		$table_labels = array();
		
		$table_labels = $this->getDataDictionary(); // Phil 20051027 using wrapper function
		$table_labels = $this->excludeCalculated($table_labels);
		$table_labels = $this->headerRowSpacer($table_labels);
		
		$sort = $this->sortTable($this->form_values['sort']);
		// $table_content = $this->getSubTableHeader($table_labels, $sort);
		
		$contact_id = $this->form_values['contact_id'];
		$item_count = $this->db_functions->countItems(); // only needed for setPagingLimits
		
		if ($this->paging_overwrite['limit']) {
			$limit = 'LIMIT 0, ' . $this->paging_overwrite['limit'];
		} else {
			$limit = $this->output->setPagingLimits($this->pref_name . 'ListSize', $item_count);
		}
		if ($this->paging_overwrite['sort']) {
			$sort = $this->paging_overwrite['sort'];
		}
		
		$items = $this->db_functions->getItems($sort, $limit);
		$alert_ids = $this->db_functions->getAlertIds($limit, array_keys($items));
		if (is_array($items) && sizeof($items)) {
			foreach ($items as $k => $item) {
				$table_content .= $this->buildListTable($item, $alert_ids);
			}
		} else {
			//no items in the table
			// $ua =& $SERVICES->getService('Authentication');
			$table_content .= $this->output->getTextRow($L[$this->mod->getModuleNameLower() . 'NoItems'] , sizeof($table_labels));
		}
		return $table_content;
	}
	
	/**
	* Builds the edit link
	*  has been changed to add the relative path to the file have been changed using path_to_module
	* 
	* @param string $id the id of the item you want to edit
	* @return string containing the link
	*/
	function getEditLink($id) {
		global $SERVICES, $L;
		if ($id) {
			if ($this->getFileSetHref($this->edit_link)) {
				$link_href = str_replace('%contact_id%', '', (str_replace('%id%', $id ,$this->getFileSetHref($this->edit_link))) );
				$link = '<a href="' . $this->getModulePath() . $link_href . '">' . $L['LinkEdit'] . '</a>';
			} else {
				$link = $this->getViewLink($id);
			}
		} else {
			$link = '&nbsp;';
		}
		return $link;
	}
	
	/**
	* Gets the custom fields edit link
	* 
	* @param string $id the id of the item you want to edit
	* @return string containing the link
	*/
	function getCustomFieldsEditLink($id) {
		global $SERVICES, $L;
		if ($id) {
			if ($custom_fields_edit_link = $this->getFileSetHref('edit_custom')) {
				$link_href = str_replace('%contact_id%', '', (str_replace('%id%', $id , $custom_fields_edit_link)) );
				$link = '<a href="' . $this->path_to_module . $link_href . '">' . $L['LinkEdit'] . '</a>';
			} 
		} else {
			$link = '&nbsp;';
		}
		return $link;
	}
	
	/**
	* Builds the Delete link
	*  has been changed to add the relative path to the file have been changed using path_to_module
	* 
	* @param string $id the id of the item you want to delete
	* @return string containing the link
	*/
	function getDeleteLink($id) {
		global $SERVICES;
		if ($id && $this->getFileSetHref($this->edit_link . '_delete')) {
			$link_href = str_replace('%id%', $id ,$this->getFileSetHref($this->edit_link . '_delete'));
			$link = $this->output->getLinkDelete($this->getModulePath() . $link_href);
		} else {
			$link = '&nbsp;';
		}
		return $link;
	}
	
	/**
	* Builds the View link
	* has been changed to add the relative path to the file have been changed using path_to_module
	* 
	* @param string $id the id of the item you want to view
	* @return string containing the link
	*/
	function getViewLink($id) {
		global $SERVICES;
		if ($id && $this->getFileSetHref('view')) {
			$link_href = str_replace( '%id%', $id ,$this->getFileSetHref('view') );
			$link = '<a href="' . $this->getModulePath() . $link_href . '">' . 'View' . '</a>';
		} else {
			$link = '&nbsp;';
		}
		return $link;
	}
}
?>