<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 */

includeWebMAT('view.inc');

/**
 * This class handles a shortened (simplified) view of an item
 *
 * Used primarily for 'pieces' view
 * @see WebMATPiecesView
 * @see WebMATDashboard
 * @see WebMATContactView
 * @package WebMAT
 * @author Phil Calder
 * @abstract
 */
class WebMATStandardShortView extends WebMATView {
	
	/**
	 * @access private
	 */
	var $begin_table_arg = '';
	
	/**
	 * Calls parent init method and sets piece identifier
	 */
	function init($module, $data_option, $post = false, $get = false, $table_name = false, $language_overwrite = false) {
		parent::init($module, $data_option, $post, $get, $table_name, $language_overwrite);
		
		// global $piece_id;
		// if (isset($piece_id) && $piece_id) {
			// $this->setPieceId($piece_id);
		// } else {
			// $this->setPieceId($module);
		// }
		// Phil 20060130 see prepend getWebMATObject
		if (isset($get['piece_identifier']) && $get['piece_identifier']) {
			$this->setPieceId($get['piece_identifier']);
		} else {
			$this->setPieceId($module);
		}
	}
	
	/**
	 * Main display method
	 */
	function main($return_string = false) {
		global $SERVICES, $USER, $L;
		$this->setReturnString($return_string);
		$this->output->setRowFlag(); // resets the row flag to its default value
		$id = $this->form_values['id'];
		
		$content = '';
		
		//$content .= $this->output->getBeginTable('pieceTable');
		//$this->output->setTagContent('zeroTable', 'class', 'tab');
		$content .= $this->output->getBeginTable($this->begin_table_arg); // phil 20050729 'pieceTable');
		//$this->output->resetTagContent('zeroTable');
		
		$content .= $this->getShortViewHeaderRow($id);
 		
 		$content .= $this->output->getBeginRow() . $this->output->getBeginCell(3);  //TODO change to 2 
 		
 		$content .= '<div class="openContent"><div class="content">';
		$content .= $this->output->getBeginTable('viewTbl'); // phil 20050729 'pieceContentTable'); // -
 		$content .= $this->getTable();
 		$content .= $this->output->getEndTable();
 		$content .= '</div></div>'; 
 
 		$content .= $this->output->getEndCell() . $this->output->getEndRow(); 
 		$content .= $this->output->getEndTable();
 		
 		return $content;
 	}
	
	/**
	 *
	 * @return string The short view header row
	 */
	function getShortViewHeaderRow($id) {
		$content = $this->output->getBeginRow('cpHeaderRow');
		
		// $content .= $this->output->getTableHeaderCell($this->getTitleImage(), 'class="icon"');
 		
 		//TODO turn links into icons
 		$tools = '<span class="tools">' . $this->getMenuItems($id) . '</span>';
 		$title = $this->getTitleText();
		// restore target to the contentframe
		// $this->output->setDefaultLinkTarget($current_target);
		
 		$content .= $this->output->getTableHeaderCell($title . $tools, 'class="label" id="' . strtolower($this->getPieceId()) . 'PieceLabel"'); 
 		
 		$content .= $this->output->getEndRow();
		
		return $content;
		
	}        	
         
	/**
	 * builds a nested table displaying all the tabular content for the page
	 *
	 * @return string returns the nested table with data
	 */
	function getTable() {
		global $SERVICES, $L;
		
		$contact_id = $this->form_values['contact_id'];
		// $items_count = $this->db_functions->countItems(); // Not needed
		
		$item = $this->db_functions->getItem($this->form_values['id']);
		
		// print '<div>short view getTable, item: </div>'; print_r($item); print '<hr>';
		
		if (is_array($item) && sizeof($item)) {
			// print '<div>' . get_class($this) . ' - in getTable</div>';
			$this->markAsSeen($item); //Marks the item as seen when you view it
			$style_flag = '';
			$this->dd->setDictionaryExcludehidden(); //Gets rid of the hidden variables
			$this->dd->setNoTrim(); //disables trim
			
			$dd_item = $this->dd->getDataStructureWithData($item);
			// $this->calculateValues($dd_item, $item);
			// print '<div>item</div>'; print_r($item); print '<hr>';
			// print '<div>dd_item</div>'; print_r($dd_item); print '<hr>';
			foreach ($dd_item as $k => $field) {
				// print '<div>field ' . $k . '</div>'; print_r($field); print '<hr>';
				bcrm_unset($field, 'list_detail_length');
				// unset($table_row);
				$table_row = array();
				$table_row[] = '<b>'. ($field['label'] ? $field['label'] : $field['list_label']) .'</b>';
				$table_row[] = str_replace(
					array(
						'<div style="text-align: right">', 
						'<div style="text-align: right;  white-space: nowrap;">','</div>'
					), 
					'', 
					$this->getStringField($field)
				);
				$table_content .= $this->output->getTableRow($table_row);
			}
			
		} else {
			//no items in the table
			// $table_content .= $this->output->getTextRow($L[$this->module_name_lower . 'NoItems'] , sizeof($table_labels));
			$table_content .= $this->output->getTextRow($L[$this->module_name_lower . 'NoItems']);
		}
		return $table_content;
	}
	
	/**
	 * @return string The path to the module (relative to other modules)
	 */
	function getModulePath() {
		return $this->path_to_module;
	}
	
	/**
	* Builds the edit link
	*  has been changed to add the relative path to the file have been changed using path_to_module
	* 
	* @param string $id the id of the item you want to edit
	* @return string containing the link
	*/
	function getNewLink($id = '') {
		if ($link = $this->getFileSetHref($this->edit_link)) {
			$link_href = $this->getModulePath() . str_replace('%id%', '', (str_replace('%contact_id%', $id, $link)) );
			// return $this->output->getLinkNew($link_href); // Phil 20050519 replaced with below (from pieces_view)
			global $L;
			if (isset($this->mod) && is_object($this->mod)) {
				$title = $L[$this->mod->getModuleNameLower() . 'TitleNew_itemedit'];
			}
			if (!$title) {
				$title = $L['LinkNew'];
			}
			$img = $this->output->getImageTag('icons_dash/tools_new.gif', $title, 'width="16" height="16"');
			return $this->output->getLinkHref(array('href'=>$link_href, 'img'=>$img, 'title' => $title), false);
		}
		return false;
	}
	
	/**
	* Builds the edit link
	*  has been changed to add the relative path to the file have been changed using path_to_module
	* 
	* @param string $id the id of the item you want to edit
	* @return string containing the link
	*/
	function getEditLink($id = '') {
		if ($link = $this->getFileSetHref($this->edit_link)) {
			$link_href = $this->getModulePath() . str_replace('%id%', $id, (str_replace('%contact_id%', '', $link)) );
			return $this->output->getLinkEdit($link_href);
		}
		return false;
	}
	
	/**
	* Gets the custom fields edit link
	* 
	* @param string $id the id of the item you want to edit
	* @return string containing the link
	*/
	function getCustomFieldsEditLink($id) {
		global $SERVICES, $L;
		if ($id) {
			if ($custom_fields_edit_link = $this->getFileSetHref('edit_custom')) {
				$link_href = str_replace('%contact_id%', '', (str_replace('%id%', $id , $custom_fields_edit_link)) );
				$link = '<a href="' . $this->path_to_module . $link_href . '">' . $L['LinkEdit'] . '</a>';
			} 
		} else {
			$link = '&nbsp;';
		}
		return $link;
	}
	
	/**
	* Gets the custom fields edit link
	* 
	* @param string $id the id of the item you want to edit
	* @return string containing the link
	*/
	function getPermissionsEditLink($id) {
		global $SERVICES, $L;
		if ($id) {
			if ($permissions_edit_link = $this->getFileSetHref('edit_permissions')) {
				$link_href = str_replace('%contact_id%', '', (str_replace('%id%', $id , $permissions_edit_link)) );
				$link = '<a href="' . $this->path_to_module . $link_href . '">' . $L['LinkEdit'] . '</a>';
			} 
		} else {
			$link = '&nbsp;';
		}
		return $link;
	}
	
	/**
	* Builds the Delete link
	*  has been changed to add the relative path to the file have been changed using path_to_module
	* 
	* @param string $id the id of the item you want to delete
	* @return string containing the link
	*/
	function getDeleteLink($id = '') {
		if ($link = $this->getFileSetHref($this->edit_link . '_delete')) {
			$link_href = $this->getModulePath() . str_replace('%id%', $id, (str_replace('%contact_id%', '', $link)) );
			return $this->output->getLinkDelete($link_href);
		}
		return false;
	}
	
	/**
	* Builds the View link
	* has been changed to add the relative path to the file have been changed using path_to_module
	* 
	* @param string $id the id of the item you want to view
	* @return string containing the link
	*/
	function getViewLink($id = '') {
		if ($link = $this->getFileSetHref('view')) {
			$link_href = $this->getModulePath() . str_replace('%id%', $id, (str_replace('%contact_id%', '', $link)) );
			return $this->output->getLinkView($link_href);
		}
		return false;
	}
	
	/**
	 * @return integer The contact id
	 */
	function getContactId() {
		return $this->contact_id;
	}
	
	/**
	 * @param integer The contact id
	 */
	function setContactId($id) {
		$this->contact_id = $id;
	}
	
	/**
	 * @return string Title text
	 */
	function getTitleText() {
		if (isset($this->title_text) && $this->title_text) {
			return $this->title_text;
		}
		return parent::getTitleText();
	}
	
	/**
	 * @param string Title text (overrides default)
	 * @see getTitleText
	 */
	function setTitleText($text = '') {
		if (empty($text)) {
			$module = $this->mod->getModuleName();
			global $SERVICES, $L;
			$SERVICES->getService('Language');
			$SERVICES->language->getLanguage($module);
			
			$text = $L[$this->mod->getModuleNameLower() . 'EventNamePlural'];
			
		}
			
		$this->title_text = $text;
		
	}
	
	/**
	 * see also WebMAT/pieces_view
	 */
	function getTitleImage() {
		$alert_flag = 'normal';
		
		// global $L;
		// if ($alerts = $this->countAlerts()) {
			// $alert_flag = 'alert';
			// $text = $L['Alerts'] . ': ' . $alerts;
		// } else {
			// $alert_flag = 'normal';
			// $text = $L['NoAlert'];
		// }
		
		if ($href = $this->getFileSetHref('list')) {
			$href = $this->getModulePath() . $href;
		} else {
			$href = $href = $this->getModulePath();
		}
		if ($query_start = strpos($href, '?')) {
			$href = substr($href, 0, $query_start);
		}
		$link = array(
			'href' => $href,
			// 'title' => $text,
			'icon' => 'icons_dash/' . strtolower($this->getPieceId()) . '_' . $alert_flag . '.gif',
			'if_exists' => true // Phil 20060510
		);
		
		$default_icon_name = 'other';
		if ($retval = $this->output->getLinkDashIcon($link)) {
			return $retval;
		}
		$link = array(
			'href' => $href,
			// 'title' => $text, // Phil 20060704 $text is not set
			'icon' => 'icons_dash/' . $default_icon_name . '_' . $alert_flag . '.gif',
		);
		return $this->output->getLinkDashIcon($link);
		
	}
	
	/**
	 * Sets the piece identifier
	 * @param string The piece identifier
	 */
	function setPieceId($id) {
		$this->piece_id = $id;
	}
	
	/**
	 * Gets the piece identifier
	 * @return string The piece identifier
	 */
	function getPieceId() {
		return $this->piece_id;
	}
	
	/**
	 * Paging row disabled
	 */
	function getPagingRow() {} 
	
	/**
	 * Menu items disabled
	 */
	function getMenuItems($contact_id) {} 
	
	/**
	 * Description row disabled
	 */
	function getDescriptionContent() {} 
	
	/**
	 * Sorting disabled
	 */
	function sortTable() {}
	
	/**
	 * Help link disabled
	 */
	function getHelpLink() {
		return -1;
	}
}
?>