<?php
/**
 * @license http://www.browsercrm.com/licence.php  BrowserCRM Software Licence
 * @copyright Copyright (c) 2003 - 2007 BrowserCRM Limited
 * @link http://www.browsercrm.com/
 * @package WebMAT
 */

includeWebMAT('item.inc');

/**
 * this class handles the edit style display of content
 *
 * @package WebMAT
 * @author  Phil Calder, Matthew Corich, Brenton, Bosco
 */
class WebMATStandardView extends WebMATItem {

	/**
	 * init function sets all the class variables
	 *
	 * @param  string $module  the current module name
	 * @param  string $data_option  the current data option used for DataDictionary
	 * @param  array $post the post array
	 * @param  array $get the get array
	 * @param  string $table_name the name of the table the page is using
	 * @param  array $language_overwrite has the keys title and description
	 * @author Phil Calder
	 */
	function init($module, $data_option, $post = false, $get = false, $table_name = false, $language_overwrite = false) {
		parent::init($module, $data_option, $post, $get, $table_name, $language_overwrite);
		$this->checkItem();
	}
	
	/** 
	 * @author Phil Calder
	 */
	function substituteTagContents() {
		$this->output->substituteTagContent('outerTable', 'outerTableView');
	}


	/**
	 * returns the WebMat type
	 */
	function getWebmatType() {
		return 'view';
	}

	/**
	 * Checks to see if the item is editable
	 *
	 * Redirects to list page if the item does not exist (deleted)
	 * Displays a message and exits if the item is not viewable to the user.
	 * @author Phil Calder
	 */
	function checkItem() {
		global $L;
		
		if ($this->form_values['id']) {
			$id = $this->form_values['id'];
		} else {
			$id = $this->form_values['item']['id'];
		}
		
		
		// print '<div>'  . get_class($this) . '::checkItem(' . $id . ', ' . ($item === false ? 'false' : 'item') . ') calling isViewable</div>'; print '<hr>'; 
		if (!$this->isViewable($id)) { // Phil 20050106
			
			// Phil 20070315 Moved isDeleted to after isViewable
			if ($this->db_functions->isDeleted($id)) {
				$this->redirectToList();
			} else {
				quickPage(
					$L['errNoViewPermissions'] . '<br> <br> ' .
					$this->output->getLinkHref(
						array(
							'href' => $_SESSION['return_page'], 
							'text' =>$L['Back']
						)
					) 
				);
			}
			exit();
		}

	}


	/**
	 * returns the title row array
	 *
	 * Disables standard title row links in printview mode
	 *
	 * @return string a string containing the title row menu items
	 */
	function getTitleRowLinks() {
		if (!isset($this->form_values['printview']) || !$this->form_values['printview']) {
			return parent::getTitleRowLinks();
		} else {
			return array('help' => -1, 'bookmark' => 'off');
		}
	}

	function getPrintViewLink() {
		global $L;
		$print_view_options = array(
			'url' => str_replace('%id%', $this->form_values['id'] ,$this->getFileSetHref('view')) . '&printview=1',
			'window_name' => 'Print_view',
			'resizable' => true,
			'scrollbars' => true,
			'width' => 640,
			'height' => 480,
		);
		$this->output->addJSPopup('printView', $print_view_options);

		$print_view_link = $this->output->getLinkHref(
			array(
				'tag_extras' =>' onClick="javascript:printView();" ',
				'href' => 'javascript:;',
				'text' => $L['PrintView']
			)
		);
		$this->output->getLinkHref($print_view_link);
	}

	/**
	 * Calls {@link setLogAction} and WebMATList main method
	 * @param  bool  $return_string  used to see if you want the content returned as a string or printed to the string
     * @return mixed if $return_string = 1 it returns a string with the content for the page else returns true.
	 * @author Phil Calder
	 */
	function main($return_string = false) {
		// print '<div>WebMATView main (return_string: ' . ($return_string ? 'yes' : 'no') . ' ) [' . $return_string . ']</div>';
		$this->setReturnString($return_string);
		$this->setLogAction(); // Phil 20060105
		parent::main($return_string);
	}
	
	/**
	 * Gets content for form fields
	 *
	 * @param array $item the item array
	 * @param array $error_msg the error message array
	 * @return mixed string or array content
	 * @author Phil Calder, Brenton Bartel
	 */
	function getFieldRows($item) {
		$table_content = false;
		$dd_item = $this->dd->getDataStructureWithData($item, $this->getPriorityEdit());  //specify alternate sort order
		// print_r($dd_item);
		
		if (is_array($dd_item)) {
			// print '<div>before CalculateValues</div>';
			$this->calculateValues($dd_item, $item);
			// print '<div>after CalculateValues</div>';
			foreach ($dd_item as $k => $field) {
				// Phil 20070815
				// See also modules/Contact/contact_view.php
				if (isset($field['hide_empty']) && $field['hide_empty']) {
					$value = $field['value'];
					switch ($field['type']) {	
					case 'dropdown':
						if ( empty($value) ) {
							// print '<div>contact field: ' . $k . ' value: "' . $field['value'] . '"</div>';
							bcrm_unset($dd_item, $k);
							continue;
						}
						break;
					case 'date':
						if ( empty($value) || ($value == '0000-00-00')) {
							// print '<div>contact field: ' . $k . ' value: "' . $field['value'] . '"</div>';
							bcrm_unset($dd_item, $k);
							continue;
						}
						break;
						
					default:
						if ( ($value !== "0") && empty($value) ) {
							// print '<div>contact field: ' . $k . ' value: "' . $field['value'] . '"</div>';
							bcrm_unset($dd_item, $k);
							continue;
						}
					}
				}
				if ($this->output->getHasTemplate()) {
					$table_content['fields'][$field['field_key']] = $this->getFieldRow($field);
				} else {
					$table_content .= $this->getFieldRow($field);
				}
			}
		}
		// print '<div>getFieldRows returning:</div>'; print_r($table_content); print '<hr>';
		return $table_content;
	}
	
	/**
	 * Gets the table links
	 *
	 * A custom get table links to remove the view link and to remove the blank links
	 *
	 * @param array $item the item array
	 * @return array an array of links
	 */
	function getTableLinks($item) {
		$links = WebMATDisplay::getTableLinks($item);
		bcrm_unset($links, 'view');
		return $links;
	}
	
	function editTableLinks(&$links, $id, $item) {
		parent::editTableLinks($links, $id, $item);
		if ($links['list'] && !$this->form_values['contact_id']) {
			$links['list'] = str_replace('%contact_id%', '', $links['list']);
		}
	}
	
	/**
	 * @deprecated version 4.150 see {@link getFieldRow}
	 * @param array a field array
	 * @return string a table row
	 */
	function getTableRow($field) {
		return $this->getFieldRow($field);
	}

	/**
	 * builds a field row
	 *
	 * @param array a feild array
	 * @return string a table row
	 */
	function getFieldRow($field) {
		return $this->output->getStringFieldRow($field);
		/*
		if ($this->output->getHasTemplate()) {
			return array(
				'label' => ($field['label'] ? $field['label'] : $field['list_label']),
				'value' => $this->output->getStringField($field) // Phil: should call this function: //'value' => $this->getStringField($field)
			);
		} else {
			bcrm_unset($field, 'list_detail_length');
			//unset($table_row);
			//$table_row[] = '<th>'. ($field['label'] ? $field['label'] : $field['list_label']) .'</th>';
				$table_row[] = array('tag' => 'th', 'content' => ($field['label'] ? $field['label'] : $field['list_label']));
			// $table_row[] = str_replace(array('<div style="text-align: right">', '<div style="text-align: right;  white-space: nowrap;">','</div>'), '', $this->output->getStringField($field));
			// Phil 20050909 - Calls getStringField in WebMAT, not Output
			$table_row[] = str_replace(
				array(
					'<div style="text-align: right">', 
					'<div style="text-align: right;  white-space: nowrap;">',
					'</div>'
				), 
				'', 
				$this->getStringField($field)
			);
			
			if (($field['type'] == 'iframe') || ($field['type'] == 'long_text')) {
				//special case, create double rows etc
				
				
			} else {
				return $this->output->getTableRow($table_row);
			}
		}
		*/
	}
	
	/**
	 * Suffix for the description row language text key
	 *
	 * 
	 * @return string the description row content language key suffix
	 */
	function getDescriptionContentSuffix() {
		return 'ViewDescription';
	}

	// Phil 20060328 see getDescriptionContentSuffix
	// Returns a string containing the description and a spacer row if the language item exists
	//
	// @return string containing the description and a spacer row
	// 
	// function getDescriptionContent() {
		// global $L;
		// $content = '';
		// if ($L[$this->module_name_lower . 'ViewDescription']) {
			// // $content = $this->output->getTextRow($L[$this->module_name_lower . 'ViewDescription']);
			// // $content .= $this->output->getSpacerRow();
			// // Phil 20060223
			// $description = $L[$this->module_name_lower . 'ViewDescription'];
			// $padding = true;
			// $content = $this->output->getBeginRow('', true) . '<td id="bodyCell"' . ($padding ? 'style="padding: 10px"' : '') . '>';
			// $content .= $description;
			// $content .= '</td>' . $this->output->getEndRow(true);
		// }
		// return $content;
	// }

	/**
	 * Marks an item as seen (1) or hidden (3)
	 *
	 * @param  array  $item  the item array
	 * @param boolean $extra_values - Whether to insert/update extra fields in the seen table
	 * @param array $extra_values An array of other fields to update in the seen table
	 */
	function markAsSeen($item, $extra_values = false) { //Change this if more criteria is required
		
		if ($this->form_values['hide']) {
			$this->db_functions->setItemSeenStatus($item['id'], 3, true, $extra_values);
			$this->redirectBack();
			exit();
		} else if ($this->form_values['unhide']) {
			// false is to make it 'unsafe' which is the same as
			// just forcing it to update the value
			$this->db_functions->setItemSeenStatus($item['id'], 1, false, $extra_values);
			$this->redirectBack();
			exit();
		} else{
			$this->db_functions->setItemSeenStatus($item['id'], 1, true, $extra_values);
			$this->db_functions->setLog('view','id=' . $item['id'], $item['id']); //Syed 20050824
			
		}
	}
	
	/**
	 * Get the specified item, if it exists
	 *
	 * @param integer $id
	 * @return mixed false or the item (array)
	 * @author Phil Calder
	 */
	function getItem($id) {
		$item = false;
		if (is_object($this->db_functions)) {
			$item = $this->db_functions->getItem($id);
		}
		return $item;
	}
}
?>